IMPLEMENTATION MODULE KeyMaster;



(*======================================================================
  Hinweis: Diverse Prozeduren verstossen gegen mindestens 10 Grundregeln
           des sauberen Programmierens. Leider hat es sich als noetig
           erwiesen, die strengen Typkontrollen von M2 mit Hilfe von
           Zeigern stellenweise zu umgehen, soweit die Verschluesselung
           der Zertifikate betroffen ist, da die Laenge eines Daten-
           blockes beim Verschluesseln zunimmt! (Es lassen sich also
           keine festen Datenstrukturen dafuer deklarieren)
  ======================================================================*)



IMPORT FIO;
IMPORT InOut;
IMPORT IO;
IMPORT RSA;
IMPORT Storage;
IMPORT MathLong;
IMPORT LInOut;
IMPORT Lib;



VAR
 keyTable: ARRAY[0..maxKeys] OF RSA.Key;



(*======================================================================
   Prozedur DirtyRead
  ----------------------------------------------------------------------
   :Semantic.   Liest einen unstrukturierten Byteblock aus einer Datei.
   :Input.      f:   Eingabedatei
   :Input.      adr: Zeiger auf einen freien Speicherbereich von
   :Input.           ausreichender Laenge(!)
   :Input.      num: Anzahl an Bytes, die gelesen werden soll
   :Result.     -
  ======================================================================*)

PROCEDURE DirtyRead(f: FIO.File; adr: ADDRESS; num: CARDINAL);
 VAR
  bufp: POINTER TO ARRAY[0..dirtySize-1] OF CHAR;
  size: CARDINAL;
 BEGIN
  bufp:=adr;
  size:=FIO.RdBin(f,bufp^,num);
 END DirtyRead;



(*======================================================================
   Prozedur DirtyWrite
  ----------------------------------------------------------------------
   :Semantic.   Schreibt einen unstrukturierten Byteblock auf eine
   :Semantic.   Datei
   :Input.      f:   Ausgabedatei
   :Input.      adr: Zeiger auf den Anfang des Byteblocks
   :Input.      num: Anzahl an Bytes, die geschrieben werden soll
   :Result.     -
  ======================================================================*)

PROCEDURE DirtyWrite(f: FIO.File; adr: ADDRESS; num: CARDINAL);
 VAR
  bufp: POINTER TO ARRAY[0..dirtySize-1] OF CHAR;
 BEGIN
  bufp:=adr;
  FIO.WrBin(f,bufp^,num);
 END DirtyWrite;



(*======================================================================
   Prozedur PutMessageType
  ----------------------------------------------------------------------
   :Semantic.   schreibt den MessageType (Art-Identifikation fuer die
   :Semantic.   nachfolgende Message) auf eine Datei
   :Input.      f:  Ausgabedatei
   :Input.      mt: MessageType-Identifikator
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE PutMessageType(f: FIO.File; mt: MessageType);
 BEGIN
  DirtyWrite(f,ADR(mt),SIZE(mt));
 END PutMessageType;



(*======================================================================
   Prozedur GetMessageType
  ----------------------------------------------------------------------
   :Semantic.   liest das erste 16-Bit-Wort einer Datenuebertragung
   :Semantic.   (dieses Wort gibt an, welche Art von Message es ist)
   :Input.      f:  Datei, von der gelesen werden soll
   :Output.     mt: Typ der Message
   :Result.     -
  ======================================================================*)

PROCEDURE GetMessageType(f: FIO.File; VAR mt: MessageType);
 BEGIN
  DirtyRead(f,ADR(mt),SIZE(mt));
 END GetMessageType;



(*======================================================================
   Prozedur CheckStamp
  ----------------------------------------------------------------------
   :Semantic.   Gueltigkeit eines Zeitstempels durch Vergleich mit einem
   :Semantic.   Referenzstempel ueberpruefen
   :Input.      stamp:     zu ueberpruefender Stempel
   :Input.      reference: Referenz-Stempel
   :Output.     -
   :Result.     TRUE:  stamp ist gueltig
   :Result.     FALSE: stamp ist ungueltig
   :Remark.     im Moment nicht implementiert
  ======================================================================*)

PROCEDURE CheckStamp(stamp,reference: Stamp): BOOLEAN;
 BEGIN
  RETURN TRUE;
 END CheckStamp;



(*======================================================================
   Prozedur MakeStamp
  ----------------------------------------------------------------------
   :Semantic.   Zeitstempel anfertigen (abhaengig von Systemzeit)
   :Input.      -
   :Output.     stamp: Zeitstempel
   :Result.     -
  ======================================================================*)

PROCEDURE MakeStamp(VAR stamp: Stamp);
 VAR
  hrs,min,sec,hsec: CARDINAL;
  dow: Lib.DayType;
 BEGIN
  Lib.GetTime(hrs,min,sec,hsec);
  stamp.x1:=hrs*60+min;
  stamp.x2:=sec*100+hsec;
 END MakeStamp;



(*======================================================================
   Prozedur MakeSignature
  ----------------------------------------------------------------------
   :Semantic.   Elektronische Unterschrift anfertigen
   :Semantic.   Verfahren: Zeitstempel mit Private-Key verschluesseln
   :Input.      in:      zu verschluesselnder Stempel
   :Input.      private: zu verwendender Private Key
   :Output.     out:     elektron. unterschriebener Zeitstempel
  ======================================================================*)

PROCEDURE MakeSignature(in: Stamp; private: RSA.Key; out: ADDRESS): INTEGER;
 VAR
  size: INTEGER;
 BEGIN
  (* willkuerliche Vorgabe einer Maximallaenge; sollte meistens reichen *)
  size:=5*SIZE(in);
  RSA.CodeByteBlock(ADR(in),SIZE(in),out,size,private);
  RETURN size;
 END MakeSignature;



PROCEDURE DumpInts(T:ARRAY OF CHAR;a: ADDRESS; n: INTEGER);
 TYPE
  iapt=POINTER TO ARRAY[0..9999] OF INTEGER;
 VAR
  i,z: INTEGER;
  iap: iapt;
 BEGIN
(*  IO.RedirectOutput("PRN"); *)
  InOut.WriteLn;InOut.WriteString(T);InOut.WriteLn;InOut.WriteLn;
  iap:=a;
  z:=0;
  FOR i:=0 TO n-1 DO
   InOut.WriteInt(iap^[i],0);
   InOut.WriteString("  ");
   INC(z);
   IF z=10 THEN z:=0; InOut.WriteLn; END;
  END;
  InOut.WriteLn;
  InOut.WriteString("---------------------------------------------");
  InOut.WriteLn;
(*  IO.RedirectOutput("CON"); *)
 END DumpInts;


(*======================================================================
   Prozedur CopyByteBlock
  ----------------------------------------------------------------------
   :Semantic.   eine bestimmte Anzahl Bytes im Hauptspeicher kopieren
   :Input.      s: Zeiger auf den Quell-Block
   :Input.      d: Zeiger auf den Ziel-Block
   :Input.      n: Anzahl Bytes, die kopiert werden sollen
   :Output.     -
   :Result.     -
   :Remark.     enthaelt keinerlei Sicherheitsabfragen!!!
   :Remark.     "s" und "d" sind vom Typ ADDRESS; intern wird zur
   :Remark.     Sicherheit ein Typ POINTER TO ARRAY OF BYTE benutzt, um
   :Remark.     in jedem Fall von der Wortlaenge der Maschine
   :Remark.     unabhaengig zu sein.
  ======================================================================*)

PROCEDURE CopyByteBlock(s,d: ADDRESS; n: INTEGER);
 VAR
  s1,d1: DirtyPtr;
  i: INTEGER;
 BEGIN
  s1:=s;
  d1:=d;
  FOR i:=0 TO n-1 DO
   d1^[i]:=s1^[i];
  END;
 END CopyByteBlock;



(*======================================================================
   Prozedur GetPublicKey
  ----------------------------------------------------------------------
   :Semantic.   oeffentlichen Schluessel eines Kommunikationspartners
   :Semantic.   aus der Schluesseltabelle holen
   :Input.      id: Teilnehmernummer des Kommunikationspartners
   :Output.     pk: zugehoeriger oeffentlicher Schluessel
   :Result.     -
  ======================================================================*)

PROCEDURE GetPublicKey(id: INTEGER; VAR pk: RSA.Key);
 BEGIN
  pk:=keyTable[id];
 END GetPublicKey;



(*======================================================================
   Prozedur AddPublicKey
  ----------------------------------------------------------------------
   :Semantic.   oeffentlichen Schluessel eines Kommunikationspartners
   :Semantic.   in die Schluesseltabelle eintragen
   :Input.      id: Teilnehmernummer des Kommunikationspartners
   :Input.      pk: zugehoeriger oeffentlicher Schluessel
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE AddPublicKey(id: INTEGER; pk: RSA.Key);
 BEGIN
  keyTable[id]:=pk;
 END AddPublicKey;



(*======================================================================
   Prozedur ExamineKeyRequest
  ----------------------------------------------------------------------
   :Semantic.   Plausibilitaetspruefung der in kr enthaltenen Daten
   :Input.      kr: Schluessel-Anforderung durch einen Teilnehmer
   :Input.          in unverschluesselter Darstellungsweise(!!!)
   :Output.     -
   :Result.     TRUE:  kr darf akzeptiert werden
   :Result.     FALSE: kr muss zurueckgewiesen werden
   :Remark.     Nur rudimentaer implementiert
  ======================================================================*)

PROCEDURE ExamineKeyRequest(kr: KeyRequestData): BOOLEAN;
 VAR
  dummy: RSA.Key;
  now: Stamp;
 BEGIN
  (* Plausibilitaet des "stamp" pruefen *)
  RETURN CheckStamp(kr.stamp,now);
 END ExamineKeyRequest;



(*======================================================================
   Prozedur BuildCertificate
  ----------------------------------------------------------------------
   :Semantic.   Ein Zertifikat erstellen und unterschreiben
   :Input.      id:   Teilnehmernummer
   :Input.      key:  Schluessel, der in das Zertifikat eingetragen
   :Input.            werden soll
   :Input.      sig:  Schluessel, mit dem das Zertifikat unterschrieben
   :Input.            werden soll
   :Input.      cert: leerer Speicherblock
   :Output.     cert: fertiges Zertifikat
   :Result.     Laenge der digitalen Unterschrift in Bytes
  ======================================================================*)

PROCEDURE BuildCertificate(id: INTEGER; key,sig: RSA.Key; VAR cert: Dirty): INTEGER;
 VAR
  ce: Certificate;
  size: INTEGER;
  c1: POINTER TO Certificate;
 BEGIN
  c1:=ADR(cert);
  ce.dest:=id;
  ce.key:=key;
  MakeStamp(ce.stamp);

  (* Daten von Zertifikat "ce" in Dirty-Block "cert" kopieren *)
  CopyByteBlock(ADR(ce),ADR(cert),SIZE(ce));

  (* Dirty-Block "cert" elektronisch unterschreiben *)
  (* size ist die Laenge der Unterschrift 1 *)
  size:=MakeSignature(ce.stamp,sig,ADR(c1^.signature));
  RETURN size;
 END BuildCertificate;


(*======================================================================
   Prozedur BuildKeyMessage
  ----------------------------------------------------------------------
   :Semantic.   Antwort auf KeyRequest zusammensetzen und
   :Semantic.   verschluesseln
   :Input.      kr:   Schluessel-Anforderung durch einen Teilnehmer
   :Input.            in unverschluesselter Darstellungsweise
   :Input.      sig:  Schluessel, mit dem die elektronischen Unter-
   :Input.            schriften angefertigt werden sollen.
   :Output.     keym: KeyMessage, die an den anfordernden Kommunikations-
   :Output.           partner geschickt werden soll (verschluesselt).
   :Output.     l:    Laenge der keymessage
   :Result.     TRUE:  Daten sind gueltig
   :Result.     FALSE: Daten sind ungueltig
  ======================================================================*)

PROCEDURE BuildKeyMessage(VAR kr: KeyRequestData; VAR keym: Dirty;
                          VAR sig: RSA.Key; VAR l: INTEGER): BOOLEAN;
 VAR
  s,d: RSA.Key;
  cert1,cert2,temp: Dirty;
  size1,size2,size3,size4,size5: INTEGER;
  km: KeyMessage;
 BEGIN
  IF ExamineKeyRequest(kr) THEN
   GetPublicKey(kr.source,s);
   GetPublicKey(kr.dest,d);
   size1:=BuildCertificate(kr.dest,d,sig,cert1);
   (* size3 ist die Laenge des ersten Zertifikates inkl. Unterschrift 1 *)
   size3:=SIZE(Certificate)-SIZE(Stamp)+size1;
(*   DumpInts("Cert.1 vor Kodierung",ADR(cert1),SIZE(Certificate));  *)

   size2:=BuildCertificate(kr.source,s,sig,cert2);
   (* size4 ist die Laenge des zweiten Zertifikates inkl. Unterschrift 2 *)
   size4:=SIZE(Certificate)-SIZE(Stamp)+size2;

   (* Daten des Dirty-Blockes "cert2" in "temp" kopieren und codieren *)
   size5:=SIZE(temp);
   RSA.CodeByteBlock(ADR(cert2),size4,ADR(temp),size5,d);
   (* size5 ist die Laenge des kodierten zweiten Zertifikates *)
(*   DumpInts("Cert.2 vor Kodierung",ADR(cert2),SIZE(Certificate)); *)

   km.sizeAdc:=size3;
   km.sizeBco:=size5;
   km.sizeBdc:=size4;
   CopyByteBlock(ADR(cert1),ADR(km.data),size3);
   CopyByteBlock(ADR(temp),ADR(km.data[size3]),size5);
   l:=SIZE(keym);
   RSA.CodeByteBlock(ADR(km),
             size3+size5+SIZE(km.sizeAdc)+SIZE(km.sizeBco)+SIZE(km.sizeBdc),
             ADR(keym),l,s);

   RETURN TRUE;
  ELSE
   RETURN FALSE;
  END;
 END BuildKeyMessage;



(*======================================================================
   Prozedur PutKeyMessageB
  ----------------------------------------------------------------------
   :Semantic.   simple Interface-Prozedur, die zuerst den richtigen
   :Semantic.   Message-Typ und dann die Message selbst schreibt.
   :Input.      out:     Ausgabedatei
   :Input.      kmb:     Verschluesseltes zweites Zertifikat
   :Input.      kmbsize: Laenge des zweiten Zertifikates in Bytes
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE PutKeyMessageB(out: FIO.File; VAR kmb: Dirty; kmbsize: INTEGER);
 BEGIN
  PutMessageType(out,mtKeyMessageB);
  DirtyWrite(out,ADR(kmb),kmbsize);
 END PutKeyMessageB;



(*======================================================================
   Prozedur DecodeKeyMessageAB
  ----------------------------------------------------------------------
   :Semantic.   Eine KeyMessage bestehend aus ZWEI Zertifikaten in ein
   :Semantic.   unkodiertes erstes und ein kodiertes zweites Zertifikat
   :Semantic.   zerlegen.
   :Input.      keymco: KeyMessage aus zwei Zertifikaten
   :Input.      mykey:  private key des Entschluesselnden
   :Input.      cpkey:  public key der Zentrale
   :Output.     certA:  entschluesseltes erstes Zertifikat
   :Output.     certB:  teilweise entschluesseltes zweites Zertifikat
   :Output.     sizeB   Laenge des zweiten Zertifikates in Bytes
   :Result.     -
  ======================================================================*)

PROCEDURE DecodeKeyMessageAB(VAR keymco: Dirty;
                             VAR mykey,cpkey: RSA.Key;
                             VAR certA: Certificate;
                             VAR certB: Dirty;
                             VAR sizeB: INTEGER);
 VAR
  keymdc: KeyMessage;
  ce: POINTER TO Certificate;
 BEGIN
  ce:=ADR(keymdc.data);
  RSA.DeCodeByteBlock(ADR(keymco),ADR(keymdc),mykey);
  CopyByteBlock(ce,ADR(certA),SIZE(Certificate));
  RSA.DeCodeByteBlock(ADR(ce^.signature),ADR(certA.signature),cpkey);
  CopyByteBlock(ADR(keymdc.data[keymdc.sizeAdc]),ADR(certB),keymdc.sizeBco);
  sizeB:=keymdc.sizeBco;
 END DecodeKeyMessageAB;



(*======================================================================
   Prozedur DecodeKeyMessageB
  ----------------------------------------------------------------------
   :Semantic.   Eine Keymessage bestehend aus einem einzelnen Zertifikat
   :Semantic.   entschluesseln
   :Input.      keymco: verschluesselte KeyMessage (=zweites Zertifikat)
   :Input.      mykey:  private key des Entschluesselnden
   :Input.      cpkey:  public key der Zentrale
   :Output.     certB:  Entschluesseltes (zweites) Zertifikat
   :Result.     -
  ======================================================================*)

PROCEDURE DecodeKeyMessageB(VAR keymco: Dirty;
                            VAR mykey,cpkey: RSA.Key;
                            VAR certB: Certificate);
 VAR
  keymdc: Dirty;
  ce: POINTER TO Certificate;
 BEGIN
  ce:=ADR(keymdc);
  RSA.DeCodeByteBlock(ADR(keymco),ADR(keymdc),mykey);
  CopyByteBlock(ce,ADR(certB),SIZE(Certificate));
  RSA.DeCodeByteBlock(ADR(ce^.signature),ADR(certB.signature),cpkey);
 END DecodeKeyMessageB;



(*======================================================================
   Prozedur BuildDupMsg
  ----------------------------------------------------------------------
   :Semantic.   Eine Duplikat-Message erzeugen (Kontrollmeldung, ist
   :Semantic.   identisch mit einem Zertifikat)
   :Input.      id:       Teilnehmernummer
   :Input.      pk:       Schluessel, der eingetragen werden soll
   :Input.      sig:      Schluessel, mit dem unterschrieben werden soll
   :Input.      key:      Schluessel, mit dem verschluesselt werden soll
   :Output.     dmsg:     fertige DupMessage (=Zertifikat)
   :Output.     dmsgsize: Laenge der fertigen DupMessage in Bytes
   :Result.     -
  ======================================================================*)

PROCEDURE BuildDupMsg(id: INTEGER;
                      VAR pk: RSA.Key;
                      VAR sig: RSA.Key;
                      VAR key: RSA.Key;
                      VAR dmsg: Dirty;
                      VAR dmsgsize: INTEGER);
 VAR
  dm: Certificate;
  temp: Dirty;
  dmp: POINTER TO Certificate;
  sigsize,outsize: INTEGER;
 BEGIN
  dmp:=ADR(temp);
  dm.dest:=id;
  dm.key:=pk;
  MakeStamp(dm.stamp);
  CopyByteBlock(ADR(dm),ADR(temp),SIZE(dm));
  sigsize:=MakeSignature(dm.stamp,sig,ADR(dmp^.signature));
  dmsgsize:=SIZE(dm)-SIZE(dm.signature)+sigsize;
  outsize:=SIZE(dmsg);
  RSA.CodeByteBlock(ADR(temp),dmsgsize,ADR(dmsg),outsize,key);
  dmsgsize:=outsize;
 END BuildDupMsg;



(*======================================================================
   Prozedur SendDupMsg
  ----------------------------------------------------------------------
   :Semantic.   simple Interface-Prozedur, schreibt zuerst korrekten
   :Semantic.   MessageType, dann Message selbst auf Datei.
   :Input.      out:      Ausgabedatei
   :Input.      dmsg:     DupMessage
   :Input.      dmsgsize: Laenge der DupMessage in Bytes
   :Result.     -
  ======================================================================*)

PROCEDURE SendDupMsg(out: FIO.File; VAR dmsg: Dirty; dmsgsize: INTEGER);
 BEGIN
  PutMessageType(out,mtDupMessage);
  DirtyWrite(out,ADR(dmsg),dmsgsize);
 END SendDupMsg;



(*======================================================================
   Prozedur DecodeDupMsg
  ----------------------------------------------------------------------
   :Semantic.   eine DupMessage (Kontrollmeldung, ist identisch mit
   :Semantic.   einem Zertifikat) entschluesseln
   :Input.      sig:  Schluessel, mit dem die Unterschrift
   :Input.            entschluesselt werden soll.
   :Input.      key:  Schluessel, mit dem die Message entschluesselt
   :Input.            werden soll
   :Input.      dmsg: DupMessage
   :Output.     dm:   daraus gewonnenes Zertifikat
   :Result.     -
  ======================================================================*)

PROCEDURE DecodeDupMsg(VAR sig: RSA.Key;
                       VAR key: RSA.Key;
                       VAR dmsg: Dirty;
                       VAR dm: Certificate);
 VAR
  temp: Dirty;
  dmp: POINTER TO Certificate;
  dmsgsize,sigsize,outsize: INTEGER;
 BEGIN
  RSA.DeCodeByteBlock(ADR(dmsg),ADR(temp),key);
  dmp:=ADR(temp);
  CopyByteBlock(ADR(temp),ADR(dm),SIZE(dm));
  RSA.DeCodeByteBlock(ADR(dmp^.signature),ADR(dm.signature),sig);
 END DecodeDupMsg;



(*======================================================================
   Prozedur HandleRegistration
  ----------------------------------------------------------------------
   :Semantic.   eine Teilnehmeranmeldung durch die Zentrale behandeln
   :Input.      inf:   Eingabedatei
   :Input.      mypub: public Key der Zentrale
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE HandleRegistration(inf: FIO.File; VAR mypub: RSA.Key);
 VAR
  reg: Registration;
 BEGIN
  DirtyRead(inf,ADR(reg),SIZE(reg));
  SendOwnKey(reg.id,mypub);
  AddPublicKey(reg.id,reg.pub);
 END HandleRegistration;



(*======================================================================
   Prozedur SendKeyMessage
  ----------------------------------------------------------------------
   :Semantic.   KeyRequest lesen und entschluesseln
   :Semantic.   Antwort auf KeyRequest zusammenstellen und abschicken
   :Input.      inf: Eingabedatei
   :Input.      pri: privater Schluessel der SVZ
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE SendKeyMessage(inf: FIO.File; VAR pri: RSA.Key);
 VAR
  size: INTEGER;
  kr: KeyRequestData;
  dirt,keym: Dirty;
  out: FIO.File;
 BEGIN
  DirtyRead(inf,ADR(dirt),SIZE(dirt));
  RSA.DeCodeByteBlock(ADR(dirt),ADR(kr),pri);
  IF BuildKeyMessage(kr,keym,pri,size) THEN
   IF kr.source=1 THEN
    out:=FIO.Create("AIN.DAT");
   ELSIF kr.source=2 THEN
    out:=FIO.Create("BIN.DAT");
   END;
   PutMessageType(out,mtKeyMessageAB);
   DirtyWrite(out,ADR(keym),size);
   FIO.Close(out);
  END;
 END SendKeyMessage;



(*======================================================================
   Prozedur SendKeyRequest
  ----------------------------------------------------------------------
   :Semantic.   KeyRequest zusammenstellen und abschicken
   :Input.      sid: id des aktiven Teilnehmers (Absender)
   :Input.      did: id des passiven Teilnehmers (gewuenschter Empf.)
   :Input.      pub: oeffentlicher Schluessel der SVZ
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE SendKeyRequest(sid,did: INTEGER; pub: RSA.Key);
 VAR
  kr: KeyRequestData;
  block: Dirty;
  bs: INTEGER;
  out: FIO.File;
 BEGIN
  kr.source:=sid;
  kr.dest:=did;
  MakeStamp(kr.stamp);
  bs:=SIZE(block);
  RSA.CodeByteBlock(ADR(kr),SIZE(kr),ADR(block),bs,pub);
  out:=FIO.Create("CIN.DAT");
  PutMessageType(out,mtKeyRequest);
  DirtyWrite(out,ADR(block),bs);
  FIO.Close(out);
 END SendKeyRequest;



(*======================================================================
   Prozedur SendRegistration;
  ----------------------------------------------------------------------
   :Semantic.   sich bei der SVZ anmelden
   :Input.      dest: ID des Teilnehmers
   :Input.      pk: oeffentlicher Schluessel des Teilnehmers
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE SendRegistration(id: INTEGER; VAR pub: RSA.Key);
 VAR
  reg: Registration;
  out: FIO.File;
 BEGIN
  reg.id:=id;
  reg.pub:=pub;
  out:=FIO.Create("CIN.DAT");
  PutMessageType(out,mtRegistration);
  DirtyWrite(out,ADR(reg),SIZE(reg));
  FIO.Close(out);
 END SendRegistration;



(*======================================================================
   Prozedur SendOwnKey;
  ----------------------------------------------------------------------
   :Semantic.   oeffentlichen Schluessel der SVZ verschicken
   :Input.      dest: id des Empfaengers
   :Input.      pk:   public key der Zentrale
   :Output.     -
   :Result.     -
  ======================================================================*)

PROCEDURE SendOwnKey(dest: INTEGER; VAR pk: RSA.Key);
 VAR
  out: FIO.File;
 BEGIN
  IF dest=1 THEN
   out:=FIO.Create("AIN.DAT");
  ELSIF dest=2 THEN
   out:=FIO.Create("BIN.DAT");
  END;
  PutMessageType(out,mtRegReply);
  DirtyWrite(out,ADR(pk),SIZE(pk));
  FIO.Close(out);
 END SendOwnKey;



END KeyMaster.