(**********************************************************************

    :Program.    MathLong
    :Contents.   Rechnen mit sehr langen natrlichen Zahlen
    :Author.     Steffen Reith
    :Address.    Hessenstr. 64, D-8700 Wrzburg
    :Phone.      0931/281859
    :Language.   Modula-2
    :Translator. TopSpeed Modula2 / M2Amiga A+L V3.32d
    :History.    V1.0  17. Nov 1990
    :History.    V1.0a 25. Dez 1990
    :History.    V1.1   6. Feb 1991
    :History.    V1.2  10. Feb 1991
    :History.    V1.3  26. Feb 1991
    :History.    V1.4   3. Mrz 1991
    :History.    V1.41 13. Mrz 1991
    :History.    V1.42 26. Mrz 1991
    :History.    V1.5  28. Mai 1991
    :Remarks.    Sollte es Probleme geben mit der Definition oder
                 werden noch andere Routinen unbedingt bentigt
                 dann ruf doch mal an.

**********************************************************************)
IMPLEMENTATION MODULE MathLong;

FROM Storage    IMPORT ALLOCATE,DEALLOCATE;
FROM InOut      IMPORT WriteString,WriteLn;
FROM MathLib0   IMPORT ln;
FROM Str        IMPORT Length;
FROM FIO        IMPORT File;
FROM Lib        IMPORT IncAddr;
FROM SYSTEM     IMPORT BYTE;

IMPORT FIO;
IMPORT ASCII;
IMPORT InOut;

CONST NBits=8*SIZE(BITSET);      (* Anzahl der Bits im einem BITSET          *)
      MSBNr=NBits-1;             (* Nr. des MSB                              *)
      MSB=BITSET{MSBNr};         (* Most Significant Bit                     *)
      HighBitNr=NBits-2;         (* Nr. des hchsten Bits                    *)
      HighBit=BITSET{HighBitNr}; (* Hchstes Bit in dieser Zahlendarstellung *)
      LSBNr=0;                   (* Nr. des LSB                              *)
      LSB=BITSET{LSBNr};         (* Least Significant Bit                    *)
      EndeMarke=ASCII.nul;       (* Dieses Zeichen beendet einen String      *)


(*--------------------------------------------------------------------------------
   Die Konstante MaxNr aus dem Definitionsmodul legt die maximale Lnge einer
   LONG-Zahl fest !!!
--------------------------------------------------------------------------------*)


TYPE L=ARRAY [0..MaxNr] OF INTEGER;
     B=ARRAY [0..MaxNr] OF BITSET; (* unsauber aber vereinfacht Bitoperationen *)
(*
   Ich gehe hier davon aus, da BITSET und INTEGER den gleichen Speicherbedarf
   haben. Dies wird im Rumpf getestet und bei Ungleichheit wird das Programm
   mit einem HALT beendet.
*)
     LONG=POINTER TO L;
     BLONG=POINTER TO B;


(* ---- Die Variablen fr die Zahlenkonstanten mit DEF-Modul ---- *)

VAR INull:L;      (* Variable fr LONG(0)    *)
    IEins:L;      (* Variable fr LONG(1)    *)
    IZwei:L;      (* Variable fr LONG(2)    *)
    IZehn:L;      (* Variable fr LONG(10)   *)
    ISechZehn:L;  (* Variable fr LONG(16)   *)
    IHundert:L;   (* Variable fr LONG(100)  *)
    ITausend:L;   (* Variable fr LONG(1000) *)


PROCEDURE ShiftRight(a:LONG); (* a:=a DIV 2 *)

VAR i:INTEGER;   (* Universalvariable *)
    Len:INTEGER; (* Lnge von a       *)
    aB:BLONG;    (* Spiegelbild von a *)

BEGIN
 aB:=BLONG(a);

 (* ---- Lnge der Zahl a ermitteln ---- *)
 Len:=a^[0];

 (* ---- Komponentenweise rechts schieben ---- *)
 FOR i:=1 TO Len-1 DO
  a^[i]:=a^[i] DIV 2; (* Compiler sollte sowas in Schiebebefehle umsetzen *)
  (* ---- Hchstes verwendbares Bit setzen ---- *)
  IF ODD(a^[i+1]) THEN
   INCL(aB^[i],HighBitNr)
  END
 END;

 a^[Len]:=a^[Len] DIV 2; (* Compiler sollte sowas in Schiebebefehle umsetzen *)
 (* ---- Wird Datenwort krzer ???? ---- *)
 IF ((a^[Len]=0) AND (Len>1)) THEN DEC(a^[0]) END

END ShiftRight;

PROCEDURE ShiftLeft(a:LONG);

VAR i:INTEGER;   (* Universalvariable *)
    Len:INTEGER; (* Lnge von a       *)
    aB:BLONG;    (* Spiegelbild von a *)

BEGIN

 aB:=BLONG(a);

 (* ---- Lnge der Zahl a ermitteln ---- *)
 Len:=a^[0];

 (*# check(overflow => off) *)
 a^[1]:=2*a^[1]; (* Compiler sollte sowas in Schiebebefehle umsetzen *)
 (*# check(overflow => on) *)

 (* ---- Komponentenweise links schieben ---- *)
 FOR i:=2 TO Len DO
  (*# check(overflow => off) *)
  a^[i]:=2*a^[i]; (* links schieben *)
  (*# check(overflow => on) *)
  IF (a^[i-1]<0) THEN
   EXCL(aB^[i-1],MSBNr);
   INCL(aB^[i],LSBNr)
  END
 END;

 (* ---- Verlngerung notwendig ???? ---- *)
 IF (a^[Len]<0) THEN
  IF Len=MaxNr THEN
   Carry:=TRUE
  ELSE
   EXCL(aB^[Len],MSBNr);
   a^[Len+1]:=1;
   INC(a^[0])
  END
 END

END ShiftLeft;

(*

Anmerkung:
Die Funktionen GE,LE,NE knnten mit Hilfe der Funktionen LT,GT,EQ for-
muliert werden.

PROCEDURE GE...

BEGIN
 RETURN (NOT LT...)
END GE;

Auf  diese Mglichkeit wird hier verzichtet,  weil eine erneutete ber-
gabe  von  Parametern  ber den Stack evtl.  eine  Laufzeitverlngerung
bewirken  knnte,  wenn diese Routinen oft verwendet werden.   Auerdem
braucht dieses Modul nicht auf kleinen Speicherbedarf optimiert werden.

*)

PROCEDURE EQ(a,b:LONG):BOOLEAN; (* a=b *)

VAR i:INTEGER;

BEGIN
 (* ---- Wenn ungleich lang, dann a#b ---- *)
 IF a^[0]#b^[0] THEN RETURN FALSE END;

 (* ---- Komponenten einzeln vergleichen ---- *)
 i:=1;
 REPEAT
  IF a^[i]#b^[i] THEN RETURN FALSE END;
  INC(i)
 UNTIL (i>a^[0]);
 RETURN TRUE

END EQ;


PROCEDURE GT(a,b:LONG):BOOLEAN; (* a>b *)

VAR i:INTEGER;

BEGIN
 (* ---- Wenn a lnger als b dann a>b ---- *)
 IF (a^[0]>b^[0]) THEN
  RETURN TRUE
 (* ---- Wenn b lnger als a dann a<b ---- *)
 ELSIF (a^[0]<b^[0]) THEN
  RETURN FALSE
 ELSE
 (* ---- Jede Komponente testen ---- *)
  i:=a^[0];
  WHILE (i>=1) DO
   IF (a^[i]>b^[i]) THEN RETURN TRUE  END;
   IF (a^[i]<b^[i]) THEN RETURN FALSE END;
   DEC(i)
  END;
  RETURN FALSE
 END
END GT;


PROCEDURE LT(a,b:LONG):BOOLEAN; (* a<b *)

VAR i:INTEGER;

BEGIN
 (* ---- Wenn b lnger als a dann a<b ---- *)
 IF (a^[0]<b^[0]) THEN
  RETURN TRUE
 (* ---- Wenn a lnger als b dann a>b ---- *)
 ELSIF (a^[0]>b^[0]) THEN
  RETURN FALSE
 ELSE
 (* ---- Jede Komponente testen ---- *)
  i:=a^[0];
  WHILE (i>=1) DO
   IF (a^[i]<b^[i]) THEN RETURN TRUE  END;
   IF (a^[i]>b^[i]) THEN RETURN FALSE END;
   DEC(i)
  END;
  RETURN FALSE
 END
END LT;


PROCEDURE NE(a,b:LONG):BOOLEAN; (* a#b *)

VAR i:INTEGER;
    Len:INTEGER; (* Maximale Lnge der Zahlen a und b *)

BEGIN
 IF (a^[0]#b^[0]) THEN RETURN TRUE END;

 (* ---- Komponenten einzeln vergleichen ---- *)
 i:=1;
 REPEAT
  IF (a^[i]#b^[i]) THEN RETURN TRUE END;
  INC(i)
 UNTIL (i>a^[0]);
 RETURN FALSE

END NE;


PROCEDURE GE(a,b:LONG):BOOLEAN; (* a>=b *)

VAR i:INTEGER;

BEGIN
 (* ---- Wenn a lnger als b dann a>=b ---- *)
 IF (a^[0]>b^[0]) THEN
  RETURN TRUE
 (* ---- Wenn b lnger als a dann a<b ---- *)
 ELSIF (a^[0]<b^[0]) THEN
  RETURN FALSE
 ELSE
 (* ---- Jede Komponente testen ---- *)
  i:=a^[0];
  WHILE (i>=1) DO
   IF (a^[i]>b^[i]) THEN RETURN TRUE  END;
   IF (a^[i]<b^[i]) THEN RETURN FALSE END;
   DEC(i)
  END;
  RETURN TRUE
 END
END GE;


PROCEDURE LE(a,b:LONG):BOOLEAN; (* a>=b *)

VAR i:INTEGER;

BEGIN
 (* ---- Wenn a lnger als b dann a>b ---- *)
 IF (a^[0]>b^[0]) THEN
  RETURN FALSE
 (* ---- Wenn b lnger als a dann a<=b ---- *)
 ELSIF (a^[0]<b^[0]) THEN
  RETURN TRUE
 ELSE
 (* ---- Jede Komponente testen ---- *)
  i:=a^[0];
  WHILE (i>=1) DO
   IF (a^[i]>b^[i]) THEN RETURN FALSE END;
   IF (a^[i]<b^[i]) THEN RETURN TRUE  END;
   DEC(i)
  END;
  RETURN TRUE
 END
END LE;


(* ---- Diese Routine testet ob a nicht Null ist ---- *)
PROCEDURE GN(a:L):BOOLEAN;

BEGIN
 RETURN NOT((a[0]=1) AND (a[1]=0))
END GN;


PROCEDURE AllocNum():LONG;

VAR Help:LONG;
   i:INTEGER;

BEGIN
 ALLOCATE(Help,SIZE(L));               (* Speicher reservieren          *)
 IF Help=NIL THEN RETURN NIL END;      (* Fehler zurckgeben            *)
 Help^[0]:=1;                          (* Die Null ist ein Element lang *)
 Help^[1]:=0;                          (* Zahl mit 0 initialisieren     *)
 FOR i:=2 TO MaxNr DO Help^[i]:=1 END; (* den Rest mit 1en auffllen    *)
 RETURN Help
END AllocNum;


PROCEDURE FreeNum(Num:LONG);

BEGIN
 DEALLOCATE(Num,SIZE(Num))             (* Speicher freigeben            *)
END FreeNum;

PROCEDURE Add(a,b,c:LONG);

VAR i:INTEGER;    (* Universalzhlvariable                    *)
    max:INTEGER;  (* Maximale Lnge der Zahlen a und b        *)
    cB:BLONG;     (* Diese Variable ist das Spiegelbild von c *)

BEGIN
 (* ---- Initialisierungen ---- *)
 Carry:=FALSE;
 cB:=BLONG(c);

 (* ---- Differenzstellen auffllen ---- *)
 IF a^[0]>b^[0] THEN
  FOR i:=b^[0]+1 TO a^[0] DO
   b^[i]:=0; (* Alle unbenutzten Werte lschen *)
  END;
  max:=a^[0]
 ELSE
  FOR i:=a^[0]+1 TO b^[0] DO
   a^[i]:=0; (* Alle unbenutzten Werte lschen *)
  END;
  max:=b^[0]
 END;

 (* ---- Erste Komponente addieren ---- *)
 (*# check(overflow => off) *)
 c^[1]:=a^[1]+b^[1];
 (*# check(overflow => on) *)

 (* ---- Den Rest des Vektors komponentenweise addieren ---- *)
 FOR i:=2 TO max DO
  IF (c^[i-1]<0) THEN (* berlauf beim letzten Addieren *)
   EXCL(cB^[i-1],MSBNr);
   (*# check(overflow => off) *)
   c^[i]:=a^[i]+b^[i]+1;
   (*# check(overflow => on) *)
  ELSE
   (*# check(overflow => off) *)
   c^[i]:=a^[i]+b^[i];
   (*# check(overflow => on) *)
  END
 END;

 (* ---- Lnge des Ergebnisvektors eintragen ---- *)
 c^[0]:=max;

 (* ---- Evtl. berlauf beim letzten Durchgang bercksichtigen ---- *)
 IF (c^[max]<0) THEN
  cB^[max]:=cB^[max]-MSB;
  IF max=MaxNr THEN
   Carry:=TRUE
  ELSE
   c^[max+1]:=1;
   INC(c^[0])
  END
 END

END Add;


PROCEDURE Inc(a:LONG;n:INTEGER);

VAR i:INTEGER;    (* Universalzhlvariable *)
    max:INTEGER;  (* Maximale Lnge der Zahlen a und b *)
    aB:BLONG;     (* Diese Variable ist das Spiegelbild von c *)

BEGIN
 (* ---- Initialisierungen ---- *)
 Carry:=FALSE;
 aB:=BLONG(a);
 max:=a^[0];

 (* ---- Erste Komponente inkrementieren ---- *)
 (*# check(overflow => off) *)
 INC(a^[1],n);
 (*# check(overflow => on) *)

 (* ---- Den Rest des Vektors auf berlauf checken ---- *)
 i:=2;
 LOOP
  IF i>max THEN EXIT END;
  IF (a^[i-1]<0) THEN (* berlauf beim letzten Addieren *)
   EXCL(aB^[i-1],MSBNr);
   (*# check(overflow => off) *)
   INC(a^[i]);
   (*# check(overflow => on) *)
  ELSE
   RETURN
  END;
  INC(i)
 END;

 (* ---- Evtl. berlauf beim letzten Inkrement bercksichtigen ---- *)
 IF (a^[i-1]<0) THEN
  EXCL(aB^[i-1],MSBNr);
  IF max=MaxNr THEN
   Carry:=TRUE
  ELSE
   a^[i]:=1;
   INC(a^[0])
  END
 END

END Inc;


PROCEDURE Sub(a,b,c:LONG);

VAR i:INTEGER;    (* Universalzhlvariable *)
    cB:BLONG;     (* Diese Variable ist das Spiegelbild von c *)

BEGIN
 (* ---- Initialisierungen ---- *)
 Carry:=FALSE;
 cB:=BLONG(c);

 (* ---- Differenzstellen auffllen ---- *)
 FOR i:=b^[0]+1 TO a^[0] DO b^[i]:=0 END;

 (* ---- Erste Komponente subtrahieren ---- *)
 (*# check(overflow => off) *)
 c^[1]:=a^[1]-b^[1];
 (*# check(overflow => on) *)

 (* ---- Den Rest des Vektors komponentenweise subtrahieren ---- *)
 i:=2; (* FOR i:=2 TO a^[0] DO *)
 WHILE (i<=a^[0]) DO
  IF (c^[i-1]<0) THEN
   EXCL(cB^[i-1],MSBNr);
   (*# check(overflow => off) *)
   c^[i]:=a^[i]-b^[i]-1;
   (*# check(overflow => on) *)
  ELSE
   (*# check(overflow => off) *)
   c^[i]:=a^[i]-b^[i];
   (*# check(overflow => on) *)
  END;
  INC(i)
 END;
 DEC(i);

 (* ---- war a<b ??? ---- *)
 IF (c^[i]<0) THEN
  Carry:=TRUE
 END;

 (* ---- Lnge des Ergebnisses bestimmen ---- *)
 WHILE ((c^[i]=0) AND (i>1)) DO DEC(i) END;
 c^[0]:=i;

END Sub;


PROCEDURE Mult(a,b,c:LONG); (* a*b *)

VAR x,y,z:L; (* Hilfsvektoren: wg. Geschwindigkeit nicht alloziert *)

BEGIN

 z[0]:=1;z[1]:=0;
 (* ---- Tauschen, um minimal viele Rotationen zu brauchen ---- *)
 IF GT(a,b) THEN
  x:=a^;
  y:=b^
 ELSE
  x:=b^;
  y:=a^
 END;

 WHILE GN(y) DO
  IF ODD(y[1]) THEN Add(ADR(z),ADR(x),ADR(z)) END;
  ShiftLeft(ADR(x));
  ShiftRight(ADR(y));
 END;
 c^:=z

END Mult;


(*
 Achtung diese Unterroutine erwartet, dass a und b Element des
 Restklassenrings Z/(n) ist. D.h  0 <= a < (n-1) und 0 <= b < (n-1).

*)


PROCEDURE MultMod(a,b,n,c:LONG); (* a*b mod n*)

VAR x,y,z:L; (* Hilfsvektoren: wg. Geschwindigkeit nicht alloziert *)

BEGIN

 z[0]:=1;z[1]:=0;
 (* ---- Tauschen, um minimal viele Rotationen zu brauchen ---- *)
 IF GT(a,b) THEN
  x:=a^;
  y:=b^
 ELSE
  x:=b^;
  y:=a^
 END;

 WHILE GN(y) DO
  IF ODD(y[1]) THEN
   Add(ADR(z),ADR(x),ADR(z));
   IF GE(ADR(z),n) THEN Sub(ADR(z),n,ADR(z)) END
  END;
  ShiftLeft(ADR(x));
  IF GE(ADR(x),n) THEN Sub(ADR(x),n,ADR(x)) END;
  ShiftRight(ADR(y));
 END;
 c^:=z

END MultMod;


PROCEDURE Div(a,b,c:LONG); (* a/b *)

VAR x,y,z,w:L; (* Hilfsvektoren: wg. Geschwindigkeit nicht alloziert *)

BEGIN

 (* ---- Lokale Kopien erstellen ---- *)
 z[0]:=1;z[1]:=0;
 x:=a^;y:=b^;w:=b^;

 (* ---- Solange y:=y*2 bis  y>x ---- *)
 WHILE LE(ADR(y),ADR(x)) DO ShiftLeft(ADR(y)) END;

 WHILE (NE(ADR(y),ADR(w))) DO
  ShiftLeft(ADR(z));
  ShiftRight(ADR(y));

  IF LE(ADR(y),ADR(x)) THEN
   Sub(ADR(x),ADR(y),ADR(x));
   Inc(ADR(z),1)
  END
 END;

 c^:=z

END Div;


PROCEDURE Mod(a,b,c:LONG); (* a MOD b *)

VAR x,y,z,w:L; (* Hilfsvektoren: wg. Geschwindigkeit nicht alloziert *)

BEGIN

 (* ---- Lokale Kopien erstellen ---- *)
 z[0]:=1;z[1]:=0;
 x:=a^;y:=b^;w:=y;

 (* ---- Solange y:=y*2 bis  y>x ---- *)
 WHILE LE(ADR(y),ADR(x)) DO ShiftLeft(ADR(y)) END;

 WHILE (NE(ADR(y),ADR(w))) DO
  ShiftLeft(ADR(z));
  ShiftRight(ADR(y));

  IF LE(ADR(y),ADR(x)) THEN
   Sub(ADR(x),ADR(y),ADR(x));
   Inc(ADR(z),1)
  END
 END;

 c^:=x

END Mod;


PROCEDURE ModDiv(a,b,c,d:LONG); (* a MOD b a DIV b *)

VAR x,y,z,w:L; (* Hilfsvektoren: wg. Geschwindigkeit nicht alloziert *)

BEGIN
 (* ---- Lokale Kopien erstellen ---- *)
 z[0]:=1;z[1]:=0;
 x:=a^;y:=b^;w:=y;

 (* ---- Solange y:=y*2 bis  y>x ---- *)
 WHILE LE(ADR(y),ADR(x)) DO ShiftLeft(ADR(y)) END;

 WHILE (NE(ADR(y),ADR(w))) DO
  ShiftLeft(ADR(z));
  ShiftRight(ADR(y));

  IF LE(ADR(y),ADR(x)) THEN
   Sub(ADR(x),ADR(y),ADR(x));
   Inc(ADR(z),1)
  END
 END;

 c^:=z;
 d^:=x

END ModDiv;


PROCEDURE Pow(a,b,c:LONG);

VAR x,y,r:L;

BEGIN
 (* ---- Lokale Kopien ---- *)
 x:=a^;y:=b^;r[0]:=1;r[1]:=1;

 WHILE GN(y) DO
  IF ODD(y[1]) THEN
   Mult(ADR(r),ADR(x),ADR(r))
  END;
  Mult(ADR(x),ADR(x),ADR(x)); (* x:=x^2 *)
  ShiftRight(ADR(y)); (* y:=y/2 *)
 END;

 (* ---- Ergebnis zurckgeben  ---- *)
 c^:=r

END Pow;


PROCEDURE PowMod(a,b,n,c:LONG);

VAR x,y,r,k:L;

BEGIN
 (* ---- Lokale Kopien ---- *)
 x:=a^;y:=b^;r[0]:=1;r[1]:=1;
 k:=n^;

 WHILE GN(y) DO
  IF ODD(y[1]) THEN
   MultMod(ADR(r),ADR(x),ADR(k),ADR(r));  (* r:=r*x mod k *)
  END;
  MultMod(ADR(x),ADR(x),ADR(k),ADR(x)); (* x:=x^2 mod k *)
  ShiftRight(ADR(y)); (* y:=y/2 *)
 END;

 (* ---- Ergebnis zurckgeben  ---- *)
 c^:=r

END PowMod;


PROCEDURE MaxLong(a:LONG);

VAR i:INTEGER;
    aB:BLONG;

BEGIN
 aB:=BLONG(a);

 a^[0]:=MaxNr;
 FOR i:=1 TO MaxNr DO
  a^[i]:=-1; (* Stimmt nur wenn Maschine Komplementdarstellung verwendet *)
  EXCL(aB^[i],MSBNr)
 END

END MaxLong;


PROCEDURE NStellen():INTEGER;

BEGIN
 RETURN INTEGER(REAL(MaxNr*(NBits-1))*ln(2.0)/ln(10.0))
END NStellen;


PROCEDURE AktStellen(a:LONG):INTEGER;

BEGIN
 RETURN INTEGER(REAL(a^[0]*(NBits-1))*ln(2.0)/ln(10.0))
END AktStellen;


PROCEDURE NumBits():INTEGER;

BEGIN
 RETURN MaxNr*(NBits-1)
END NumBits;


PROCEDURE Trans(b,a:LONG);

BEGIN
 b^:=a^
END Trans;


PROCEDURE Long2Str(a:LONG;VAR Str:ARRAY OF CHAR;VAR err:BOOLEAN);

VAR Rest:L; (* Der Rest bei der Mod-Operation *)
    c:L; (* Kopie des eingegeben Werts *)
    i:INTEGER; (* Indexzhler *)
    Help:CHAR; (* Hilfsvariable zum verdrehen des Strings *)
    Len:INTEGER; (* Lnge des Strings *)

BEGIN
 (* ---- Initialisierungen  ---- *)
 err:=FALSE;
 i:=0;
 c:=a^;

 (* ---- Wenn Feld zu klein dann immer ein Fehler !! ---- *)
 IF (AktStellen(a)+1>INTEGER(HIGH(Str))) THEN err:=TRUE; RETURN END;

 IF NOT(GN(a^)) THEN Str[0]:='0'; RETURN END;

 (* ---- Umwandlung  ---- *)
 WHILE GN(c) DO
  ModDiv(ADR(c),Zehn,ADR(c),ADR(Rest));
  Str[i]:=CHAR(INTEGER('0')+Rest[1]);
  INC(i)
 END;

 (* ---- Hiermit wird String beendet ---- *)
 Str[i]:=EndeMarke;
 Len:=Length(Str);

 (* ---- String noch umdrehen ---- *)
 FOR i:=0 TO (Len DIV 2)-1 DO
  Help:=Str[i];
  Str[i]:=Str[Len-i-1];
  Str[Len-i-1]:=Help
 END

END Long2Str;


PROCEDURE Str2Long(Str:ARRAY OF CHAR;a:LONG;VAR err:BOOLEAN);

VAR Erg:L; (* Hier wird das Ergebnis aufgebaut *)
    Ziff:INTEGER; (* INTEGER-Darstellung i-Dezimalziffer *)
    i:INTEGER;

BEGIN
 (* ---- Initialisierungen ---- *)
 err:=FALSE;
 Erg[0]:=1;Erg[1]:=0;

 (* ---- Umrechnung ---- *)
 FOR i:=0 TO Length(Str)-1 DO
  CASE Str[i] OF
   |'0':Ziff:=0;
   |'1':Ziff:=1;
   |'2':Ziff:=2;
   |'3':Ziff:=3;
   |'4':Ziff:=4;
   |'5':Ziff:=5;
   |'6':Ziff:=6;
   |'7':Ziff:=7;
   |'8':Ziff:=8;
   |'9':Ziff:=9;
  ELSE
   err:=TRUE;
   RETURN
  END;
  Mult(ADR(Erg),Zehn,ADR(Erg));
  Inc(ADR(Erg),Ziff);
 END;

 (* ---- Ergebnis kopieren ---- *)
 a^:=Erg

END Str2Long;


PROCEDURE Int2Long(Zahl:INTEGER;a:LONG);

BEGIN
 IF Zahl>=0 THEN
  a^[1]:=Zahl;
  a^[0]:=1
 ELSE
  Trans(a,Null)
 END
END Int2Long;


PROCEDURE WriteRawLONG(out:File;Z:LONG);

VAR i:INTEGER;

BEGIN
 FOR i:=0 TO Z^[0] DO
  FIO.WrChar(out,CHAR(Z^[i] DIV 256));
  FIO.WrChar(out,CHAR(Z^[i] MOD 256))
 END
END WriteRawLONG;


PROCEDURE ReadRawLONG(in:File;Z:LONG);

VAR i:INTEGER; (* Universalzhlvariable *)
    ch1,ch2:CHAR; (* Zwischenspeicher fr Zeicheneingaben *)

BEGIN
  (* ---- Die ersten zwei Bytes mssen durch in erreichbar sein!! ---- *)
  ch1:=FIO.RdChar(in);
  ch2:=FIO.RdChar(in);   (* Die in Z^[0] angegebene Zahl gibt an *)
  Z^[0]:=256*ORD(ch1)+ORD(ch2);  (* wieviele INTEGERs noch lesbar sind!  *)

  (* ---- Komponentenweises Einlesen ---- *)
  FOR i:=1 TO Z^[0] DO
   ch1:=FIO.RdChar(in);
   ch2:=FIO.RdChar(in);
   Z^[i]:=256*ORD(ch1)+ORD(ch2)
  END

END ReadRawLONG;


PROCEDURE Split(Kom:INTEGER;VAR High,Low:BYTE);

BEGIN
 High:=BYTE(CHAR(Kom DIV 256));
 Low:=BYTE(CHAR(Kom MOD 256))
END Split;


PROCEDURE Join(High,Low:BYTE;VAR Kom:INTEGER);

BEGIN
 Kom:=256*INTEGER(CHAR(High))+INTEGER(CHAR(Low))
END Join;


PROCEDURE LONG2ByteBlock(Z:LONG;VAR Block:ByteFeld);

VAR i,j:INTEGER;

BEGIN
 (* ---- Zhler fr Bytefeld initialisieren ---- *)
 j:=0;

 FOR i:=0 TO Z^[0] DO
  (* ---- Lowbyte vor Highbyte ablegen ==> INTEL *)
  Split(Z^[i],Block[j+1],Block[j]);
  INC(j,2)
 END;

 (* ---- Den Rest noch mit Dummy-Werten auffllen um einem Verschwinden ---- *)
 (* ----               von Nullen beim Kodieren vorzubeugen             ---- *)
 FOR i:=j TO ByteZahl DO
  Block[i]:=BYTE(1)
 END

END LONG2ByteBlock;


PROCEDURE ByteBlock2LONG(Block:ByteFeld;Z:LONG);

VAR i,j:INTEGER;

BEGIN
 (* ---- Lnge des BYTE-Blocks ermitteln INTEL beachten! ---- *)
 Join(Block[1],Block[0],Z^[0]);
 (* ---- Zhler fr BYTEs initialisieren ---- *)
 j:=2;

 FOR i:=1 TO Z^[0] DO
  (* Erst Low dann HighByte INTEL *)
  Join(Block[j+1],Block[j],Z^[i]);
  INC(j,2)
 END

END ByteBlock2LONG;


PROCEDURE RSACode(in,out:File;e,n:LONG;Code:Func);

VAR Kompo:INTEGER; (* Anzahl der Komponenten in der aktuellen Zahl *)
    ch1,ch2:CHAR;
    Zahl:L;

BEGIN

 FIO.EOF:=FALSE;

 LOOP

  Kompo:=1; (* In erste Komponente der LONG-Zahl schreiben *)

  (* ---- Eine Zahl lesen  ---- *)
  WHILE NOT(FIO.EOF) AND (Kompo<n^[0]) DO

   ch1:=FIO.RdChar(in);ch1:=Code(ch1);
   IF FIO.EOF THEN
    ch2:=ASCII.nul;
   ELSE
    ch2:=FIO.RdChar(in);ch2:=Code(ch2);
   END;

   (* ---- In INTEGER umwandeln ---- *)
   Zahl[Kompo]:=256*ORD(ch1)+ORD(ch2);

   INC(Kompo)
  END;
  Zahl[0]:=Kompo-1;

  (* ---- Kodieren der Zahl ---- *)
  PowMod(ADR(Zahl),e,n,ADR(Zahl));

  (* ---- Schreiben einer LONG-Zahl ---- *)
  WriteRawLONG(out,ADR(Zahl));

  IF FIO.EOF THEN EXIT END

 END

END RSACode;


PROCEDURE RSADeCode(in,out:File;d,n:LONG;DeCode:Func);

VAR Kompo:INTEGER; (* Anzahl der Komponenten in der aktuellen Zahl *)
    ch1,ch2:CHAR;
    Zahl:L;
    i:INTEGER;

BEGIN

 FIO.EOF:=FALSE;

 LOOP

  ch1:=FIO.RdChar(in);
  IF FIO.EOF THEN EXIT END;
  ch2:=FIO.RdChar(in);
  IF FIO.EOF THEN EXIT END;
  Zahl[0]:=256*ORD(ch1)+ORD(ch2);

  Kompo:=1; (* In erste Komponente der LONG-Zahl schreiben *)

  (* ---- Eine Zahl lesen  ---- *)
  WHILE NOT(FIO.EOF) AND (Kompo<=Zahl[0]) DO
   ch1:=FIO.RdChar(in);
   IF FIO.EOF THEN
    ch2:=ASCII.nul;
   ELSE
    ch2:=FIO.RdChar(in)
   END;

   (* ---- In INTEGER umwandeln ---- *)
   Zahl[Kompo]:=256*ORD(ch1)+ORD(ch2);

   INC(Kompo)
  END;

  (* ---- Kodieren der Zahl ---- *)
  PowMod(ADR(Zahl),d,n,ADR(Zahl));

  (* ---- Ausgabe des Ergebnisses ---- *)
  FOR i:=1 TO Zahl[0] DO
   FIO.WrChar(out,DeCode(CHAR(Zahl[i] DIV 256)));
   FIO.WrChar(out,DeCode(CHAR(Zahl[i] MOD 256)))
  END;

 END

END RSADeCode;


PROCEDURE CodeByteBlock(Source:BytePtr;SourceLen:INTEGER;
                        Destination:BytePtr;VAR DestinationLen:INTEGER;
                        e,n:LONG);

VAR bl,bh:BYTE; (* Ein Low und ein High-Byte *)
    Kompo,i,Free:INTEGER;
    Zahl:L;
    CryptPtr:BytePtr; (* Hier wird die Lnge des verschlsselten Byteblocks abgelegt *)

BEGIN

 (* ---- In DestinationLen bekomme ich die Lnge des Zielblocks ---- *)
 Free:=DestinationLen;

 (* ---- Sourceblocklnge in Zielblock eintragen ---- *)
 Split(SourceLen,bh,bl);

 (* ---- Erst die Lnge des unverschlsselten Datenblocks ablegen ---- *)
 Destination^:=bl;IncAddr(Destination,SIZE(BYTE));
 Destination^:=bh;IncAddr(Destination,SIZE(BYTE));

 (* ---- Zeiger auf die Lnge des verschlsselten Byteblocks merken ---- *)
 CryptPtr:=Destination;
 (* ---- Zeiger um 2 Bytes inkrementieren ---- *)
 IncAddr(Destination,2*SIZE(BYTE));

 (* ---- Zielnge ist aktuell 0 ohne Header!! ---- *)
 DestinationLen:=0;

 (* ---- 4 Bytes sind schon im Zielblock ---- *)
 DEC(Free,4);

 LOOP

  Kompo:=1;

  (* ---- Eine Komponente aus Buffer holen und LONG-Zahl basteln ---- *)
  WHILE (SourceLen>0) AND (Kompo<n^[0]) DO

   (* ---- Erst Low und dann Highbyte holen ---- *)
   bl:=Source^;IncAddr(Source,SIZE(BYTE));DEC(SourceLen);
   IF SourceLen>0 THEN
    bh:=Source^;IncAddr(Source,SIZE(BYTE));DEC(SourceLen)
   ELSE
    bh:=0
   END;

   (* ---- Und in LONG-Zahl eintragen ---- *)
   Join(bh,bl,Zahl[Kompo]);

   INC(Kompo)
  END;

  Zahl[0]:=Kompo-1;

  (* ---- Kodieren der Zahl ---- *)
  PowMod(ADR(Zahl),e,n,ADR(Zahl));

  (* ---- Ausgabe in Zielfeld ---- *)
  FOR i:=0 TO Zahl[0] DO

   Split(Zahl[i],bh,bl);

   Destination^:=bl;IncAddr(Destination,SIZE(BYTE));INC(DestinationLen);DEC(Free);
   IF Free=0 THEN
    InOut.WriteString("Free ist Null! (Low) CodeByte Zielblock zu klein !!");
    InOut.WriteLn;
    HALT
   END;
   Destination^:=bh;IncAddr(Destination,SIZE(BYTE));INC(DestinationLen);DEC(Free);
   IF Free=0 THEN
    InOut.WriteString("Free ist Null! (High) CodeByte Zielblock zu klein !!");
    InOut.WriteLn;
    HALT
   END

  END;

  (* ---- Source schon leer ? ---- *)
  IF SourceLen=0 THEN EXIT END

 END;

 (* ---- Und noch die Lnge des codierten Byteblocks zu erhalten ---- *)
 Split(DestinationLen,bh,bl);
 CryptPtr^:=bl;IncAddr(CryptPtr,SIZE(BYTE));
 CryptPtr^:=bh;IncAddr(CryptPtr,SIZE(BYTE));

 (* ---- Destinationlnge noch um 4 erhhen wegen Header (2*INTEGER) ---- *)
 INC(DestinationLen,4)

END CodeByteBlock;


PROCEDURE DeCodeByteBlock(Source:BytePtr;Destination:BytePtr;
                          VAR DestinationLen:INTEGER;d,n:LONG);

VAR Kompo:INTEGER;
    bl,bh:BYTE;
    Zahl:L;
    i:INTEGER;
    SourceLen:INTEGER;
    (* Hier wird die Lnge der verschlsselten Blocks gespeichert *)

BEGIN

 (* ---- Lnge des (unverschlsselten) Zielblocks holen ---- *)
 bl:=Source^;IncAddr(Source,SIZE(BYTE));
 bh:=Source^;IncAddr(Source,SIZE(BYTE));
 Join(bh,bl,DestinationLen);

 (* ---- Lnge des (verschlsselten) Sourceblocks holen ---- *)
 bl:=Source^;IncAddr(Source,SIZE(BYTE));
 bh:=Source^;IncAddr(Source,SIZE(BYTE));
 Join(bh,bl,SourceLen);

 LOOP

  bl:=Source^;IncAddr(Source,SIZE(BYTE));DEC(SourceLen);
  bh:=Source^;IncAddr(Source,SIZE(BYTE));DEC(SourceLen);

  Join(bh,bl,Zahl[0]);

  (* ---- Es geht weiter in der ersten Komponente ---- *)
  Kompo:=1;

  (* ---- Eine Zahl erzeugen ---- *)
  WHILE (SourceLen>0) AND (Kompo<=Zahl[0]) DO
   bl:=Source^;IncAddr(Source,SIZE(BYTE));DEC(SourceLen);
   IF SourceLen>0 THEN
    bh:=Source^;IncAddr(Source,SIZE(BYTE));DEC(SourceLen)
   ELSE
    bh:=0
   END;

   (* ---- Und eintragen ---- *)
   Join(bh,bl,Zahl[Kompo]);

   INC(Kompo)
  END;

  (* ---- Dekodieren der Zahl ---- *)
  PowMod(ADR(Zahl),d,n,ADR(Zahl));

  (* ---- Und in Zielfeld schreiben ---- *)
  FOR i:=1 TO Zahl[0] DO
   (* ---- Im INTEL-Format wieder ausgeben ---- *)
   Split(Zahl[i],bh,bl);
   Destination^:=bl; IncAddr(Destination,SIZE(BYTE));
   Destination^:=bh; IncAddr(Destination,SIZE(BYTE))
  END;

  IF SourceLen=0 THEN EXIT END;

 END

END DeCodeByteBlock;


BEGIN

 IF SIZE(BITSET)#SIZE(INTEGER) THEN
  WriteString('Achtung: BITSET und INTEGER Typ verschieden gro!');
  WriteLn;
  HALT
 END;

 (*
   Leider kennt der TopSpeed keinen LONGSET-Typ mit 32 Bit. Der Grund ist
   wohl in der Busbreite von 16 Bit des 8086 zu suchen. Sonst htte man
   auch mit LONGSET und LONGINT arbeiten knnen, was einiges an Teiloperationen
   gespart htte.
 *)

 (* ---- Konstanten initialisieren ---- *)
 INull[0]:=1;INull[1]:=0;
 Null:=ADR(INull);

 IEins[0]:=1;IEins[1]:=1;
 Eins:=ADR(IEins);

 IZwei[0]:=1;IZwei[1]:=2;
 Zwei:=ADR(IZwei);

 IZehn[0]:=1;IZehn[1]:=10;
 Zehn:=ADR(IZehn);

 ISechZehn[0]:=1;ISechZehn[1]:=16;
 SechZehn:=ADR(ISechZehn);

 IHundert[0]:=1;IHundert[1]:=100;
 Hundert:=ADR(IHundert);

 ITausend[0]:=1;ITausend[1]:=1000;
 Tausend:=ADR(ITausend);

END MathLong.
