(**********************************************************************

    :Program.    RSA
    :Contents.   Eigentliches Kernmodul fr die Verschlsselung
    :Contents.   und Schlsselerzeugung
    :Author.     Steffen Reith
    :Address.    Hessenstr. 64, D-8700 Wrzburg
    :Phone.      0931/281859
    :Language.   Modula-2
    :Translator. Topspeed Modula2 / M2Amiga A+L V3.32d
    :History.    V1.0 20. Feb 1991
    :History.    V1.1 25. Feb 1991
    :History.    V1.2 11. Mrz 1991
    :History.    V1.3 28. Mai 1991

***********************************************************************)
IMPLEMENTATION MODULE RSA;

FROM MathLong IMPORT AllocNum,FreeNum,LONG;
FROM MathLong IMPORT Mult,Sub,NE,Add,Str2Long,Trans,Int2Long,RSACode;
FROM MathLong IMPORT RSADeCode,Eins;
FROM LongMath IMPORT EulerPhi,Berle;
FROM InOut    IMPORT WriteString,WriteLn;
FROM MRT      IMPORT MR;
FROM Lib      IMPORT RANDOM;
FROM FIO      IMPORT File;

IMPORT ASCII;
IMPORT MathLong;
IMPORT InOut;
IMPORT LInOut;

CONST Schranke='76772353'; (* Ab dieser Grenze beginnt die Primzahlsuche *)


VAR Ab:LONG; (* Untere Schranke f. Primzahlensuche global verwendet !! *)
    Flag:BOOLEAN; (* Wandlungsfehler *)


PROCEDURE CalculatePseudoPrim(Prim:LONG);

VAR RN:LONG; (* Zufallswert *)

BEGIN
 RN:=AllocNum();
 IF RN=LONG(NIL) THEN
  WriteString('RSA-CalculatePseudoPrim No Memory');WriteLn;
  HALT
 END;
 Int2Long(INTEGER(RANDOM(10000)),RN);
 (* Zufallszahlen sollen sich um einige Stellen unterscheiden *)

 LOOP
  IF MR(Ab) THEN
   Trans(Prim,Ab);
   Add(Ab,Eins,Ab);
   EXIT
  END;
  Add(Ab,Eins,Ab) (* INC(Ab) *)
 END;

 FreeNum(RN)

END CalculatePseudoPrim;

PROCEDURE LONGs2Key(e,n:LONG;VAR key:Key);

BEGIN
 MathLong.LONG2ByteBlock(e,key.a);
 MathLong.LONG2ByteBlock(n,key.n)
END LONGs2Key;


PROCEDURE Key2LONGs(key:Key;e,n:LONG);

BEGIN
 MathLong.ByteBlock2LONG(key.a,e);
 MathLong.ByteBlock2LONG(key.n,n)
END Key2LONGs;

PROCEDURE CreateKeys(VAR offen,geheim:Key);

VAR p,q,Phi,e,d,n:LONG; (* Die Bedeutungen der Variablen sind im Skript beschrieben *)
    ggT:LONG; (* Der ggT muss 1 sein !! *)

BEGIN
 (* ---- Variablen allokieren ---- *)
 e:=AllocNum();
 IF e=LONG(NIL) THEN
  WriteString('RSA-Kein Speicher fr Schlssel e');WriteLn;
  HALT
 END;

 d:=AllocNum();
 IF d=LONG(NIL) THEN
  WriteString('RSA-Kein Speicher fr Schlssel d');WriteLn;
  HALT
 END;

 n:=AllocNum();
 IF n=LONG(NIL) THEN
  WriteString('RSA-Kein Speicher fr Modul n');WriteLn;
  HALT
 END;

 (* ---- Erste Primzahl allokieren ---- *)
 p:=AllocNum();
 IF p=LONG(NIL) THEN
  WriteString('RSA-Kein Speicher fr Primzahl p');WriteLn;
  HALT
 END;

 (* ---- Zweite Primzahl allokieren ---- *)
 q:=AllocNum();
 IF q=LONG(NIL) THEN
  WriteString('RSA-Kein Speicher fr Primzahl q');WriteLn;
  HALT
 END;

 (* ---- n allokieren ---- *)
 Phi:=AllocNum();
 IF Phi=LONG(NIL) THEN
  WriteString('RSA-Kein Speicher fr Eulersche Phi-funktion');WriteLn;
  HALT
 END;

 (* ---- ggT allokieren ---- *)
 ggT:=AllocNum();
 IF ggT=LONG(NIL) THEN
  WriteString('RSA-Kein Speicher fr ggT');WriteLn;
  HALT
 END;

 (* ---- Erste Primzahl holen ---- *)
 CalculatePseudoPrim(p);

 (* ---- Zweite Primzahl holen ---- *)
 CalculatePseudoPrim(q);

 (* ---- n berechnen (n=p*q)  ---- *)
 Mult(p,q,n);

 (* ---- Phi berechnen Phi=(p-1)(q-1) ---- *)
 EulerPhi(p,q,Phi);

 (* ---- Whle e als Primzahl ---- *)
 CalculatePseudoPrim(e);

(* ---- Berlekampalgorithmus durchfhren ---- *)
 Berle(e,Phi,d,ggT);
 IF NE(ggT,Eins) THEN
  WriteString('FATALER FEHLER !!');
  WriteString('RSA-ggT # 1 PseudoPrimzahlen oder Berlekamp gescheitert');
  WriteLn;
  HALT
 END;

 (* ---- Das Ergebnis zusammensetzen ---- *)
 LONGs2Key(e,n,offen);
 LONGs2Key(d,n,geheim);

 (* ---- Verwendete Zahlen wieder freigeben ---- *)
 FreeNum(p);FreeNum(q);FreeNum(e);FreeNum(d);FreeNum(n);
 FreeNum(Phi);FreeNum(ggT);

END CreateKeys;


(* ---- Vernderte Quellkodierung ---- *)
PROCEDURE Code(ch:CHAR):CHAR;

BEGIN
 IF ORD(ch)<=127 THEN
  RETURN ch
 ELSE
  RETURN ' '
 END
END Code;

PROCEDURE EnCrypt(in,out:File;key:Key);

VAR e,n:LONG;

BEGIN
 (* ---- Zahlen allokieren ---- *)
 e:=AllocNum();
 IF e=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt e');WriteLn;
  HALT
 END;
 n:=AllocNum();
 IF n=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt n');WriteLn;
  HALT
 END;

 (* ---- Auseinandernehmen des Key-Blocks ---- *)
 Key2LONGs(key,e,n);

 (* ---- Verschlsseln ---- *)
 RSACode(in,out,e,n,Code);

 (* ---- Freigeben der Zahlen ---- *)
 FreeNum(e);FreeNum(n)

END EnCrypt;


(* ---- Vernderte Quellkodierung ---- *)
PROCEDURE DeCode(ch:CHAR):CHAR;

BEGIN
 RETURN ch
END DeCode;

PROCEDURE DeCrypt(in,out:File;key:Key);

VAR d,n:LONG;

BEGIN
 (* ---- Zahlen allokieren ---- *)
 d:=AllocNum();
 IF d=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt d');WriteLn;
  HALT
 END;
 n:=AllocNum();
 IF n=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt n');WriteLn;
  HALT
 END;

 (* ---- Auseinandernehmen des Key-Blocks ---- *)
 Key2LONGs(key,d,n);

 (* ---- Entschlsseln ----*)
 RSADeCode(in,out,d,n,DeCode);

 (* ---- Freigeben der Zahlen ---- *)
 FreeNum(d);FreeNum(n)

END DeCrypt;

PROCEDURE CodeByteBlock(Source:ADDRESS;SourceLen:INTEGER;
                        Destination:ADDRESS;VAR DestinationLen:INTEGER;
                        key:Key);

VAR e,n:LONG;

BEGIN
 (* ---- Zahlen allokieren ---- *)
 e:=AllocNum();
 IF e=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt e');WriteLn;
  HALT
 END;
 n:=AllocNum();
 IF n=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt n');WriteLn;
  HALT
 END;

 (* ---- Auseinandernehmen des Key-Blocks ---- *)
 Key2LONGs(key,e,n);

 (* ---- Verschlsseln ---- *)
 MathLong.CodeByteBlock(Source,SourceLen,Destination,DestinationLen,e,n);

 (* ---- Zahlen freigeben ---- *)
 FreeNum(e);FreeNum(n)

END CodeByteBlock;

PROCEDURE DeCodeByteBlock(Source:ADDRESS;Destination:ADDRESS;key:Key);

VAR d,n:LONG;
    Len:INTEGER;

BEGIN
 (* ---- Zahlen allokieren ---- *)
 d:=AllocNum();
 IF d=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt d');WriteLn;
  HALT
 END;
 n:=AllocNum();
 IF n=LONG(NIL) THEN
  WriteString('Speichermangel in RSA-EnCrypt n');WriteLn;
  HALT
 END;

 (* ---- Auseinandernehmen des Key-Blocks ---- *)
 Key2LONGs(key,d,n);

 (* ---- Entschlsseln ---- *)
 MathLong.DeCodeByteBlock(Source,Destination,Len,d,n);

 (* ---- Freigeben der Zahlen ---- *)
 FreeNum(d);FreeNum(n)

END DeCodeByteBlock;

PROCEDURE WriteRawLONG(out:File;Z:LONG);

BEGIN
 MathLong.WriteRawLONG(out,Z)
END WriteRawLONG;


PROCEDURE ReadRawLONG(in:File;Z:LONG);

BEGIN
 MathLong.ReadRawLONG(in,Z)
END ReadRawLONG;


BEGIN
 (* ---- Unter Schranke fr Primzahlensuche allokieren ---- *)
 Ab:=AllocNum();
 IF Ab=LONG(NIL) THEN
  WriteString('RSA- Kein Speicher fr untere Schranke Primzahlen');WriteLn;
  HALT
 END;

 (* ---- Schranke initialisieren ---- *)
 Str2Long(Schranke,Ab,Flag);
 IF Flag THEN
  WriteString('RSA- Kann String nicht in untere Schranke Primzahlen wandeln');
  WriteLn;
  HALT
 END

END RSA.
