(**********************************************************************

    :Program.    LongMathLib
    :Contents.   Komplexere Rechenroutinen fr lange Zahlen
    :Author.     Steffen Reith
    :Address.    Hessenstr. 64, D-8700 Wrzburg
    :Phone.      0931/281859
    :Language.   Modula-2
    :Translator. TopSpeed Modula / M2Amiga A+L V3.32d
    :History.    V1.0  1. Jan 1991

**********************************************************************)
IMPLEMENTATION MODULE LONGMathLib;

FROM InOut    IMPORT WriteString,WriteLn;
FROM MathLONG IMPORT AllocNum,FreeNum,Add,ModDiv,Trans,LONG,Str2Long,Mult,
                     NE,Null,Eins,Zwei,EQ,Sub;

PROCEDURE EulerPhi(p,q,phi:LONG);

VAR a,b:LONG;

BEGIN

 (* ---- Speicherplatz fr die bentigten Zahlen holen ---- *)
 a:=AllocNum();
 IF a=NIL THEN
  WriteString('Fehler beim Allokieren (a) EulerPhi');WriteLn;
  HALT
 END;

 b:=AllocNum();
 IF b=NIL THEN
  WriteString('Fehler beim Allokieren (b) EulerPhi');WriteLn;
  HALT
 END;

 (* ---- Kopien der Zahlen anfertigen ---- *)
 Trans(a,p);
 Trans(b,q);

 (* ---- Berechnung von a-1 und b-1 ---- *)
 Sub(a,Eins,a);
 Sub(b,Eins,b);

 (* ---- Berechnung von (a-1)*(b-1) Phi-Funktion fr zwei Primzahlen ---- *)
 Mult(a,b,phi);

 (* ---- Speicher freigeben ---- *)
 FreeNum(a);FreeNum(b)

END EulerPhi;


PROCEDURE Berle(e,phi,d,ggT:LONG);

VAR r,r1,r2,a,a1,a2,b,b1,b2,q:LONG;
    Counter:LONGINT;
    err:BOOLEAN;

BEGIN

 (* ---- Speicherplatz fr die bentigten Zahlen holen ---- *)
 r:=AllocNum();
 IF r=NIL THEN
  WriteString('Fehler beim Allokieren (r) Berle');WriteLn;
  HALT
 END;

 r1:=AllocNum();
 IF r1=NIL THEN
  WriteString('Fehler beim Allokieren (r1) Berle');WriteLn;
  HALT
 END;

 r2:=AllocNum();
 IF r2=NIL THEN
  WriteString('Fehler beim Allokieren (r2) Berle');WriteLn;
  HALT
 END;

 a:=AllocNum();
 IF a=NIL THEN
  WriteString('Fehler beim Allokieren (a) Berle');WriteLn;
  HALT
 END;

 a1:=AllocNum();
 IF a1=NIL THEN
  WriteString('Fehler beim Allokieren (a1) Berle');WriteLn;
  HALT
 END;

 a2:=AllocNum();
 IF a2=NIL THEN
  WriteString('Fehler beim Allokieren (a2) Berle');WriteLn;
  HALT
 END;

 b:=AllocNum();
 IF b=NIL THEN
  WriteString('Fehler beim Allokieren (b) Berle');WriteLn;
  HALT
 END;

 b1:=AllocNum();
 IF b1=NIL THEN
  WriteString('Fehler beim Allokieren (b1) Berle');WriteLn;
  HALT
 END;

 b2:=AllocNum();
 IF b2=NIL THEN
  WriteString('Fehler beim Allokieren (b2) Berle');WriteLn;
  HALT
 END;

 q:=AllocNum();
 IF q=NIL THEN
  WriteString('Fehler beim Allokieren (q) Berle');WriteLn;
  HALT
 END;

 (* ---- Initialisieren der Zahlen ---- *)
 Trans(r2,e);Trans(r1,phi);

 Trans(a2,Eins);Trans(a1,Null);
 Trans(b1,Eins);Trans(b2,Null);
 Trans(r,Eins);
 Counter:=1;

 (* ---- Berlekampalgorithmus durchfhren ---- *)
 WHILE NE(r,Null) DO

  ModDiv(r2,r1,q,r);

  Mult(q,a1,a);Add(a,a2,a);
  Mult(q,b1,b);Add(b,b2,b);

  Trans(r2,r1);Trans(r1,r);
  Trans(a2,a1);Trans(a1,a);
  Trans(b2,b1);Trans(b1,b);

  INC(Counter)

 END;

 (* ---- Inverses berechnen ---- *)
 IF EQ(r2,Eins) THEN
  IF NOT(ODD(Counter)) THEN
   Sub(phi,a2,d)
  ELSE
   Trans(d,a2)
  END
 END;

 (* ---- ggT zurckgeben ---- *)
 Trans(ggT,r2);

 (* ---- Benutzte Zahlen wieder freigeben ---- *)
 FreeNum(r);FreeNum(r1);FreeNum(r2);
 FreeNum(a);FreeNum(a1);FreeNum(a2);
 FreeNum(b);FreeNum(b1);FreeNum(b2);
 FreeNum(q);

END Berle;

BEGIN
END LONGMathLib.
