(*************************************************************************

:Program.    EdCmd2.mod
:Contents.   Commands for AmokEd
:Author.     Hartmut Goebel
:Language.   Oberon
:Translator. Amiga Oberon Compiler V2.13
:Imports.    SupLib (Hartmut Goebel)
:History.    V0.1, 23 Mar 1991 Hartmut Goebel
:History.    V1.0, 14 Apr 1991 Hartmut Goebel [hG]
:History.    V1.0b 28 Apr 1991 [hG] changed dNewWindow <- edL.TextInit
:History.    V1.0c 06 Jun 1991 [hG] +Vars lineno, colno
:History.    V1.1  12 Jun 1991 [hG] Scanf complited (SScanf v. V.Rudolph)
:History.    V1.1b 17 Jun 1991 [hG] eigener Screen mglich
:Date.       01 Feb 1992 12:24:27

*************************************************************************)
(* $Debug- *)
MODULE EdCmd2;

IMPORT
  d:   Dos,
  e:   Exec,
  edD: EdDisplay,
  edE: EdErrors,
  edG: EdGlobalVars,
  edK: EdKeyboard,
  edL: EdLowLevel,
  eMn: EdMenu,
  g:   Graphics,
  I:   Intuition,
  lst: EdLists,
  ol:  OberonLib,
       Printf,
  sl:  SupLib,
  str: Strings,
  s  : SYSTEM;

TYPE
  Variable = STRUCT (node: lst.Node)
    name: edG.StringPtr;
    string: edG.StringPtr;
  END;
  VarPtr = POINTER TO Variable;

  (* WICHTIG: Bei nderungen Offsets in FindReplace.asm berprfen!! *)
  FindReplaceStruct = STRUCT
    findStr: edG.StringPtr;
    line: edG.LinePtr;
    flen: INTEGER;
    pos: INTEGER;
  END;

CONST
  (* Flags fr einzelne Kommandos *)
  replace*=0;
  previous*=1;
  oldStrings*=2;
  top*=0;
  height*=1;
  temp*=2;
  leftEdge*={top,height};

  ReplaceQuestion = "Replace? (y/n/a/t/q)";
  NoFindPattern = "No find pattern";
  NoReplacePattern = "No replace pattern";
  PatternNotFound = "Pattern Not Found";
  ReplaceLineTooLong = "Replace: Line Too Long";
  TurboReplacePleaseWait = "Turbo Replace: Please Wait!";
  WindowTooBig = "window too big (try moving to upper left corner and retrying)";
  ExecuteFailed = "Execute failed!";

VAR
  VarList: lst.List;
  VarBuffer: ARRAY 14 OF CHAR;
  wasTurboReplace: BOOLEAN;

(*-----------------------------------------------------------------------*)

PROCEDURE dUndo*;
BEGIN
  edD.TextLoad;
  edD.TextRedisplayCurrentLine;
END dUndo;


PROCEDURE dNull*; (* macht absolut nichts *)
BEGIN END dNull;


PROCEDURE dAbort*; (* subject of change *)
BEGIN
  edG.Rc := edE.abortLevel;
END dAbort;


PROCEDURE dVersion*;
VAR
  str: edG.StringPtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  str := s.ADR(edG.Version[5]);
  edL.Title(str^); edG.Rc := edE.cmdValid2;
END dVersion;

(*-----------------------------------------------------------------------*)

PROCEDURE dScanf*;
CONST
  StrLen = 80;
VAR
  i,j:INTEGER;
  maxWidth:INTEGER;
  allAllowed:BOOLEAN;
  negAllowed:BOOLEAN;
  allowedChars:ARRAY StrLen OF CHAR;
  fmtLen:INTEGER;
  argLen:INTEGER;
  argStr: edG.StringPtr;

  PROCEDURE IsAllowed(ch:CHAR):BOOLEAN;
  VAR
    i:INTEGER;
  BEGIN
    IF ch = ' ' THEN RETURN FALSE END;
    IF allAllowed THEN RETURN TRUE END;
  (* $IFNOT ClearVars *)
    i := 0;
  (* $END *)
    WHILE (allowedChars[i] # ch) AND
          (allowedChars[i] # 0X) DO
      INC(i);
    END; (* WHILE *)
    IF allowedChars[i] = 0X THEN
      RETURN negAllowed
    ELSE
      RETURN NOT negAllowed;
    END;(* IF *)
  END IsAllowed;

  PROCEDURE BadArgument;
  BEGIN
    edG.Rc := edE.cmdError; edL.Title(edG.BadArgument);
  END BadArgument;

BEGIN
  i := 0;
  argStr := s.ADR(edG.LineBuffer[edG.Text.pos]);
  fmtLen := edL.Length(edG.Arg[0]^);
  argLen := edL.Length(argStr^);;
  DISPOSE(edG.ScanStr);

  (* scan format string *)

  (* skip leading garbage *)
  i := 0;
  WHILE (i < fmtLen) AND (edG.Arg[0][i] # '%') DO
    INC(i); END;
  IF i = fmtLen THEN
    BadArgument; RETURN;
  END;

  (* check for '*' (ignore argument) *)
  INC(i);
  IF edG.Arg[0][i] = '*' THEN RETURN END;

  (* calculate maximal width of string *)
  maxWidth := 0;
  WHILE (i < fmtLen) AND
        (edG.Arg[0][i] >= '0') AND
        (edG.Arg[0][i] <= '9') DO
    maxWidth := maxWidth * 10 + ORD(edG.Arg[0][i])-ORD('0');
    INC(i);
  END; (* WHILE *)
  IF i = fmtLen THEN
    BadArgument; RETURN;
  END;
  IF maxWidth = 0 THEN maxWidth := edG.MaxLineLength-1 END;

  (* is conversion-specifier 's' or '[..]' ? *)
  IF edG.Arg[0][i] = 's' THEN
    allAllowed := TRUE;
  ELSIF edG.Arg[0][i] = '[' THEN
    allAllowed := FALSE;
    negAllowed := FALSE;

    (* check for negation flags '~' and '^' *)
    INC(i);
    IF (edG.Arg[0][i] = '~') OR (edG.Arg[0][i] = '^') THEN
      negAllowed := TRUE;
      INC(i);
    END; (* IF *)

    j := 0;
    allowedChars := "";
    WHILE (i < fmtLen) AND
          (j < (StrLen-1)) AND
          (edG.Arg[0][i] # ']') DO
      allowedChars[j] := edG.Arg[0][i];
      INC(i);
      INC(j);
    END; (* WHILE *)
    allowedChars[j] := 0X;
    IF edG.Arg[0][i] # ']' THEN
      BadArgument; RETURN;
    END;
  ELSE
    BadArgument; RETURN;
  END; (* IF *)

  (* scan argument string *)

  (* skip leading blanks in argStr *)
  i := 0;
  WHILE (i < argLen) AND (argStr[i] = ' ') DO
    INC(i); END;

  j := 0;
  WHILE (i < argLen) AND
        (j < maxWidth) AND
        IsAllowed(argStr[i]) DO
    INC(i);
    INC(j);
  END; (* WHILE *)

  ol.New(edG.ScanStr,j+1);
  IF edG.ScanStr=NIL THEN
    INCL(edG.Status,edG.memoryFail); edG.Rc := edE.cmdSevere;
    RETURN;
  END;
  e.CopyMem(edG.LineBuffer[edG.Text.pos],edG.ScanStr^,j);
  edG.ScanStr[j] := 0X;
  edL.Title(edG.ScanStr^); edG.Rc := edE.cmdValid2;
END dScanf;

(*-------------------------------------------------------------------------*)

PROCEDURE ReleaseVar(var: lst.NodePtr);
BEGIN
  lst.Remove(VarList,var);
  DISPOSE(var(Variable).string);
  (*DISPOSE(var(Variable).name);  wird mit var alocciert *)
  DISPOSE(var);
END ReleaseVar;


PROCEDURE FreeVars;
VAR
  var: lst.NodePtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  WHILE VarList.head#NIL DO
    var := VarList.head;
    VarList.head := var.next;
    ReleaseVar(var);
  END;
END FreeVars;


PROCEDURE dSet*;
VAR
  var: lst.NodePtr;
  len: INTEGER;

  PROCEDURE SetString;
  BEGIN
    len := edL.Length(edG.Arg[1]^)+1;
    ol.New(var(Variable).string,len);
    IF var(Variable).string = NIL THEN
      edG.Rc := edE.cmdSevere; INCL(edG.Status,edG.memoryFail);
      ReleaseVar(var);
      RETURN;
    END;
    e.CopyMem(edG.Arg[1]^,var(Variable).string^,len);
  END SetString;

  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  var := VarList.head;
  WHILE var # NIL DO
    IF var(Variable).name^ = edG.Arg[0]^ THEN
      DISPOSE(var(Variable).string);
      SetString;
      RETURN;
    END;
    var := var.next;
  END;
  len := edL.Length(edG.Arg[0]^)+1;
  ol.New(var,s.SIZE(Variable)+len);
  IF var = NIL THEN
    edG.Rc := edE.cmdSevere; INCL(edG.Status,edG.memoryFail);
    RETURN;
  END;
  (* $TypeChk- *)
  s.INIT(var(Variable)); (* $TypeChk= *)
  var(Variable).name := s.VAL(s.ADDRESS,s.VAL(LONGINT,var)+s.SIZE(Variable));
  e.CopyMem(edG.Arg[0]^,var(Variable).name^,len);
  lst.AddTail(VarList,var);
  SetString;
END dSet;


PROCEDURE dSetEnv*;
BEGIN
  IF NOT sl.SetDEnv(edG.Arg[0]^,edG.Arg[1]^) THEN
    edG.Rc := edE.cmdSevere;
  END;
END dSetEnv;


PROCEDURE dUnsetEnv*;
BEGIN
  IF NOT sl.UnSetDEnv(edG.Arg[0]^) THEN
    edG.Rc := edE.cmdSevere;
  END;
END dUnsetEnv;


PROCEDURE dUnset*;
VAR
  var: lst.NodePtr;
  len: INTEGER;
BEGIN
  var := VarList.head;
  WHILE var # NIL DO
    IF var(Variable).name^ = edG.Arg[0]^ THEN
      ReleaseVar(var);
      RETURN;
    END;
    var := var.next;
  END;
END dUnset;


(* search order: (1) intern variables, (2) ENV:, (3) key-macros *)

PROCEDURE GetVar*(find: edG.StringPtr): edG.StringPtr;
VAR
  var: lst.NodePtr;
  string: edG.StringPtr;
  len: INTEGER;
  (* $ClearVars- *)

  PROCEDURE Convert(i: LONGINT);
  BEGIN
    Printf.SPrintf1(VarBuffer,"%ld",i);
  END Convert;

BEGIN
  (* $ClearVars= *)
  IF    find^ = "findstr"  THEN RETURN edG.FindStr;
  ELSIF find^ = "repstr"   THEN RETURN edG.ReplaceStr;
  ELSIF find^ = "scanf"    THEN RETURN edG.ScanStr;
  ELSIF find^ = "filename" THEN RETURN s.ADR(edG.Text.name);
  ELSIF find^ = "rexxport" THEN RETURN s.ADR(edG.AEdrxPort);
  ELSIF find^ = "currentline" THEN RETURN s.ADR(edG.LineBuffer);
  ELSIF find^ = "colno"    THEN
    Convert(edG.Text.pos+1); RETURN s.ADR(VarBuffer);
  ELSIF find^ = "lineno" THEN
    Convert(edG.Text.line+1); RETURN s.ADR(VarBuffer);
  ELSIF find^ = "modified" THEN
    VarBuffer := "0";
    IF edG.modified IN edG.Text.status THEN VarBuffer := "1"; END;
    RETURN s.ADR(VarBuffer);
  ELSIF find^ = "margin" THEN
    Convert(edG.Text.margin); RETURN s.ADR(VarBuffer);
  END;
  var := VarList.head;
  WHILE var # NIL DO
    IF var(Variable).name^ = find^ THEN
      RETURN var(Variable).string; END;
    var := var.next;
  END;
  string := sl.GetDEnv(find^);
  IF string # NIL THEN
    INCL(edG.Status,edG.disposeString); (* _mu_ DISPOSE sein, wg. SupLib *)
    RETURN string;
  END;
  string := edK.KeySpectroMacro(find);
  IF string = NIL THEN string := eMn.MenuToMacro(find); END;
  RETURN string;
END GetVar;

(*-----------------------------------------------------------------------*)

PROCEDURE dFindStr*;
VAR
  sp: POINTER TO edG.StringPtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF replace IN edG.ArgSet THEN sp := s.ADR(edG.ReplaceStr);
                  ELSE sp := s.ADR(edG.FindStr); END;
  DISPOSE(sp^);
  sp^ := edL.CopyString(edG.Arg[0]);
END dFindStr;

(*-----------------------------------------------------------------------*)

PROCEDURE Next{"FindNext"}(VAR findStruct{8}:FindReplaceStruct): LONGINT;
PROCEDURE Prev{"FindPrev"}(VAR findStruct{8}:FindReplaceStruct): LONGINT;
PROCEDURE NCNext{"FindNCNext"}(VAR findStruct{8}:FindReplaceStruct): LONGINT;
PROCEDURE NCPrev{"FindNCPrev"}(VAR findStruct{8}:FindReplaceStruct): LONGINT;


PROCEDURE Replace;
VAR
  Rlen,Flen: INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  Rlen := edL.Length(edG.ReplaceStr^);
  Flen := edL.Length(edG.FindStr^);
  IF (edG.LineBufferLen+Rlen-Flen >= edG.MaxLineLength) THEN
    edG.Rc := edE.cmdError; edL.Title(ReplaceLineTooLong); RETURN;
  END;
  str.Delete(edG.LineBuffer,edG.Text.pos,Flen);
  str.Insert(edG.LineBuffer,edG.Text.pos,edG.ReplaceStr^);
  IF (edG.multiMode IN edG.Status) & (Rlen > 0) THEN
    DEC(Rlen); END;
  INC(edG.Text.pos,Rlen);
  edD.PutBackLine;
  edD.TextRedisplayCurrentLine;
END Replace;


PROCEDURE dFind*;
VAR
  LineChg: LONGINT;
  FindData: FindReplaceStruct;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  edD.PutBackLine;
  IF NOT (oldStrings IN edG.ArgSet) THEN
    DISPOSE(edG.FindStr);
    edG.FindStr := edL.CopyString(edG.Arg[0]);
    IF replace IN edG.ArgSet THEN
      DISPOSE(edG.ReplaceStr);
      edG.ReplaceStr := edL.CopyString(edG.Arg[1]);
      IF edG.multiMode IN edG.Status THEN EXCL(edG.ArgSet,replace); END;
    END;
    IF edG.memoryFail IN edG.Status THEN RETURN; END;
  END;
  IF edG.FindStr = NIL THEN
    edG.Rc := edE.cmdError; edL.Title(NoFindPattern); RETURN; END;
  IF (replace IN edG.ArgSet) AND (edG.ReplaceStr = NIL) THEN
    edG.Rc := edE.cmdError; edL.Title(NoReplacePattern); RETURN; END;
  FindData.findStr := edG.FindStr; FindData.flen := edL.Length(edG.FindStr^);
  FindData.line := edG.Text.actLinePtr; FindData.pos := edG.Text.pos+1;
  LOOP
    IF edG.ignoreCase IN edG.Text.status THEN
      IF previous IN edG.ArgSet THEN
        LineChg := NCPrev(FindData);
      ELSE
        LineChg := NCNext(FindData);
      END;
    ELSE
      IF previous IN edG.ArgSet THEN
        LineChg := Prev(FindData);
      ELSE
        LineChg := Next(FindData);
      END;
    END;
    IF (LineChg < 0) THEN
      edG.Rc := edE.cmdFailed; edL.Title(PatternNotFound);
      RETURN;
    END;
    edG.Text.actLinePtr := FindData.line; edG.Text.pos := FindData.pos;
    IF previous IN edG.ArgSet THEN DEC(edG.Text.line,LineChg);
                              ELSE INC(edG.Text.line,LineChg); END;
    edD.TextLoad;
    edD.TextSync(FALSE);

    IF replace IN edG.ArgSet THEN
      Replace;
    ELSIF edG.multiMode IN edG.Status THEN
      edL.Title(ReplaceQuestion); edG.Rc := edE.cmdValid2;
    END;
    IF ~((edG.multiMode IN edG.Status) & (replace IN edG.ArgSet))
    OR (edG.Rc >= edE.abortLevel) THEN
      EXIT; END;
    IF edL.BreakCheck() THEN edG.Rc := edE.cmdFailed5; EXIT; END;
    INC(FindData.pos,edL.Length(edG.ReplaceStr^));
  END;
END dFind;

(*-----------------------------------------------------------------------*)

(**** multi-replace ***)

PROCEDURE dMultiReplace*;
BEGIN
  INCL(edG.Status,edG.multiMode);
  dFind;
END dMultiReplace;


PROCEDURE MultiReplace*(IMsg: I.IntuiMessagePtr);
VAR
  Buffer: ARRAY 2 OF CHAR;
  Blen: INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  Blen := SHORT(sl.DeadKeyConvert(IMsg,Buffer,2,NIL));
  IF Blen = 1 THEN
     CASE Buffer[0] OF
     "t","a":
          IF Buffer[0] = "t" THEN
            INC(edG.NoScreenUpdate);
            wasTurboReplace := TRUE;
            edL.Title(TurboReplacePleaseWait);
          END;
          Replace;
          IF edG.Rc < edE.abortLevel THEN
            edG.ArgSet := {replace,oldStrings};
            dFind;
            EXCL(edG.Status,edG.multiMode);
          END;|
     "y": Replace;
          edG.ArgSet := {oldStrings};
          dFind;|
     "n": edG.ArgSet := {oldStrings};
          dFind;|
     "q",03X:  (* Ctrl-C *)
          EXCL(edG.Status,edG.multiMode);
          edG.Rc := edE.cmdValid1;  (* ob das so sinnvoll ist? *)
          edL.WindowTitle;
          RETURN;
     ELSE
     END;
  END;
  IF edG.Rc >= edE.abortLevel THEN
    EXCL(edG.Status,edG.multiMode);
    IF wasTurboReplace THEN
      DEC(edG.NoScreenUpdate);
      wasTurboReplace := FALSE;
      edD.TextSync(TRUE);
    END;
  ELSIF edG.multiMode IN edG.Status THEN
    edL.Title(ReplaceQuestion);
  END;
END MultiReplace;

(*-------------------------------------------------------------------------*)

PROCEDURE dNewWindow*;
VAR
  newText: edG.TextHeaderPtr;
  nw: I.NewWindow;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  nw := edG.StdWindow;
  e.CopyMemQuick(edG.Config.edges,nw.leftEdge,s.SIZE(edG.Config.edges));
  IF edG.TempHeight#0 THEN
    nw.height := edG.TempHeight; edG.TempHeight := 0;
  END;
  IF edG.TempWidth #0 THEN
    nw.width  := edG.TempWidth; edG.TempWidth := 0;
  END;
  IF edG.Config.screenDepth # 0 THEN
    nw.screen := edG.Screen;
    nw.type := I.customScreen;
  END;
  newText := edL.TextInit(nw);
  IF newText=NIL THEN
    INCL(edG.Status,edG.memoryFail); edG.Rc := edE.cmdSevere;
    RETURN;
  END;
  eMn.MenuStrip(newText.win);
  edD.SwitchEdit(newText);
END dNewWindow;


(* set LeftEdge/TopEdge/Width/Height/tempWidth/tempHeight *)
PROCEDURE dWinEdges*;
VAR
  ip: POINTER TO INTEGER;
  long: LONGINT;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF temp IN edG.ArgSet THEN
    IF height IN edG.ArgSet THEN
      ip := s.ADR(edG.TempHeight);
    ELSE
      ip := s.ADR(edG.TempWidth);
    END;
  ELSIF (leftEdge-edG.ArgSet={}) THEN
    ip := s.ADR(edG.Config.edges.left);
  ELSIF top IN edG.ArgSet     THEN
    ip := s.ADR(edG.Config.edges.top);
  ELSIF height IN edG.ArgSet THEN
    ip := s.ADR(edG.Config.edges.height);
  ELSE
    ip := s.ADR(edG.Config.edges.width);
  END;
  IF edL.StrToInt(edG.Arg[0],long) AND (long < MAX(INTEGER)) THEN
    ip^ := ABS(SHORT(long));
  ELSE
    edL.Title(edG.BadArgument); edG.Rc := edE.cmdError;
  END;
END dWinEdges;


PROCEDURE dResize*;
VAR
  Win: I.WindowPtr;
  cols, rows: LONGINT;
  width, height: LONGINT;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF NOT (edL.StrToInt(edG.Arg[0],cols))
  OR NOT (edL.StrToInt(edG.Arg[1],rows))
  OR (cols > MAX(INTEGER)) OR (rows > MAX(INTEGER)) THEN
    edL.Title(edG.BadArgument); edG.Rc := edE.cmdError;
    RETURN;
  END;
  Win := edG.Text.win;
  width := (SHORT(cols)*Win.rPort.font.xSize)+Win.borderLeft+Win.borderRight;
  height:= (SHORT(rows)*Win.rPort.font.ySize)+Win.borderTop+Win.borderBottom;
  IF (width > MAX(INTEGER)) OR (height > MAX(INTEGER)) THEN
    edL.Title(edG.BadArgument); edG.Rc := edE.cmdError;
    RETURN;
  END;
  IF (width < 16) OR (height < 16)
  OR (width > Win.wScreen.width - Win.leftEdge)
  OR (height > Win.wScreen.height - Win.topEdge) THEN
    edL.Title(WindowTooBig); edG.Rc := edE.cmdError;
    RETURN;
  END;
  I.SizeWindow(Win,width-Win.width, height-Win.height);
  edD.SetWindowParams;
  d.Delay(50*2); (* wait 2 seconds *)
END dResize;

(*-------------------------------------------------------------------------*)

PROCEDURE dMyPri*;
VAR
  long: LONGINT;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF edL.StrToInt(edG.Arg[0],long) AND (long<=MAX(SHORTINT))
  AND (long>=MIN(SHORTINT)) THEN
    IF e.SetTaskPri(e.FindTask(NIL),SHORT(SHORT(long))) = 0 THEN END;
  ELSE
    edL.Title(edG.BadArgument); edG.Rc := edE.cmdError;
  END;
END dMyPri;


PROCEDURE dExecute*;
VAR
  OldLock: d.FileLockPtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  OldLock := d.CurrentDir(edG.Text.dirLock);
  IF NOT d.Execute(edG.Arg[0]^,NIL,NIL) THEN
    edG.Rc := edE.cmdFailed; edL.Title(ExecuteFailed); END;
  OldLock := d.CurrentDir(OldLock);
END dExecute;

(*-------------------------------------------------------------------------*)

PROCEDURE dNoScreenUpdate*;
BEGIN
  INC(edG.NoScreenUpdate);
END dNoScreenUpdate;

(*-------------------------------------------------------------------------*)

BEGIN
  lst.Init(VarList);
  edG.FindStr := NIL; edG.ReplaceStr := NIL; edG.ScanStr := NIL;
  edG.Screen := NIL; edG.Config.screenDepth := 0;

CLOSE
  IF edG.Config.screenDepth # 0 THEN I.OldCloseScreen(edG.Screen); END;
  FreeVars;
END EdCmd2.

