(*************************************************************************

:Program.    EdFileTools.mod
:Contents.   File-Management for AmokEd
:Author.     Hartmut Goebel
:Language.   Oberon
:Translator. Amiga Oberon Compiler V2.13
:Imports.    Printf (Volker Rudolph)
:History.    V0.1, 25 Sep 1990 Hartmut Goebel
:History.    V1.0, 14 Apr 1991 Hartmut Goebel [hG]
:History.    V1.0b 22 Apr 1991 [hG] Source-Bug in LoadLines removed
:History.    V1.1  22 Apr 1991 [hG] +doSaveConfig, GetConfig
:History.    V1.2  11 Jun 1991 [hg], Volker Rudolph: +_Fast_LoadLines etc.
:History.    V1.2b 21 Oct 1991 [hg] - Bug in ArpLoad (File/path-Split)
:History.    V1.2c 26 Oct 1991 [hG] asl.FileReq jetzt zu Beginn alloziert
:Date.       08 Feb 1992 13:28:17

*************************************************************************)
(* $Debug- *)
MODULE EdFileTools;

IMPORT
  ASCII, Printf,
  asl: ASL,
  d  : Dos,
  e  : Exec,
  edB: EdBlocks,
  edD: EdDisplay,
  edE: EdErrors,
  edG: EdGlobalVars,
  edK: EdKeyboard,
  edL: EdLowLevel,
  fs : EdFileSystem,
  g  : Graphics,
  I  : Intuition,
  lst: EdLists,
  ol : OberonLib,
  str: Strings,
  req: Requests,
  s  : SYSTEM,
  u  : Utility;

(*------------------------------------------------------------------------*)

CONST
  newFile*=0;     (* NewFile oder InsFile  }                         *)
  saveAs*=1;      (* SaveAs oder SaveOld   } mssen unterschiedlich  *)
  saveBlock* = 2; (*                       } sein wegen ArpLoad/Save *)
  source* = 3;    (*                       }                         *)
  sourceDoNotTitle* = 4 (*source*);
  saveSysMap* = 0;

  NewFile*="NEWFILE";
  InsFile*="INSFILE";
  SaveAs*="SAVEAS";
  SaveOld*="SAVEOLD";
  SaveBlock*="SAVEBLOCK";
  Source* = "SOURCE";

  Loading = "Loading...";
  Saving = "Saving...";
  FileNotFound = "File Not Found";
  UnableToOpenFile = "Unable to open file";
  ReadError = "Read Error!!";
  WriteFailed = "Write failed!";
  FileReqNotAvailable ="File Requester not available";
  UnableToCD = "Unable to CD";
  ConfigFileName = "s:AmokEd.config";
  DelModImage = "Delete modified Image?";

  (*
   * Solange NoGuruRq drin ist, wird Requests eh benutzt, darum
   * diesen Requester auch ber Requests
   *
  ReqBody = I.IntuiText(0,1,g.jam2,12,8,NIL,s.ADR(DelModImage),NIL);
  ReqOkay = I.IntuiText(0,1,g.jam2,6,3,NIL,s.ADR(edG.Okay),NIL);
  ReqCancel = I.IntuiText(0,1,g.jam2,6,3,NIL,s.ADR(edG.Cancel),NIL);
  *)

VAR
  arpbase: e.LibraryPtr;
  fr: asl.FileRequesterPtr;
CONST
  longDSize = 255;
  longFSize = 126;
  longPath = 0;

TYPE
  DirName = ARRAY longDSize+1 OF CHAR;
  FileName = ARRAY longFSize+1 OF CHAR;
  PatternString = ARRAY 80 OF CHAR;

  DirNamePtr = POINTER TO DirName;
  FileNamePtr = POINTER TO FileName;

VAR
  Dirname: DirName; (* wird auch als Buffer fr SaveLines benutzt *)
  Filename: FileName;
  Pattern: PatternString;

TYPE
  FileRequester = STRUCT
    hail      : e.ADDRESS;    (* Hailing text                 *)
    file      : FileNamePtr;  (* Filename array (FCHARS + 1)  *)
    dir       : DirNamePtr;   (* Directory array (DSIZE + 1)  *)
    window    : I.WindowPtr;  (* Window requesting or NULL    *)
    funcFlags : SHORTSET;     (* Set bitdef's below           *)
    flags2    : SHORTSET;     (* New flags...                 *)
    function  : PROCEDURE;    (* Your function, see bitdef's  *)
    leftEdge  : INTEGER;      (* To be used later...          *)
    topEdge   : INTEGER;
    reserved  : ARRAY 3 OF INTEGER;
    argList   : LONGINT;
    userData  : e.ADDRESS;
  END;

  FileRequesterPtr = POINTER TO FileRequester;

(*------------------------------------------------------------------------*)

PROCEDURE FileRequest {arpbase,-294}(fr{8}:FileRequesterPtr): BOOLEAN;

(*------------------------------------------------------------------------*)

PROCEDURE FileReq*(hail: edG.StringPtr; save: BOOLEAN): BOOLEAN;
VAR
  j: INTEGER;
  FR: FileRequester;
  flags: LONGINT;
  res: BOOLEAN;
BEGIN
  LOOP
    j := edL.Length(Filename);
    WHILE (j>=0) AND (Filename[j]#":") AND (Filename[j]#"/") DO DEC(j) END;
    IF j >= 0 THEN
      str.Cut(Filename,0,j+1,Dirname);
      str.Delete(Filename,0,j+1);
      IF Dirname[j] = "/" THEN
        Dirname[j] := 0X; END;
    END;
    IF asl.asl#NIL THEN
      flags := ASH(1,asl.patGad);
      IF save THEN INC(flags,ASH(1,asl.save)) END;
      res := asl.AslRequestTags(fr,
                                asl.hail,    hail,
                                asl.file,    s.ADR(Filename),
                                asl.dir,     s.ADR(Dirname),
                                asl.window,  edG.Text.win,
                                asl.pattern, s.ADR(Pattern),
                                asl.funcFlags,flags,
                                u.done)#NIL;
      COPY(fr.dir^,Dirname);
      COPY(fr.file^,Filename);
      IF NOT res THEN EXIT END;
    ELSE
      FR.hail     := hail;
      FR.file     := s.ADR(Filename);
      FR.dir      := s.ADR(Dirname);
      FR.window   := edG.Text.win;
      FR.funcFlags:= SHORTSET{};
      FR.flags2   := SHORTSET{longPath};
      FR.function := NIL;
      FR.leftEdge := 0;
      FR.topEdge  := 0;
      IF NOT FileRequest(s.ADR(FR)) THEN EXIT END;
    END;
    j := edL.Length(Dirname);
    IF (j>0) AND (Dirname[j-1]#"/") AND (Dirname[j-1]#":") THEN
      Dirname[j] := "/"; INC(j);
      Dirname[j] := 0X;
    END;
    IF s.SIZE(Filename)>j+edL.Length(Filename) THEN
      str.Insert(Filename,0,Dirname);
      RETURN TRUE;
    END;
  END;
  RETURN FALSE;
END FileReq;


PROCEDURE GetPathAndName(string: ARRAY OF CHAR;  (* $CopyArrays- *)
                         VAR path, name: ARRAY OF CHAR);
(* <string> kann die gleiche Var sein wie <path> *)

VAR
  i: INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  i := edL.Length(string);
  WHILE (i > 0) AND (string[i]#"/") AND (string[i]#":") DO
    DEC(i);
  END;
  str.Cut(string,i+1,SHORT(LEN(name)),name);
  str.Cut(string,0,i,name);
END GetPathAndName;

(*-----------------------------------------------------------------------*)
(*
 * Ldt File <Name> in <List>, <Num> ist Anzahl geladener Zeilen
 *
 * Nicht Lschen! Wird bei MMU einsatz wieder gebraucht!
 *)
(*
PROCEDURE LoadLines*(VAR List: lst.List; Source: BOOLEAN;
                     (* $CopyArrays- *)
                     Name: edG.StringPtr): LONGINT;
VAR
  len: INTEGER;
  newLine: edG.LinePtr;
  Buffer: edG.StringPtr;
  Num: LONGINT;
  File: fs.File;
BEGIN
  NEW(Buffer);
  IF Buffer=NIL THEN
    INCL(edG.Status,edG.memoryFail); edG.Rc := edE.cmdSevere;
    RETURN -1;
  END;
  IF NOT fs.Open(File,Name^,FALSE) THEN
    DISPOSE(Buffer); RETURN -1; END;
  Buffer^ := "";
  Num := 0;
  LOOP
    IF NOT fs.ReadString(File,Buffer^) THEN
      IF File.status # fs.eof THEN
        edG.Rc := edE.cmdSevere; edL.Title(ReadError);
        Num := -1;
      END;
      EXIT;
    END;
    IF Source THEN
      IF Buffer^ # "" THEN
        Num := 0;
        REPEAT
          IF Buffer[Num]=ASCII.ht THEN Buffer[Num] := 20X; END;
          INC(Num);
        UNTIL (Num = edG.MaxLineLength) OR (Buffer[Num] = 0X);
        edG.ExecCmd(Buffer);
      END;
    ELSE
      len := edL.Length(Buffer^);
      REPEAT
        DEC(len);
      UNTIL (len < 0) OR (Buffer[len]#20X);
      INC(len);
      Buffer[len] := 0X;
      len := len+edG.MemStepSize-(len MOD edG.MemStepSize);
      edL.NewLine(newLine,len);
      IF newLine#NIL THEN
        e.CopyMem(Buffer^,newLine(edG.Line).string^,len);
        lst.AddTail(List,newLine);
        INC(Num);
      ELSE
        INCL(edG.Status,edG.memoryFail); edG.Rc := edE.cmdSevere;
        Num := -1;
      END; (* IF newLine # NIL *)
    END;
    IF Num < 0 THEN (* MemoryFailed *)
      EXIT; END;
  END; (* LOOP *)
  IF fs.Close(File) THEN END;
  DISPOSE(Buffer);
  RETURN Num;
END LoadLines;
*)

(*
 * FastLoadLines hat einen groen Nachteil: es funktioniert nicht mit
 * der MMU, weil die die Stckchenweise Rckgabe eines angeforderten
 * Speicherblocks nicht ermglicht. D.H., die Zeilen mssen doch einzeln
 * alloziert werden :-(
 *)
PROCEDURE FastLoadLines(VAR list:lst.List;name: edG.StringPtr):LONGINT;
CONST
  BlockSize = 10000;
  MaxLineSize = edG.LineAllocSize + edG.MaxLineLength;
  ChunkSize = s.SIZE(e.MemChunk);

VAR
  line:POINTER TO edG.Line;
  node:lst.NodePtr;
  end:s.ADDRESS;
  file:fs.File;
  len:LONGINT;
  num:LONGINT;

BEGIN
  IF NOT fs.Open(file,name^,FALSE) THEN
    RETURN -1
  END; (* IF *)

  lst.Init(list);
  line := NIL;
  num := 0;

  REPEAT

    (* do we need more memory ? *)
    IF (line = NIL) OR
       (s.VAL(LONGINT,line) > (s.VAL(LONGINT,end) - MaxLineSize)) THEN

      (* free unused memory of last block *)
      IF line # NIL THEN
        e.FreeMem(line,s.VAL(LONGINT,end)-s.VAL(LONGINT,line));
      END; (* IF *)

      (* get new memory block *)
      line := e.AllocMem(BlockSize,LONGSET{});
      IF line = NIL THEN

        (* free memory used so far *)
        node := lst.RemHead(list);
        WHILE node # NIL DO
          WITH node:edG.Line DO
            e.FreeMem(node.string,node.len+edG.LineAllocSize);
          END;
          node := lst.RemHead(list);
        END; (* WHILE *)

        s.SETREG(0,fs.Close(file));
        INCL(edG.Status,edG.memoryFail);
        edG.Rc := edE.cmdSevere;
        RETURN -1;
      END;

      end := s.VAL(s.ADDRESS,s.VAL(LONGINT,line)+BlockSize);

    END; (* IF *)

    (* read line *) (* $TypeChk- *)
    s.INIT(line); (* $TypeChk= *)
    line.string := s.VAL(s.ADDRESS,s.VAL(LONGINT,line) + edG.LineAllocSize);
    len := fs.ReadStringLenTab(file,line.string^,8);
    IF len >= 0 THEN
      line.string[len] := "\o";
      line.len := SHORT(len + ChunkSize-(len MOD ChunkSize));
      lst.AddTail(list,line);
      line := s.VAL(s.ADDRESS,s.VAL(LONGINT,line)+line.len+edG.LineAllocSize);
      INC(num);
    END; (* IF *)

  UNTIL (len = -1);

  IF s.VAL(LONGINT,line) < s.VAL(LONGINT,end) THEN
    e.FreeMem(line,s.VAL(LONGINT,end)-s.VAL(LONGINT,line));
  END; (* IF *)

  IF fs.Close(file) THEN END;
  RETURN num;
END FastLoadLines;


PROCEDURE dLoad*;
VAR
  oldlock: d.FileLockPtr;
  NumNewLines: LONGINT;
  TempLine: edG.LinePtr;
  NewList: lst.List;
BEGIN
  edD.PutBackLine;
  IF (newFile IN edG.ArgSet) THEN
    IF (edG.modified IN edG.Text.status)
    (*
    AND NOT I.AutoRequest(edG.Text.win,ReqBody,s.ADR(ReqOkay),ReqCancel,
                          LONGSET{},LONGSET{},320,58)
    *)
    AND NOT req.RequestWin(""(*edG.Version*),DelModImage,edG.Okay,edG.Cancel,
                       edG.Text.win)
    THEN
      edG.Rc := edE.cmdValid1; RETURN;
    END;
    e.CopyMemQuick(edG.Arg[0]^,edG.Text.name,edG.FileNameLength);
    edG.Text.name[edG.FileNameLength-1] := 0X;
    IF edG.Text = edG.Block.Owner THEN
      edB.dUnblock; END;
  END;

  oldlock := d.CurrentDir(edG.Text.dirLock);
  edL.Title(Loading); edG.Rc := edE.cmdValid2;
  NumNewLines := FastLoadLines(NewList,edG.Arg[0]);
  IF NumNewLines = -1 THEN
    IF edG.Rc < edE.failLevel THEN
      edG.Rc := edE.cmdError; edL.Title(FileNotFound);
    END;
    RETURN;
  END;
  edL.Title(edG.Okay);
  oldlock := d.CurrentDir(oldlock);

  (* Newfile oder leeres File *)
  IF (newFile IN edG.ArgSet) OR (edG.Text.actLinePtr(edG.Line).string^ = "")
                       AND (edG.Text.numberOfLines=1) THEN
    edL.FreeLines(edG.Text.lineList,edG.Text.lineList.head(edG.Line),
                  edG.Text.lineList.tail(edG.Line));
  END;
  IF edG.Text.lineList.head=NIL THEN
    edG.Text.line := 0; edG.Text.pos := 0; edG.Text.topLine := 0;
    EXCL(edG.Text.status,edG.modified);
    IF NewList.head#NIL THEN
      edG.Text.numberOfLines := NumNewLines;
      lst.AddMarkHead(edG.Text.lineList,NewList);
    ELSE
      edG.Text.numberOfLines := 1;
      edL.NewLine(TempLine,edG.ChunkSize);
      lst.AddHead(edG.Text.lineList,TempLine);
    END;
    edG.Text.actLinePtr := edG.Text.lineList.head(edG.Line);
    edG.Text.topLinePtr := edG.Text.actLinePtr;
  ELSE
    INC(edG.Text.numberOfLines,NumNewLines);
    lst.AddMarkBefore(edG.Text.lineList,NewList,edG.Text.actLinePtr);
    INCL(edG.Text.status,edG.modified);
    edG.Text.actLinePtr := NewList.head(edG.Line);
    IF edG.Text.line = edG.Text.topLine THEN
      edG.Text.topLinePtr := edG.Text.actLinePtr; END;
  END;
  edD.TextLoad;
  edD.SetWindowParams;
  edD.TextRedisplay;
END dLoad;


PROCEDURE dSource*;
VAR
  len: INTEGER;
  Buffer: edG.StringPtr;
  File: fs.File;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  NEW(Buffer);
  IF Buffer=NIL THEN
    INCL(edG.Status,edG.memoryFail); edG.Rc := edE.cmdSevere;
    RETURN;
  END;
  IF fs.Open(File,edG.Arg[0]^,FALSE) THEN
    LOOP
      len := fs.ReadStringLenTab(File,Buffer^,1);
      IF len = -1 THEN
        IF File.status # fs.eof THEN
          edG.Rc := edE.cmdSevere; edL.Title(ReadError);
        END;
        EXIT;
      ELSIF len # 0 THEN
        (*   macht ReadStringLenTab
        REPEAT
          DEC(len);
          IF Buffer[len]=ASCII.ht THEN Buffer[len] := 20X; END;
        UNTIL len = 0;
        *)
        edG.ExecCmd(Buffer);
      END;
    END; (* LOOP *)
    IF fs.Close(File) THEN END;
  ELSIF NOT (sourceDoNotTitle IN edG.ArgSet) THEN
    edG.Rc := edE.cmdError; edL.Title(FileNotFound);
  END;
  DISPOSE(Buffer);
END dSource;

(*-----------------------------------------------------------------------*)

PROCEDURE SaveLines(Name: edG.StringPtr; start,end: edG.LinePtr);
VAR
  next: edG.LinePtr;
  File: fs.File;
  ok: BOOLEAN;
  i, j, k: INTEGER;
  ptr: e.STRPTR;
  Buffer: edG.String;
BEGIN
  IF NOT fs.Open(File,Name^,TRUE) THEN
    edL.Title(UnableToOpenFile); edG.Rc := edE.cmdSevere;
    RETURN;
  END;
  edL.Title(Saving);
  REPEAT
    IF edG.saveTabs IN edG.Text.status THEN
      ptr := s.VAL(e.STRPTR,start(edG.Line).string);
      i := 0; j := 0;
      WHILE ptr^[0] # "\o" DO
        Buffer[i] := ptr^[0];
        IF (j = 7) AND (Buffer[i] = ' ') AND (Buffer[i-1] = ' ') THEN
          k := j;
          WHILE (k >= 0) AND (Buffer[i] = ' ') DO
             DEC(k);
             DEC(i);
          END;
          INC(i);
          Buffer[i] := "\t";
        ELSE
          CASE Buffer[i] OF
          "\"", "\'", "\`", "(":
            j := edL.Length(ptr^);
            e.CopyMem(ptr^,Buffer[i],j+1);
            INC(i,j);
            ptr := s.VAL(s.ADDRESS,s.VAL(LONGINT,ptr)+j);
          ELSE END;
        END;
        ptr := s.VAL(s.ADDRESS,s.VAL(LONGINT,ptr)+1);
        INC(i);
        j := (j+1) MOD 8;
      END;
      Buffer[i] := "\o";
      ok := fs.WriteString(File,Buffer);
    ELSE
      ok := fs.WriteString(File,start(edG.Line).string^);
    END;
    IF ok AND (start # end) THEN
      start := start.next(edG.Line);
    ELSE
      start := NIL;
    END;
  UNTIL start = NIL;
  IF ok THEN
    edL.Title(edG.Okay); edG.Rc := edE.cmdValid2;
  ELSE
    edL.Title(WriteFailed); edG.Rc := edE.cmdSevere;
  END;
  ok := fs.Close(File);
END SaveLines;


PROCEDURE dSave*;
VAR
  SaveList: lst.List;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  edD.PutBackLine;
  SaveList := edG.Text.lineList;
  IF (saveBlock IN edG.ArgSet) THEN
    IF NOT edB.BlockSpecified() THEN RETURN; END;
    SaveList := edG.Block.mark;
  ELSIF NOT (saveAs IN edG.ArgSet) THEN
    edG.Arg[0] := s.ADR(edG.Text.name);
  END;
  SaveLines(edG.Arg[0],SaveList.head(edG.Line),SaveList.tail(edG.Line));
  IF (edG.Rc < edE.abortLevel) THEN
    IF NOT (saveBlock IN edG.ArgSet) THEN
      EXCL(edG.Text.status,edG.modified);
    ELSIF saveAs IN edG.ArgSet THEN
      GetPathAndName(edG.Arg[0]^,edG.Arg[0]^,edG.Text.name);
    END;
  END;
END dSave;

(*-----------------------------------------------------------------------*)

PROCEDURE dFileReq*;
VAR
  doDo: edG.ComProc;
  hail: edG.StringPtr;
  save, src: BOOLEAN;
  pat: PatternString;
BEGIN
  IF edG.fileReqAvail IN edG.Status THEN
    Filename := "";
    save := FALSE;
    IF saveAs IN edG.ArgSet THEN
      hail := s.ADR(SaveAs);
      doDo := dSave;
      save := TRUE;
      e.CopyMemQuick(edG.Text.name,Filename,edG.FileNameLength);
    ELSIF saveBlock IN edG.ArgSet THEN
      IF NOT edB.BlockSpecified() THEN RETURN; END;
      hail := s.ADR(SaveBlock);
      doDo := dSave;
      save := TRUE;
    ELSIF newFile IN edG.ArgSet THEN
      hail := s.ADR(NewFile);
      doDo := dLoad;
    ELSIF source IN edG.ArgSet THEN
      hail := s.ADR(Source);
      doDo := dSource;
      pat := Pattern;
      Pattern := "#?.aedrc";
      src := TRUE;
    ELSE
      hail := s.ADR(InsFile);
      doDo := dLoad;
    END;
    IF FileReq(hail,save) THEN
      edG.Arg[0] := s.ADR(Filename);
      doDo;
    ELSE
      edG.Rc := edE.cmdValid1;
    END;
    IF src THEN
      Pattern := pat;
    END;
  ELSE
    edL.Title(FileReqNotAvailable); edG.Rc := edE.cmdError;
  END;
END dFileReq;

(*-----------------------------------------------------------------------*)

PROCEDURE dChFilename*;
BEGIN
  e.CopyMemQuick(edG.Arg[0]^,edG.Text.name,edG.FileNameLength);
END dChFilename;


PROCEDURE dCD*;
VAR
  oldLock, Lock: d.FileLockPtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  oldLock := d.CurrentDir(edG.Text.dirLock);
  Lock := d.Lock(edG.Arg[0]^,d.sharedLock);
  IF Lock # NIL THEN
    d.UnLock(d.CurrentDir(oldLock));
    edG.Text.dirLock := Lock;
  ELSE
    IF d.CurrentDir(oldLock)=NIL THEN END;
    edG.Rc := edE.cmdFailed; edL.Title(UnableToCD);
  END;
END dCD;

(*-----------------------------------------------------------------------*)

PROCEDURE dSaveMap*;
VAR
  ptr: edG.StringPtr;
  hash: edK.HashPtr;
  i,len : INTEGER;
  soc, eoc, ksoc, keoc: CHAR;
  sysalso: BOOLEAN;
  File: fs.File;
  Buffer: edG.String;
BEGIN
  IF NOT fs.Open(File,edG.Arg[0]^,TRUE) THEN
    edL.Title(UnableToOpenFile); edG.Rc := edE.cmdSevere;
    RETURN;
  END;
  edL.Title(Saving);
  sysalso := saveSysMap IN edG.ArgSet;
  i := 0;
  REPEAT
    hash := edK.HashList[i];
    WHILE hash # NIL DO
      IF (hash.len # 0) OR sysalso THEN
        soc := "("; eoc := ")"; ksoc := "("; keoc := ")";
        ptr := hash.map;
        LOOP
          IF (ptr[0] = 0X) OR (ptr[0] = "(") THEN EXIT; END;
          IF ptr^ = "`" THEN
            soc := "`"; eoc := "'";
            EXIT;
          END;
          ptr := s.VAL(s.ADDRESS,s.VAL(LONGINT,ptr)+1);
        END; (* LOOP *)
        ptr := edK.cqTOa(hash.code,hash.qual);
        len := edL.Length(ptr^)-1;
        IF (ptr[len] = "(") OR (ptr[len] = ")") THEN
          ksoc := "`"; keoc := "'"; END;
        Printf.SPrintf6(Buffer,"map %lc%s%lc %lc%s%lc",
          ORD(ksoc),ptr,ORD(keoc),ORD(soc),hash.map,ORD(eoc));
        IF NOT fs.WriteString(File,Buffer) THEN
          IF fs.Close(File) THEN END;
          edL.Title(WriteFailed); edG.Rc := edE.cmdSevere;
          RETURN;
        END;
      END; (* IF hash.len#0 ... *)
      hash := hash.next;
    END; (* WHILE *)
    INC(i);
  UNTIL i = edK.Hashsize;
  IF fs.Close(File) THEN END;
  edL.Title(edG.Okay);
END dSaveMap;

(*-----------------------------------------------------------------------*)

PROCEDURE GetConfig*;
VAR
  config: fs.File;
BEGIN
  e.CopyMemQuick(edG.StdWindow,edG.Config.edges,s.SIZE(edG.Config.edges));
  edG.Config.screenDepth := 0;
  IF fs.Open(config,ConfigFileName,FALSE) THEN
    IF NOT fs.Read(config,edG.Config) THEN
      CASE config.status OF
        fs.toofar, fs.eof:|
      ELSE
        edL.Title(ReadError); edG.Rc := edE.cmdSevere;
      END;
    END;
    IF fs.Close(config) THEN END;
  END;
END GetConfig;


PROCEDURE dSaveConfig*;
VAR
  win: I.WindowPtr;
  config: fs.File;
BEGIN
  IF fs.Open(config,ConfigFileName,TRUE) THEN
    e.CopyMemQuick(edG.Text.win.leftEdge,
                   edG.Config.edges,s.SIZE(edG.Config.edges));
    IF NOT fs.Write(config,edG.Config) THEN
      edL.Title(WriteFailed); edG.Rc := edE.cmdSevere; END;
    IF fs.Close(config) THEN END;
  END;
END dSaveConfig;

(*-----------------------------------------------------------------------*)

BEGIN
  INCL(edG.Status,edG.fileReqAvail);
  Pattern := "~(#?.info)";
  IF asl.asl # NIL THEN
    fr := asl.AllocAslRequestTags(asl.fileRequest,
                                  asl.leftEdge,20,
                                  asl.topEdge, 20,
                                  asl.width,   300,
                                  asl.height,  200,
                                  u.done);
    IF fr = NIL THEN
      EXCL(edG.Status,edG.fileReqAvail);
    END;
  ELSE
    arpbase := e.OpenLibrary("arp.library",39);
    IF arpbase = NIL THEN
      EXCL(edG.Status,edG.fileReqAvail);
    END;
  END;
CLOSE
  IF arpbase#NIL THEN e.CloseLibrary(arpbase) END;

END EdFileTools.

