(*************************************************************************

:Program.    EdKeyboard.mod
:Contents.   Keyboard-Routines for Amok-Editor
:Author.     Hartmut Goebel
:Copyright.  Copyright  1987 by Matthew Dillon
:Copyright.  Oberon implementation Copyright  1990 by Hartmut Goebel
:Language.   Oberon
:Translator. Amiga Oberon Compiler V2.13
:History.    V0.1, 07 Nov 1990 Hartmut Goebel
:History.    V1.0, 14 Apr 1991 Hartmut Goebel [hG]
:History.    V1.1, 25 Apr 1991 [hG] -Bug NKx = KKx, Code optimiert
:History.    V1.1, 25 Apr 1991 [hG] z.T. e.Alloc/FreeMem statt New/Disp.
:History.    V1.1a 24 May 1991 [hG] -SpecKeyCode-Dummy (war Bug in V1.17.1)
:History.    V1.1b 07 Aug 1991 [hG] -Bug in GetCodeQual (-str.Upper(key))
:History.    V1.2  16 Dec 1991 [hG] schreibt nicht mehr in Konstanten!
:History.    V1.3  06 Feb 1992 [hG] jetzt ALLE Tasten belegbar, auch Umlaute
:Date.       08 Feb 1992 14:13:04

*************************************************************************)
(* $Debug- *)
MODULE EdKeyboard;

IMPORT
  asc: ASCII,
  e  : Exec,
  con: Console,
  edE: EdErrors,
  edG: EdGlobalVars,
  edL: EdLowLevel,
  ie : InputEvent,
  I  : Intuition,
  ol : OberonLib,
  sl : SupLib,
  str: Strings,
  s  : SYSTEM;

CONST
  qualShift = 1; qualCtrl = 2; qualAmiga = 3; qualAlt = 4;
  qualLmb   = 5; qualMmb  = 6; qualRmb   = 7;

  Hashsize* = 64 (* war: KeyMap.maxKeys *);
  NumRawKeyCodes = 128; (* lt. ARKM Libs & Devs max. mgl. Raw-Codes *)

  NumDefaultKeys = 60;
  SpecKeyNum = 43;

  qMove* = 06BX;
  ReturnRawCode = 44X;
  Return = "return";
  UnknownKey = "Unknown Key";

TYPE
  SpecKeyArray = ARRAY SpecKeyNum,3 OF CHAR;
  SpecKeyCodes = ARRAY SpecKeyNum OF BYTE;

  HashPtr* = POINTER TO Hash;
  Hash* = STRUCT
    next*: HashPtr;
    code*: BYTE;     (* Keycode *)
    (*mask: SHORTSET;  (* qual. mask fr was?? *)*)
    qual*: SHORTSET; (* qual. comp *)
    len*: INTEGER;  (* incl. 00C, --> len = 0 fr SysMap *)
    map*: edG.StringPtr; (* Command String *)
  END;

  DefMapArray = ARRAY NumDefaultKeys OF STRUCT
    from, to: edG.StringPtr;
  END;

CONST
  SpecKey = SpecKeyArray(
    "ESC", "F1",  "F2",  (*  0-2  *)
    "F3",  "F4",  "F5",  (*  3-5  *)
    "F6",  "F7",  "F8",  (*  6-8  *)
    "F9",  "F10", "DEL", (*  9-11 *)
    "BAC", "BS",  "TAB", (* 12-14 *)
    "HEL", "RET", "UP",  (* 15-17 *)
    "DOW", "RIG", "LEF", (* 18-20 *)
    "ENT", "NK-", "NK.", (* 21-23 *)
    "NK0",               (* 24    *)
    "NK1", "NK2", "NK3", (* 25-27 *)
    "NK4", "NK5", "NK6", (* 28-30 *)
    "NK7", "NK8", "NK9", (* 31-33 *)
    "NK(", "NK)", "NK/", (* 34-36 *)
    "NK*", "NK+",        (* 37-38 *)
    "LMB", "MMB",        (* 39-40 *)
    "RMB", "MMO");       (* 41-42 *)

  DefaultMap = DefMapArray(
    s.ADR("up"),      s.ADR("up"),
    s.ADR("c-esc"),   s.ADR("recall"),
    s.ADR("return"),  s.ADR("return"),
    s.ADR("enter"),   s.ADR("return"),
    s.ADR("esc"),     s.ADR("esc"),
    s.ADR("down"),    s.ADR("down"),
    s.ADR("right"),   s.ADR("right"),
    s.ADR("left"),    s.ADR("left"),
    s.ADR("bs"),      s.ADR("bs"),
    s.ADR("del"),     s.ADR("del"),
    s.ADR("tab"),     s.ADR("tab"),
    s.ADR("a-up"),    s.ADR("scrollup"),
    s.ADR("a-down"),  s.ADR("scrolldown"),
    s.ADR("a-r"),     s.ADR("nextr"),
    s.ADR("a-u"),     s.ADR("while cl (tlate -32 right)"),
    s.ADR("a-l"),     s.ADR("while cu (tlate +32 right)"),
    s.ADR("s-up"),    s.ADR("top"),
    s.ADR("s-down"),  s.ADR("bottom"),
    s.ADR("s-right"), s.ADR("last"),
    s.ADR("s-left"),  s.ADR("first"),
    s.ADR("s-tab"),   s.ADR("backtab"),
    s.ADR("s-del"),   s.ADR("deline"),
    s.ADR("s- "),     s.ADR("( )"),        (* shift space to space *)
    s.ADR("c-1"),     s.ADR("goto block"),
    s.ADR("c-c"),     s.ADR(""),           (* break.. map to a nop *)
    s.ADR("c-l"),     s.ADR("wleft"),
    s.ADR("c-r"),     s.ADR("wright"),
    s.ADR("c-i"),     s.ADR("insertmode on"),
    s.ADR("c-o"),     s.ADR("insertmode off"),
    s.ADR("c-j"),     s.ADR("join"),
    s.ADR("c-s"),     s.ADR("split first down"),
    s.ADR("c-del"),   s.ADR("remeol"),
    s.ADR("c-n"),     s.ADR("next"),
    s.ADR("c-p"),     s.ADR("prev"),
    s.ADR("c-/"),     s.ADR("escimm (find )"),
    (*s.ADR("c-]"),     s.ADR("ref"),*)
    (*s.ADR("c-["),     s.ADR("ctags"),*)
    s.ADR("c-g"),     s.ADR("escimm (goto )"),
    s.ADR("c-up"),    s.ADR("pageup"),
    s.ADR("c-down"),  s.ADR("pagedown"),
    s.ADR("c-q"),     s.ADR("quit"),
    s.ADR("c-f"),     s.ADR("reformat"),
    s.ADR("c-w"),     s.ADR("toggle wordwrap"),
    s.ADR("f1"),      s.ADR("escimm (insfile )"),
    s.ADR("f2"),      s.ADR("escimm (newfile )"),
    s.ADR("f3"),      s.ADR("escimm (newwindow newfile)"),
    s.ADR("f6"),      s.ADR("saveold iconify"),
    s.ADR("f7"),      s.ADR("escimm (bsave )"),
    s.ADR("f8"),      s.ADR("saveold escimm (newfile )"),
    s.ADR("f9"),      s.ADR("saveold"),
    s.ADR("f10"),     s.ADR("saveold quit"),
    s.ADR("c-b"),     s.ADR("block"),
    s.ADR("c-u"),     s.ADR("unblock"),
    s.ADR("a-d"),     s.ADR("bdelete"),
    s.ADR("a-c"),     s.ADR("bcopy"),
    s.ADR("a-m"),     s.ADR("bmove"),
    s.ADR("a-s"),     s.ADR("bsource"),
    s.ADR("a-S"),     s.ADR("unblock block block bsource"),
    s.ADR("L-lmb"),   s.ADR("tomouse"), (* left button *)
    s.ADR("L-mmo"),   s.ADR("tomouse"), (* mouse move with left held down *)
    s.ADR("R-rmb"),   s.ADR("iconify"), (* right button *)
    s.ADR("a-("),     s.ADR("`:-)'"));  (* Smily *)

VAR
  HashList*: ARRAY Hashsize OF HashPtr;
  cTOa, csTOa: ARRAY NumRawKeyCodes OF CHAR;
  SpecCodes: SpecKeyCodes;
  RetOvrString: edG.StringPtr; (* Zwischenspeicher fr *)
  RetOvrLen: INTEGER;          (* PROC ReturnOveride *)
  cqTOaBuffer: ARRAY 16 OF CHAR;

(*
 * Ermittelt aus Raw-code und qualifiern einen String, der die Taste
 * in der Form `qual-Key' beschreibt (Umkehrfunktion von GetCodeQual)
 *)

PROCEDURE cqTOa*(code: BYTE; qual: SHORTSET): edG.StringPtr;
VAR
  i,j: INTEGER;
BEGIN
(* $IFNOT ClearVars THEN *)
  i := 0;
  j := 0;
(* $END *)
  IF qualShift IN qual THEN cqTOaBuffer[i] := "s"; INC(i); END;
  IF qualCtrl  IN qual THEN cqTOaBuffer[i] := "c"; INC(i); END;
  IF qualAlt   IN qual THEN cqTOaBuffer[i] := "a"; INC(i); END;
  IF qualAmiga IN qual THEN cqTOaBuffer[i] := "A"; INC(i); END;
  IF qualLmb   IN qual THEN cqTOaBuffer[i] := "L"; INC(i); END;
  IF qualMmb   IN qual THEN cqTOaBuffer[i] := "M"; INC(i); END;
  IF qualRmb   IN qual THEN cqTOaBuffer[i] := "R"; INC(i); END;
  cqTOaBuffer[i] := "-"; INC(i);
  LOOP
    IF SpecCodes[j] = code THEN          (* Sondertaste? *)
      cqTOaBuffer[i] := SpecKey[j,0];
      cqTOaBuffer[i+1] := SpecKey[j,1];
      cqTOaBuffer[i+2] := SpecKey[j,2];
      INC(i,3);
      EXIT;
    END;
    INC(j);
    IF j = SpecKeyNum THEN                  (* Normale Taste *)
      cqTOaBuffer[i] := cTOa[ORD(code)];
      INC(i);
      EXIT;
    END;
  END;
  cqTOaBuffer[i] := 0X;
  RETURN s.ADR(cqTOaBuffer);
END cqTOa;


(*
 * Ermittelt aus einem String, der die Taste in der Form `qual-Key'
 * beschreibt, Rawcode und qualifier (Umkehrfunktion von cqTOa)
 *)

(* $Debug= *)
PROCEDURE GetCodeQual(key: edG.StringPtr;
                      VAR code: BYTE; VAR qual: SHORTSET): BOOLEAN;
(*
 * key : zu untersuchender String, z.B. "s-d"
 * code: Tastaturcode, qual: Qualifiers
 *)
VAR
  i, j, len: INTEGER;
  help : ARRAY 3 OF CHAR;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  i := 0;
  qual := SHORTSET{};
  len := edL.Length(key^);
  IF len > 1 THEN
    WHILE (i < len) AND (key[i] # "-") DO
      INC(i); END;
    j := i;
    WHILE j > 0 DO
      DEC(j);
      CASE key[j] OF
      |"s" : INCL(qual,qualShift);
      |"c" : INCL(qual,qualCtrl);
      |"a" : INCL(qual,qualAlt);
      |"A" : INCL(qual,qualAmiga);
      |"L" : INCL(qual,qualLmb);
      |"M" : INCL(qual,qualMmb);
      |"R" : INCL(qual,qualRmb);
      ELSE
        qual := SHORTSET{};
        j := i;
        i := 0;
      END;
    END; (* WHILE *)
    IF key[i] = "-" THEN INC(i); END;
  END; (* len > 1 *)
  IF len > i+1 THEN           (* long name : more than this Byte left  *)
    help[0] := CAP(key[i]);   (* nicht schoen, aber sicher *)
    help[1] := CAP(key[i+1]); (* eine kurze Mglichkeit    *)
    help[2] := CAP(key[i+2]);
    i := SpecKeyNum;
    REPEAT                      (* entsprechenden Rawcode suchen *)
      DEC(i);
      IF help = SpecKey[i] THEN
        code := SpecCodes[i];
        RETURN TRUE;
      END;
    UNTIL i = 0;
  ELSIF len = i+1 THEN                 (* single character keycap *)
    j := 0;            (* wichtig: von 0 aufwrts suchen, wg. NKx *)
    REPEAT
      IF key[i] = cTOa[j] THEN   (* ohne Shift *)
        code := SHORT(j);
        RETURN TRUE;
      END;
      INC(j);
    UNTIL j = NumRawKeyCodes;
    j := 0;
    REPEAT
      IF key[i] = csTOa[j] THEN  (* mit Shift *)
        code := SHORT(j);
        INCL(qual,qualShift);
        RETURN TRUE;
      END;
      INC(j);
    UNTIL j = NumRawKeyCodes;
  END; (* IF len > i *)
  RETURN FALSE;
END GetCodeQual;
(* $Debug- *)


(*
 * Fgt einen String als Tastaturbelegung der mit code & qual angegebenen
 * Taste in die Tasaturbelegung (Hash!!) ein. Die Hash wird aus
 * (Rawcode AND 64) gebildet, und alle Tastaturbelegungen mit dieser Hash
 * an eine Kette gehngt.
 *)

PROCEDURE AddHash(code: BYTE; (*mask,*) qual:SHORTSET;
                  map: edG.StringPtr; sysMap: BOOLEAN);
VAR
  thisHash: HashPtr;
  hPtr: POINTER TO HashPtr;
  HashValue: INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  HashValue := ORD(code) MOD Hashsize;
  thisHash := HashList[HashValue];
  hPtr := s.ADR(HashList[HashValue]);
  LOOP
    IF thisHash = NIL THEN  (* noch nicht belegt, einrichten *)
      thisHash := e.AllocMem(s.SIZE(Hash),LONGSET{e.memClear});
      IF thisHash=NIL THEN
        INCL (edG.Status,edG.memoryFail); edG.Rc := edE.cmdSevere;
        RETURN;
      END;
      hPtr^ := thisHash;
      thisHash.next := NIL;
      thisHash.code := code;
      (*thisHash.mask := mask;*)
      thisHash.qual := qual;
      EXIT;
    END;
    IF (thisHash.code = code) AND     (* richtige Taste? *)
       (*(thisHash.mask = mask) AND*)
       (thisHash.qual = qual) THEN
      (* Sysmap-Keys drfen nicht DISPOSEd werden *)
      IF thisHash.len # 0 THEN        (* vom User belegt? *)
        DISPOSE(thisHash.map); END;   (* lschen *)
      EXIT;
    END;
    hPtr := s.ADR(thisHash.next);
    thisHash := thisHash.next;
  END; (* LOOP *)
  IF NOT sysMap THEN
    thisHash.map := edL.CopyString(map);
    thisHash.len := edL.Length(map^);
  ELSE
    thisHash.map := map; (* SysMap *)
    thisHash.len := 0;
  END;
END AddHash;


PROCEDURE RemHash(code: BYTE; (*mask,*) qual: SHORTSET);
VAR
  thisHash, hnext,hlast: HashPtr;
  HashValue: INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  HashValue := ORD(code) MOD Hashsize;
  thisHash := HashList[HashValue];
  WHILE thisHash # NIL DO
    hnext := thisHash.next;
    IF (thisHash.code = code) (*AND (thisHash.mask = mask)*)
    AND (thisHash.qual = qual) THEN
      IF thisHash.len # 0 THEN
        DISPOSE(thisHash.map);
      END;
      IF thisHash = HashList[HashValue] THEN
        HashList[HashValue] := thisHash.next;
      ELSE
        hlast.next := thisHash.next;
      END;
      e.FreeMem(thisHash,s.SIZE(Hash));
      RETURN;
    END;
    hlast := thisHash;
    thisHash := hnext;
  END;
END RemHash;


(*
 * Lscht die komplette Tastatur Hash => Tastaturbelegung ist weg
 *)
PROCEDURE DeallocHash*;
VAR
 thisHash, hnext: HashPtr;
 i: SHORTINT;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  i := Hashsize;
  REPEAT
    DEC(i);
    thisHash := HashList[i];
    WHILE thisHash # NIL DO
      hnext := thisHash.next;
      IF thisHash.len # 0 THEN DISPOSE(thisHash.map); END;
      e.FreeMem(thisHash,s.SIZE(Hash));
      thisHash := hnext;
    END;
    HashList[i] := NIL;
  UNTIL i = 0;
END DeallocHash;


(*
 * Stellt die Default-Belegung fr die Tasten her
 *)
PROCEDURE ResetHash*;
VAR
  i: INTEGER;
  code: BYTE;
  qual: SHORTSET;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  DeallocHash;
  i := NumDefaultKeys;
  REPEAT
    DEC(i);
    IF GetCodeQual(DefaultMap[i].from,code,qual) THEN
      AddHash(code,(*s.VAL(SHORTSET,0FFX),*)qual,DefaultMap[i].to,TRUE);
      IF edG.memoryFail IN edG.Status THEN RETURN END;
    END;
  UNTIL i=0;
END ResetHash;

(* $Debug= *)
(*
 * gibt die Adresse des String zurck, mit dem einem Key in der Form
 * `qual-Key' belegt ist
 *)
PROCEDURE KeySpectroMacro*(string: edG.StringPtr): edG.StringPtr;
VAR
  thisHash: HashPtr;
  code: BYTE;
  qual: SHORTSET;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF GetCodeQual(string,code,qual) THEN
    thisHash := HashList[ORD(code) MOD Hashsize];
    WHILE thisHash # NIL DO
      IF (thisHash.code = code)
      AND (thisHash.qual = (qual (** thisHash.mask*))) THEN
        RETURN thisHash.map;
      END;
      thisHash := thisHash.next;
    END; (* WHILE *)
  END;
  RETURN NIL;
END KeySpectroMacro;
(* $Debug- *)


(*
 * belegt den Key in der Form `qual-Key' mit edG.Arg[1]
 *)
PROCEDURE dMap*;
VAR
  code: CHAR;
  qual: SHORTSET;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF GetCodeQual(edG.Arg[0],code,qual) THEN       (* code und qual suchen *)
    AddHash(code,(*s.VAL(SHORTSET,0FFX),*)qual,edG.Arg[1],FALSE);
  ELSE
    edG.Rc := edE.cmdError; edL.Title(UnknownKey);
  END;
END dMap;


(*
 * lscht die Belegung des Key's in der Form `qual-Key'
 *)
PROCEDURE dUnmap*;
VAR
  code: CHAR;
  qual: SHORTSET;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF GetCodeQual(edG.Arg[0],code,qual) THEN     (* code und qual suchen *)
    RemHash(code,(*s.VAL(SHORTSET,0FFX),*)qual);  (* lschen *)
  ELSE
    edG.Rc := edE.cmdError; edL.Title(UnknownKey);
  END;
END dUnmap;

(*
 * ermittelt fr Raw-Codes 0..127, um welche Taste es sich handelt
 *)
PROCEDURE KeyboardInit*;
VAR
  i, len: INTEGER;
  iEvent: ie.InputEventAdr;
  buffer: ARRAY 32 OF CHAR;
  qual: SHORTSET;
BEGIN
  SpecCodes[16] := 44X; SpecCodes[21] := 43X;
  SpecCodes[39] := ie.lButton; SpecCodes[40] := ie.mButton;
  SpecCodes[41] := ie.rButton; SpecCodes[42] := qMove;

  iEvent.nextEvent := NIL;
  iEvent.class := ie.rawkey;
  iEvent.subClass := 0;
  i := NumRawKeyCodes;
  REPEAT
    DEC(i);
    iEvent.code := i;
    iEvent.qualifier := {};
    iEvent.addr := NIL;
    len := SHORT(con.RawKeyConvert(s.ADR(iEvent),buffer,
                                   s.SIZE(buffer),NIL));
 (*
  * this first part is quiet dirty: changing consts (Bad!!). But it should
  * not matter, cause they are only depenting on the Keymaping. So every
  * call from a Process/Task using Std_Keymap gives the _same_ result.
  *)
    CASE len OF
     1: IF ((ORD(buffer[0]) MOD 80H) >= 32) AND (buffer[0]#asc.del) THEN
          cTOa[i] := buffer[0]; (* nicht 0-31, 128-129 *)
        END;
        CASE buffer[0] OF  (* esc/del/tab/bs/nkx *)
         1BX: SpecCodes[ 0] := CHR(i);
        |7FX: SpecCodes[11] := CHR(i);
        |09X: SpecCodes[14] := CHR(i);
        |08X: SpecCodes[12] := CHR(i); SpecCodes[13] := CHR(i);
        |"-": IF (i > 3AH) THEN SpecCodes[22] := CHR(i); END;
        |".": IF (i > 3AH) THEN SpecCodes[23] := CHR(i); END;
        |"(": IF (i > 3AH) THEN SpecCodes[34] := CHR(i); END;
        |")": IF (i > 3AH) THEN SpecCodes[35] := CHR(i); END;
        |"/": IF (i > 3AH) THEN SpecCodes[36] := CHR(i); END;
        |"*": IF (i > 3AH) THEN SpecCodes[37] := CHR(i); END;
        |"+": IF (i > 3AH) THEN SpecCodes[38] := CHR(i); END;
        ELSE
          IF (buffer[0] <= "9") AND (buffer[0] >= "0") AND (i >= 0FH) THEN
            SpecCodes[ORD(buffer[0])+(24-ORD("0"))] := CHR(i);   (* nkn *)
          END;
        END; (* CASE buffer[0] *)
    |2: IF buffer[0] = asc.csi THEN   (* cursor *)
           CASE (buffer[1]) OF
             "A": SpecCodes[17] := CHR(i); |
             "B": SpecCodes[18] := CHR(i); |
             "C": SpecCodes[19] := CHR(i); |
             "D": SpecCodes[20] := CHR(i);
           ELSE;
           END;
        END;
    |3: IF (buffer[0] = asc.csi) AND (buffer[2] = 7EX) THEN (* funct/help *)
          IF (buffer[1] >= "0") AND (buffer[1] <= "9") THEN
            SpecCodes[ORD(buffer[1])-(ORD("0")-1)] := CHR(i);
          ELSIF buffer[1] = "?" THEN
            SpecCodes[15] := CHR(i);
          END;
        END;
    ELSE;
    END; (* CASE len *)
  UNTIL i = 0;
  i := NumRawKeyCodes;
  REPEAT
    DEC(i);
    iEvent.code := i;
    iEvent.qualifier := {ie.lShift};
    iEvent.addr := NIL;
    len := SHORT(con.RawKeyConvert(s.ADR(iEvent),buffer,
                                   s.SIZE(buffer),NIL));
    IF len = 1 THEN csTOa[i] := buffer[0]; END;
  UNTIL i = 0;
  IF GetCodeQual(s.ADR("c"),edG.CtrlC,qual) THEN END;
END KeyboardInit;


(*
 * berschreibt '-return' mit return bzw. (ggf. vorher gespeicherter)
 * alter belegung
 *)
PROCEDURE ReturnOveride*(overide: BOOLEAN);
VAR
  thisHash: HashPtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  thisHash := HashList[ORD(ReturnRawCode) MOD Hashsize];
  WHILE thisHash # NIL DO
    IF (thisHash.code = ReturnRawCode) AND
       (thisHash.qual = SHORTSET{}) THEN    (* schon belegt *)
      IF overide THEN
        RetOvrString := thisHash.map;
        RetOvrLen := thisHash.len;
        thisHash.map := s.ADR(Return);
        thisHash.len := 0;
      ELSE
        IF RetOvrString = NIL THEN
          RemHash(ReturnRawCode,(*s.VAL(SHORTSET,0FFX),*)SHORTSET{});
        ELSE
          thisHash.map := RetOvrString;
          thisHash.len := RetOvrLen;
          RetOvrString := NIL;
        END;
      END;
      RETURN;
    END;
    thisHash := thisHash.next;
  END;
  IF overide THEN                      (* noch nicht belgt *)
    AddHash(ReturnRawCode,(*s.VAL(SHORTSET,0FFX),*)SHORTSET{},
            s.ADR(Return),TRUE);
    RetOvrString := NIL;
  END;
END ReturnOveride;


PROCEDURE GetKeyText*(imsg: I.IntuiMessagePtr;
                      code: BYTE; qual: SET; VAR buffer: edG.String): INTEGER;
VAR
  thisHash: HashPtr;
  q2: SHORTSET;
  blen, i: INTEGER;
  bufPtr: edG.StringPtr;
BEGIN
(* $IFNOT ClearVars THEN *)
  blen := 0;
  q2 := SHORTSET{};
(* $END *)
  IF imsg # NIL THEN
    IF NOT (edG.kick20 IN edG.Status) THEN
      EXCL(imsg.qualifier,ie.repeat); END;
    bufPtr := s.ADR(buffer[1]);
    blen := SHORT(sl.DeadKeyConvert(imsg,bufPtr^,edG.MaxLineLength-2,NIL));
    IF blen <= 0 THEN RETURN -1; END;
  END; (* imsg # NIL *)

  (*
   * Qualifiers herausfinden
   *)
  IF (qual * {ie.lCommand,ie.rCommand})#{} THEN INCL(q2,qualAmiga); END;
  IF (qual * {ie.lShift,ie.rShift})#{} THEN INCL(q2,qualShift); END;
  IF (qual * {ie.lAlt,  ie.rAlt})  #{} THEN INCL(q2,qualAlt); END;
  IF ie.control     IN qual THEN INCL(q2,qualCtrl); END;
  IF ie.leftButton  IN qual THEN INCL(q2,qualLmb); END;
  IF ie.midButton   IN qual THEN INCL(q2,qualMmb); END;
  IF ie.rightButton IN qual THEN INCL(q2,qualRmb); END;
  IF (ie.capsLock IN qual) AND (blen = 1) THEN
    INCL(q2,qualShift); END;
  code := CHR(ORD(code) MOD NumRawKeyCodes);

  IF edG.numLock IN edG.Status THEN
    i := 23; (* auf nk. *)
    REPEAT
      IF SpecCodes[i] = code THEN
        buffer[0] := "'"; buffer[1] := SpecKey[i,2]; buffer[2] := 0X;
        RETURN 1;
      END;
      INC(i);
    UNTIL i > 33; (* hinter nk9 *)
  END;
  thisHash := HashList[ORD(code) MOD Hashsize];
  LOOP
    IF thisHash = NIL THEN EXIT; END;  (* Taste belegt? *)
    IF (thisHash.code = code)
    AND (q2 (* * thisHash.mask*) = thisHash.qual) THEN
      EXIT; END;
    thisHash := thisHash.next;
  END;

    (* use HashList entry only if not in commandline or if the entry *)
    (* does not correspont to an alpha key *)

  IF (thisHash # NIL)
  AND ((q2 # SHORTSET{}) OR NOT (edG.commLineMode IN edG.Status)
       OR (blen>1) OR (cTOa[ORD(code)] = 0X))
  THEN
    blen := edL.Length(thisHash.map^);
    e.CopyMem(thisHash.map^,buffer,blen+1);
  ELSIF blen = 1 THEN  (* einzelnes Zeichen *)
    buffer[0] := "'";
    buffer[2] := 0X;
  ELSIF buffer[1] # asc.csi THEN (* keine Sondertaste *)
    buffer[0] := "`";
    buffer[blen+1] := "'";
    buffer[blen+2] := 0X;
  ELSE
    buffer[0] := 0X;
  END;
  RETURN blen;
END GetKeyText;

(*-----------------------------------------------------------------------
 * folgende beiden Prozeduren sind zum Debuggen. Es brauchen nur zwei
 * Befehle eingebunden zu werden, die diese beiden Proc. ohne Parameter
 * aufrufen.
 * GetHash: zeigt alle Belegungen zu einem Raw-Key-Value
 * GetMap:  zeigt die Belegung einer Taste (Eingabe wie bei Map)
 *)

PROCEDURE dGetMap*;
BEGIN
  edG.Arg[1] := KeySpectroMacro(edG.Arg[0]);
  IF edG.Arg[1] # NIL THEN
    edL.Title(edG.Arg[1]^); edG.Rc := edE.cmdValid2;
  END;
END dGetMap;


PROCEDURE dGetHash*;
VAR text: edG.StringPtr;
  thisHash : HashPtr;
  code : LONGINT;
BEGIN
  IF edL.StrToInt(edG.Arg[0],code) THEN
    thisHash := HashList[code MOD Hashsize];
    WHILE thisHash # NIL DO
      IF thisHash.code = s.VAL(BYTE,SHORT(SHORT(code))) THEN
        edL.Title(thisHash.map^); END;
      thisHash := thisHash.next;
    END; (* WHILE *)
  END;
END dGetHash;


BEGIN
  (*RetOvrStr := NIL; RetOvrLen := 0;*)
  KeyboardInit; (* MainPort mu! bereits geffnet sein *)
  ResetHash;
  IF edG.memoryFail IN edG.Status THEN HALT(20); END;

CLOSE
  ReturnOveride(FALSE);
  DeallocHash;
END EdKeyboard.

