(*************************************************************************

:Program.    EdText.mod
:Contents.   Text handling for AmokEd
:Author.     Hartmut Goebel
:Language.   Oberon
:Translator. Amiga Oberon Compiler V2.13
:History.    V1.0, 28 Oct 1991 Hartmut Goebel [hG] taken from edM, ed3
:History.    V1.1, 16 Nov 1991 [hG] -Bug in dReturn (no Redisplay)
:Date.       06 Feb 1992 20:54:18

*************************************************************************)
(* $Debug- *)
MODULE EdText;

IMPORT
  e  : Exec,
  edB: EdBlocks,
  ed1: EdCmd1,
  edD: EdDisplay,
  edE: EdErrors,
  edG: EdGlobalVars,
  edL: EdLowLevel,
  edM: EdMovement,
  g  : Graphics,
  lst: EdLists,
  Printf,
  str: Strings,
  s  : SYSTEM;

CONST
  (* Flags fr einzelne Kommandos *)
  insert* = 0;     (* \  unterschiedlich wg.  *)
  rfmtParag* = 1;  (* /  Reformat!!           *)
  tabIndent* = 0;

  LineTooLong = "Line too long!";

(*--------------------------------------------------------------------------*)

PROCEDURE^ dReformat*;  (* Forward, wird in dDel und dBS gebraucht *)

(*--------------------------------------------------------------------------*)

PROCEDURE dDownAdd*;
VAR
  newLine : edG.LinePtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF edG.Text.line+1 = edG.Text.numberOfLines THEN
    edL.NewLine(newLine,edG.ChunkSize);
    IF newLine # NIL THEN
      lst.AddTail(edG.Text.lineList,newLine);
      INC(edG.Text.numberOfLines);
      INCL(edG.Text.status,edG.modified);
    END;
  END;
  edM.dDown;
END dDownAdd;

(*--------------------------------------------------------------------------*)

PROCEDURE dDel*;
VAR
  buf : edG.StringPtr;
BEGIN
  IF edG.LineBuffer[edG.Text.pos] # 0X THEN
    str.Delete(edG.LineBuffer,edG.Text.pos,1);
    DEC(edG.LineBufferLen);
    edD.FastScrollRaster(edG.XSize,0,
           edD.Col(edG.Text.pos-edG.Text.topPos),
           edD.Row(SHORT(edG.Text.line-edG.Text.topLine)),
           edD.Col(edG.Columns)-1,
           edD.Row(SHORT(edG.Text.line-edG.Text.topLine+1))-1);
    IF (edG.LineBufferLen >= edG.Text.topPos+edG.Columns) THEN
      edD.SetPen(edG.Text.line);
      g.Move(edG.RPort,edD.ColT(edG.Columns-1),
            edD.RowT(SHORT(edG.Text.line-edG.Text.topLine)));
      buf := s.VAL(s.ADDRESS,s.VAL(LONGINT,s.ADR(edG.LineBuffer))
                                           +edG.Text.topPos+edG.Columns-1);
      g.Text(edG.RPort,buf^,1);
    END;
    IF NOT (edG.commLineMode IN edG.Status)
    AND (edG.wordWrap IN edG.Text.status) THEN
      edG.ArgSet := {};
      dReformat;
    END;
  ELSE
    edG.Rc := edE.cmdFailed5;
  END;
END dDel;


PROCEDURE dBs*;
VAR
  buf : edG.StringPtr;
BEGIN
  IF edG.Text.pos # 0 THEN
    str.Delete(edG.LineBuffer,edG.Text.pos-1,1);
    DEC(edG.Text.pos);
    DEC(edG.LineBufferLen);
    IF edG.Text.pos < edG.Text.topPos THEN
      edD.TextSync(FALSE);
    ELSE
      edD.FastScrollRaster(edG.XSize, 0,
           edD.Col(edG.Text.pos-edG.Text.topPos),
           edD.Row(SHORT(edG.Text.line-edG.Text.topLine)),edD.Col(edG.Columns)-1,
           edD.Row(SHORT(edG.Text.line-edG.Text.topLine+1))-1);
      IF (edG.LineBufferLen >= edG.Text.topPos+edG.Columns) THEN
        edD.SetPen(edG.Text.line);
        g.Move(edG.RPort,edD.ColT(edG.Columns-1),
               edD.RowT(SHORT(edG.Text.line-edG.Text.topLine)));
        buf := s.VAL(s.ADDRESS,s.VAL(LONGINT,s.ADR(edG.LineBuffer))
                                             +edG.Text.topPos+edG.Columns-1);
        g.Text(edG.RPort,buf^,1);
      END;
    END;
    IF NOT (edG.commLineMode IN edG.Status)
    AND (edG.wordWrap IN edG.Text.status) THEN
      edG.ArgSet := {};
      dReformat;
    END;
  ELSE
    edG.Rc := edE.cmdFailed5;
  END;
END dBs;


PROCEDURE dRemEOL*;
BEGIN
  edG.LineBufferLen := edG.Text.pos;
  edG.LineBuffer[edG.LineBufferLen] := 0X;
  edD.TextRedisplayCurrentLine;
END dRemEOL;


PROCEDURE dRemToFirst*;
BEGIN
  str.Delete(edG.LineBuffer,0,edG.Text.pos);
  edG.Text.pos := 0;
  edD.TextSync(TRUE);
END dRemToFirst;

(*----------------------------------------------------------------------*)

PROCEDURE InsertLine(newLine: edG.LinePtr; Behind: BOOLEAN);
BEGIN
  edD.PutBackLine;
  IF Behind THEN
    lst.AddBehind(edG.Text.lineList,newLine,edG.Text.actLinePtr);
    edB.HoldBlockLines(FALSE);
  ELSE
    lst.AddBefore(edG.Text.lineList,newLine,edG.Text.actLinePtr);
    IF edG.Text.topLinePtr = edG.Text.actLinePtr THEN
      edG.Text.topLinePtr := newLine; END;
    edG.Text.actLinePtr := newLine;
    edD.TextLoad;
    DEC(edG.Text.line);
    edB.HoldBlockLines(FALSE);
    INC(edG.Text.line);
  END;
  INC(edG.Text.numberOfLines);
  INCL(edG.Text.status,edG.modified);
  IF edG.NoScreenUpdate = 0 THEN
    g.ScrollRaster(edG.RPort,0,-edG.YSize,edD.Col(0),
                   edD.Row(SHORT(edG.Text.line-edG.Text.topLine)),
                   edD.Col(edG.Columns)-1,edD.Row(edG.Rows)-1);
    edD.TextDisplaySeg(SHORT(edG.Text.line-edG.Text.topLine),2);
  END;
END InsertLine;


PROCEDURE dInsLine*;
VAR
  newLine: edG.LinePtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  edL.NewLine(newLine,edG.ChunkSize);
  IF edG.memoryFail IN edG.Status THEN
    edG.Rc := edE.cmdSevere;
    RETURN;
  END;
  InsertLine(newLine,FALSE);
END dInsLine;


PROCEDURE dUndelLine*;
VAR
  newLine: edG.LinePtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF edG.DelLinePtr = NIL THEN edG.Rc := edE.cmdFailed5; RETURN END;
  newLine := edL.CreateLineCopy(edG.DelLinePtr);
  IF newLine=NIL THEN edG.Rc := edE.cmdSevere; RETURN END;
  InsertLine(newLine,FALSE);
END dUndelLine;


PROCEDURE dDelLine*;
VAR
  This: edG.LinePtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  This := edG.Text.actLinePtr;
  edD.PutBackLine;
  INCL(edG.Text.status,edG.modified);
  edG.Text.actLinePtr := edG.Text.actLinePtr.next(edG.Line);
  lst.Remove(edG.Text.lineList,This);
  IF edG.LineBufferLen > 0 THEN (* nur retten, wenn was drin steht *)
    IF edG.DelLinePtr # NIL THEN
      e.FreeMem(edG.DelLinePtr(edG.Line).string,edG.DelLinePtr(edG.Line).len);
      e.FreeMem(edG.DelLinePtr,edG.LineAllocSize);
    END;
    edG.DelLinePtr := This;
  ELSE
    e.FreeMem(This(edG.Line).string,This(edG.Line).len);
    e.FreeMem(This,edG.LineAllocSize);
  END;
  edB.HoldBlockLines(TRUE);
  DEC(edG.Text.numberOfLines);
  IF edG.Text.numberOfLines = 0 THEN
    edL.TextUnInit(edG.Text);
    EXCL(edG.Text.status,edG.modified);
    edD.TextRedisplayCurrentLine;
    RETURN;
  ELSIF edG.Text.actLinePtr = NIL THEN (* es war die letzte Ziele *)
    edG.Text.actLinePtr := edG.Text.lineList.tail(edG.Line);
    DEC(edG.Text.line);    (* eine Zeile vor *)
  END;
  edD.TextLoad;
  IF edG.Text.line < edG.Text.topLine THEN
    edD.TextSync(FALSE);
    RETURN;
  ELSIF edG.Text.line = edG.Text.topLine THEN
    edG.Text.topLinePtr := edG.Text.actLinePtr;
  END;
  IF edG.NoScreenUpdate = 0 THEN
    IF edG.Text.line-edG.Text.topLine+1 < edG.Rows THEN
      g.ScrollRaster(edG.RPort,0,edG.YSize,edD.Col(0),
                     edD.Row(SHORT(edG.Text.line-edG.Text.topLine+1)),
                     edD.Col(edG.Columns)-1,edD.Row(edG.Rows)-1);
      edD.TextRedisplayCurrentLine;
    END;
    edD.TextDisplaySeg(edG.Rows-1,1);
  END;
END dDelLine;


PROCEDURE dJoin*;
VAR
  i: INTEGER;
  next: edG.LinePtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  (*
   * LineBuffer krzen auf MAX(LastNoSpace,edG.Text.pos);
   *)
  i := edL.LastNoSpace(s.ADR(edG.LineBuffer));
  IF i < edG.Text.pos THEN i := edG.Text.pos; END;
  edG.LineBuffer[i] := "\o";
  edG.LineBufferLen := i;

  next := edG.Text.actLinePtr.next(edG.Line);
  IF (edG.Text.line+1 < edG.Text.numberOfLines)
  AND (i+edL.Length(next(edG.Line).string^)<edG.MaxLineLength)
  THEN
    IF (i > 0) AND (edG.LineBuffer[i-1] # 20X) THEN
      (* Space anhngen *)
      edG.LineBuffer[i] := " "; edG.LineBuffer[i+1] := "\o";
    END;
    i := 0;
    WHILE next(edG.Line).string[i] = 20X DO
      INC(i); END;
    IF i > 0 THEN str.Delete(next(edG.Line).string^,0,i); END;
    str.Append(edG.LineBuffer,next(edG.Line).string^);
    edG.LineBufferLen := edL.Length(edG.LineBuffer);
    edD.PutBackLine;
    INC(edG.Text.line);
    edB.HoldBlockLines(TRUE);
    DEC(edG.Text.line);
    edL.FreeLines(edG.Text.lineList,next,next);
    DEC(edG.Text.numberOfLines);
    edD.TextLoad;
    IF edG.NoScreenUpdate = 0 THEN
      g.ScrollRaster(edG.RPort,0,edG.YSize,edD.Col(0),
                     edD.Row(SHORT(edG.Text.line-edG.Text.topLine+1)),
                     edD.Col(edG.Columns)-1,edD.Row(edG.Rows)-1);
      edD.TextRedisplayCurrentLine;
      edD.TextDisplaySeg(edG.Rows-1,1);
    END;
  ELSE
    edG.Rc := edE.cmdFailed5;
  END;
END dJoin;


PROCEDURE dSplit*;
VAR
  len : INTEGER;
  newLine : edG.LinePtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  len := edG.LineBufferLen-edG.Text.pos;
  edL.NewLine(newLine,len+edG.ChunkSize-(len MOD edG.ChunkSize));
  IF edG.memoryFail IN edG.Status THEN
    edG.Rc := edE.cmdSevere;
    RETURN;
  END;
  str.Cut(edG.LineBuffer,edG.Text.pos,len,newLine(edG.Line).string^);
  edG.LineBuffer[edG.Text.pos] := 0X;
  len := SHORT(edG.Text.line-edG.Text.topLine);
  g.SetAPen(edG.RPort,0);
  g.RectFill(edG.RPort,edD.Col(0),edD.Row(len),
             edG.XBase+edG.XPixs,edD.Row(len+1)-1);
  InsertLine(newLine,TRUE);
  edM.dDown;
  edM.dUp;
END dSplit;

(*----------------------------------------------------------------------*)

PROCEDURE dReturn*;
VAR
  Buffer: edG.StringPtr;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF edG.commLineMode IN edG.Status THEN
    Buffer := edL.CopyString(s.ADR(edG.LineBuffer));
    ed1.EscapeCommLineMode;
    IF Buffer # NIL THEN
      edG.ExecCmd(Buffer);
      DISPOSE(Buffer);
    END;
    IF edG.Particial # NIL THEN
      edG.ExecCmd(edG.Particial);
      DISPOSE(edG.Particial);
    END;
  ELSE
    edM.dFirstColumn;
    dDownAdd;
  END;
END dReturn;

(*----------------------------------------------------------------------*)

PROCEDURE dIndent*;
VAR
  indent: INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF (tabIndent IN edG.ArgSet) THEN
    indent := edG.Text.tabStop;
  ELSE
    indent := 0;
    IF edG.Text.line > 0 THEN
      indent := edL.FirstNoSpace(edG.Text.actLinePtr.prev(edG.Line).string);
    END;
  END;
  IF indent <= 0 THEN RETURN; END;
  IF edG.LineBuffer[0] = 0X THEN
    edG.Text.pos := indent;
    edL.FillUpSpaces;
  ELSE
    edL.StripEndSpaces;
    IF indent + edG.LineBufferLen < 253 THEN
      INC(edG.Text.pos,indent);
      edL.bmove(s.ADR(edG.LineBuffer),
                s.ADR(edG.LineBuffer[indent]),edG.LineBufferLen+1);
      edL.bfill(s.ADR(edG.LineBuffer)," ",indent);
      INC(edG.LineBufferLen,indent);
    END;
  END;
  edD.TextSync(FALSE);
  IF NOT (edG.alreadyRedrawn IN edG.Status) THEN
    edD.TextRedisplayCurrentLine; END;
  IF (edG.wordWrap IN edG.Text.status) THEN
    edG.ArgSet := {};
    dReformat;
  END;
END dIndent;


PROCEDURE dUnIndent*;
VAR
  line: edG.LinePtr;
  indent, j: INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  indent := edL.FirstNoSpace(s.ADR(edG.LineBuffer));
  IF indent = 0 THEN RETURN; END;
  IF (tabIndent IN edG.ArgSet) THEN
    IF edG.Text.tabStop < indent THEN indent := edG.Text.tabStop; END;
  ELSE
    IF edG.Text.line > 0 THEN
      line := edG.Text.actLinePtr;
      j := indent;
      WHILE (j >= indent) AND (line # NIL) DO
        line := line.prev;
        j := edL.FirstNoSpace(line(edG.Line).string);
      END;
      DEC(indent,j);
    END;
  END;
  IF indent = 0 THEN RETURN; END;
  edL.bmove(s.ADR(edG.LineBuffer[indent]),
            s.ADR(edG.LineBuffer),edG.LineBufferLen+1-indent);
  DEC(edG.LineBufferLen,indent);
  DEC(edG.Text.pos,indent);
  IF edG.Text.pos < 0 THEN
    edG.Text.pos := 0; END;
  edD.TextSync(FALSE);
  IF NOT (edG.alreadyRedrawn IN edG.Status) THEN
    edD.TextRedisplayCurrentLine; END;
  IF (edG.wordWrap IN edG.Text.status) THEN
    edG.ArgSet := {};
    dReformat;
  END;
END dUnIndent;

(*----------------------------------------------------------------------*)
(*
 * schreibt <text> in den Buffer und auf den Screen, beachtet InsertMode
 *)

PROCEDURE TextWrite*(text: edG.StringPtr);
VAR
  len,i : INTEGER;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  len := edL.Length(text^);
  IF edG.LineBufferLen+len >= edG.MaxLineLength-1 THEN
    edD.TextSync(FALSE);
  END;
  IF NOT (edG.insertMode IN edG.Text.status) THEN
    i := edG.Text.pos+len;
    IF (i >= edG.MaxLineLength-1) THEN
      edL.Title(LineTooLong); edG.Rc := edE.cmdFailed;
      RETURN;
    END;
    e.CopyMem(text^,edG.LineBuffer[edG.Text.pos],len);
    IF (i > edG.LineBufferLen) THEN
      edG.LineBufferLen := i;
      edG.LineBuffer[edG.LineBufferLen] := 0X;
    END;
  ELSE
    IF (edG.LineBufferLen+len >= edG.MaxLineLength-1) THEN
      edL.Title(LineTooLong); edG.Rc := edE.cmdFailed;
      RETURN;
    END;
    str.Insert(edG.LineBuffer,edG.Text.pos,text^);
    INC(edG.LineBufferLen,len);
    IF (edG.NoScreenUpdate = 0)
    AND (len < edG.Columns - (edG.Text.pos-edG.Text.topPos)) THEN
      edD.FastScrollRaster(-len*edG.XSize,0,
         edD.Col(edG.Text.pos-edG.Text.topPos),
         edD.Row(SHORT(edG.Text.line-edG.Text.topLine)),edD.Col(edG.Columns),
         edD.Row(SHORT(edG.Text.line-edG.Text.topLine+1))-1);
    END;
  END; (* IF NOT InsertMode *)
  IF (edG.NoScreenUpdate > 0) THEN
    INC(edG.Text.pos,len);
    RETURN;
  END;
  IF (edG.Text.pos-edG.Text.topPos+len > edG.Columns) THEN
    i := edG.Columns-edG.Text.pos+edG.Text.topPos;
  ELSE
    i := len;
  END;
  edD.SetPen(edG.Text.line);
  g.Move(edG.RPort,edD.ColT(edG.Text.pos-edG.Text.topPos),
         edD.RowT(SHORT(edG.Text.line-edG.Text.topLine)));
  g.Text(edG.RPort,text^,i);
  INC(edG.Text.pos,len);
  IF (edG.Text.pos-edG.Text.topPos >= edG.Columns) THEN
    edD.TextSync(FALSE); END;
  IF NOT (edG.commLineMode IN edG.Status)
  AND (edG.wordWrap IN edG.Text.status) THEN
    EXCL(edG.ArgSet,rfmtParag);
    dReformat;
  END;
END TextWrite;

(*-------------------------------------------------------------------------*)
(*
 *  rfmtParag  : force reformat entire paragraph
 * ~rfmtParag  : only until line equalizes (from TextWrite())
 *
 * What is a paragraph?   A paragraph ends whenever the left justification
 * gets larger, or on a blank line.
 *)

PROCEDURE dReformat*;
VAR
  string: edG.StringPtr;
  lnsc, fnst, fnsc: INTEGER;
  column: INTEGER;
  srow, crow, erow: LONGINT;
  cptr: edG.LinePtr;     (* origin line *)
  dins: INTEGER;         (* relative insert lines/delete lines   *)
  moded: BOOLEAN;        (* any modifications done at all?       *)
  checked: BOOLEAN;      (* for cursor positioning.              *)
  nlok: BOOLEAN;
  indent, lastWordSpace: INTEGER;
BEGIN
  column := edG.Text.pos;
  cptr  := edG.Text.actLinePtr;
  srow := edG.Text.line; erow := srow; crow := srow;
(* $IFNOT ClearVars THEN *)
  dins := 0; moded := FALSE; checked := FALSE; indent := 0;
(* $END *)

  IF (edG.Text.margin = 0) THEN edG.Text.margin := 75; END;
  INC(edG.NoScreenUpdate);

  LOOP
    fnsc := edL.FirstNoSpace(s.ADR(edG.LineBuffer));
    IF edG.Text.actLinePtr.next # NIL THEN
      string := edG.Text.actLinePtr.next(edG.Line).string;
      fnst := edL.FirstNoSpace(string);
    ELSE
      string := NIL;
      fnst := 0;
    END;
    nlok := (edG.Text.line < edG.Text.numberOfLines-1) AND (fnsc >= fnst)
             AND (string # NIL) AND (string[0] # 0X);
    lnsc := edL.LastNoSpace(s.ADR(edG.LineBuffer));
    IF (lnsc < edG.Text.margin) THEN
      (*
       * space at end of line for margin-lnsc-2 letter word
       *)
      IF NOT nlok THEN      (* but no more data to joinup *)
        EXIT; END;          (* done *)
      IF (edG.Text.margin-lnsc-2 >= edL.WordLen(s.ADR(string[fnst]))) THEN
        edG.LineBufferLen := edL.LastNoSpace(s.ADR(edG.LineBuffer));
        (*
        IF edG.LineBufferLen < edG.Text.pos THEN
          edG.LineBufferLen := edG.Text.pos; END;
        *)
        IF (edG.LineBuffer[edG.LineBufferLen] # 0X) THEN
          INC(edG.LineBufferLen); END;
        moded := TRUE;
        DEC(dins);
        dJoin;
        IF edG.Rc >= edE.abortLevel THEN
          INC(dins);
          edL.Title(edG.ErrorMarginOver124);
          EXIT;          (* stop reformating *)
        END;
      ELSIF NOT (rfmtParag IN edG.ArgSet) THEN   (* if couldn't mod line, and *)
          EXIT;                          (* TextWrite, don't update any more *)
      ELSE
        edM.dDown;
        erow := edG.Text.line;
      END;
    ELSE                    (* no space, need to split      *)
      LOOP                  (* find start of prev word      *)
        lastWordSpace := lnsc;
        WHILE (lnsc # 0) & (edG.LineBuffer[lnsc] # ' ') DO
          DEC(lnsc); END;
        IF (lnsc < edG.Text.margin) THEN
          EXIT; END;
        DEC(lnsc);          (* edG.Text.margin >= 1 !! *)
      END;
      IF lnsc > 0 THEN     (* ok to split at word          *)
        INC(lnsc);
        INC(dins);
        edG.Text.pos := lnsc;
        dSplit;                 (* Split at point LNSC        *)
        edG.Text.pos := 0;      (* avoid FillUpSpaces         *)
        edM.dDown;              (* must insert proper amount? *)
        erow := edG.Text.line;
        IF nlok THEN
          indent :=  fnst;
        ELSE
          indent := fnsc;
        END;
        IF (indent > 0) AND (edG.LineBufferLen + indent < 253) THEN
          edL.bmove(s.ADR(edG.LineBuffer),
                    s.ADR(edG.LineBuffer[indent]),edG.LineBufferLen+1);
          edL.bfill(s.ADR(edG.LineBuffer)," ",indent);
          INC(edG.LineBufferLen,indent);
        END;
        IF NOT checked THEN  (* has cursor to be moved to the next line? *)
          checked := TRUE;
          moded := TRUE;
          IF (lnsc <= column) THEN   (* if split before cursor   *)
            column := column - lnsc + indent;
            cptr := cptr.next(edG.Line);
            INC(crow);
          END;
        END;
      END;
      IF (lastWordSpace-lnsc+indent >= edG.Text.margin) THEN (* Lnge des Wortes *)
        EXIT; END;
    END;
  END; (* LOOP *)

  IF (column < 0) OR (column > 200) THEN
    column := 0; END;
  edD.PutBackLine;
  edG.Text.actLinePtr := cptr;
  edG.Text.line := crow;
  edG.Text.pos := column;
  DEC(edG.NoScreenUpdate);
  edD.TextLoad;
  edD.TextSync(FALSE);

  IF moded AND NOT (edG.alreadyRedrawn IN edG.Status) THEN
    IF (dins # 0) OR (srow >= edG.Text.numberOfLines)
    OR (srow < edG.Text.topLine) OR (srow >= edG.Text.topLine + edG.Rows) THEN
      edD.TextRedisplay;
    ELSIF (erow = srow) THEN
      edD.TextRedisplayCurrentLine;
    ELSE
      INC(erow);
      IF (erow - edG.Text.topLine > edG.Rows) THEN
        erow := edG.Text.topLine + edG.Rows; END;
      g.SetAPen(edG.RPort, 0);
      g.RectFill(edG.RPort, edD.Col(0), edD.Row(SHORT(srow - edG.Text.topLine)),
                 edG.XBase+edG.XPixs, edD.Row(SHORT(erow - edG.Text.topLine))-1);
      g.SetAPen(edG.RPort, 1);
      edD.TextDisplaySeg(SHORT(srow-edG.Text.topLine),SHORT(erow-srow));
    END;
  END;
END dReformat;

(*-----------------------------------------------------------------------*)

PROCEDURE dTLate*;
VAR
  ch: CHAR;
  long: LONGINT;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  IF NOT edL.StrToInt(edG.Arg[0],long) OR (long > 255) THEN
    edG.Rc := edE.cmdError; edL.Title(edG.BadArgument);
    RETURN;
  END;
  ch := edG.LineBuffer[edG.Text.pos];
  IF ch = 0X THEN
    ch := 20X; END;
  IF edG.Arg[0][0] < "0" THEN (* mit Vorzeichen => relativ *)
    ch := CHR(ORD(edG.LineBuffer[edG.Text.pos])+long);
  ELSE
    ch := CHR(long);
  END;
  IF ch # 0X THEN
    IF edG.LineBuffer[edG.Text.pos] = 0X THEN (* letztes Zeichen *)
      INC(edG.LineBufferLen);
      edG.LineBuffer[edG.LineBufferLen] := 0X;
    END;
    edG.LineBuffer[edG.Text.pos] := ch;
    IF NOT (edG.NoScreenUpdate > 0) THEN
      edL.MoveToCursor;
      edD.SetPen(edG.Text.line);
      edG.Arg[1] := s.ADR(edG.LineBuffer[edG.Text.pos]);
      g.Text(edG.RPort,edG.Arg[1]^,1);
    END;
  ELSE
    edG.Rc := edE.cmdError; edL.Title(edG.BadArgument);
  END;
END dTLate;


PROCEDURE dAscii*;
VAR
  Buffer: ARRAY 5 OF CHAR;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  Printf.SPrintf1(Buffer,"%ld",ORD(edG.LineBuffer[edG.Text.pos]));
  edG.Arg[0] := s.ADR(Buffer);
  edL.dTitle;
END dAscii;

(*-----------------------------------------------------------------------*)

PROCEDURE dOverWrite*;
VAR
  insertMode: BOOLEAN;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  insertMode := edG.insertMode IN edG.Text.status;
  IF ~(insert IN edG.ArgSet) THEN EXCL(edG.Text.status,edG.insertMode);
                             ELSE INCL(edG.Text.status,edG.insertMode); END;
  TextWrite(edG.Arg[0]);
  IF insertMode THEN INCL(edG.Text.status,edG.insertMode);
                ELSE EXCL(edG.Text.status,edG.insertMode); END;
END dOverWrite;


PROCEDURE dSwapChar*;
VAR
  i: INTEGER;
  ch: CHAR;
  (* $ClearVars- *)
BEGIN
  (* $ClearVars= *)
  i := edG.Text.pos;
  IF i < edG.LineBufferLen-1 THEN
    ch := edG.LineBuffer[i];
    edG.LineBuffer[i] := edG.LineBuffer[i+1];
    edG.LineBuffer[i+1] := ch;
    edD.TextRedisplayCurrentLine;
  ELSE
    edG.Rc := edE.cmdFailed5;
  END;
END dSwapChar;

(*-----------------------------------------------------------------------*)

PROCEDURE dUnJustify*;
VAR
  from, to: INTEGER;
  c: BYTE;
  wasWhite: BOOLEAN;
BEGIN
  wasWhite := FALSE;

  from := edL.FirstNoSpace(s.ADR(edG.LineBuffer));
  to := from;
  WHILE edG.LineBuffer[from] # 0X DO
    c := edG.LineBuffer[from];
    edG.LineBuffer[to] := edG.LineBuffer[from];
    IF (c # ' ') OR (NOT wasWhite) THEN
      INC(to); END;
    wasWhite := (c = 20X);
    INC(from);
  END;
  edG.LineBuffer[to] := 0X;

  IF from # to THEN
    edD.TextSync(FALSE);
    IF NOT (edG.alreadyRedrawn IN edG.Status) THEN
      edD.TextRedisplayCurrentLine;
    END;
  END;
END dUnJustify;


PROCEDURE dJustify*;
VAR
  i: INTEGER;
  firstnb, lastnb, n, fill, sp: INTEGER;
  changed: BOOLEAN;
  string: edG.StringPtr;
BEGIN
  changed := FALSE;
  CASE CAP(edG.Arg[0][0]) OF
  |"C": (* noch nicht implementiert *)
  |"F":
    firstnb := edL.FirstNoSpace(s.ADR(edG.LineBuffer));
    lastnb := edL.LastNoSpace(s.ADR(edG.LineBuffer))-1;
    IF (firstnb < lastnb) AND (edG.Text.margin < edG.MaxLineLength-1) THEN
      edG.LineBuffer[lastnb + 1] := 0X;
      edG.LineBufferLen := edG.Text.margin;
      fill := edG.Text.margin - lastnb - 1;
      n := 0;
      i := firstnb;
      WHILE i <= lastnb DO
        WHILE (edG.LineBuffer[i] # 0X) AND (edG.LineBuffer[i] # 20X) DO
          INC(i); END;
        IF (i <= lastnb) THEN
          INC(n);                        (* Anzahl Lcken *)
          WHILE (edG.LineBuffer[i] = 20X) DO
            INC(i); END;
        END;
      END;
      i := firstnb;
      IF (n > 0) AND (fill > 0) THEN
        changed := TRUE; END;
      WHILE (n > 0) AND (fill > 0) AND (edG.LineBuffer[i] # 0X) DO
        WHILE (edG.LineBuffer[i] # 0X) AND (edG.LineBuffer[i] # 20X) DO
          INC(i); END;
        sp := fill DIV n;
        string := s.ADR(edG.LineBuffer[i]);
        edL.bmove(string,s.ADR(string[sp]),edL.Length(string^)+1);
        edL.bfill(string, ' ', sp);
        WHILE (edG.LineBuffer[i] = 20X) DO
          INC(i); END;
        DEC(fill,sp);
        DEC(n);
      END;
    END;
  ELSE END;

  IF changed THEN
    edD.TextSync(FALSE);
    IF NOT (edG.alreadyRedrawn IN edG.Status) THEN
      edD.TextRedisplayCurrentLine;
    END;
  END;
END dJustify;

END EdText.

