(* --------------------------------------------------------------------------

:Program.    CPicScreen.mod
:Contents.   Procedure to generate Screen from Inline- Datas
:Author.     Steffen Khler
:Address.    Breite Str. 1 2880 Brake, Germany
:Phone.      04401/70164
:Copyright.  This Routine based on the Shareware-Routines by
             Thomas Zipproth
             Dr. Jochner Weg 10, 8948 Mindelheim, Germany
             08261/4838
             (see the CPic.doc on AMOK#42 for more Informations)
:Language.   Modula2 + Assembler
:Translator. M2-Amiga A+L V3.32d
:History.    V1.0 26-Jan-90

-------------------------------------------------------------------------- *)

IMPLEMENTATION MODULE CPicScreen;

FROM Arts      IMPORT TermProcedure,DetectCtrlC;
FROM SYSTEM    IMPORT ADR,ADDRESS,INLINE;
FROM Exec      IMPORT CopyMem,UByte,AllocMem,FreeMem,MemReqSet,MemReqs;
FROM Graphics  IMPORT BitMapPtr,RastPortPtr,ViewModes,ViewModeSet,LoadRGB4,
                      ViewPortPtr,RectFill,SetAPen,TextAttr,FontStyles,
                      FontStyleSet;
FROM Intuition IMPORT OpenScreen,CloseScreen,customScreen,NewScreen,ScreenPtr,
                      ScreenToBack,ScreenFlagSet,ScreenFlags,MoveScreen;


CONST RecLen = 132; (* Length of CPicInfo - Record *)

VAR
    Buffer2 : ADDRESS;    (* Buffer for crunching and decrunching *)
    BufSize1,BufSize2 : LONGINT;  (* = one Bitplane of rectangle + 800 *)
    MyFont            : TextAttr;


PROCEDURE CleanUp;
BEGIN
  IF Buffer2 # NIL THEN FreeMem(Buffer2,BufSize2) END;
END CleanUp;



(* ----------------------------------------------------------------------*)


(* $E- *)
PROCEDURE StringToRect(a{8},b{9} : ADDRESS; c{0},d{1},e{2} : LONGINT);
BEGIN
  INLINE(048E7H,07F00H); (* Movem.l d1-d7,-(a7) *)
  INLINE(048E7H,000FEH); (* Movem.l a0-a6,-(a7) *)
  INLINE(05380H,05381H,02601H,02449H,01498H,0D5C2H,
         051CBH,0FFFAH,05289H,051C8H,0FFF0H);
  INLINE(04CDFH,07F00H); (* Movem.l (a7)+,a0-a6 *)
  INLINE(04CDFH,000FEH); (* Movem.l (a7)+,d1-d7 *)
  INLINE(04E75H); (* RTS *)
END StringToRect;


(* $E- *)
PROCEDURE Decrunch(a{8} : ADDRESS; b{9} : ADDRESS; c{0} : LONGINT) : LONGINT;
BEGIN
  INLINE(048E7H,07F00H); (* Movem.l d1-d7,-(a7) *)
  INLINE(048E7H,000FEH); (* Movem.l a0-a6,-(a7) *)
  INLINE(02E08H,02449H,0D5C0H,07200H,07400H,07620H,0183CH,00080H,
         01A3CH,000A0H,01C3CH,000FFH,07000H,01018H,0B003H,06408H,
         012C2H,051C8H,0FFFCH,06026H,0B004H,0640AH,09003H,012D8H,
         051C8H,0FFFCH,06018H,0B005H,0640AH,09004H,012C6H,051C8H,
         0FFFCH,0600AH,01218H,09005H,012C1H,051C8H,0FFFCH,0B5C9H,
         06EC6H,02008H,09087H);
  INLINE(04CDFH,07F00H); (* Movem.l (a7)+,a0-a6 *)
  INLINE(04CDFH,000FEH); (* Movem.l (a7)+,d1-d7 *)
  INLINE(04E75H); (* RTS *)
END Decrunch;


PROCEDURE Refresh(CpInlinePtr: ADDRESS; VAR sp: ScreenPtr;
                        VAR ci : CPicInfo) : BOOLEAN;

VAR i,
    depth,
    BpR,
    rows       : INTEGER;
    PicSize,
    PicStart,
    dummy      : LONGINT;
    ad1,
    ad2,
    ad3        : ADDRESS;
    num        : UByte;
    x0,
    y0         : INTEGER;
    bp         : BitMapPtr;
    rp         : RastPortPtr;

BEGIN
  rp := ADR(sp^.rastPort);
  SetAPen(ADR(sp^.rastPort),0);
  IF sp^.title = NIL THEN
    RectFill(ADR(sp^.rastPort),0,0,ci.Width-1,ci.Height-1);
  ELSE
    RectFill(ADR(sp^.rastPort),0,10,ci.Width-1,ci.Height-11);
  END;
  bp := rp^.bitMap;
  x0:=ci.PicX0;
  y0:=ci.PicY0;
  Cle:=0;
  depth := bp^.depth;
  BpR   := bp^.bytesPerRow;
  rows  := bp^.rows;
  IF (x0<0) OR (y0<0) THEN Cle:=13; RETURN FALSE END;
  IF ((x0+ci.PicX>BpR) OR (y0+ci.PicY>rows)) THEN Cle:=13; RETURN FALSE END;
  IF ci.Depth>depth THEN Cle:=14; RETURN FALSE END;
  PicSize := LONGINT(ci.PicX) * LONGINT(ci.PicY);
  BufSize2 := PicSize + 800;
  Buffer2:=NIL;  Buffer2:=AllocMem(BufSize2,MemReqSet{public,memClear});
  IF Buffer2=NIL THEN Cle:=15; RETURN FALSE END;
  ad1:=Buffer2 + BufSize2 - 20;
  PicStart := LONGINT(y0) * LONGINT(BpR) + LONGINT(x0);

  FOR i:=0 TO ci.Depth-1 DO
     (* dummy:=Seek(fp,ci.Planes[i],-1)*)
    ad2:=ad1-ci.Length[i];
    ad3:=bp^.planes[i]+PicStart;
    CopyMem(CpInlinePtr+ci.Planes[i],ad2,ci.Length[i]); (*dummy:=Read(fp,ad2,ci.Length[i])*)
    IF Decrunch(ad2,Buffer2,PicSize) # ci.Length[i] THEN
      FreeMem(Buffer2,BufSize2);
      Buffer2:=NIL; Cle:=17; RETURN FALSE;
    END;
    StringToRect(Buffer2,ad3,ci.PicX,ci.PicY,BpR);
  END;
  FreeMem(Buffer2,BufSize2);
  Buffer2:=NIL;RETURN TRUE;
END Refresh;


PROCEDURE MakeCPicScreen( CpInlinePtr: ADDRESS; TitlePtr:ADDRESS;
                         VAR sp : ScreenPtr;
                         x0,y0 : INTEGER; Front : BOOLEAN) : BOOLEAN;
VAR bp : BitMapPtr;

PROCEDURE ReadInfo(VAR ci     : CPicInfo     ) : BOOLEAN;
VAR l  : LONGINT;
    i  : INTEGER;
    dn : ARRAY[0..80] OF CHAR;
BEGIN
  Cle:=0;
  (*Seek(fp,0,-1)*)
  CopyMem (CpInlinePtr,ADR(ci),RecLen); (* l:=Read(fp,ADR(ci),RecLen)*)
  IF ci.Ident # 1129343331 THEN
    Cle:=11; RETURN FALSE;
  END;
  RETURN TRUE;
END ReadInfo;


PROCEDURE OpenCPicScreen(VAR ci : CPicInfo;
                         VAR sp : ScreenPtr;
                         x0,y0 : INTEGER;
                         title : ADDRESS; Front : BOOLEAN) : BOOLEAN;
VAR ns : NewScreen;
BEGIN
  Cle:=0; sp:=NIL; bp:=NIL;
  WITH MyFont DO
      name:=ADR('topaz.font');
      ySize:=8;
      style:=FontStyleSet{};
   END;
  WITH ns DO
    leftEdge:=x0;
    topEdge:=y0;
    width:=ci.Width;
    height:=ci.Height;
    depth:=ci.Depth;
    detailPen:=0;
    IF title=NIL THEN
      blockPen:=0;
      type:=customScreen+ScreenFlagSet{screenQuiet};
    ELSE
      blockPen:=1;
      type:=customScreen;
    END;
    viewModes:=ci.Mode;
    font:=ADR(MyFont);
    defaultTitle:=title;
    gadgets:=NIL;
    customBitMap:=NIL;
  END;
  IF NOT Front THEN ns.topEdge:=ci.Height-1 END;

  DetectCtrlC(FALSE);
  sp:=OpenScreen(ns);
  IF sp=NIL THEN Cle:=12; DetectCtrlC(TRUE); RETURN FALSE; END;
  DetectCtrlC(TRUE);

  IF NOT Front THEN
     ScreenToBack(sp);
  END;
  LoadRGB4(ADR(sp^.viewPort),ADR(ci.Colors),32);
  IF title = NIL THEN
    SetAPen(ADR(sp^.rastPort),0);RectFill(ADR(sp^.rastPort),0,0,ci.Width-1,9)
  END;
  RETURN TRUE;
END OpenCPicScreen;




BEGIN
  IF NOT ReadInfo(ci) THEN RETURN FALSE END;
  IF NOT OpenCPicScreen(ci,sp,x0,y0,TitlePtr,Front) THEN
    RETURN FALSE;
  END;
  IF NOT Refresh(CpInlinePtr,sp,ci) THEN
    CloseScreen(sp); sp:=NIL; RETURN FALSE;
  END;
  RETURN TRUE;
END MakeCPicScreen;


BEGIN

TermProcedure(CleanUp);

END CPicScreen.
