(*---------------------------------------------------------------------------
 :Program.       RememberBirth
 :Contents.      Scan an ASCII-file and remember you if somebody, listed in
                 the file, has birthday (in next week or the two previous
                 days).
 :Author.        Rainer Bauer
 :Shortcut.      [rb]
 :Address.       Sandberg 6
 :Address.       7092 Rosenberg, FRG (west)
 :Phone.         49-7967-6959
 :Address.       Fido: 2:244/8580.24     email: r_bauer@guru.hamm.sub.org
 :Language.      Modula-2
 :Translator.    M2Amiga V4.0
 :Copyright.     Real Public Domain
 :Bugs.          none (hope so)
 :Remark.        Inputfile format: date, name, remark.
                 Date must be given in one of the following two ways:
                    DD-MM-YY   or
                    DD-MM-YYYY
                 The first way indicates, that year of birth is in the
                 19th century. In the date-field separator is the '-' and
                 only numbers are allowed.
                 The data fields are seperated by semicolons. The remark
                 field can be left blank.
                 Incorrect lines are ignored (so you may comment your
                 file).

                 EXAMPLE:
                 06-12-413 ; Santa Claus ; my personally friend
 :History.       V1.0, 26-Dec-91: First release.                         [rb]
 :Update.        No further versions; I just buyed M2Apse. An enviroment
 :Update.        for the M2Amiga System!
 :Usage.         Workbench: double click
                 CLI      : RememberBirth [?|Inputfile]
---------------------------------------------------------------------------*)

(*$ StackChk    := FALSE
    NilChk      := FALSE
    RangeChk    := FALSE
    OverflowChk := FALSE
    CaseChk     := FALSE
    ReturnChk   := FALSE
    LongAlign   := FALSE
    Volatile    := FALSE
    LargeVars   := FALSE
    StackParms  := FALSE
*)


MODULE RememberBirth;

(*********  import  *********)

IMPORT sys: SYSTEM,
       Arts,
       arg: Arguments,
       str: String,
       io: InOut,
       fs: FileSystem,
       sto: Storage,
       ASCII,
       DD: DosD,
       DL: DosL,
       DC: DateConversions,
       conv: Conversions;

(*********  declaration  *********)

CONST   StandartFile  = "s:Birthdays";
        Datedelimiter = "-";
        Datadelimiter = ";";


TYPE    String  = ARRAY [0..255] OF CHAR;
        ShortString = ARRAY [0..19] OF CHAR;
        LinePtr = POINTER TO Line;
        Line = RECORD
           Inh   : String;
           NLine : LinePtr;
        END;
        verbs  = (future, past);
        times  = (week, fdays, tdays, datom, tom, tod, yes, dbyes);


CONST   LSize = SIZE(Line);


VAR     Diff, Alter           : INTEGER;
        InputFName            : ARRAY [0..149] OF CHAR;
        FirstL, ActualL, NextL: LinePtr;
        InFile                : fs.File;
        DosDate               : DD.Date;
        DInfo                 : DC.DateInfo;
        Rday, Rmonth, Ryear   : INTEGER;
        Wday, Wmonth, Wyear   : INTEGER;
        RTotaldays, WTotaldays: INTEGER;
        name, remark          : String;
        v                     : ARRAY [future..past] OF ShortString;
        vvar                  : verbs;
        t                     : ARRAY [week..dbyes]  OF ShortString;
        tvar                  : times;
        RSchalt, WSchalt,
        S1, S2                : BOOLEAN;


  (********* text initialisation   *********)

  PROCEDURE InitText();
  BEGIN
    v[future] := " hat";     v[past] := " hatte";
    t[week]   := " in einer Woche";
    t[fdays]  := " in fnf Tagen";
    t[tdays]  := " in drei Tagen";
    t[datom]  := " bermorgen";
    t[tom]    := "\e[1m morgen\e[0m";
    t[tod]    := "\e[1m\e[33m HEUTE\e[0m";
    t[yes]    := " gestern";
    t[dbyes]  := " vorgestern";
  END InitText;


  (*********  where are the birthday-datas ?  *********)

  PROCEDURE GetBirthFilename(VAR FName : ARRAY OF CHAR);

    (*********  how TO use  *********)

    PROCEDURE Usage();
    BEGIN
      io.WriteString(" Usage: RememberBirth [?|Inputfile]\n");
      Arts.Exit(20)
    END Usage;

  VAR   ArgAnz, Len : INTEGER;
        Argu        : ARRAY [0..149] OF CHAR;
  BEGIN
    ArgAnz := arg.NumArgs();
    IF (ArgAnz > 1) THEN Usage END;
    IF (ArgAnz = 1) THEN
      arg.GetArg(1, Argu, Len);
      IF ((Len = 1) AND (Argu[0] = "?")) THEN Usage END;
      str.Copy(FName, Argu);
    ELSE
      str.Copy(FName, StandartFile);
    END;
  END GetBirthFilename;


  (*********  Deallocate readed lines  *********)

  PROCEDURE Freelines(ok: BOOLEAN);
  BEGIN
    ActualL := FirstL;
    WHILE (ActualL # NIL) DO
      NextL := ActualL^.NLine;
      sto.DEALLOCATE(ActualL, LSize);
      ActualL := NextL;
    END;
    FirstL  := NIL;           (* falls CLOSE nochmals durchlaufen wird *)
    IF NOT ok THEN (* while reading birthday file memory went out !*)
      io.WriteString(" NOT enough memory");
      fs.Close(InFile);
      Arts.Exit(20);
    END;
  END Freelines;


  (*********  read input-file and return a ptr to the 1st line  *********)

  PROCEDURE ReadFile(VAR InFile: fs.File): LinePtr;

    (*********  read a line  *********)

    PROCEDURE ReadLine(VAR ALine: LinePtr);
    VAR i : SHORTCARD;
        ch: CHAR;
    BEGIN
      i := 0;
      REPEAT
        fs.ReadChar(InFile, ch);
        ALine^.Inh[i] := ch;
        INC(i);
      UNTIL ((ch = ASCII.eol) OR InFile.eof OR (i = 255));
    END ReadLine;

  VAR  FLine, ALine, NLine: LinePtr;

  BEGIN
    sto.ALLOCATE(FLine, LSize);
    IF (FLine = NIL) THEN
      io.WriteString(" NOT enough memory");
      Arts.Exit(20);
    END;

    ALine := FLine;
    ReadLine(ALine);
    WHILE NOT InFile.eof DO
      sto.ALLOCATE(NLine, LSize);
      IF (NLine = NIL) THEN
        ALine^.NLine := NIL;
        Freelines(FALSE);
      END;
      ALine^.NLine := NLine;
      ALine        := NLine;
      ReadLine(ALine);
    END;
    ALine^.NLine := NIL;
    RETURN FLine;
  END ReadFile;


  (*********  parse a given line  *********)

  PROCEDURE ParseLine(ALine: LinePtr; VAR day, month, year: INTEGER;
                      VAR name, remark: ARRAY OF CHAR): BOOLEAN;
  VAR     Pos, l      : LONGINT;
          HStr        : String;
          Err, sign   : BOOLEAN;

  BEGIN

    (*********  delete blanks in date  *********)

    l := 0;
    REPEAT
      IF ((ALine^.Inh[l] = ASCII.sp) OR (ALine^.Inh[l] = ASCII.ht)) THEN
        str.DeleteChar(ALine^.Inh, l);
      ELSE
        INC(l);
      END;
    UNTIL ((ALine^.Inh[l] = Datadelimiter) OR (ALine^.Inh[l] = ASCII.eol)
          OR (l = 255));

    (*********  find out day  *********)

    Pos := str.FirstPos(ALine^.Inh, 0, Datedelimiter);
    IF ((Pos > 2) OR (Pos = str.noOccur)) THEN RETURN FALSE END;
                                          (* day longer than 2 chars *)
    str.CopyPart(HStr, ALine^.Inh, 0, Pos);
    str.Delete(ALine^.Inh, 0, (Pos + 1));
    conv.StrToVal(HStr, l, sign, 10, Err);
    IF (Err OR sign OR (l = 0) OR (l > 31)) THEN
      RETURN FALSE;
    ELSE
      day := INTEGER(l);
    END;

    (*********  find out month  *********)

    Pos := str.FirstPos(ALine^.Inh, 0, Datedelimiter);
    IF (Pos > 2) THEN RETURN FALSE END; (* month longer than 2 chars *)
    str.CopyPart(HStr, ALine^.Inh, 0, Pos);
    str.Delete(ALine^.Inh, 0, (Pos + 1));
    conv.StrToVal(HStr, l, sign, 10, Err);
    IF (Err OR sign OR (l = 0) OR (l > 12)) THEN
      RETURN FALSE;
    ELSE
      month := INTEGER(l);
    END;

    (*********  find out year  *********)

    Pos := str.FirstPos(ALine^.Inh, 0, Datadelimiter);
    IF (Pos > 4) THEN RETURN FALSE END; (* year longer than 4 chars *)
    str.CopyPart(HStr, ALine^.Inh, 0, Pos);
    str.Delete(ALine^.Inh, 0, (Pos + 1));
    conv.StrToVal(HStr, l, sign, 10, Err);
    IF (Err OR sign) THEN
      RETURN FALSE;
    ELSE
      year := INTEGER(l);
      IF (year < 100) THEN year := year + 1900 END;
    END;

    (*********  find out name  *********)

    Pos := str.FirstPos(ALine^.Inh, 0, Datadelimiter);
    IF (Pos = str.noOccur) THEN (* no remark *)
      IF (str.Length(ALine^.Inh) = 1) THEN RETURN FALSE END;
                                          (* no name ! (attention: eol)*)
      str.Copy(name, ALine^.Inh);
      remark[0] := ASCII.nul;
      RETURN TRUE;
    ELSE
      IF (Pos = 0) THEN                   (* no name ! *)
        RETURN FALSE
      END;
      str.CopyPart(name, ALine^.Inh, 0, Pos);
      str.Delete(ALine^.Inh, 0, (Pos + 1));
    END;

    (*********  find out remark  *********)

    IF (str.Length(ALine^.Inh) = 1) THEN  (* no remark (attention: eol) *)
      remark[0] := ASCII.nul;
    ELSE
      str.Copy(remark, ALine^.Inh);
    END;
    RETURN TRUE
  END ParseLine;


  (*********  kills blanks (and eol) at beginning and end of line  *********)

  PROCEDURE KillBlAndEol(VAR sent: ARRAY OF CHAR);
  VAR i : INTEGER;
  BEGIN
    WHILE ((sent[0] = ASCII.sp) OR (sent[0] = ASCII.ht)) DO
      str.DeleteChar(sent, 0);
    END;
    i := str.Length(sent) - 1;
    WHILE ((sent[i] = ASCII.sp) OR (sent[i] = ASCII.ht) OR
           (sent[i] = ASCII.eol)) DO
      str.DeleteChar(sent, i);
      DEC(i);
    END;
  END KillBlAndEol;


  (*********  test wether a "Schaltjahr" or not  *********)

  PROCEDURE Schaltjahr(y: INTEGER): BOOLEAN;
  BEGIN
    IF (((y MOD 4) = 0) AND ((y MOD 400) # 0)) THEN RETURN TRUE
    ELSE RETURN FALSE END
  END Schaltjahr;


  (*********  calculate totals days FOR a specific date  *********)

  PROCEDURE CalcDays(day, month, year: INTEGER): INTEGER;
  VAR dpmonth : ARRAY [0..12] OF INTEGER;
      i, sum  : INTEGER;
  BEGIN
    dpmonth[0] := 0;
    dpmonth[1] := 31;
    IF Schaltjahr(year) THEN
      dpmonth[2] := 29;
    ELSE
      dpmonth[2] := 28;
    END;
    dpmonth[3] := 31;         dpmonth[4] := 30;
    dpmonth[5] := 31;         dpmonth[6] := 30;
    dpmonth[7] := 31;         dpmonth[8] := 31;
    dpmonth[9] := 30;         dpmonth[10] := 31;
    dpmonth[11] := 30;        dpmonth[12] := 31;

    sum := 0;
    FOR i := 0 TO (month - 1) DO
      sum := sum + dpmonth[i]
    END;
    RETURN (sum + day)
  END CalcDays;


BEGIN

  InitText;

  (*********  parsing arguments  *********)

  GetBirthFilename(InputFName);

  (*********  open Birthday file  *********)

  fs.Lookup(InFile, InputFName, 1024, FALSE);

  IF (InFile.res # fs.done) THEN
    io.WriteString(" Could not open input file !! \n");
    Arts.Exit(20)
  END;

  (*********  read Birthday file  *********)

  FirstL := ReadFile(InFile);

  (*********  close Birthday file  *********)

  fs.Close(InFile);

  (*********  get actual date  *********)

  DL.DateStamp(sys.ADR(DosDate));
  DC.FromDos(DosDate, DInfo);
  DC.DecodeDate(DInfo.minutes, Rday, Rmonth, Ryear);
  RTotaldays := CalcDays(Rday, Rmonth, Ryear);
  RSchalt := Schaltjahr(Ryear);

  (*********  calculate persons and output  *********)

  ActualL := FirstL;
  REPEAT
    S1 := FALSE;
    S2 := FALSE;

  (*********  valid line ??  *********)

    IF (ParseLine(ActualL, Wday, Wmonth, Wyear, name, remark)) THEN
      WTotaldays := CalcDays(Wday, Wmonth, Ryear); (* RYear --> days in
                       this year ! (NOT birthyear !)                 *)
      WSchalt := Schaltjahr(Wyear);

  (*********  case 1 *********)

            (* Case 1 : end of this year and birthday is at beginning
                              of next                                *)
      IF ((RSchalt AND (RTotaldays > 359)) OR
          ((NOT RSchalt) AND (RTotaldays > 358))) THEN
        IF (WTotaldays < 8) THEN         (* test for one week *)
          S1 := TRUE;
          CASE RSchalt OF
            TRUE  : WTotaldays := WTotaldays + 366;|
            FALSE : WTotaldays := WTotaldays + 365;
          END;
        END;
      END;

  (*********  case 1 *********)

            (* Case 2 : beginning of year and birthday is at the end
                              of last                                *)
      IF (RTotaldays < 3) THEN           (* test only UNTIL day before
                                            yesterday                *)
        IF (WSchalt AND (WTotaldays > 364)) THEN
          S2 := TRUE;
          WTotaldays := WTotaldays - 366
        END;
        IF ((NOT WSchalt) AND (WTotaldays > 363)) THEN
          S2 := TRUE;
          WTotaldays := WTotaldays - 365
        END;
      END;

      Diff := WTotaldays - RTotaldays;

  (*********  person relevant ?  *********)

      IF (((-3 < Diff) AND (Diff < 4)) OR (Diff = 5) OR (Diff = 7)) THEN
        IF (Diff < 0) THEN
          vvar := past;
        ELSE
          vvar := future;
        END;
        CASE Diff OF
          -2: tvar := dbyes;|
          -1: tvar := yes;|
           0: tvar := tod;|
           1: tvar := tom;|
           2: tvar := datom;|
           3: tvar := tdays;|
           5: tvar := fdays;|
           7: tvar := week;
        END;

        IF S1 THEN
          Alter := (Ryear - Wyear + 1);
        ELSIF S2 THEN
          Alter := (Ryear - Wyear - 1);
        ELSE
          Alter := (Ryear - Wyear);
        END;

  (*********  output  *********)

        KillBlAndEol(name);
        io.WriteLn;
        io.WriteString(name);
        io.WriteString(v[vvar]);
        io.WriteString(t[tvar]);
        io.WriteString(" den ");
        io.WriteInt(Alter,0);
        IF (Alter > 19) THEN
          io.WriteString("sten");
        ELSE
          io.WriteString("ten");
        END;
        IF (remark[0] # ASCII.nul) THEN
          KillBlAndEol(remark);
          io.WriteString(" Geburtstag !\n   (");
          io.WriteString(remark);
          io.WriteString(")\n");
        ELSE
          io.WriteString(" Geburtstag !\n");
        END;

      END;
    END;

    ActualL := ActualL^.NLine;
  UNTIL (ActualL = NIL);
  io.WriteLn;

CLOSE
  Freelines(TRUE);
END RememberBirth.
