(*-------------------------------------------------------------------------
:Program.       AudioSupport.mod
:Contents.      Procedures for easy sound programming
:Author.        Christian Stiens
:Address.       Heustiege 2, D-4710 Ldinghausen
:Phone.         02591/6299
:Copyright.     PD
:Language.      Modula-2
:Translator.    M2Amiga V3.3d,  AMSoft
:History.       V1.0, 25-Aug-90
-------------------------------------------------------------------------*)

IMPLEMENTATION MODULE AudioSupport;

  FROM SYSTEM IMPORT
    LONGSET,ADR,ADDRESS;
  FROM Audio IMPORT
    pervol,IOAudioPtr,IOAudio,perVol,audioName;
  FROM ExecD IMPORT
    MsgPortPtr,write,quick,MemReqs,MemReqSet,IOFlagSet;
  FROM ExecL IMPORT
    AllocMem,OpenDevice,CloseDevice,DoIO,FreeMem,CheckIO,WaitIO;
  FROM ExecSupport IMPORT
    BeginIO,AbortIO,CreateExtIO,DeleteExtIO,CreatePort,DeletePort;
  FROM Arts IMPORT
    Assert;


  CONST
    cantOpen = "Can't open channel";
    notOpen  = "Channel not open";

  VAR
    mp1,mp2     : ARRAY Channel OF MsgPortPtr;
    ioa1,ioa2   : ARRAY Channel OF IOAudioPtr;
    open        : ARRAY Channel OF BOOLEAN;


  PROCEDURE OpenChannel(channelMask: ChannelSet): Channel;
    VAR
      allocMask : ARRAY [0..3] OF ChannelSet;
      ch        : Channel;
      i         : INTEGER;
      mp        : MsgPortPtr;
      ioa       : IOAudioPtr;
  BEGIN
    i := 0;
    mp := CreatePort(NIL,0);
    IF mp # NIL THEN
      ioa := CreateExtIO(mp,SIZE(IOAudio));
      IF ioa # NIL THEN
        FOR ch := left0 TO left1 DO
          IF ch IN channelMask THEN
            allocMask[i] := ChannelSet{ch};
            INC(i);
          END
        END;
        WITH ioa^ DO
          request.message.node.pri := 0;
          data   := ADR(allocMask);
          length := i*SIZE(ChannelSet);
        END;
        OpenDevice(ADR(audioName),0,ioa,LONGSET{});
        IF ioa^.request.error=0 THEN
          CASE LONGINT(ioa^.request.unit) OF
          | 1: ch := left0;
          | 2: ch := right0;
          | 4: ch := right1;
          | 8: ch := left1;
          END;
          mp1[ch] := mp;
          ioa1[ch] := ioa;
          mp2[ch] := CreatePort(NIL,0);
          IF mp2[ch] # NIL THEN
            ioa2[ch] := AllocMem(SIZE(IOAudio),MemReqSet{memClear,public});
            IF ioa2[ch] # NIL THEN
              ioa2[ch]^ := ioa1[ch]^;
              ioa2[ch]^.request.message.replyPort := mp2[ch];
              open[ch] := TRUE;
              RETURN ch
            END;
            DeletePort(mp2[ch]);
          END;
          CloseDevice(ioa);
        END;
        DeleteExtIO(ioa);
      END;
      DeletePort(mp)
    END;
    Assert(FALSE,ADR(cantOpen));
  END OpenChannel;

  PROCEDURE CloseChannel(ch: Channel);
  BEGIN
    IF open[ch] THEN
      CloseDevice(ioa1[ch]);
      FreeMem(ioa2[ch],SIZE(IOAudio));
      DeletePort(mp2[ch]);
      DeleteExtIO(ioa1[ch]);
      DeletePort(mp1[ch]);
      open[ch] := FALSE;
    END
  END CloseChannel;

  PROCEDURE CloseAllChannels;
  BEGIN
    CloseChannel(left0);
    CloseChannel(right0);
    CloseChannel(right1);
    CloseChannel(left1);
  END CloseAllChannels;

  PROCEDURE PlaySound(ch:Channel; sound:ADDRESS; size:LONGCARD;
                      per,vol,cyc:CARDINAL);
  BEGIN
    Assert(open[ch],ADR(notOpen));
    AbortIO(ioa1[ch]);
    WITH ioa1[ch]^ DO
      request.command := write;
      request.flags := pervol+quick;
      data := sound;
      length := size;
      period := per;
      volume := vol;
      cycles := cyc;
    END;
    BeginIO(ioa1[ch]);
  END PlaySound;

  PROCEDURE StopSound(ch: Channel);
  BEGIN
    Assert(open[ch],ADR(notOpen));
    AbortIO(ioa1[ch])
  END StopSound;

  PROCEDURE ModifySound(ch:Channel; per,vol: CARDINAL);
  BEGIN
    Assert(open[ch],ADR(notOpen));
    WITH ioa2[ch]^ DO
      request.command := perVol;
      request.flags := quick;
      period := per;
      volume := vol;
    END;
    DoIO(ioa2[ch])
  END ModifySound;

  PROCEDURE WaitSound(ch: Channel);
  BEGIN
    Assert(open[ch],ADR(notOpen));
    WaitIO(ioa1[ch]);
  END WaitSound;

  PROCEDURE CheckSound(ch: Channel):BOOLEAN;
  BEGIN
    Assert(open[ch],ADR(notOpen));
    RETURN NOT CheckIO(ioa1[ch])
  END CheckSound;

  PROCEDURE Filter(onOff: BOOLEAN);
    VAR ciaapra[0BFE001H]: SET OF [0..7];
  BEGIN
    IF onOff THEN
      EXCL(ciaapra,1)
    ELSE
      INCL(ciaapra,1)
    END
  END Filter;

BEGIN
  open[left0]  := FALSE;
  open[right0] := FALSE;
  open[right1] := FALSE;
  open[left1]  := FALSE;
CLOSE
  CloseAllChannels;
END AudioSupport.
