(******************************************************************************

    :Program.    Clock.mod
    :Contents.   procedures to read and reset an ever-running clock
    :Version.    0.1
    :Date.       18.10.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    18.10.91 --- 0.1 --- created

******************************************************************************)
IMPLEMENTATION MODULE Clock;


FROM Arts IMPORT
  Assert;

FROM ExecD IMPORT
  DevicePtr,MsgPortPtr;

FROM ExecL IMPORT
  CloseDevice,DoIO,OpenDevice;

FROM ExecSupport IMPORT
  CreateExtIO,CreatePort,DeleteExtIO,DeletePort;

FROM Timer IMPORT
  getSysTime,SubTime,TimeRequest,TimeRequestPtr,timerName,TimeVal,vBlank;

FROM SYSTEM IMPORT
  ADR,LONGSET;


VAR
  timerPort : MsgPortPtr;
  timeReqPtr: TimeRequestPtr;
  timerBase : DevicePtr;
  startTime : TimeVal;


PROCEDURE ResetClock;

BEGIN
  timeReqPtr^.node.command:=getSysTime;
  DoIO(timeReqPtr);
  startTime:=timeReqPtr^.time;
END ResetClock;

(* after playing longer than 255 minutes the procedure may be buggy !!! *)
PROCEDURE ReadClock(VAR mins,secs : CARDINAL);

VAR actualTime: TimeVal;

BEGIN
  timeReqPtr^.node.command:=getSysTime;
  DoIO(timeReqPtr);
  actualTime:=timeReqPtr^.time;
  SubTime(timerBase,ADR(actualTime),ADR(startTime));
  secs:=CARDINAL(actualTime.secs MOD 60);
  mins:=CARDINAL(actualTime.secs DIV 60);
  IF mins>99 THEN
    mins:=99;
    secs:=59;
  END;
END ReadClock;


BEGIN
  timeReqPtr:=NIL;
  timerBase:=NIL;
  timerPort:=CreatePort(NIL,0);  (* create a private port *)
  Assert(timerPort#NIL,ADR("Can't create TimerPort"));
  timeReqPtr:=CreateExtIO(timerPort,SIZE(TimeRequest));
  Assert(timeReqPtr#NIL,ADR("Can't create TimeRequest"));
  OpenDevice(ADR(timerName),vBlank,timeReqPtr,LONGSET{});
  Assert(timeReqPtr^.node.error=0,ADR("Can't open timer.device"));
  timerBase:=timeReqPtr^.node.device;
  ResetClock;
CLOSE
  IF timeReqPtr#NIL THEN
    CloseDevice(timeReqPtr);
    DeleteExtIO(timeReqPtr);
    timeReqPtr:=NIL;
  END;
  IF timerPort#NIL THEN
    DeletePort(timerPort);
    timerPort:=NIL;
  END;
END Clock.

