(******************************************************************************

    :Program.    Game.mod
    :Contents.   procedure to play the game
    :Version.    0.1
    :Date.       18.10.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    18.10.91 --- 0.1 --- created

******************************************************************************)
IMPLEMENTATION MODULE Game;


(*====== Import from project Pushem =========================================*)
FROM Clock IMPORT
  ReadClock,ResetClock;

FROM Definitions IMPORT
  Dests,Direction,LevelFile,MaxX,MaxY,Modes,Moves,NameString,Playfield,Status,
  StatusSet;

FROM GraphicSupport IMPORT
  BlitObject,ChangeScreens,DrawLevel,DrawNumber,HideSprites,obFallingBall,
  obPlatform,rp,ShowTop5,spFallingMan,spBall,spMan,spr1,spr2,spr3,vp,xOff,yOff;

FROM InputControl IMPORT
  Fire,GetDir;

FROM LevelLoader IMPORT
  LevelNumber,SaveLevel;

FROM SoundSupport IMPORT
  dropSound,fallSound,PlaySample;

(*====== Import from standard library =======================================*)
FROM DosL IMPORT
  Delay;

FROM GraphicsL IMPORT
  ChangeSprite,MoveSprite;

FROM SYSTEM IMPORT
  ADDRESS;


TYPE
  ActionType=(wait,walk,pushBall,getBall,leaveBall,dropBall,fall);


VAR
  mode       : Modes;
  field      : Playfield;
  readyDests : Dests;
  man        : ARRAY[0..1] OF RECORD
                                xpos  : [0..MaxX];
                                ypos  : [0..MaxY];
                                newDir: Direction;
                                x,y   : INTEGER;
                                xp,xp2: INTEGER;
                                yp,yp2: INTEGER;
                                dir   : Direction;
                                hball : BOOLEAN;
                                step  : [0..15];
                                action: ActionType;
                                dead  : BOOLEAN;
                              END;
  player     : Modes;


PROCEDURE InitLevel(level : LevelFile);

BEGIN
  mode:=level.mode;
  field:=level.field;
  readyDests:=level.readyDests;
  FOR player:=0 TO 1 DO
    WITH man[player] DO
      xpos:=level.man[player].x;
      ypos:=level.man[player].y;
      newDir:=noDir;
      x:=xOff+15*INTEGER(xpos);
      y:=yOff+15*INTEGER(ypos);
      dir:=north;
      hball:=FALSE;
      step:=0;
      dead:=FALSE;
    END;
    ChangeSprite (vp,spr3[player],spBall[player]);
    spr1[player]^.height:=16;
    spr2[player]^.height:=16;
    spr3[player]^.height:=16;
  END;
END InitLevel;


PROCEDURE NextMove(VAR moves : Moves) : Dests;

VAR
  inRange  : BOOLEAN;
  i        : CARDINAL;
  dummy    : BOOLEAN;
  b1,b2,b3 : BOOLEAN;
  picnum   : [0..3];
  sprAdr   : ADDRESS;

BEGIN
  FOR player:=0 TO mode DO
    WITH man[player] DO
      IF NOT dead THEN
        IF step=0 THEN
          newDir:=GetDir(player);
          IF (newDir#dir) AND hball THEN
            action:=leaveBall;
            hball:=FALSE;
            CASE dir OF
            | north: xp:=xpos; yp:=ypos-1;
            | south: xp:=xpos; yp:=ypos+1;
            | east: xp:=xpos+1; yp:=ypos;
            | west: xp:=xpos-1; yp:=ypos;
            ELSE
            END;
            EXCL (field[xp,yp],occupied);
            INCL (field[xp,yp],ball);
            BlitObject(obPlatform[dest IN field[xp,yp],TRUE],rp,
                       xp*15+xOff,yp*15+yOff);
          ELSIF newDir=noDir THEN
            action:=wait;
          ELSIF NOT hball THEN
            dir:=newDir;
            inRange:=FALSE;
            CASE dir OF
            | north: xp:=xpos; yp:=INTEGER(ypos)-1;
                     IF yp>=0 THEN
                       inRange:=TRUE;
                     END;
            | south: xp:=xpos; yp:=INTEGER(ypos)+1;
                     IF yp<=MaxY THEN
                       inRange:=TRUE;
                     END;
            | east: xp:=INTEGER(xpos)+1; yp:=ypos;
                    IF xp<=MaxX THEN
                      inRange:=TRUE;
                    END;
            | west: xp:=INTEGER(xpos)-1; yp:=ypos;
                    IF xp>=0 THEN
                      inRange:=TRUE;
                    END;
            ELSE
            END;
            action:=wait;
            IF inRange THEN
              IF ball IN field[xp,yp] THEN
                action:=getBall;
                EXCL (field[xp,yp],ball);
                INCL (field[xp,yp],occupied);
              ELSIF gorge IN field[xp,yp] THEN
                action:=fall;
                INC(moves);
                EXCL (field[xpos,ypos],man1);
                INCL (field[xp,yp],man1);
                xpos:=xp; ypos:=yp;
              ELSIF field[xp,yp]*StatusSet{man1,occupied}=StatusSet{} THEN
                action:=walk;
                INC(moves);
                EXCL (field[xpos,ypos],man1);
                INCL (field[xp,yp],man1);
                xpos:=xp; ypos:=yp;
              END;
            END;
          ELSE
            inRange:=FALSE;
            CASE dir OF
            | north: xp:=xpos; yp:=ypos-1;
                     xp2:=xpos; yp2:=INTEGER(ypos)-2;
                     IF yp2>=0 THEN
                       inRange:=TRUE;
                     END;
            | south: xp:=xpos; yp:=ypos+1;
                     xp2:=xpos; yp2:=INTEGER(ypos)+2;
                     IF yp2<=MaxY THEN
                       inRange:=TRUE;
                     END;
            | east: xp:=xpos+1; yp:=ypos;
                    xp2:=INTEGER(xpos)+2; yp2:=ypos;
                    IF xp2<=MaxX THEN
                      inRange:=TRUE;
                    END;
            | west: xp:=xpos-1; yp:=ypos;
                    xp2:=INTEGER(xpos)-2; yp2:=ypos;
                    IF xp2>=0 THEN
                      inRange:=TRUE;
                    END;
            END;
            action:=wait;
            IF inRange THEN
              IF field[xp2,yp2]*StatusSet{man1,occupied,ball}
                 =StatusSet{} THEN
                action:=pushBall;
                INC(moves);
                EXCL (field[xpos,ypos],man1);
                EXCL (field[xp,yp],occupied);
                INCL (field[xp,yp],man1);
                INCL (field[xp2,yp2],occupied);
                xpos:=xp; ypos:=yp;
                IF dest IN field[xp,yp] THEN
                  DEC (readyDests);
                END;
                IF dest IN field[xp2,yp2] THEN
                  INC (readyDests);
                END;
              END;
              IF field[xp2,yp2]=StatusSet{gorge,occupied} THEN
                action:=dropBall;
                EXCL (field[xp2,yp2],occupied);
              END;
            END;
          END;
        END;
        IF action#wait THEN
          INC(step);
          IF action=dropBall THEN
            IF step<9 THEN
              IF step=4 THEN
                CASE dir OF
                | north: DEC(y);
                | south: INC(y);
                | east: INC(x);
                | west: DEC(x);
                END;
              ELSE
                CASE dir OF
                | north: DEC(y,2);
                | south: INC(y,2);
                | east: INC(x,2);
                | west: DEC(x,2);
                END;
              END;
            ELSE
              hball:=FALSE;
              IF xp2<MaxX THEN
                b1:=field[xp2+1,yp2]#StatusSet{gorge};
              ELSE
                b1:=FALSE;
              END;
              IF yp2<MaxY THEN
                b2:=field[xp2,yp2+1]#StatusSet{gorge};
              ELSE
                b2:=FALSE;
              END;
              IF (xp2<MaxX) AND (yp2<MaxY) THEN
                b3:=field[xp2+1,yp2+1]#StatusSet{gorge};
              ELSE
                b3:=FALSE;
              END;
              BlitObject(obFallingBall[step-9,b1,b2,b3],rp,
                         xp2*15+xOff+5,yp2*15+yOff+5);
              MoveSprite(vp,spr3[player],310,300);
            END;
            IF step=10 THEN
              PlaySample(dropSound,player);
            END;
            IF step=12 THEN
              step:=1;
              action:=leaveBall;
            END;
          END;
          CASE action OF
          | walk: IF step=4 THEN
                    CASE dir OF
                    | north: DEC (y);
                    | south: INC (y);
                    | east: INC (x);
                    | west: DEC (x);
                    END;
                  ELSE
                    CASE dir OF
                    | north: DEC (y,2);
                    | south: INC (y,2);
                    | east: INC (x,2);
                    | west: DEC (x,2);
                    END;
                  END;
                  IF step=8 THEN
                    step:=0;
                  END;
          | fall: IF xp<MaxX THEN
                    b1:=NOT(gorge IN field[xp+1,yp]);
                  ELSE
                    b1:=FALSE;
                  END;
                  IF yp<MaxY THEN
                    b2:=NOT(gorge IN field[xp,yp+1]);
                  ELSE
                    b2:=FALSE;
                  END;
                  IF (xp<MaxX) AND (yp<MaxY) THEN
                    b3:=NOT(gorge IN field[xp+1,yp+1]);
                  ELSE
                    b3:=FALSE;
                  END;
                  IF step<4 THEN
                    CASE dir OF
                    | north: DEC(y,2);
                    | south: INC(y,2);
                    | east: INC(x,2);
                    | west: DEC(x,2);
                    END;
                  ELSIF step=4 THEN
                    CASE dir OF
                    | north: DEC(y);
                    | south: INC(y);
                    | east: INC(x);
                    | west: DEC(x);
                    END;
                    PlaySample(fallSound,player);
                  ELSIF step=5 THEN
                    CASE dir OF
                    | north: INC(x,3);
                             DEC(y,3);
                    | south: INC(x,5);
                             INC(y,8);
                    | east: INC(x,9);
                            INC(y,5);
                    | west: DEC(x,3);
                            INC(y,3);
                    END;
                    MoveSprite (vp,spr3[player],310,300);
                  ELSIF step=6 THEN
                    CASE dir OF
                    | north: INC (x,2);
                    | south: INC (y,5);
                    | east: INC (x,4);
                    | west: INC (y,2);
                    END;
                  ELSIF step=7 THEN
                    dead:=TRUE;
                  END;
          | pushBall: IF step=4 THEN
                        CASE dir OF
                        | north: DEC (y);
                        | south: INC (y);
                        | east: INC (x);
                        | west: DEC (x);
                        END;
                      ELSE
                        CASE dir OF
                        | north: DEC (y,2);
                        | south: INC (y,2);
                        | east: INC (x,2);
                        | west: DEC (x,2);
                        END;
                      END;
                      IF step=8 THEN
                        step:=0;
                      END;
          | getBall: IF step=1 THEN
                       CASE dir OF
                       | north: MoveSprite (vp,spr3[player],x-1,y-15);
                       | south: MoveSprite (vp,spr3[player],x-1,y+15);
                       | east: MoveSprite (vp,spr3[player],x+14,y);
                       | west: MoveSprite (vp,spr3[player],x-16,y);
                       END;
                     END;
                     CASE dir OF
                     | north: DEC (y,2);
                     | south: INC (y,2);
                     | east: INC (x,2);
                     | west: DEC (x,2);
                     END;
                     IF step=2 THEN
                       step:=0;
                       hball:=TRUE;
                       BlitObject(obPlatform[dest IN field[xp,yp],FALSE],rp,
                                  xp*15+xOff,yp*15+yOff);
                     END;
          | leaveBall: IF step=1 THEN
                         MoveSprite (vp,spr3[player],310,300);
                       END;
                       CASE dir OF
                       | north: INC (y,2);
                       | south: DEC (y,2);
                       | east: DEC (x,2);
                       | west: INC (x,2);
                       END;
                       IF step=2 THEN
                         step:=0;
                       END;
          ELSE
          END;
        END;
        MoveSprite (vp,spr1[player],x-1,y);
        MoveSprite (vp,spr2[player],x-1,y);
        IF hball THEN
          CASE dir OF
          | north: MoveSprite (vp,spr3[player],x-1,y-11);
          | south: MoveSprite (vp,spr3[player],x-1,y+11);
          | east: MoveSprite (vp,spr3[player],x+10,y);
          | west: MoveSprite (vp,spr3[player],x-12,y);
          END;
        END;
        IF (step<8) AND (action#fall) THEN
          sprAdr:=spMan[player,hball,dir,step];
          ChangeSprite (vp,spr1[player],sprAdr);
          ChangeSprite (vp,spr2[player],sprAdr+72);
        ELSIF action=fall THEN
          IF step<5 THEN
            sprAdr:=spMan[player,hball,dir,step];
            ChangeSprite (vp,spr1[player],sprAdr);
            ChangeSprite (vp,spr2[player],sprAdr+72);
          ELSE
            picnum:=0;
            CASE dir OF
            | north: IF b1 THEN
                       picnum:=1;
                       IF b3 THEN
                         picnum:=3;
                       END;
                     ELSIF b3 THEN
                       picnum:=2;
                     END;
            | south: IF b1 THEN
                       picnum:=1;
                     END;
            | east: IF b2 THEN
                      picnum:=1;
                    END;
            | west: IF b2 THEN
                      picnum:=2;
                      IF b3 THEN
                        picnum:=3;
                      END;
                    ELSIF b3 THEN
                      picnum:=1;
                    END;
            ELSE
            END;
            sprAdr:=spFallingMan[player,dir,picnum,step-5];
            ChangeSprite (vp,spr1[player],sprAdr);
            ChangeSprite (vp,spr2[player],sprAdr+72);
          END;
        END;
      END;
    END;
  END;
  Delay(1);
  RETURN (readyDests);
END NextMove;


PROCEDURE PlayGame (level        : LevelFile;
                    name         : NameString;
                    showTop5     : BOOLEAN;
                    currentLevel : LevelNumber;
                    maxLevel     : LevelNumber) : BOOLEAN;

VAR
  readyDest : Dests;
  moves     : Moves;
  solved    : BOOLEAN;
  mins,secs : CARDINAL;
  x,y       : INTEGER;

BEGIN
  InitLevel (level);
  DrawLevel (level,TRUE);
  ChangeScreens (FALSE);
  solved:=FALSE;
  ResetClock;
  DrawNumber (currentLevel,76,232,31,3);
  REPEAT
    ReadClock (mins,secs);
    DrawNumber (mins,252,232,31,2);
    DrawNumber (secs,276,232,31,2);
    DrawNumber (moves,164,232,31,4);
    readyDests:=NextMove (moves);
    IF (readyDests=level.dests) THEN
      IF NOT man[0].dead THEN
        solved:=man[0].action=wait;
      ELSIF NOT man[1].dead THEN
        solved:=TRUE;
      END;
      IF (mode=1) AND (NOT man[1].dead) THEN
        solved:=solved AND (man[1].action=wait);
      END;
    END;
  UNTIL Fire(1) OR Fire(0) OR solved OR (moves=9999);
  HideSprites;
  IF (readyDests=level.dests) THEN
    IF showTop5 THEN
      x:=4;
      WHILE (x>=0) AND (moves<level.top5[x].moves) DO
        DEC (x);
      END;
      WHILE (x>=0) AND
            (moves=level.top5[x].moves) AND
            (mins*60+secs<level.top5[x].mins*60+level.top5[x].secs) DO
        DEC (x);
      END;
      INC (x);
      IF x<5 THEN
         FOR y:=3 TO x BY -1 DO
           level.top5[y+1]:=level.top5[y];
         END;
         level.top5[x].name:=name;
         level.top5[x].moves:=moves;
         level.top5[x].mins:=mins;
         level.top5[x].secs:=secs;
         maxLevel:=SaveLevel (currentLevel,level);
      END;
      ShowTop5 (level.top5,currentLevel,x);
    END;
    RETURN (TRUE);
  END;
  IF showTop5 THEN
    ShowTop5 (level.top5,currentLevel,6);
  END;
  RETURN (FALSE);
END PlayGame;


END Game.
