(******************************************************************************

    :Program.    GfxToObj.mod
    :Contents.   Converts graphics data to Gfx.obj
    :Version.    0.3
    :Date.       01.11.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    29.07.91 --- 0.1 --- created
                 11.09.91 --- 0.2 --- bug removed
                 01.11.91 --- 0.3 --- fixed another bug

******************************************************************************)

MODULE GfxToObj;


FROM Arts IMPORT
  Assert;

FROM ExecL IMPORT
  CopyMem;

FROM FileSystem IMPORT
  Close,File,Length,Lookup,ReadBytes,Response,SetPos;

FROM GraphicsD IMPORT
  BitMap;

FROM GraphicsL IMPORT
  InitBitMap;

FROM Heap IMPORT
  Allocate,Deallocate;

FROM ObjFileCreator IMPORT
  AddData,AddDataBlock,AddLabel,CreateDataHunk,DefReloc,HunkPos,WriteObjFile;

FROM String IMPORT
  Compare;

FROM SYSTEM IMPORT
  ADDRESS,ADR;

FROM Terminal IMPORT
  WriteInt, WriteLn;


TYPE
  BlOb=RECORD
         bitMap : LONGCARD;  (* pointer to the bitmap with the image *)
         mask   : LONGCARD;  (* pointer to a mask for BltMask... or 0 *)
         yOffset: CARDINAL;  (* offset to the first row of the image *)
         width  : CARDINAL;  (* size of     *)
         height : CARDINAL;  (*   the image *)
       END;
  BMHDPtr=POINTER TO BMHD;
  BMHD=RECORD
         w,h        : CARDINAL;
         x,y        : INTEGER;
         nPlanes    : SHORTCARD;
         masking    : SHORTCARD;
         compression: SHORTCARD;
         pad1       : SHORTCARD;
         transCol   : SHORTCARD;
         xAspect    : SHORTCARD;
         yAspect    : SHORTCARD;
         pageWidth  : INTEGER;
         pageHeight : INTEGER;
       END;
  String4=ARRAY [0..4] OF CHAR;
  Chunk=RECORD
          id  : String4;
          len : LONGCARD;
          data: ADDRESS;
        END;


VAR
  dataHunk,structHunk: CARDINAL;
  chunk              : Chunk;
  bmhd               : BMHDPtr;
  file               : File;
  pic,src,dest       : POINTER TO SHORTINT;
  plane              : [0..4];
  line,row           : CARDINAL;
  byte               : SHORTINT;
  data               : SHORTCARD;
  dataPtr            : POINTER TO SHORTCARD;
  i                  : CARDINAL;
  bitMap             : BitMap;
  bmAdr              : LONGCARD;
  blob               : BlOb;
  maskAdr            : ARRAY [0..3] OF LONGCARD;
  b1,b2,b3           : BOOLEAN;


PROCEDURE ReadChunk(VAR chunk:Chunk): BOOLEAN;

VAR act: LONGINT;

BEGIN
  ReadBytes(file,ADR(chunk.id),4,act);
  IF act#4 THEN
    RETURN(FALSE);
  END;
  IF (Compare(chunk.id,"BMHD")=0) OR
     (Compare(chunk.id,"BODY")=0) OR
     (Compare(chunk.id,"CMAP")=0) THEN
    ReadBytes(file,ADR(chunk.len),4,act);
    Allocate(chunk.data,chunk.len);
    Assert(chunk.data#NIL,ADR("Not enough memory"));
    ReadBytes(file,chunk.data,chunk.len,act);
  END;
  RETURN(TRUE);
END ReadChunk;


PROCEDURE FreeChunk(VAR chunk: Chunk);

BEGIN
  Deallocate(chunk.data);
END FreeChunk;


PROCEDURE AddPlane(x,y,plane:LONGCARD);

VAR
  i: LONGCARD;

BEGIN
  FOR i:=0 TO 14 DO
    AddData(dataHunk,LONGCARD(pic)+10*60*plane+(y*15+i)*10+2*x,2);
  END;
END AddPlane;


BEGIN
  Lookup(file,"Grafik/Gfx80x60.IFF",1024,FALSE);
  Assert(file.res=done,ADR("Gfx80x60.IFF not found"));
  SetPos(file,12);
  dataHunk:=CreateDataHunk("GfxData",TRUE);
  structHunk:=CreateDataHunk("GfxStructs",FALSE);
  WHILE ReadChunk(chunk) DO
    IF Compare(chunk.id,"BODY")=0 THEN
      IF bmhd^.compression=0 THEN
        pic:=chunk.data;
      ELSE
        Allocate(pic,10*60*5);
        Assert(pic#NIL,ADR("Not enough memory"));
        src:=chunk.data;
        dest:=pic;
        FOR line:=0 TO 59 DO
          FOR plane:=0 TO 4 DO
            row:=0;
            REPEAT
              dest:=ADDRESS(LONGCARD(pic)+10*60*plane+line*10+row);
              byte:=src^;
              INC(src);
              IF byte>=0 THEN
                CopyMem(src,dest,SHORTCARD(byte)+1);
                INC(src,SHORTCARD(byte)+1);
                row:=row+CARDINAL(byte)+1;
              ELSIF byte>-128 THEN
                byte:=-byte;
                FOR i:=0 TO SHORTCARD(byte) DO
                  dest^:=src^;
                  INC(dest);
                END;
                INC(src,1);
                row:=row+CARDINAL(byte)+1;
              END;
            UNTIL row>=10;
          END;
        END;
        FreeChunk(chunk);
      END;
    ELSIF Compare(chunk.id,"CMAP")=0 THEN
      AddLabel(structHunk,"Gfx_cmap");
      FOR i:=0 TO chunk.len-1 DO
        dataPtr:=ADDRESS(chunk.data+ADDRESS(i));
        IF i>2 THEN
          data:=dataPtr^ DIV 16;
        ELSE
          data:=0; (* background color is black *)
        END;
        AddDataBlock(structHunk,data);
      END;
      FreeChunk(chunk);
    ELSIF Compare(chunk.id,"BMHD")=0 THEN
      bmhd:=chunk.data;
    END;
  END;
  Close(file);
  InitBitMap(bitMap,5,15,165);
  bmAdr:=HunkPos(structHunk);
  FOR plane:=0 TO 4 DO
    bitMap.planes[plane]:=ADDRESS(HunkPos(dataHunk));
    DefReloc(structHunk,bmAdr+8+4*plane,dataHunk);
    FOR i:=0 TO 3 DO
      AddPlane(2,i,plane);
    END;
    FOR i:=0 TO 3 DO
      AddPlane(3,i,plane);
    END;
    FOR i:=0 TO 2 DO
      AddPlane(4,i,plane);
    END;
  END;
  AddDataBlock(structHunk,bitMap);
  WITH blob DO
    bitMap:=bmAdr;
    mask:=0;
    yOffset:=0;
    width:=15;
    height:=15;
    AddLabel(structHunk,"Gfx_platform");
    FOR i:=0 TO 3 DO
      DefReloc(structHunk,HunkPos(structHunk),structHunk);
      AddDataBlock(structHunk,blob);
      INC(yOffset,15);
    END;
    AddLabel(structHunk,"Gfx_man");
    FOR i:=4 TO 7 DO
      DefReloc(structHunk,HunkPos(structHunk),structHunk);
      AddDataBlock(structHunk,blob);
      INC(yOffset,15);
    END;
    AddLabel(structHunk,"Gfx_gorge");
    DefReloc(structHunk,HunkPos(structHunk),structHunk);
    AddDataBlock(structHunk,blob);
    INC(yOffset,15);
    AddLabel(structHunk,"Gfx_shadow");
    DefReloc(structHunk,HunkPos(structHunk),structHunk);
    AddDataBlock(structHunk,blob);
    INC(yOffset,15);
    AddLabel(structHunk,"Gfx_gorge2");
    DefReloc(structHunk,HunkPos(structHunk),structHunk);
    AddDataBlock(structHunk,blob);
    INC(yOffset,15);
  END;
  InitBitMap(bitMap,5,15,60);
  bmAdr:=HunkPos(structHunk);
  FOR plane:=0 TO 4 DO
    bitMap.planes[plane]:=ADDRESS(HunkPos(dataHunk));
    DefReloc(structHunk,bmAdr+8+4*plane,dataHunk);
    FOR i:=0 TO 3 DO
      AddPlane(0,i,plane);
    END;
  END;
  AddDataBlock(structHunk,bitMap);
  FOR i:=0 TO 3 DO
    maskAdr[i]:=HunkPos(dataHunk);
    AddPlane(1,i,0);
  END;
  AddLabel(structHunk,"Gfx_fallingBall");
  FOR i:=0 TO 3 DO
    FOR b1:=FALSE TO TRUE DO
      FOR b2:=FALSE TO TRUE DO
        FOR b3:=FALSE TO TRUE DO
          WITH blob DO
            bitMap:=bmAdr;
            mask:=0;
            yOffset:=15*i;
            width:=15;
            height:=15;
            IF b3 THEN
              IF b1 THEN
                width:=10;
              ELSIF NOT b2 THEN
                mask:=maskAdr[0];
              END;
              IF b2 THEN
                height:=10;
              END;
            ELSE
              IF b1 THEN
                IF b2 THEN
                  mask:=maskAdr[3];
                ELSE
                  mask:=maskAdr[2];
                END;
              ELSIF b2 THEN
                mask:=maskAdr[1];
              END;
            END;
          END;
          DefReloc(structHunk,HunkPos(structHunk),structHunk);
          IF blob.mask#0 THEN
            DefReloc(structHunk,HunkPos(structHunk)+4,dataHunk);
          END;
          AddDataBlock(structHunk,blob);
        END;
      END;
    END;
  END;
  WriteObjFile("obj/Gfx.obj","Gfx");
END GfxToObj.
