(******************************************************************************

    :Program.    GraphicSupport.mod
    :Contents.   definition of graphic elements and supporting procedures
    :Version.    0.4
    :Date.       11.01.92
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    17.10.91 --- 0.1 --- created
                 13.11.91 --- 0.2 --- bug fixed in DrawLevel (did not draw
                                      man on dest)
                 16.11.91 --- 0.3 --- changed MenuLine in DrawMenuLine
                 11.01.92 --- 0.4 --- overscan-bug fixed

******************************************************************************)
IMPLEMENTATION MODULE GraphicSupport;


(*====== Import from project Push'em ========================================*)
FROM Definitions IMPORT
  Direction,LevelFile,MaxX,MaxY,Modes,Status,Top5;

FROM InputControl IMPORT
  Controllertype,Fire;

FROM LevelLoader IMPORT
  LevelNumber;

(*====== Import from standard library =======================================*)
FROM Arts IMPORT
  Assert;

FROM DosL IMPORT
  Delay;

FROM ExecL IMPORT
  CopyMem;

FROM Heap IMPORT
  Allocate,AllocMem,Deallocate;

FROM GraphicsD IMPORT
  BitMapPtr,FontFlags,FontFlagSet,FontStyleSet,RastPortPtr,SimpleSprite,
  SimpleSpritePtr,TextAttr,ViewModes,ViewModeSet,ViewPortPtr;

FROM GraphicsL IMPORT
  BltBitMapRastPort,BltMaskBitMapRastPort,Draw,FreeSprite,GetRGB4,GetSprite,
  Move,MoveSprite,RectFill,SetAPen,SetBPen,SetRast,SetRGB4,Text,VBeamPos;

FROM IntuitionD IMPORT
  customScreen,IDCMPFlags,IDCMPFlagSet,NewScreen,NewWindow,Preferences,
  ScreenFlags,ScreenFlagSet,ScreenPtr,WindowFlags,WindowFlagSet,WindowPtr;

FROM IntuitionL IMPORT
  ActivateWindow,ClearPointer,CloseScreen,CloseWindow,GetPrefs,OpenScreen,
  OpenWindow,SetPointer,ScreenToFront;

IMPORT R;

FROM SYSTEM IMPORT
  ADDRESS,ADR;


VAR
  titleCMap ["Title_cmap"] : ColMap;
  cmap      ["Gfx_cmap"]   : ColMap;
  titlePic  ["Title_data"] : CARDINAL;
  s1,s2,s3                 : ARRAY Modes OF INTEGER;
  pointer                  : INTEGER;
  cset                     : ARRAY [0..44] OF SHORTCARD;
  player                   : Modes;
  mouse                    : ADDRESS;
  topazFont                : TextAttr;
  prefs                    : Preferences;


PROCEDURE TitleToScreen;

TYPE
  SIPtr = POINTER TO SHORTINT;

VAR
  planes  : ARRAY [0..4] OF ADDRESS;
  plane   : [0..4];
  line,row: CARDINAL;
  src,dest: SIPtr;
  byte    : SHORTINT;

  PROCEDURE FillMem(srcByte{R.A0},dest{R.A1}:SIPtr; count{R.D0}:SHORTCARD);
  VAR i{R.D1}: INTEGER;
  BEGIN
    FOR i:=1 TO count DO
      dest^:=srcByte^;
      INC(dest);
    END;
  END FillMem;

BEGIN
  FOR plane:=0 TO 4 DO
    planes[plane]:=brp^.bitMap^.planes[plane];
  END;
  src:=ADR(titlePic);
  FOR line:=0 TO 255 DO
    FOR plane:=0 TO 4 DO
      row:=0;
      REPEAT
        dest:=ADDRESS(planes[plane]+ADDRESS(line*40+row));
        byte:=src^;
        INC(src);
        IF byte>=0 THEN
          CopyMem(src,dest,SHORTCARD(byte)+1);
          INC(src,SHORTCARD(byte)+1);
          row:=row+CARDINAL(byte)+1;
        ELSIF byte>-128 THEN
          byte:=-byte;
          FillMem(src,dest,SHORTCARD(byte)+1);
          INC(src,1);
          row:=row+CARDINAL(byte)+1;
        END;
      UNTIL row>=40;
    END;
  END;
END TitleToScreen;


PROCEDURE BlitObject(object:BlOb; destRP:RastPortPtr; x,y:CARDINAL);

VAR
  maskPtr: ADDRESS;

BEGIN
  WITH object DO
    IF mask#NIL THEN
      maskPtr:=mask-ADDRESS(bitMap^.bytesPerRow*yOffset);
      BltMaskBitMapRastPort(bitMap,0,yOffset,destRP,x,y,width,height,
                            0E0H,maskPtr);
    ELSE
      BltBitMapRastPort(bitMap,0,yOffset,destRP,x,y,width,height,0C0H);
    END;
  END;
END BlitObject;


PROCEDURE HideSprites;

VAR
  player: Modes;

BEGIN
  FOR player:=0 TO 1 DO
    MoveSprite (vp,spr1[player],310,300);
    MoveSprite (vp,spr2[player],310,300);
    MoveSprite (vp,spr3[player],310,300);
  END;
  MoveSprite (vp,pointerSpr,310,300);
END HideSprites;


PROCEDURE ChangeScreens(titleColMap : BOOLEAN);

CONST
  Steps=8;

VAR
  count    : CARDINAL;
  col,i    : CARDINAL;
  temp     : ADDRESS;
  oldcm    : ColMap;
  rgb4     : LONGCARD;
  newCMap  : ColMap;

BEGIN
  HideSprites;
  IF titleColMap THEN
    newCMap:=titleCMap;
  ELSE
    newCMap:=cmap;
  END;
  FOR col:=0 TO 31 DO
    rgb4:=GetRGB4(vp^.colorMap,col);
    oldcm[col*3]:=rgb4 DIV 256;
    rgb4:=rgb4-oldcm[col*3]*256;
    oldcm[col*3+1]:=rgb4 DIV 16;
    rgb4:=rgb4-oldcm[col*3+1]*16;
    oldcm[col*3+2]:=rgb4;
  END;
  FOR count:=Steps-1 TO 0 BY -1 DO
    FOR i:=0 TO 31 DO
      col:=3*i;
      SetRGB4(vp,i,oldcm[col]*count DIV Steps,
                   oldcm[col+1]*count DIV Steps,
                   oldcm[col+2]*count DIV Steps);
    END;
  END;
  ScreenToFront(bsp);
  temp:=sp; sp:=bsp; bsp:=temp;
  temp:=wp; wp:=bwp; bwp:=temp;
  temp:=rp; rp:=brp; brp:=temp;
  temp:=vp; vp:=bvp; bvp:=temp;
  ActivateWindow(wp);
  FOR count:=1 TO Steps DO
    FOR i:=0 TO 31 DO
      col:=3*i;
      SetRGB4(vp,i,newCMap[col]*count DIV Steps,
                   newCMap[col+1]*count DIV Steps,
                   newCMap[col+2]*count DIV Steps);
    END;
  END;
END ChangeScreens;


PROCEDURE OpenScreens;

VAR
  ns       : NewScreen;
  nw       : NewWindow;
  i        : INTEGER;

BEGIN
  WITH topazFont DO
    name:=ADR("topaz.font");
    ySize:=8;
    style:=FontStyleSet{};
    flags:=FontFlagSet{romFont};
  END;
  GetPrefs(ADR(prefs),SIZE(Preferences));
  WITH ns DO
    leftEdge:=-prefs.viewXOffset;topEdge:=0;
    width:=320;height:=256;
    depth:=5;
    detailPen:=0;blockPen:=1;
    viewModes:=ViewModeSet{sprites};
    type:=customScreen+ScreenFlagSet{screenQuiet};
    font:=ADR(topazFont);
    defaultTitle:=NIL;
    gadgets:=NIL;
    customBitMap:=NIL;
  END;
  bsp:=OpenScreen(ns);
  Assert (bsp#NIL,ADR("Kann Screen 1 nicht ffnen!"));
  WITH nw DO
    leftEdge:=0;topEdge:=0;
    width:=320;height:=256;
    detailPen:=0;blockPen:=1;
    idcmpFlags:=IDCMPFlagSet{rawKey};
    flags:=WindowFlagSet{activate,borderless,rmbTrap};
    firstGadget:=NIL;
    checkMark:=NIL;
    title:=NIL;
    screen:=bsp;
    bitMap:=NIL;
    type:=customScreen;
  END;
  bwp:=OpenWindow(nw);
  Assert (bwp#NIL,ADR("Kann Window 1 nicht ffnen!"));
  sp:=OpenScreen(ns);
  Assert (sp#NIL,ADR("Kann Screen 2 nicht ffnen!"));
  nw.screen:=sp;
  wp:=OpenWindow(nw);
  Assert (wp#NIL,ADR("Kann Window 2 nicht ffnen!"));
  AllocMem(mouse,8,TRUE);
  SetPointer(wp,mouse,0,0,0,0);
  SetPointer(bwp,mouse,0,0,0,0);
  rp:=wp^.rPort;
  vp:=ADR(sp^.viewPort);
  brp:=bwp^.rPort;
  bvp:=ADR(bsp^.viewPort);
  FOR i:=0 TO 31 DO
    SetRGB4(vp,i,0,0,0);
    SetRGB4(bvp,i,0,0,0);
  END;
END OpenScreens;


PROCEDURE InitSprites;

VAR
  ph      : ARRAY [0..7] OF CARDINAL;
  player  : Modes;
  dir     : Direction;
  phase,x : CARDINAL;

BEGIN
  FOR player:=0 TO 1 DO
    Allocate (spr1[player],SIZE(SimpleSprite));
    WITH spr1[player]^ DO
      posctldata:=spMan[player,FALSE,north,0];
      height:=16;
      x:=310; y:=300;
      num:=2+2*player;
    END;
    s1[player]:=GetSprite(ADR(spr1[player]),2+2*player);
    Allocate (spr2[player],SIZE(SimpleSprite));
    WITH spr2[player]^ DO
      posctldata:=spMan[player,FALSE,north,0]+72;
      height:=16;
      x:=310; y:=300;
      num:=3+2*player;
    END;
    s2[player]:=GetSprite(ADR(spr2[player]),3+2*player);
    Allocate (spr3[player],SIZE(SimpleSprite));
    WITH spr3[player]^ DO
      posctldata:=spBall[player];
      height:=16;
      x:=310; y:=300;
      num:=6+player;
    END;
    s3[player]:=GetSprite(ADR(spr3[player]),6+player);
  END;
  Allocate (pointerSpr,SIZE(SimpleSprite));
  WITH pointerSpr^ DO
    posctldata:=spPointer;
    height:=16;
    x:=310; y:=300;
    num:=1;
  END;
  pointer:=GetSprite(ADR(pointerSpr),1);
END InitSprites;


PROCEDURE ChangeCMap(colorSet:CARDINAL);

VAR
  i: CARDINAL;

BEGIN
  FOR i:=0 TO 2 DO
    cmap[9+i]:=cset[9*colorSet+i];
    cmap[18+i]:=cset[9*colorSet+3+i];
  END;
END ChangeCMap;


PROCEDURE DrawNumber (number : CARDINAL;
                      x,y    : INTEGER;
                      color  : INTEGER;
                      digits : SHORTCARD);

VAR
  numStr : ARRAY[1..6] OF CHAR;
  i      : INTEGER;

BEGIN
  SetAPen (rp,color);
  Move (rp,x,y);
  FOR i:=digits TO 1 BY -1 DO
    numStr[i]:=CHAR(number MOD 10+48);
    number:=number DIV 10;
  END;
  Text (rp,ADR(numStr),digits);
END DrawNumber;


PROCEDURE DrawLevel(level:LevelFile; shadow : BOOLEAN);

VAR
  x,y,xc,yc,num : INTEGER;
  player        : Modes;

BEGIN
  IF shadow THEN
    SetRast (brp,0);
  END;
  WITH level DO
    FOR y:=0 TO MaxY DO
      FOR x:=0 TO MaxX DO
        xc:=x*15+xOff;
        yc:=y*15+yOff;
        IF gorge IN field[x,y] THEN
          IF shadow THEN
            BlitObject(obGorge,brp,xc+5,yc+5);
          ELSE
            BlitObject(obGorge2,brp,xc,yc);
          END;
        ELSE
          IF shadow THEN
            BlitObject(obShadow,brp,xc+5,yc+5);
          END;
          BlitObject(obPlatform[dest IN field[x,y],ball IN field[x,y]],
                     brp,xc,yc);
        END;
      END;
    END;
  END;
  IF shadow THEN
    Move (brp,28,232);
    Text (brp,ADR("Level"),5);
    Move (brp,116,232);
    Text (brp,ADR("Steps"),5);
    Move (brp,212,232);
    Text (brp,ADR("Time"),4);
    Move (brp,268,232);
    Text (brp,ADR(": "),1);
  ELSE
    SetAPen (brp,7);
    SetBPen (brp,0);
    Move (brp,10,8);
    Text (brp,ADR("LevelEd"),7);
    BlitObject(obGorge2,brp,10,225);
    BlitObject(obPlatform[FALSE,FALSE],brp,30,225);
    BlitObject(obMan[FALSE,0],brp,50,225);
    BlitObject(obPlatform[TRUE,TRUE],brp,70,225);
    BlitObject(obPlatform[TRUE,FALSE],brp,90,225);
    BlitObject(obPlatform[FALSE,TRUE],brp,110,225);
    BlitObject(obMan[FALSE,1],brp,130,225);
    DrawButton (150,225,36,15,2,11,"Save");
    DrawButton (190,225,36,15,2,11,"Load");
    DrawButton (230,225,36,15,2,11,"Mode");
    DrawButton (270,225,36,15,2,11,"Quit");
    FOR player:=0 TO 1 DO
      WITH level.man[player] DO
        IF (man1 IN level.field[x,y]) THEN
          BlitObject (obMan[dest IN level.field[x,y],player],brp,
                      INTEGER(x)*15+xOff,INTEGER(y)*15+yOff);
        END;
      END;
    END;
  END;
  num:=VBeamPos() DIV 128;
  IF num>3 THEN
    num:=3;
  END;
  ChangeCMap(num);
END DrawLevel;


PROCEDURE DrawButton (x,y,w,h   : INTEGER;
                      xOff,yOff : INTEGER;
                      text      : ARRAY OF CHAR);

BEGIN
  DEC (w);
  DEC (h);
  SetAPen (brp,5);
  SetBPen (brp,5);
  RectFill (brp,x,y,x+w,y+h);
  SetAPen (brp,1);
  Move (brp,x+w,y);
  Draw (brp,x+w,y+h);
  Draw (brp,x,y+h);
  SetAPen (brp,2);
  Draw (brp,x,y);
  Draw (brp,x+w,y);
  SetAPen (brp,7);
  Move (brp,x+xOff,y+yOff);
  Text (brp,ADR(text),HIGH(text)+1);
  SetAPen (brp,31);
  SetBPen (brp,0);
END DrawButton;


PROCEDURE ShowTop5 (top5         : Top5;
                    currentLevel : LevelNumber;
                    pos          : INTEGER);

VAR
  x,col : INTEGER;

BEGIN
  SetAPen (rp,0);
  RectFill (rp,10,75,300,165);
  SetAPen (rp,30);
  Move (rp,20,87);
  Text (rp,ADR("Top5 Level "),10);
  DrawNumber (currentLevel,108,87,30,3);
  FOR x:=0 TO 4 DO
    col:=22;
    IF x=pos THEN
      col:=28;
    END;
    SetAPen (rp,col);
    Move (rp,20,100+x*10);
    Text (rp,ADR(top5[x].name),20);
    DrawNumber (top5[x].moves,204,100+x*10,col,4);
    DrawNumber (top5[x].mins,255,100+x*10,col,2);
    Text (rp,ADR(": "),1);
    DrawNumber (top5[x].secs,279,100+x*10,col,2);
  END;
  SetAPen (rp,28);
  Move (rp,20,160);
  CASE pos OF
  | 0..4 : Text (rp,ADR("Well Done!"),10);
  | 5    : Text (rp,ADR("You did it!"),11);
  | 6    : Text (rp,ADR("Loser ...!"),10);
  END;
  Delay (20);
  REPEAT
  UNTIL Fire(1) OR Fire(0);
END ShowTop5;


PROCEDURE DrawCtrl (ctrl : Controllertype);

BEGIN
  Move (rp,30,MenuLine+8);
  CASE ctrl OF
  | key : Text (rp,ADR("Key"),3);
  | joy : Text (rp,ADR("Joy"),3);
  END;
END DrawCtrl;


PROCEDURE DrawMenuLine (mode         : Modes;
                        currentLevel : LevelNumber;
                        ctrl         : Controllertype);

BEGIN
  SetAPen (rp,31);
  Move (rp,206,MenuLine);
  Text (rp,ADR("Editor"),6);
  Move (rp,86,MenuLine);
  Text (rp,ADR("Player"),6);
  Move (rp,28,MenuLine);
  Text (rp,ADR("Ctrl"),4);
  IF mode=1 THEN
    DrawCtrl (ctrl);
  END;
  Move (rp,268,MenuLine);
  Text (rp,ADR("Quit"),4);
  Move (rp,98,MenuLine+8);
  IF mode=0 THEN
    Text (rp,ADR("One"),3);
  ELSE
    Text (rp,ADR("Two"),3);
  END;
  Move (rp,152,MenuLine);
  Text (rp,ADR("Level"),5);
  DrawNumber(currentLevel,160,MenuLine+8,31,3);
END DrawMenuLine;


BEGIN
  cset[0]:=2; cset[1]:=12; cset[2]:=13;
  cset[3]:=1; cset[4]:=5; cset[5]:=6;
  cset[6]:=1; cset[7]:=9; cset[8]:=10;
  cset[9]:=7; cset[10]:=13; cset[11]:=7;
  cset[12]:=1; cset[13]:=6; cset[14]:=1;
  cset[15]:=1; cset[16]:=10; cset[17]:=2;
  cset[18]:=13; cset[19]:=11; cset[20]:=2;
  cset[21]:=6; cset[22]:=5; cset[23]:=1;
  cset[24]:=10; cset[25]:=8; cset[26]:=1;
  cset[27]:=5; cset[28]:=11; cset[29]:=15;
  cset[30]:=1; cset[31]:=6; cset[32]:=8;
  cset[33]:=4; cset[34]:=8; cset[35]:=11;
  cset[36]:=14; cset[37]:=14; cset[38]:=14;
  cset[39]:=5; cset[40]:=5; cset[41]:=5;
  cset[42]:=10; cset[43]:=10; cset[44]:=10;
  sp:=NIL;
  bsp:=NIL;
  wp:=NIL;
  bwp:=NIL;
  InitSprites;
  OpenScreens;
CLOSE
  ClearPointer(wp);
  ClearPointer(bwp);
  Deallocate(mouse);
  IF wp#NIL THEN
    CloseWindow(wp);
    wp:=NIL;
  END;
  IF bwp#NIL THEN
    CloseWindow(bwp);
    bwp:=NIL;
  END;
  IF sp#NIL THEN
    CloseScreen(sp);
    sp:=NIL;
  END;
  IF bsp#NIL THEN
    CloseScreen(bsp);
    bsp:=NIL;
  END;
  FOR player:=0 TO 1 DO
    IF s1[player]#-1 THEN
      FreeSprite(s1[player]);
      s1[player]:=-1;
    END;
    IF s2[player]#-1 THEN
      FreeSprite(s2[player]);
      s2[player]:=-1;
    END;
    IF s3[player]#-1 THEN
      FreeSprite(s3[player]);
      s3[player]:=-1;
    END;
  END;
  IF pointer#-1 THEN
    FreeSprite(pointer);
    pointer:=-1;
  END;
END GraphicSupport.
