(******************************************************************************

    :Program.    InputControl.mod
    :Contents.   joystick and keyboard controlling procedures
    :Version.    0.2
    :Date.       29.10.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    17.10.91 --- 0.1 --- created
                 29.10.91 --- 0.2 --- SetStatus now GetMsg's everytime it is
                                      called

******************************************************************************)
IMPLEMENTATION MODULE InputControl;
(*$ Volatile:=FALSE *)


(*====== Import from project Pushem =========================================*)
FROM Definitions IMPORT
  Direction,Modes,NameString;

FROM GraphicSupport IMPORT
  brp,bwp,ChangeScreens,rp,wp;

IMPORT Joystick;

(*====== Import from standard library =======================================*)
FROM ExecL IMPORT
  GetMsg,ReplyMsg;

FROM GraphicsD IMPORT
  DrawModes,DrawModeSet,jam2;

FROM GraphicsL IMPORT
  Move,SetAPen,SetDrMd,SetRast,Text;

FROM InputEvent IMPORT
  Qualifiers,QualifierSet,upPrefix;

FROM IntuitionD IMPORT
  IDCMPFlags,IDCMPFlagSet,IntuiMessagePtr;

FROM IntuitionL IMPORT
  ModifyIDCMP;

FROM SYSTEM IMPORT
  ADR;


VAR
  esc  : BOOLEAN;
  fire : ARRAY Modes OF BOOLEAN;
  dir  : ARRAY Modes OF Direction;
  ctrl : Controllertype;



PROCEDURE SetStatus;
(*$ NilChk:=FALSE *)

VAR
  msg     : IntuiMessagePtr;
  code    : CARDINAL;
  qual    : QualifierSet;
  oldFire : BOOLEAN;
  oldDir  : Direction;

BEGIN
  msg:=GetMsg(wp^.userPort);
  IF msg#NIL THEN
    code:=msg^.code;
    qual:=msg^.qualifier;
    ReplyMsg (msg);
    IF NOT (repeat IN qual) THEN
      IF code=045H+upPrefix THEN
        esc:=TRUE;
      ELSIF ctrl=key THEN
        oldFire:=fire[1];
        oldDir:=dir[1];
        CASE code OF
        | 04CH          : oldDir:=north;
        | 04DH          : oldDir:=south;
        | 04EH          : oldDir:=east;
        | 04FH          : oldDir:=west;
        | 040H          : oldFire:=TRUE;
        | 04CH+upPrefix : IF oldDir=north THEN
                            oldDir:=noDir;
                          END;
        | 04DH+upPrefix : IF oldDir=south THEN
                            oldDir:=noDir;
                          END;
        | 04EH+upPrefix : IF oldDir=east THEN
                            oldDir:=noDir;
                          END;
        | 04FH+upPrefix : IF oldDir=west THEN
                            oldDir:=noDir;
                          END;
        | 040H+upPrefix : IF oldFire THEN
                            oldFire:=FALSE;
                          END;
        ELSE
        END;
        dir[1]:=oldDir;
        fire[1]:=oldFire;
      END;
    END;
  END;
  (*$ StackParms:=FALSE *)
  dir[0]:=Joystick.Joy(1);
  fire[0]:=Joystick.Fire(1);
  IF ctrl#key THEN
    dir[1]:=Joystick.Joy(0);
    fire[1]:=Joystick.Fire(0);
  END;
  (*$ POP StackParms  POP NilChk *)
END SetStatus;


PROCEDURE ResetController (controller : Controllertype);

VAR
  player  : Modes;
  message : IntuiMessagePtr;

BEGIN
  esc:=FALSE;
  FOR player:=0 TO 1 DO
    fire[player]:=FALSE;
    dir[player]:=noDir;
  END;
  message:=GetMsg(wp^.userPort);
  WHILE (message#NIL) DO
    ReplyMsg (message);
    message:=GetMsg(wp^.userPort);
  END;
  ctrl:=controller;
END ResetController;


PROCEDURE GetDir (player : Modes) : Direction;

BEGIN
  SetStatus;
  RETURN (dir[player]);
END GetDir;


PROCEDURE Fire (player : Modes) : BOOLEAN;

BEGIN
  SetStatus;
  RETURN (fire[player]);
END Fire;


PROCEDURE Escape () : BOOLEAN;

BEGIN
  IF esc THEN
    esc:=FALSE;
    RETURN (TRUE);
  ELSE
    RETURN (FALSE);
  END;
END Escape;


PROCEDURE GetName (VAR name : NameString);

CONST
  xBase = 80;
  yBase = 100;
  cursor = "  ";

VAR
  message : IntuiMessagePtr;
  code    : CARDINAL;
  class   : IDCMPFlagSet;
  length  : INTEGER;

BEGIN
  ModifyIDCMP (bwp,IDCMPFlagSet{vanillaKey});
  SetRast (brp,0);
  SetAPen (brp,31);
  length:=19;
  WHILE (name[length]=" ") AND (length>0) DO
    DEC (length);
  END;
  IF length#0 THEN
    INC (length);
  END;
  Move (brp,100,60);
  Text (brp,ADR("Enter your name"),15);
  Move (brp,32,80);
  Text (brp,ADR("Press RETURN or FIRE to continue"),32);
  Move (brp,xBase,yBase);
  Text (brp,ADR(name),length);
  SetDrMd (brp,DrawModeSet{inversvid}+jam2);
  Text (brp,ADR(cursor),1);
  SetDrMd (brp,jam2);
  ChangeScreens (TRUE);
  REPEAT
    message:=GetMsg(wp^.userPort);
    IF (message#NIL) THEN
      ReplyMsg (message);
      code:=message^.code;
      class:=message^.class;
      IF vanillaKey IN class THEN
        CASE code OF
        | 32..126 : IF length<SIZE(name) THEN
                      name[length]:=CHAR(code);
                      INC (length);
                    END;
        | 8       : IF length>0 THEN
                      DEC (length);
                    END;
                    name[length]:=CHAR(32);
        ELSE
        END;
        Move (rp,xBase,yBase);
        Text (rp,ADR(name),length);
        SetDrMd (rp,DrawModeSet{inversvid}+jam2);
        Text (rp,ADR(cursor),1);
        SetDrMd (rp,jam2);
        Text (rp,ADR(cursor),1);
      END;
    END;
  UNTIL (code=13) OR Joystick.Fire(0) OR Joystick.Fire(1);
  ModifyIDCMP (wp,IDCMPFlagSet{rawKey});
END GetName;


END InputControl.
