(******************************************************************************

    :Program.    LevelCopy.mod
    :Contents.   program to copy and sort Push'em levels
    :Version.    0.1
    :Date.       26.10.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    25.10.91 --- 0.1 --- created
    :History.    13.11.91 --- 0.2 --- fixed bug in LoadLevel
                                      LevelLoader.LoadLevel is better !

******************************************************************************)
MODULE LevelCopy;
(*$ LargeVars:=FALSE  Volatile:=FALSE *)


FROM Arts IMPORT
  Assert,Requester;

FROM DosL IMPORT
  Delay;

FROM ExecL IMPORT
  CopyMem,GetMsg,ReplyMsg,WaitPort;

FROM FileSystem IMPORT
  Close,Delete,File,Lookup,ReadByteBlock,ReadBytes,Response,WriteByteBlock;

FROM GraphicsD IMPORT
  DrawModes,DrawModeSet,FontFlags,FontFlagSet,FontStyleSet,jam2,RastPortPtr,
  TextAttr;

FROM GraphicsL IMPORT
  Move,RectFill,SetAPen,SetDrMd,Text;

FROM Heap IMPORT
  AllocMem,Deallocate;

FROM IntuitionD IMPORT
  ActivationFlags,ActivationFlagSet,boolGadget,Border,Gadget,GadgetPtr,
  gadgHComp,gadgHNone,IDCMPFlags,IDCMPFlagSet,IntuiMessagePtr,IntuiText,
  Image,maxBody,maxPot,NewWindow,propGadget,PropInfo,PropInfoFlags,
  PropInfoFlagSet,ScreenFlags,ScreenFlagSet,strGadget,StringInfo,
  WindowFlags,WindowFlagSet,WindowPtr;

FROM IntuitionL IMPORT
  CloseWindow,DrawImage,ModifyProp,OpenWindow;

FROM String IMPORT
  Concat,ConcatChar,Copy,Length;

FROM SYSTEM IMPORT
  ADDRESS,ADR,LONGSET;

FROM Definitions IMPORT
  LevelFile,MaxX,MaxY,Modes,Status,StatusSet;


TYPE
  Str255=ARRAY [0..255] OF CHAR;
  XYArray=ARRAY [0..9] OF INTEGER;
  PropGad=RECORD
    gad  : Gadget;
    info : PropInfo;
    img  : Image;
  END;
  BoolGad=RECORD
    gad    : Gadget;
    border : Border;
    xy     : XYArray;
    text   : IntuiText;
  END;
  StringGad=RECORD
    gad    : Gadget;
    border : Border;
    xy     : XYArray;
    info   : StringInfo;
    buffer : Str255;
    undo   : Str255;
  END;

  LevelFilePtr=POINTER TO LevelFile;
  Level=RECORD
    oldNum    : INTEGER;      (* -1 = don't know old level number *)
    levelFile : LevelFilePtr;
  END;
  ExtLevel=RECORD
    copied : BOOLEAN;
  END;

  ImgDataPtr=POINTER TO ImgData;
  ImgData=ARRAY [0..3] OF LONGCARD;


VAR
  (* global VARs *)
  win           : WindowPtr;
  rp            : RastPortPtr;
  gadgets       : RECORD
                    prop : ARRAY [0..1] OF PropGad;
                    bool : ARRAY [0..8] OF BoolGad;
                    str  : StringGad;
                  END;
  topaz8        : TextAttr;
  msg           : IntuiMessagePtr;
  class         : IDCMPFlagSet;
  mouseY        : INTEGER;
  gad           : GadgetPtr;
  level         : LevelFile;
  intMode       : Modes;
  intLevels     : ARRAY [0..999] OF Level;
  extLevels     : ARRAY [0..999] OF ExtLevel;
  intSelected   : INTEGER;
  extSelected   : INTEGER;
  intNumEntries : CARDINAL;
  extNumEntries : CARDINAL;
  i             : INTEGER;
  img           : Image;
  imgMan,imgGorge,imgPlatform,imgDest,imgBall,imgDestBall : ImgDataPtr;


PROCEDURE GetPropGadget(VAR gad     : PropGad;
                        gadID       : INTEGER;
                        xpos,ypos   : INTEGER;
                        xsize,ysize : INTEGER);

VAR
  adr : LONGCARD;

BEGIN
  adr:=LONGCARD(ADR(gad));
  WITH gad.gad DO
    nextGadget:=ADDRESS(adr+SIZE(PropGad));
    leftEdge:=xpos ; topEdge:=ypos;
    width:=xsize   ; height:=ysize;
    flags:=gadgHComp;
    activation:=ActivationFlagSet{relVerify};
    gadgetType:=propGadget;
    gadgetRender:=adr+SIZE(Gadget)+SIZE(PropInfo);
    selectRender:=NIL;
    gadgetText:=NIL;
    mutualExclude:=LONGSET{};
    specialInfo:=adr+SIZE(Gadget);
    gadgetID:=gadID;
    userData:=NIL;
  END;
  WITH gad.info DO
    flags:=PropInfoFlagSet{autoKnob,freeVert};
    horizPot:=0  ; vertPot:=0;
    horizBody:=0 ; vertBody:=maxBody DIV 100;
  END;
END GetPropGadget;


PROCEDURE GetBoolGadget(VAR gad     : BoolGad;
                        gadID       : INTEGER;
                        gadText     : ADDRESS;
                        textLen     : INTEGER;
                        xpos,ypos   : INTEGER;
                        xsize,ysize : INTEGER);

VAR
  adr : LONGCARD;

BEGIN
  adr:=LONGCARD(ADR(gad));
  WITH gad.gad DO
    nextGadget:=ADDRESS(adr+SIZE(BoolGad));
    leftEdge:=xpos ; topEdge:=ypos;
    width:=xsize   ; height:=ysize;
    flags:=gadgHComp;
    activation:=ActivationFlagSet{relVerify};
    gadgetType:=boolGadget;
    gadgetRender:=adr+SIZE(Gadget);
    selectRender:=NIL;
    IF gadText#NIL THEN
      gadgetText:=ADDRESS(adr+SIZE(Gadget)+SIZE(Border)+SIZE(XYArray));
    ELSE
      gadgetText:=NIL;
    END;
    mutualExclude:=LONGSET{};
    specialInfo:=NIL;
    gadgetID:=gadID;
    userData:=NIL;
  END;
  WITH gad.border DO
    leftEdge:=0; topEdge:=0;
    frontPen:=1; backPen:=0;
    drawMode:=jam2;
    count:=5;
    xy:=adr+SIZE(Gadget)+SIZE(Border);
    nextBorder:=NIL;
  END;
  WITH gad DO
    xy[0]:=0       ; xy[1]:=0;
    xy[2]:=xsize-1 ; xy[3]:=0;
    xy[4]:=xsize-1 ; xy[5]:=ysize-1;
    xy[6]:=0       ; xy[7]:=ysize-1;
    xy[8]:=0       ; xy[9]:=0;
  END;
  IF gadText#NIL THEN
    WITH gad.text DO
      frontPen:=1; backPen:=0;
      drawMode:=jam2;
      leftEdge:=(72-textLen*8) DIV 2;
      topEdge:=2;
      iTextFont:=ADR(topaz8);
      iText:=gadText;
      nextText:=NIL;
    END;
  END;
END GetBoolGadget;


PROCEDURE GetStringGadget(VAR gad     : StringGad;
                          gadID       : INTEGER;
                          xpos,ypos   : INTEGER;
                          xsize,ysize : INTEGER);

VAR
  adr : LONGCARD;

BEGIN
  adr:=LONGCARD(ADR(gad));
  WITH gad.gad DO
    nextGadget:=NIL; (* this is the last gadget!!! *)
    leftEdge:=xpos+2 ; topEdge:=ypos+2;
    width:=xsize-4   ; height:=ysize-4;
    flags:=gadgHComp;
    activation:=ActivationFlagSet{relVerify};
    gadgetType:=strGadget;
    gadgetRender:=adr+SIZE(Gadget);
    selectRender:=NIL;
    gadgetText:=NIL;
    mutualExclude:=LONGSET{};
    specialInfo:=adr+SIZE(Gadget)+SIZE(Border)+SIZE(XYArray);
    gadgetID:=gadID;
    userData:=NIL;
  END;
  WITH gad.border DO
    leftEdge:=0; topEdge:=0;
    frontPen:=1; backPen:=0;
    drawMode:=jam2;
    count:=5;
    xy:=adr+SIZE(Gadget)+SIZE(Border);
    nextBorder:=NIL;
  END;
  WITH gad DO
    xy[0]:=-2      ; xy[1]:=-2;
    xy[2]:=xsize-3 ; xy[3]:=-2;
    xy[4]:=xsize-3 ; xy[5]:=ysize-3;
    xy[6]:=-2      ; xy[7]:=ysize-3;
    xy[8]:=-2      ; xy[9]:=-2;
  END;
  adr:=adr+SIZE(Gadget)+SIZE(Border)+SIZE(XYArray);
  WITH gad.info DO
    buffer:=adr+SIZE(StringInfo);
    undoBuffer:=adr+SIZE(StringInfo)+SIZE(Str255);
    bufferPos:=0 ; maxChars:=255;
    dispPos:=0   ; undoPos:=0;
    numChars:=0  ; dispCount:=0;
    cLeft:=0     ; cTop:=0;
    extension:=NIL;
    longInt:=0;
    altKeyMap:=NIL;
  END;
END GetStringGadget;


PROCEDURE WriteNum(num: INTEGER);

VAR
  ch : CHAR;

BEGIN
  IF (num>=0) AND (num<=999) THEN
    ch:=CHR(num DIV 100+48);
    Text(rp,ADR(ch),1);
    num:=num MOD 100;
    ch:=CHR(num DIV 10+48);
    Text(rp,ADR(ch),1);
    num:=num MOD 10;
    ch:=CHR(num+48);
    Text(rp,ADR(ch),1);
  ELSE
    Text(rp,ADR("   "),3);
  END;
END WriteNum;


PROCEDURE SetWindow;

VAR
  nw : NewWindow;

BEGIN
  WITH topaz8 DO
    name:=ADR("topaz.font");
    ySize:=8;
    style:=FontStyleSet{};
    flags:=FontFlagSet{romFont}
  END;
  GetPropGadget(gadgets.prop[0],1,81,28,16,92);
  GetPropGadget(gadgets.prop[1],2,239,28,16,92);
  GetBoolGadget(gadgets.bool[0],3,NIL,0,8,28,64,92);
  gadgets.bool[0].gad.flags:=gadgHNone;
  gadgets.bool[0].gad.activation:=ActivationFlagSet{gadgImmediate};
  GetBoolGadget(gadgets.bool[1],4,NIL,0,264,28,80,92);
  gadgets.bool[1].gad.flags:=gadgHNone;
  gadgets.bool[1].gad.activation:=ActivationFlagSet{gadgImmediate};
  GetBoolGadget(gadgets.bool[2],5,ADR("Add"),3,140,28,72,12);
  GetBoolGadget(gadgets.bool[3],6,ADR("Insert"),6,140,42,72,12);
  GetBoolGadget(gadgets.bool[4],7,ADR("Delete"),6,140,56,72,12);
  GetBoolGadget(gadgets.bool[5],8,ADR("Add all"),7,140,70,72,12);
  GetBoolGadget(gadgets.bool[6],9,ADR("Del all"),7,140,84,72,12);
  GetBoolGadget(gadgets.bool[7],10,ADR("Write"),5,140,101,72,12);
  GetBoolGadget(gadgets.bool[8],11,ADR("Append"),6,140,116,72,12);
  GetStringGadget(gadgets.str,12,8,13,204,12);
  WITH nw DO
    leftEdge:=144 ; topEdge:=2;
    width:=352    ; height:=195;
    detailPen:=0  ; blockPen:=1;
    idcmpFlags:=IDCMPFlagSet{closeWindow,gadgetDown,gadgetUp};
    flags:=WindowFlagSet{windowDrag,windowDepth,windowClose,activate};
    firstGadget:=ADR(gadgets);
    checkMark:=NIL;
    title:=ADR("Push'em LevelCopy");
    screen:=NIL   ; bitMap:=NIL;
    minWidth:=352 ; minHeight:=195;
    maxWidth:=352 ; maxHeight:=195;
    type:=ScreenFlagSet{wbenchScreen};
  END;
  win:=OpenWindow(nw);
  Assert(win#NIL,ADR("Can't open window"));
  rp:=win^.rPort;
  SetDrMd(rp,jam2);
  SetAPen(rp,1);
  Move(rp,222,21);
  Text(rp,ADR("Internal Memory"),15);
END SetWindow;


PROCEDURE InitImages;

BEGIN
  AllocMem(imgGorge,SIZE(ImgData),TRUE);
  Assert(imgGorge#NIL,ADR("Not enough chip mem"));
  imgGorge^[0]:=0;
  imgGorge^[1]:=0;
  imgGorge^[2]:=0;
  imgGorge^[3]:=0;
  AllocMem(imgPlatform,SIZE(ImgData),TRUE);
  Assert(imgPlatform#NIL,ADR("Not enough chip mem"));
  imgPlatform^[0]:=00000000000000000000000000000000L;
  imgPlatform^[1]:=00000000000000000000000000000000L;
  imgPlatform^[2]:=11111111000000001111111100000000L;
  imgPlatform^[3]:=11111111000000001111111100000000L;
  AllocMem(imgMan,SIZE(ImgData),TRUE);
  Assert(imgMan#NIL,ADR("Not enough chip mem"));
  imgMan^[0]:=00000000000000000111111000000000L;
  imgMan^[1]:=01111110000000000000000000000000L;
  imgMan^[2]:=11111111000000001001100100000000L;
  imgMan^[3]:=10011001000000001111111100000000L;
  AllocMem(imgDest,SIZE(ImgData),TRUE);
  Assert(imgDest#NIL,ADR("Not enough chip mem"));
  imgDest^[0]:=01111110000000001100001100000000L;
  imgDest^[1]:=11000011000000000111111000000000L;
  imgDest^[2]:=11111111000000001111111100000000L;
  imgDest^[3]:=11111111000000001111111100000000L;
  AllocMem(imgBall,SIZE(ImgData),TRUE);
  Assert(imgBall#NIL,ADR("Not enough chip mem"));
  imgBall^[0]:=00011000000000000111111000000000L;
  imgBall^[1]:=01111110000000000001100000000000L;
  imgBall^[2]:=11100111000000001000000100000000L;
  imgBall^[3]:=10000001000000001110011100000000L;
  AllocMem(imgDestBall,SIZE(ImgData),TRUE);
  Assert(imgDestBall#NIL,ADR("Not enough chip mem"));
  imgDestBall^[0]:=01111110000000001111111100000000L;
  imgDestBall^[1]:=11111111000000000111111000000000L;
  imgDestBall^[2]:=11100111000000001000000100000000L;
  imgDestBall^[3]:=10000001000000001110011100000000L;
  WITH img DO
    leftEdge:=0  ; topEdge:=0;
    width:=8     ; height:=4;
    depth:=2     ; imageData:=imgGorge;
    planePick:=3 ; planeOnOff:=0;
    nextImage:=NIL;
  END;
END InitImages;


PROCEDURE UpdateScrollBar(gadget     : PropGad;
                          numEntries : CARDINAL);

VAR
  vBody : CARDINAL;

BEGIN
  WITH gadget.info DO
    IF numEntries>10 THEN
      vBody:=CARDINAL(LONGCARD(maxBody)*10 DIV numEntries);
    ELSE
      vBody:=maxBody;
    END;
    ModifyProp(ADR(gadget.gad),win,NIL,flags,0,vertPot,0,vBody);
  END;
END UpdateScrollBar;


PROCEDURE GetTop(gadget     : PropGad;
                 numEntries : CARDINAL): CARDINAL;

BEGIN
  IF numEntries>10 THEN
    RETURN(CARDINAL(LONGCARD(gadget.info.vertPot)*(numEntries-10)
                    DIV maxPot));
  ELSE
    RETURN(0);
  END;
END GetTop;


PROCEDURE ShowInternals;

VAR
  i      : CARDINAL;
  intTop : CARDINAL;

BEGIN
  intTop:=GetTop(gadgets.prop[1],intNumEntries);
  FOR i:=intTop TO intTop+9 DO
    Move(rp,268,36+9*(i-intTop));
    IF i<intNumEntries THEN
      IF intSelected=INTEGER(i) THEN
        SetDrMd(rp,jam2+DrawModeSet{inversvid});
      END;
      WriteNum(i);
      IF intLevels[i].oldNum>=0 THEN
        Text(rp,ADR(" ("),2);
        WriteNum(intLevels[i].oldNum);
        Text(rp,ADR(") "),1);
      ELSE
        Text(rp,ADR(" ( * )"),6);
      END;
      IF intSelected=INTEGER(i) THEN
        SetDrMd(rp,jam2);
      END;
    ELSE
      Text(rp,ADR("         "),9);
    END;
  END;
END ShowInternals;


PROCEDURE ShowExternals;

VAR
  i      : CARDINAL;
  extTop : CARDINAL;

BEGIN
  extTop:=GetTop(gadgets.prop[0],extNumEntries);
  FOR i:=extTop TO extTop+9 DO
    Move(rp,12,36+9*(i-extTop));
    IF i<extNumEntries THEN
      IF extSelected=INTEGER(i) THEN
        SetDrMd(rp,jam2+DrawModeSet{inversvid});
      END;
      WriteNum(i);
      IF extLevels[i].copied THEN
        Text(rp,ADR("  * "),4);
      ELSE
        Text(rp,ADR("    "),4);
      END;
      IF extSelected=INTEGER(i) THEN
        SetDrMd(rp,jam2);
      END;
    ELSE
      Text(rp,ADR("       "),7);
    END;
  END;
END ShowExternals;


PROCEDURE CreateLevelString(VAR str : Str255;
                            num     : INTEGER);

VAR
  len    : INTEGER;
  numStr : ARRAY [0..3] OF CHAR;

BEGIN
  Copy(str,gadgets.str.buffer);
  len:=Length(str);
  IF len>0 THEN
    IF (str[len-1]#":") AND (str[len-1]#"/") THEN
      ConcatChar(str,"/");
    END;
  END;
  IF num>=0 THEN
    Concat(str,"Level");
    numStr[0]:=CHR(num DIV 100 + 48);
    numStr[1]:=CHR(num MOD 100 DIV 10 + 48);
    numStr[2]:=CHR(num MOD 10 + 48);
    numStr[3]:=0C;
    Concat(str,numStr);
  ELSE
    Concat(str,"MaxLevel");
  END;
END CreateLevelString;


PROCEDURE LoadLevel(VAR num : INTEGER);

VAR
  f      : File;
  name   : Str255;
  len    : INTEGER;
  numStr : ARRAY [0..3] OF CHAR;
  dummy  : LONGINT;

BEGIN
  CreateLevelString(name,num);
  Lookup(f,name,512,FALSE);
  ReadBytes (f,ADR(level),SIZE(level),dummy);
(* Besser an LevelLoader.LoadLevel orientieren, damit Datensicherheit
   gewhrleistet bleibt. Alte Version hat alte Levelformate gelesen
   und geschrieben und somit die Datensicherheit gefhrdet ! (Die alten
   Levelformate sind fr Pushem absolut unbrauchbar und gehren deshalb
   nicht gelesen oder geschrieben)
 *)
  IF f.res#done THEN
    num:=-1;
  END;
  Close(f);
END LoadLevel;


PROCEDURE ReadNumLevels(VAR numLevels : CARDINAL);

VAR
  f      : File;
  name   : Str255;
  len    : INTEGER;
  numStr : ARRAY [0..3] OF CHAR;

BEGIN
  CreateLevelString(name,-1);
  Lookup(f,name,512,FALSE);
  ReadByteBlock(f,numLevels);
  IF f.res#done THEN
    numLevels:=0;
  ELSE
    INC(numLevels);
  END;
  Close(f);
END ReadNumLevels;


PROCEDURE WriteLevels;

VAR
  str : Str255;
  i   : INTEGER;
  f   : File;

BEGIN
  Copy(str,gadgets.str.buffer);
  Concat(str," ?");
  IF Requester(ADR("Overwrite all levels in"),
               ADR(str),ADR("Write"),ADR("Cancel")) THEN
    FOR i:=0 TO INTEGER(extNumEntries)-1 DO
      CreateLevelString(str,i);
      Lookup(f,str,512,FALSE);
      Delete(f);
      Close(f);
    END;
    FOR i:=0 TO INTEGER(intNumEntries)-1 DO
      CreateLevelString(str,i);
      Lookup(f,str,512,TRUE);
      WriteByteBlock(f,intLevels[i].levelFile^);
      Close(f);
    END;
    CreateLevelString(str,-1);
    Lookup(f,str,512,TRUE);
    i:=intNumEntries-1;
    WriteByteBlock(f,i);
    Close(f);
  END;
END WriteLevels;


PROCEDURE AppendLevels;

VAR
  str : Str255;
  i   : INTEGER;
  f   : File;
  num : INTEGER;

BEGIN
  Copy(str,gadgets.str.buffer);
  Concat(str," ?");
  IF Requester(ADR("Append all levels to"),
               ADR(str),ADR("Append"),ADR("Cancel")) THEN

    FOR i:=0 TO INTEGER(intNumEntries)-1 DO
      num:=INTEGER(extNumEntries)+i;
      IF num<=999 THEN
        CreateLevelString(str,num);
        Lookup(f,str,512,TRUE);
        WriteByteBlock(f,intLevels[i].levelFile^);
        Close(f);
      END;
    END;
    CreateLevelString(str,-1);
    Lookup(f,str,512,TRUE);
    WriteByteBlock(f,num);
    Close(f);
  END;
END AppendLevels;


PROCEDURE DrawLevel(level     : LevelFile;
                    xOff,yOff : CARDINAL);

VAR
  x,y    : CARDINAL;
  status : StatusSet;
  adr    : ADDRESS;

BEGIN
  FOR x:=0 TO MaxX DO
    FOR y:=0 TO MaxY DO
      status:=level.field[x,y];
      IF gorge IN status THEN
        adr:=imgGorge;
      ELSIF man1 IN status THEN
        adr:=imgMan;
      ELSIF status=StatusSet{dest} THEN
        adr:=imgDest;
      ELSIF status=StatusSet{ball} THEN
        adr:=imgBall;
      ELSIF status=StatusSet{dest,ball} THEN
        adr:=imgDestBall;
      ELSE
        adr:=imgPlatform;
      END;
      img.imageData:=adr;
      DrawImage(rp,ADR(img),xOff+8*x,yOff+4*y);
    END;
  END;
END DrawLevel;


PROCEDURE SelectInternal(num : INTEGER);

BEGIN
  IF (num>=INTEGER(intNumEntries)) OR (num<0) THEN
    intSelected:=-1;
    Move(rp,272,131);
    Text(rp,ADR("         "),9);
    RectFill(rp,182,133,343,190);
  ELSE
    intSelected:=num;
    Move(rp,272,131);
    Text(rp,ADR("Level "),6);
    WriteNum(intSelected);
    DrawLevel(intLevels[intSelected].levelFile^,183,134);
  END;
END SelectInternal;


PROCEDURE SelectExternal(num : INTEGER);

BEGIN
  IF (num>=INTEGER(extNumEntries)) OR (num<0) THEN
    extSelected:=-1;
    Move(rp,8,131);
    Text(rp,ADR("         "),9);
    RectFill(rp,8,133,169,190);
  ELSE
    extSelected:=num;
    LoadLevel(extSelected);
    IF extSelected>=0 THEN
      Move(rp,8,131);
      Text(rp,ADR("Level "),6);
      WriteNum(extSelected);
      DrawLevel(level,9,134);
    ELSE
      Move(rp,8,131);
      Text(rp,ADR("         "),9);
      RectFill(rp,8,133,169,190);
    END;
  END;
END SelectExternal;


PROCEDURE AddLevel(level    : LevelFilePtr;
                   levelNum : CARDINAL);

BEGIN
  IF intNumEntries=0 THEN
    intMode:=level^.mode;
  END;
  IF (intNumEntries<999) AND (intMode=level^.mode) THEN
    WITH intLevels[intNumEntries] DO
      oldNum:=levelNum;
      AllocMem(levelFile,SIZE(LevelFile),FALSE);
      IF levelFile#NIL THEN
        CopyMem(level,levelFile,SIZE(LevelFile));
        INC(intNumEntries);
        SelectInternal(intNumEntries-1);
      END;
    END;
  END;
END AddLevel;


PROCEDURE InsertLevel(level    : LevelFilePtr;
                      levelNum : CARDINAL);

BEGIN
  IF (intNumEntries<999) AND (intSelected>=0) AND (intMode=level^.mode) THEN
    FOR i:=intNumEntries-1 TO intSelected BY -1 DO
      intLevels[i+1]:=intLevels[i];
    END;
    WITH intLevels[intSelected] DO
      AllocMem(levelFile,SIZE(LevelFile),FALSE);
      IF levelFile#NIL THEN
        oldNum:=levelNum;
        CopyMem(level,levelFile,SIZE(LevelFile));
        INC(intNumEntries);
        SelectInternal(intSelected);
      ELSE
        FOR i:=intSelected+1 TO intNumEntries-1 DO
          intLevels[i-1]:=intLevels[i];
        END;
      END;
    END;
  END;
END InsertLevel;


PROCEDURE DeleteActual;

VAR
  i : CARDINAL;

BEGIN
  IF intSelected>=0 THEN
    FOR i:=intSelected+1 TO intNumEntries-1 DO
      intLevels[i-1]:=intLevels[i];
    END;
    DEC(intNumEntries);
    IF intSelected>=INTEGER(intNumEntries) THEN
      SelectInternal(intSelected-1);
    ELSE
      SelectInternal(intSelected);
    END;
  END;
END DeleteActual;


PROCEDURE DelInternals;

VAR
  i : INTEGER;

BEGIN
  FOR i:=1 TO intNumEntries DO
    Deallocate(intLevels[i-1].levelFile);
  END;
  intNumEntries:=0;
  SelectInternal(-1);
END DelInternals;


PROCEDURE MouseYToNum(mouseY : INTEGER): INTEGER;

BEGIN
  mouseY:=(mouseY-29) DIV 9;
  IF mouseY<0 THEN
    RETURN(0);
  ELSIF mouseY>9 THEN
    RETURN(9);
  END;
  RETURN(mouseY);
END MouseYToNum;


PROCEDURE NewDir;

VAR
  i : INTEGER;

BEGIN
  ReadNumLevels(extNumEntries);
  SelectExternal(-1);
  FOR i:=0 TO INTEGER(extNumEntries)-1 DO
    extLevels[i].copied:=FALSE;
  END;
  FOR i:=0 TO INTEGER(intNumEntries)-1 DO
    intLevels[i].oldNum:=-1;
  END;
  UpdateScrollBar(gadgets.prop[0],extNumEntries);
  ShowExternals;
  ShowInternals;
END NewDir;


BEGIN
  InitImages;
  SetWindow;

  intNumEntries:=0;
  SelectInternal(-1);
  UpdateScrollBar(gadgets.prop[1],0);
  ShowInternals;

  extNumEntries:=0;
  SelectExternal(-1);
  UpdateScrollBar(gadgets.prop[0],0);
  ShowExternals;

  LOOP
    WaitPort(win^.userPort);
    msg:=GetMsg(win^.userPort);
    IF msg#NIL THEN
      class:=msg^.class;
      gad:=msg^.iAddress;
      mouseY:=msg^.mouseY;
      ReplyMsg(msg);
      IF class=IDCMPFlagSet{closeWindow} THEN
        EXIT;
      ELSIF class=IDCMPFlagSet{gadgetDown} THEN
        CASE gad^.gadgetID OF
        | 3: SelectExternal(MouseYToNum(mouseY)+
                            INTEGER(GetTop(gadgets.prop[0],extNumEntries)));
             ShowExternals;
        | 4: SelectInternal(MouseYToNum(mouseY)+
                            INTEGER(GetTop(gadgets.prop[1],intNumEntries)));

(* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

             SelectInternal(INTEGER(GetTop(gadgets.prop[1],intNumEntries))+
                            MouseYToNum(mouseY));

   Ist das ein Denkfehler von mir oder ein Bug von M2Amiga, da die obere
   Version funktioniert, diese aber nicht???
   -- INTEGER(GetTop(...))+MouseYToNum(...) liefert immer den doppelten Wert
      von MouseYToNum(...)+INTEGER(GetTop(...)) --

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! *)

             ShowInternals;
        END;
      ELSIF class=IDCMPFlagSet{gadgetUp} THEN
        CASE gad^.gadgetID OF
        | 1: ShowExternals;
        | 2: ShowInternals;
        | 5: IF extSelected>=0 THEN
               AddLevel(ADR(level),extSelected);
               extLevels[extSelected].copied:=TRUE;
               UpdateScrollBar(gadgets.prop[1],intNumEntries);
               ShowInternals;
               ShowExternals;
             END;
        | 6: IF extSelected>=0 THEN
               InsertLevel(ADR(level),extSelected);
               extLevels[extSelected].copied:=TRUE;
               UpdateScrollBar(gadgets.prop[1],intNumEntries);
               ShowInternals;
               ShowExternals;
             END;
        | 7: DeleteActual;
             UpdateScrollBar(gadgets.prop[1],intNumEntries);
             ShowInternals;
        | 8: FOR i:=0 TO INTEGER(extNumEntries)-1 DO
               SelectExternal(i);
               AddLevel(ADR(level),extSelected);
               extLevels[extSelected].copied:=TRUE;
               UpdateScrollBar(gadgets.prop[1],intNumEntries);
               ShowInternals;
               ShowExternals;
             END;
        | 9: DelInternals;
             UpdateScrollBar(gadgets.prop[1],0);
             ShowInternals;
        | 10: IF intNumEntries>0 THEN
                WriteLevels;
                NewDir;
              END;
        | 11: IF intNumEntries>0 THEN
                AppendLevels;
                NewDir;
              END;
        | 12: NewDir;
        END;
      END;
    END;
  END;

CLOSE
  IF win#NIL THEN
    CloseWindow(win);
    win:=NIL;
  END;
END LevelCopy.
