(******************************************************************************

    :Program.    LevelEd.mod
    :Contents.   leveleditor for game
    :Version.    0.3
    :Date.       28.11.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    18.10.91 --- 0.1 --- created
                 13.11.91 --- 0.2 --- all bugs fixed (from bug report 1.11.91,
                                      Peter Groth)
                 28.11.91 --- 0.3 --- change mode bug fixed
                 07.12.91 --- 0.4 --- all bugs fixed (from bug report 28.11.91,
                                      Peter Groth)

******************************************************************************)
IMPLEMENTATION MODULE LevelEd;


(*====== Import from project Pushem =========================================*)
FROM Definitions IMPORT
  Direction,LevelFile,MaxX,MaxY,Modes,NameString,Status,StatusSet;

FROM Game IMPORT
  PlayGame;

FROM GraphicSupport IMPORT
  BlitObject,brp,ChangeScreens,DrawButton,DrawLevel,HideSprites,obGorge2,obMan,
  obPlatform,pointerSpr,rp,spMan,spr1,spr2,spr3,vp,xOff,yOff;

FROM InputControl IMPORT
  Controllertype,Escape,Fire,GetDir,ResetController;

FROM LevelLoader IMPORT
  LevelNumber,LoadLevel,RequestLevel,SaveLevel,SetMode;

(*====== Import from standard library =======================================*)
FROM DosL IMPORT
  Delay;

FROM GraphicsD IMPORT
  RastPortPtr;

FROM GraphicsL IMPORT
  ChangeSprite,Draw,Move,MoveSprite,SetAPen,SetBPen,SetRast,Text;

FROM SYSTEM IMPORT
  ADDRESS,ADR;
(* debug *)
FROM Arts IMPORT
  BreakPoint;


TYPE
  Buttons = (sel0,  (* gorge button        *)
             sel1,  (* freeposition button *)
             sel2,  (* man1 button         *)
             sel3,  (* ball on dest button *)
             sel4,  (* dest button         *)
             sel5,  (* ball button         *)
             sel6,  (* man2 button         *)
             save,  (* save button         *)
             load,  (* load button         *)
             mode,  (* mode button         *)
             quit); (* quit button         *)
   ModeButtons = (player, (* one or two player mode  *)
                  play,   (* play current level mode *)
                  move,   (* move men mode           *)
                  reset,  (* reset level             *)
                  cancel);(* cancel menu             *)
   PointerData = RECORD
                   xpos          : INTEGER;   (* x-coordinate in window    *)
                   ypos          : INTEGER;   (* y-coordinate in window    *)
                   menu          : BOOLEAN;   (* in menu = TRUE            *)
                   button        : Buttons;   (* button in menu            *)
                   oldbutton     : Buttons;   (* old selected button       *)
                   x             : [0..MaxX]; (* x-coordinate in playfield *)
                   y             : [0..MaxY]; (* y-coordinate in playfield *)
                   selectedItems : StatusSet; (* items for playfield       *)
                 END;


PROCEDURE ResetLevel (VAR level : LevelFile);

VAR
  x,y    : INTEGER;
  player : Modes;

BEGIN
  WITH level DO
    FOR x:=0 TO MaxX DO
      FOR y:=0 TO MaxY DO
        field[x,y]:=StatusSet{gorge};
      END;
    END;
    FOR player:=0 TO 1 DO
      WITH man[player] DO
        x:=0;
        y:=0;
      END;
    END;
    readyDests:=0;
    dests:=0;
  END;
END ResetLevel;


PROCEDURE DrawLevelInRast (level : LevelFile);

VAR
  temp  : RastPortPtr;

BEGIN
  temp:=brp;
  brp:=rp;
  rp:=temp;
  DrawLevel (level,FALSE);
  temp:=brp;
  brp:=rp;
  rp:=temp;
END DrawLevelInRast;


PROCEDURE CanSave (level : LevelFile) : BOOLEAN;

VAR
  player : Modes;

BEGIN
  WITH level DO
    FOR player:=0 TO mode DO
      WITH man[player] DO
        IF ~(man1 IN field[x,y]) THEN
          RETURN (FALSE);
        END;
      END;
    END;
  END;
  RETURN (TRUE);
END CanSave;


PROCEDURE Possible (m     : Modes;
                    dir   : Direction;
                    level : LevelFile) : BOOLEAN;

BEGIN
  WITH level DO
    CASE dir OF
    | north : IF (man[m].y=0) THEN
                RETURN (FALSE);
              ELSIF (field[man[m].x,man[m].y-1]=StatusSet{}) OR
                    (field[man[m].x,man[m].y-1]=StatusSet{dest}) THEN
                RETURN (TRUE);
              ELSE
                RETURN (FALSE);
              END;
    | south : IF (man[m].y=MaxY) THEN
                RETURN (FALSE);
              ELSIF (field[man[m].x,man[m].y+1]=StatusSet{}) OR
                    (field[man[m].x,man[m].y+1]=StatusSet{dest}) THEN
                RETURN (TRUE);
              ELSE
                RETURN (FALSE);
              END;
    | west  : IF (man[m].x=0) THEN
                RETURN (FALSE);
              ELSIF (field[man[m].x-1,man[m].y]=StatusSet{}) OR
                    (field[man[m].x-1,man[m].y]=StatusSet{dest}) THEN
                RETURN (TRUE);
              ELSE
                RETURN (FALSE);
              END;
    | east  : IF (man[m].x=MaxX) THEN
                RETURN (FALSE);
              ELSIF (field[man[m].x+1,man[m].y]=StatusSet{}) OR
                    (field[man[m].x+1,man[m].y]=StatusSet{dest}) THEN
                RETURN (TRUE);
              ELSE
                RETURN (FALSE);
              END;
    END;
  END;
END Possible;


PROCEDURE MoveMen (VAR level : LevelFile);

VAR
  m           : Modes;     (* number of man                  *)
  x,y         : INTEGER;   (* coordinates of man[m] in field *)
  xcoor,ycoor : INTEGER;   (* coordinates in window          *)
  dir         : Direction; (* current direction              *)
  sprAdr      : ADDRESS;   (* address of Sprites             *)

BEGIN
  Move (rp,94,7);
  Text (rp,ADR("Press ESC to quit move men"),26);
  Delay (10);
  WITH level DO
    FOR m:=0 TO 1 DO
      IF (man1 IN field[man[m].x,man[m].y]) THEN
        sprAdr:=spMan[m,FALSE,north,0];
        ChangeSprite (vp,spr1[m],sprAdr);
        ChangeSprite (vp,spr2[m],sprAdr+72);
        MoveSprite (vp,spr1[m],INTEGER(man[m].x)*15+xOff-1,
                    INTEGER(man[m].y)*15+yOff);
        BlitObject(obPlatform[dest IN field[man[m].x,man[m].y],FALSE],rp,
                   INTEGER(man[m].x)*15+xOff,INTEGER(man[m].y)*15+yOff);
      END;
    END;
    REPEAT
      FOR m:=0 TO mode DO
        dir:=GetDir(m);
        x:=INTEGER(man[m].x);
        y:=INTEGER(man[m].y);
        xcoor:=x*15+xOff;
        ycoor:=y*15+yOff;
        IF (dir=north) AND Possible(m,dir,level) THEN
          sprAdr:=spMan[m,FALSE,south,0];
          ChangeSprite (vp,spr1[m],sprAdr);
          ChangeSprite (vp,spr2[m],sprAdr+72);
          EXCL (field[x,y],man1);
          INCL (field[x,y-1],man1);
          IF (y<MaxY) AND (ball IN field[x,y+1]) AND Fire(m) THEN
            EXCL (field[x,y+1],ball);
            IF (dest IN field[x,y+1]) THEN
              DEC (readyDests);
            END;
            BlitObject(obPlatform[dest IN field[x,y+1],FALSE],rp,
                       xcoor,ycoor+15);
            INCL (field[x,y],ball);
            IF (dest IN field[x,y]) THEN
              INC (readyDests);
            END;
          END;
          DEC (man[m].y);
          BlitObject(obPlatform[dest IN field[x,y],ball IN field[x,y]],rp,
                     xcoor,ycoor);
          MoveSprite (vp,spr1[m],xcoor-1,ycoor-15);
          MoveSprite (vp,spr2[m],xcoor-1,ycoor-15);
        END;
        IF (dir=south) AND Possible(m,dir,level) THEN
          sprAdr:=spMan[m,FALSE,north,0];
          ChangeSprite (vp,spr1[m],sprAdr);
          ChangeSprite (vp,spr2[m],sprAdr+72);
          EXCL (field[x,y],man1);
          INCL (field[x,y+1],man1);
          IF (y>0) AND (ball IN field[x,y-1]) AND Fire(m) THEN
            EXCL (field[x,y-1],ball);
            IF (dest IN field[x,y-1]) THEN
              DEC (readyDests);
            END;
            BlitObject(obPlatform[dest IN field[x,y-1],FALSE],rp,
                       xcoor,ycoor-15);
            INCL (field[x,y],ball);
            IF (dest IN field[x,y]) THEN
              INC (readyDests);
            END;
          END;
          INC (man[m].y);
          BlitObject(obPlatform[dest IN field[x,y],ball IN field[x,y]],
                     rp,xcoor,ycoor);
          MoveSprite (vp,spr1[m],xcoor-1,ycoor+15);
          MoveSprite (vp,spr2[m],xcoor-1,ycoor+15);
        END;
        IF (dir=west) AND Possible(m,dir,level) THEN
          sprAdr:=spMan[m,FALSE,east,0];
          ChangeSprite (vp,spr1[m],sprAdr);
          ChangeSprite (vp,spr2[m],sprAdr+72);
          EXCL (field[x,y],man1);
          INCL (field[x-1,y],man1);
          IF (x<MaxX) AND (ball IN field[x+1,y]) AND Fire(m) THEN
            EXCL (field[x+1,y],ball);
            IF (dest IN field[x+1,y]) THEN
              DEC (readyDests);
            END;
            BlitObject(obPlatform[dest IN field[x+1,y],FALSE],rp,
                       xcoor+15,ycoor);
            INCL (field[x,y],ball);
            IF (dest IN field[x,y]) THEN
              INC (readyDests);
            END;
          END;
          DEC (man[m].x);
          BlitObject(obPlatform[dest IN field[x,y],ball IN field[x,y]],rp,
                     xcoor,ycoor);
          MoveSprite (vp,spr1[m],xcoor-15-1,ycoor);
          MoveSprite (vp,spr2[m],xcoor-15-1,ycoor);
        END;
        IF (dir=east) AND Possible(m,dir,level) THEN
          sprAdr:=spMan[m,FALSE,west,0];
          ChangeSprite (vp,spr1[m],sprAdr);
          ChangeSprite (vp,spr2[m],sprAdr+72);
          EXCL (field[x,y],man1);
          INCL (field[x+1,y],man1);
          IF (x>0) AND (ball IN field[x-1,y]) AND Fire(m) THEN
            EXCL (field[x-1,y],ball);
            IF (dest IN field[x-1,y]) THEN
              DEC (readyDests);
            END;
            BlitObject(obPlatform[dest IN field[x-1,y],FALSE],rp,
                       xcoor-15,ycoor);
            INCL (field[x,y],ball);
            IF (dest IN field[x,y]) THEN
              INC (readyDests);
            END;
          END;
          INC (man[m].x);
          BlitObject(obPlatform[dest IN field[x,y],ball IN field[x,y]],rp,
                     xcoor,ycoor);
          MoveSprite (vp,spr1[m],xcoor+15-1,ycoor);
          MoveSprite (vp,spr2[m],xcoor+15-1,ycoor);
        END;
      END;
      Delay (10);
    UNTIL Escape();
  END;
  Move (rp,94,7);
  Text (rp,ADR("                          "),26);
  HideSprites;
END MoveMen;


PROCEDURE DrawGetNewModeWindow (level : LevelFile);

VAR
  temp : RastPortPtr;

BEGIN
  temp:=brp;
  brp:=rp;
  rp:=temp;
  DrawButton (100,70,120,105,13,10,"Mode");
  IF level.mode=0 THEN
    DrawButton (110,85,52,15,3,10,"One");
  ELSE
    DrawButton (110,85,52,15,3,10,"Two");
  END;
  DrawButton (110,103,52,15,3,10,"Play");
  DrawButton (110,121,52,15,3,10,"Move");
  DrawButton (110,139,52,15,3,10,"Reset");
  DrawButton (110,157,52,15,3,10,"Cancel");
  temp:=brp;
  brp:=rp;
  rp:=temp;
  SetAPen (rp,7);
  SetBPen (rp,5);
  Move (rp,165,95);
  Text (rp,ADR("Player"),6);
  Move (rp,165,113);
  Text (rp,ADR("Level"),5);
  Move (rp,165,131);
  Text (rp,ADR("Men"),3);
  Move (rp,165,149);
  Text (rp,ADR("Level"),5);
  SetAPen (rp,31);
  SetBPen (rp,0);
END DrawGetNewModeWindow;


PROCEDURE ModeButtonposition (button : ModeButtons;VAR xpos,ypos : INTEGER);

BEGIN
  xpos:=125;
  CASE button OF
  | player : ypos:=85;
  | play   : ypos:=103;
  | move   : ypos:=121;
  | reset  : ypos:=139;
  | cancel : ypos:=157;
  END;
END ModeButtonposition;


PROCEDURE Buttonposition (button : Buttons;VAR xpos,ypos : INTEGER);

BEGIN
  ypos:=225;
  CASE button OF
  | sel0 : xpos:=9;
  | sel1 : xpos:=29;
  | sel2 : xpos:=49;
  | sel3 : xpos:=69;
  | sel4 : xpos:=89;
  | sel5 : xpos:=109;
  | sel6 : xpos:=129;
  | save : xpos:=159;
  | load : xpos:=199;
  | mode : xpos:=239;
  | quit : xpos:=279;
  END;
END Buttonposition;


PROCEDURE Select (VAR pdat : PointerData);

VAR
  xcoor,ycoor : INTEGER;

BEGIN
  WITH pdat DO
    IF button#oldbutton THEN
      CASE button OF
      | sel0 : selectedItems:=StatusSet{gorge};
      | sel1 : selectedItems:=StatusSet{};
      | sel2 : selectedItems:=StatusSet{man1};
      | sel3 : selectedItems:=StatusSet{ball,dest};
      | sel4 : selectedItems:=StatusSet{dest};
      | sel5 : selectedItems:=StatusSet{ball};
      | sel6 : selectedItems:=StatusSet{man1};
      END;
      Buttonposition (oldbutton,xcoor,ycoor);
      SetAPen (rp,0);
      DEC (xcoor,2);
      DEC (ycoor,3);
      Move (rp,xcoor,ycoor);
      Draw (rp,xcoor+20,ycoor);
      Draw (rp,xcoor+20,ycoor+20);
      Draw (rp,xcoor,ycoor+20);
      Draw (rp,xcoor,ycoor);
      Buttonposition (button,xcoor,ycoor);
      SetAPen (rp,7);
      DEC (xcoor,2);
      DEC (ycoor,3);
      Move (rp,xcoor,ycoor);
      Draw (rp,xcoor+20,ycoor);
      Draw (rp,xcoor+20,ycoor+20);
      Draw (rp,xcoor,ycoor+20);
      Draw (rp,xcoor,ycoor);
      oldbutton:=button;
    END;
  END;
END Select;


PROCEDURE GetNewMode (VAR level    : LevelFile;
                      VAR number   : LevelNumber;
                      VAR maxLevel : LevelNumber;
                      VAR pdat     : PointerData;
                          ctrl     : Controllertype);

VAR
  button    : ModeButtons; (* current mode button              *)
  xpos,ypos : INTEGER;     (* coordinates of pointer in window *)

BEGIN
  DrawGetNewModeWindow (level);
  Delay (10);
  button:=cancel;
  ModeButtonposition (button,xpos,ypos);
  MoveSprite (vp,pointerSpr,xpos,ypos);
  REPEAT
    IF (GetDir(0)=north) AND (button>player) THEN
      DEC (button);
    END;
    IF (GetDir(0)=south) AND (button<cancel) THEN
      INC (button);
    END;
    ModeButtonposition (button,xpos,ypos);
    MoveSprite (vp,pointerSpr,xpos,ypos);
    Delay (5);
  UNTIL Fire(0);
  CASE button OF
  | player : number:=0;
             WITH level DO
               mode:=1-mode;
               IF mode=0 THEN
                 EXCL (field[man[1].x,man[1].y],man1);
                 IF pdat.oldbutton=sel6 THEN
                   pdat.button:=sel2;
                   Select (pdat);
                   pdat.button:=Buttons(9);(* pdat.button:=mode *)
                 END;
               END;
             END;
             maxLevel:=SetMode(level.mode);
  | play   : IF CanSave(level) THEN
               ResetController (ctrl);
               IF PlayGame(level,"",FALSE,number,maxLevel) THEN
               END;
               ChangeScreens (FALSE);
             END;
  | move   : HideSprites;
             DrawLevelInRast (level);
             ResetController (ctrl);
             MoveMen (level);
             DrawLevelInRast (level);
  | reset  : ResetLevel (level);
  | cancel : ;
  END;
END GetNewMode;


PROCEDURE Edit (level : LevelFile;number : LevelNumber;ctrl : Controllertype);

VAR
  pdat        : PointerData; (* current pointer datas          *)
  xcoor,ycoor : INTEGER;     (* x- and y-coordinate of pointer *)
  index       : INTEGER;     (* index variable for highScore   *)
  maxLevel    : LevelNumber;

BEGIN
  maxLevel:=SetMode (level.mode);
  WITH pdat DO
    x:=MaxX DIV 2;
    y:=MaxY;
    menu:=TRUE;
    oldbutton:=sel1;
    button:=sel0;
    Select (pdat);
    button:=quit;
    Buttonposition (button,xpos,ypos);
    REPEAT
      MoveSprite (vp,pointerSpr,xpos,ypos);
      REPEAT
        Delay (5);
        IF (GetDir(0)=north) AND (y>0) THEN
          IF menu THEN
            menu:=FALSE;
            ypos:=INTEGER(MaxY)*15+yOff;
            xpos:=INTEGER(x)*15+xOff-1;
          ELSE
            DEC (y);
            DEC (ypos,15);
          END;
          MoveSprite (vp,pointerSpr,xpos,ypos);
        END;
        IF (GetDir(0)=south) AND NOT(menu) THEN
          IF (y=MaxY) THEN
            menu:=TRUE;
            Buttonposition (button,xpos,ypos);
          ELSE
            INC (y);
            INC (ypos,15);
          END;
          MoveSprite (vp,pointerSpr,xpos,ypos);
        END;
        IF (GetDir(0)=west) THEN
          IF (button>sel0) AND menu THEN
            DEC (button);
            Buttonposition (button,xpos,ypos);
          ELSIF (x>0) AND NOT(menu) THEN
            DEC (x);
            DEC (xpos,15);
          END;
          MoveSprite (vp,pointerSpr,xpos,ypos);
        END;
        IF (GetDir(0)=east) THEN
          IF (button<quit) AND menu THEN
            INC (button);
            Buttonposition (button,xpos,ypos);
          ELSIF (x<MaxX) AND NOT(menu) THEN
            INC (x);
            INC (xpos,15);
          END;
          MoveSprite (vp,pointerSpr,xpos,ypos);
        END;
        Delay (5);
      UNTIL Fire(0);
      IF menu THEN
        CASE button OF
        | sel0 : Select (pdat);
        | sel1 : Select (pdat);
        | sel2 : Select (pdat);
        | sel3 : Select (pdat);
        | sel4 : Select (pdat);
        | sel5 : Select (pdat);
        | sel6 : IF level.mode=1 THEN
                   Select (pdat);
                 END;
        | save : IF CanSave(level) & RequestLevel(number,TRUE,maxLevel) THEN
                   WITH level DO
                     FOR index:=0 TO 4 DO
                       top5[index].name:="                    ";
                       top5[index].mins:=99;
                       top5[index].secs:=59;
                       top5[index].moves:=9999;
                     END;
                   END;
                   maxLevel:=SaveLevel (number,level);
                 END;
                 DrawLevelInRast (level);
        | load : IF RequestLevel(number,FALSE,maxLevel) THEN
                   LoadLevel (number,level);
                 END;
                 DrawLevelInRast (level);
        | mode : GetNewMode (level,number,maxLevel,pdat,ctrl);
                 DrawLevelInRast (level);
        ELSE
        END;
      ELSE
        xcoor:=15*INTEGER(x)+xOff;
        ycoor:=15*INTEGER(y)+yOff;
        WITH level DO
          IF (field[x,y]=StatusSet{ball,dest}) THEN
            DEC (dests);
            DEC (readyDests);
          ELSIF (dest IN field[x,y]) THEN
            DEC (dests);
          END;
          IF selectedItems=StatusSet{gorge} THEN
            BlitObject(obGorge2,rp,xcoor,ycoor);
          ELSIF (selectedItems=StatusSet{man1}) AND (oldbutton=sel2) THEN
            IF (man1 IN field[man[0].x,man[0].y]) THEN
              BlitObject(obPlatform[dest IN field[man[0].x,man[0].y],FALSE],rp,
                         INTEGER(man[0].x)*15+xOff,INTEGER(man[0].y)*15+yOff);
            END;
            EXCL (field[man[0].x,man[0].y],man1);
            IF dest IN field[x,y] THEN
              INCL (selectedItems,dest);
            END;
            man[0].x:=x;
            man[0].y:=y;
            BlitObject(obMan[dest IN field[x,y],0],rp,xcoor,ycoor);
          ELSIF (selectedItems=StatusSet{man1}) AND (oldbutton=sel6) THEN
            IF (man1 IN field[man[1].x,man[1].y]) THEN
              BlitObject(obPlatform[dest IN field[man[1].x,man[1].y],FALSE],rp,
                         INTEGER(man[1].x)*15+xOff,INTEGER(man[1].y)*15+yOff);
            END;
            EXCL (field[man[1].x,man[1].y],man1);
            IF dest IN field[x,y] THEN
              INCL (selectedItems,dest);
            END;
            man[1].x:=x;
            man[1].y:=y;
            BlitObject(obMan[dest IN field[x,y],1],rp,xcoor,ycoor);
          ELSE
            BlitObject(obPlatform[dest IN selectedItems,
                       ball IN selectedItems],rp,xcoor,ycoor);
          END;
          field[x,y]:=selectedItems;
          IF (selectedItems=StatusSet{man1,dest}) THEN
            EXCL (selectedItems,dest);
          END;
          IF (field[x,y]=StatusSet{ball,dest}) THEN
            INC (dests);
            INC (readyDests);
          ELSIF (dest IN field[x,y]) THEN
            INC (dests);
          END;
        END;
      END;
    UNTIL (button=quit) AND menu;
  END;
END Edit;


PROCEDURE Editor (number : LevelNumber;ctrl : Controllertype);

VAR
  level : LevelFile;

BEGIN
  LoadLevel (number,level);
  SetRast (brp,0);
  DrawLevel (level,FALSE);
  ChangeScreens (FALSE);
  Edit (level,number,ctrl);
  SetBPen (rp,0);
  HideSprites;
END Editor;


END LevelEd.
