(******************************************************************************

    :Program.    Pushem.mod
    :Contents.   the stuff that keeps it all together (main module)
    :Version.    0.4
    :Date.       11.01.92
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :Copyright.   1991 P. Groth and M. Uhlendahl
    :History.    17.10.91 --- 0.1 --- created
                 13.11.91 --- 0.2 --- default controller is keyboard,
                                      start game on every fire event
                 16.11.91 --- 0.3 --- new MenuLine
                 11.01.92 --- 0.4 --- Show always Ctrl

******************************************************************************)
MODULE Pushem;


(*====== Import from project Pushem =========================================*)
FROM Definitions IMPORT
  Direction,LevelFile,Modes,NameString;

FROM LevelEd IMPORT
  Editor;

FROM Game IMPORT
  PlayGame;

FROM GraphicSupport IMPORT
  brp,ChangeScreens,DrawCtrl,DrawMenuLine,DrawNumber,MenuLine,rp,TitleToScreen;

FROM InputControl IMPORT
  Controllertype,Fire,GetDir,GetName,ResetController;

FROM LevelLoader IMPORT
  LevelNumber,LoadLevel,SelectLevel,SetMode;

FROM SoundSupport IMPORT
  AllocChannels,FreeChannels,StartMusic,StopMusic;

(*====== Import from standard library =======================================*)
FROM DosL IMPORT
  Delay;

FROM GraphicsL IMPORT
  Draw,Move,SetAPen,SetRast,Text;

FROM SYSTEM IMPORT
  ADR;


TYPE
  Actions = (selectCtrl,changeMode,selectLevel,edit,quit);


VAR
  currentLevel : LevelNumber;
  mode         : Modes;
  maxLevel     : LevelNumber;
  action       : Actions;
  level        : LevelFile;
  name         : NameString;
  ctrl         : Controllertype;


PROCEDURE DoIntro;

VAR
  r : INTEGER;

BEGIN
  Move (brp,32,100);
  Text (brp,ADR("Peter Groth and Markus Uhlendahl"),32);
  Move (brp,132,120);
  Text (brp,ADR("present"),7);
  ChangeScreens (TRUE);
  Delay (4*50);
  SetRast (brp,0);
  Move (brp,136,110);
  Text (brp,ADR("A Game"),6);
  ChangeScreens (TRUE);
  Delay (3*50);
  SetRast (brp,0);
  Move (brp,108,100);
  Text (brp,ADR("Programmed by"),13);
  Move (brp,32,120);
  Text (brp,ADR("Peter Groth and Markus Uhlendahl"),32);
  ChangeScreens (TRUE);
  Delay (3*50);
  SetRast (brp,0);
  Move (brp,128,100);
  Text (brp,ADR("Music by"),8);
  Move (brp,104,120);
  Text (brp,ADR("Harald Schfer"),14);
  ChangeScreens (TRUE);
  Delay (3*50);
  SetRast (brp,0);
  Move (brp,116,80);
  Text (brp,ADR("Graphics by"),11);
  Move (brp,116,100);
  Text (brp,ADR("Peter Groth"),11);
  Move (brp,100,130);
  Text (brp,ADR("Titlegraphic by"),15);
  Move (brp,60,150);
  Text (brp,ADR("Andreas 'Leini' Leinichen"),25);
  ChangeScreens (TRUE);
  Delay (5*50);
  SetRast (brp,0);
  Move (brp,104,100);
  Text (brp,ADR("Leveldesign by"),14);
  Move (brp,52,120);
  Text (brp,ADR("Markus and Thomas Uhlendahl"),27);
  ChangeScreens (TRUE);
  Delay (3*50);
  SetRast (brp,0);
  TitleToScreen;
  ChangeScreens (TRUE);
END DoIntro;


PROCEDURE DrawBorder (x,y,color : INTEGER);

BEGIN
  SetAPen (rp,color);
  Move (rp,x,y);
  Draw (rp,x+58,y);
  Draw (rp,x+58,y-18);
  Draw (rp,x,y-18);
  Draw (rp,x,y);
END DrawBorder;


PROCEDURE SelectController (VAR ctrl : Controllertype);

VAR
  old : Controllertype;

BEGIN
  old:=ctrl;
  IF GetDir(0)=north THEN
    ctrl:=key;
  END;
  IF GetDir(0)=south THEN
    ctrl:=joy;
  END;
  DrawCtrl (ctrl);
END SelectController;


PROCEDURE ChangePlayMode (VAR mode     : Modes;
                          VAR levelNum : LevelNumber;
                          VAR maxLevel : LevelNumber);

VAR
  old : Modes;

BEGIN
  old:=mode;
  IF (GetDir(0)=north) THEN
    mode:=0;
  END;
  IF (GetDir(0)=south) THEN
    mode:=1;
  END;
  IF (old#mode) THEN
    CASE mode OF
    | 0 :  Move (rp,98,MenuLine+8);
           Text (rp,ADR("One"),3);
           Move (rp,30,MenuLine+8);
           Text (rp,ADR("   "),3);
    | 1 :  Move (rp,98,MenuLine+8);
           Text (rp,ADR("Two"),3);
           DrawCtrl (ctrl);
    END;
    maxLevel:=SetMode(mode);
    levelNum:=0;
    DrawNumber (levelNum,160,MenuLine+8,31,3);
  END;
END ChangePlayMode;


PROCEDURE ButtonPosition (button : Actions;VAR x,y : INTEGER);

BEGIN
  y:=MenuLine+10;
  CASE button OF
  | selectCtrl  : x:=15;
  | changeMode  : x:=81;
  | selectLevel : x:=143;
  | quit        : x:=255;
  | edit        : x:=201;
  ELSE
  END;
END ButtonPosition;


PROCEDURE GetAction (VAR action : Actions;VAR ctrl : Controllertype);

VAR
  x,y : INTEGER;
  esc : BOOLEAN;

BEGIN
  ButtonPosition (action,x,y);
  DrawBorder (x,y,31);
  REPEAT
    IF (GetDir(0)=west) AND (action>selectCtrl) THEN
      IF ~((mode=0) & (action=changeMode)) THEN
        DEC (action);
        DrawBorder (x,y,0);
        ButtonPosition (action,x,y);
        DrawBorder (x,y,31);
        Delay (10);
      END;
    END;
    IF (GetDir(0)=east) AND (action<quit) THEN
      INC (action);
      DrawBorder (x,y,0);
      ButtonPosition (action,x,y);
      DrawBorder (x,y,31);
      Delay (10);
    END;
    CASE action OF
    | changeMode  : ChangePlayMode (mode,currentLevel,maxLevel);
    | selectLevel : SelectLevel (currentLevel,maxLevel,160,MenuLine+8,31);
    | selectCtrl  : SelectController (ctrl);
    ELSE
    END;
  UNTIL Fire(0);
END GetAction;


BEGIN
  DoIntro;
  ctrl:=key;
  ResetController (ctrl);
  StartMusic;
  name:="                    ";
  action:=selectLevel;
  currentLevel:=0;
  mode:=0;
  maxLevel:=SetMode(mode);
  DrawMenuLine (mode,currentLevel,ctrl);
  REPEAT
    GetAction (action,ctrl);
    CASE action OF
    | edit        : StopMusic;
                    Editor(currentLevel,ctrl);
                    TitleToScreen;
                    ChangeScreens(TRUE);
                    maxLevel:=SetMode(mode);
                    DrawMenuLine (mode,currentLevel,ctrl);
                    StartMusic;
    | selectLevel,
      changeMode,
      selectCtrl  : StopMusic;
                    GetName (name);
                    ResetController (ctrl);
                    LoadLevel(currentLevel,level);
                    AllocChannels;
                    WHILE PlayGame(level,name,TRUE,currentLevel,maxLevel) AND
                          (currentLevel<maxLevel) DO
                      INC(currentLevel);
                      LoadLevel (currentLevel,level);
                    END;
                    FreeChannels;
                    TitleToScreen;
                    ChangeScreens(TRUE);
                    DrawMenuLine (mode,currentLevel,ctrl);
                    StartMusic;
    ELSE
    END;
  UNTIL action=quit;
CLOSE
  StopMusic;
END Pushem.
