(******************************************************************************

    :Program.    SoundToObj.mod
    :Contents.   Converts sound data to Sound.obj
    :Version.    0.2
    :Date.       08.11.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    28.07.91 --- 0.1 --- created
                 08.11.91 --- 0.2 --- forgot the new version number...

******************************************************************************)

MODULE SoundToObj;


FROM Arts IMPORT
  Assert;

FROM FileSystem IMPORT
  Close,File,Length,Lookup,ReadBytes,Response;

FROM Heap IMPORT
  Allocate,Deallocate;

FROM ObjFileCreator IMPORT
  AddData,AddDataBlock,AddLabel,CreateDataHunk,DefReloc,HunkPos,WriteObjFile;

FROM SYSTEM IMPORT
  ADDRESS,ADR;


TYPE
  Sample=RECORD
          data  : ADDRESS;
          length: LONGCARD;
          period: CARDINAL;
        END;


VAR
  dataHunk,structHunk: CARDINAL;


PROCEDURE AddSample(name:ARRAY OF CHAR; labelName:ARRAY OF CHAR);

VAR
  file  : File;
  act   : LONGINT;
  sample: Sample;
  data  : ADDRESS;

BEGIN
  Lookup(file,name,1024,FALSE);
  Assert(file.res=done,ADR("Sample not found"));
  Length(file,act);
  sample.length:=LONGCARD(act);
  Allocate(data,sample.length);
  Assert(data#NIL,ADR("Not enough memory"));
  ReadBytes(file,data,sample.length,act);
  Close(file);
  sample.data:=HunkPos(dataHunk);
  sample.period:=3563220 DIV 8800;
  AddLabel(structHunk,labelName);
  DefReloc(structHunk,HunkPos(structHunk),dataHunk);
  AddData(dataHunk,data,sample.length);
  AddDataBlock(structHunk,sample);
  Deallocate(data);
END AddSample;


BEGIN
  dataHunk:=CreateDataHunk("SampleData",TRUE);
  structHunk:=CreateDataHunk("SampleStructs",FALSE);
  AddSample("Musik/Glass.raw","Sound_drop");
  AddSample("Musik/ManFall.raw","Sound_fall");
  WriteObjFile("obj/Sound.obj","Sounds");
END SoundToObj.
