(******************************************************************************

    :Program.    SprToObj.mod
    :Contents.   Converts sprite data to Spr.obj
    :Version.    0.4
    :Date.       01.11.91
    :Author.     Markus Uhlendahl    Peter Groth
    :Address.    Vorm Burgtor 16     Kleine Koppel 12
    :Address.    D-4408 Dlmen       D-4408 Dlmen
    :Phone.      02594/81540         02594/82375
    :Language.   Modula-2
    :Translator. M2Amiga 4.0d
    :History.    05.08.91 --- 0.1 --- created
                 02.10.91 --- 0.2 --- removed a serious bug
                 03.10.91 --- 0.3 --- another bug...
                 01.11.91 --- 0.4 --- cosmetics

******************************************************************************)

MODULE SprToObj;


FROM Arts IMPORT
  Assert;

FROM ExecL IMPORT
  CopyMem;

FROM FileSystem IMPORT
  Close,File,Length,Lookup,ReadBytes,Response,SetPos;

FROM GraphicsD IMPORT
  spriteAttached;

FROM Heap IMPORT
  Allocate,Deallocate;

FROM ObjFileCreator IMPORT
  AddData,AddDataBlock,AddLabel,AddReloc,CreateDataHunk,HunkPos,WriteObjFile;

FROM String IMPORT
  Compare;

FROM SYSTEM IMPORT
  ADDRESS,ADR;

FROM Terminal IMPORT
  WriteInt,WriteLn,WriteString;


TYPE
  BMHDPtr=POINTER TO BMHD;
  BMHD=RECORD
         w,h        : CARDINAL;
         x,y        : INTEGER;
         nPlanes    : SHORTCARD;
         masking    : SHORTCARD;
         compression: SHORTCARD;
         pad1       : SHORTCARD;
         transCol   : SHORTCARD;
         xAspect    : SHORTCARD;
         yAspect    : SHORTCARD;
         pageWidth  : INTEGER;
         pageHeight : INTEGER;
       END;
  String4=ARRAY [0..4] OF CHAR;
  Chunk=RECORD
          id  : String4;
          len : LONGCARD;
          data: ADDRESS;
        END;


VAR
  dataHunk,structHunk: CARDINAL;
  chunk              : Chunk;
  bmhd               : BMHDPtr;
  file               : File;
  pic,src,dest       : POINTER TO SHORTINT;
  plane              : [0..4];
  line,row           : CARDINAL;
  data,byte          : SHORTINT;
  i                  : CARDINAL;

  player,hball,dir,phase: CARDINAL;
  ph                    : ARRAY [0..7] OF CARDINAL;
  lc                    : LONGCARD;


PROCEDURE ReadChunk(VAR chunk:Chunk): BOOLEAN;

VAR act: LONGINT;

BEGIN
  ReadBytes(file,ADR(chunk.id),4,act);
  IF act#4 THEN
    RETURN(FALSE);
  END;
  IF (Compare(chunk.id,"BMHD")=0) OR
     (Compare(chunk.id,"BODY")=0) OR
     (Compare(chunk.id,"CMAP")=0) THEN
    ReadBytes(file,ADR(chunk.len),4,act);
    Allocate(chunk.data,chunk.len);
    Assert(chunk.data#NIL,ADR("Not enough memory"));
    ReadBytes(file,chunk.data,chunk.len,act);
  END;
  RETURN(TRUE);
END ReadChunk;


PROCEDURE FreeChunk(VAR chunk: Chunk);

BEGIN
  Deallocate(chunk.data);
END FreeChunk;


PROCEDURE AddSprite(xnum,ynum:CARDINAL; attached:BOOLEAN);

VAR
  src : POINTER TO CARDINAL;
  line: CARDINAL;
  null: CARDINAL;

BEGIN
  null:=0;
  ynum:=ynum*16;
  AddDataBlock(dataHunk,null);
  IF attached THEN
    null:=spriteAttached;
    AddDataBlock(dataHunk,null);
    null:=0;
  ELSE
    AddDataBlock(dataHunk,null);
  END;
  FOR line:=0 TO 15 DO
    src:=ADDRESS(LONGCARD(pic)+2*xnum+272 DIV 8 * (ynum+line));
    AddData(dataHunk,src,2);
    src:=ADDRESS(LONGCARD(pic)+34*176+2*xnum+272 DIV 8 * (ynum+line));
    AddData(dataHunk,src,2);
  END;
  AddDataBlock(dataHunk,null);
  AddDataBlock(dataHunk,null);
  IF attached THEN
    AddDataBlock(dataHunk,null);
    null:=spriteAttached;
    AddDataBlock(dataHunk,null);
    null:=0;
    FOR line:=0 TO 15 DO
      src:=ADDRESS(LONGCARD(pic)+34*176*2+2*xnum+272 DIV 8 * (ynum+line));
      AddData(dataHunk,src,2);
      src:=ADDRESS(LONGCARD(pic)+34*176*3+2*xnum+272 DIV 8 * (ynum+line));
      AddData(dataHunk,src,2);
    END;
    AddDataBlock(dataHunk,null);
    AddDataBlock(dataHunk,null);
  END;
END AddSprite;


BEGIN
  Lookup(file,"Grafik/Spr272x176.IFF",1024,FALSE);
  Assert(file.res=done,ADR("Spr272x176.IFF not found"));
  SetPos(file,12);
  dataHunk:=CreateDataHunk("SprData",TRUE);
  structHunk:=CreateDataHunk("SprStructs",FALSE);
  WHILE ReadChunk(chunk) DO
    IF Compare(chunk.id,"BODY")=0 THEN
      IF bmhd^.compression=0 THEN
        pic:=chunk.data;
      ELSE
        Allocate(pic,34*176*5);
        Assert(pic#NIL,ADR("Not enough memory"));
        src:=chunk.data;
        dest:=pic;
        FOR line:=0 TO 175 DO
          FOR plane:=0 TO 4 DO
            row:=0;
            REPEAT
              dest:=ADDRESS(LONGCARD(pic)+34*176*plane+line*34+row);
              byte:=src^;
              INC(src);
              IF byte>=0 THEN
                CopyMem(src,dest,SHORTCARD(byte)+1);
                INC(src,SHORTCARD(byte)+1);
                row:=row+CARDINAL(byte)+1;
              ELSIF byte>-128 THEN
                byte:=-byte;
                FOR i:=0 TO SHORTCARD(byte) DO
                  dest^:=src^;
                  INC(dest);
                END;
                INC(src);
                row:=row+CARDINAL(byte)+1;
              END;
            UNTIL row>=34;
          END;
        END;
        FreeChunk(chunk);
      END;
    ELSIF Compare(chunk.id,"BMHD")=0 THEN
      bmhd:=chunk.data;
    ELSIF Compare(chunk.id,"CMAP")=0 THEN
      FreeChunk(chunk);
    END;
  END;
  Close(file);
  ph[0]:=0;
  ph[1]:=1;
  ph[2]:=2;
  ph[3]:=1;
  ph[4]:=0;
  ph[5]:=3;
  ph[6]:=4;
  ph[7]:=3;
  AddLabel(structHunk,"Spr_man");
  FOR player:=0 TO 1 DO
    FOR hball:=0 TO 1 DO
      FOR dir:=0 TO 3 DO
        FOR phase:=0 TO 7 DO
          AddReloc(structHunk,dataHunk,HunkPos(dataHunk));
          AddSprite(8*player+4*hball+dir,ph[phase],TRUE);
        END;
      END;
    END;
  END;
  AddLabel(structHunk,"Spr_fallingMan");
  lc:=0;
  FOR player:=0 TO 1 DO
    FOR i:=0 TO 11 DO
      FOR phase:=0 TO 2 DO
        AddReloc(structHunk,dataHunk,HunkPos(dataHunk));
        AddSprite(i,5+phase+3*player,TRUE);
      END;
      IF (i=5) OR (i=7) THEN
        FOR phase:=0 TO 5 DO
          AddDataBlock(structHunk,lc);
        END;
      END;
    END;
  END;
  AddLabel(structHunk,"Spr_ball");
  FOR player:=0 TO 1 DO
    AddReloc(structHunk,dataHunk,HunkPos(dataHunk));
    AddSprite(16,0,FALSE);
  END;
  AddLabel(structHunk,"Spr_pointer");
  AddReloc(structHunk,dataHunk,HunkPos(dataHunk));
  AddSprite(16,1,FALSE);
  WriteObjFile("obj/Spr.obj","Spr");
END SprToObj.
