(*************************************************************************

:Program.    DisplayIO.mod
:Contents.   IO-functions for Oberon-Module 'Display'
:Author.     Hartmut Goebel
:Address.    Aufseplatz 5, D-8500 Nrnberg 40
:Copyright.  Freeware, Copyright  1990 by Hartmut Goebel
:Language.   Oberon
:Translator. Amiga Oberon V2.13
:History.    V1.0, 30 Jan 1992 hartmut Goebel [hG]
:Date.       30 Jan 1992 12:13:54

:Support.    most parts were taken form Oberon-Std-Module 'io'
:Update.     ReadHex now also accepts lower-case charakters

:Remark.     Works just like 'io', but in a Display.Window
:Usage.      ExecSupport.CreatePort(), Display.OpenWindow(),
:Usage.      DisplayIO.AttachPort() - perform IO - DisplayIO.DetachPort(),
:Usage.      Display.Close(), ExecSupport.DeletePort(),

*************************************************************************)

MODULE DisplayIO;

IMPORT
  asc: ASCII,
  dpl*: Display,
  e  : Exec,
  I  : Intuition,
  s  : SYSTEM;

TYPE
  String = ARRAY 40 OF CHAR;

VAR
  ftemp: ARRAY 80 OF CHAR;
  helpstr: String;

(*----------------------------------------------------------------------*)

PROCEDURE * RFProc; (* $EntryExitCode- *)
BEGIN
  s.INLINE(016C0U,  (* MOVE.B D0,(A3)+ *)
           04E75U); (* RTS             *)
END RFProc;

(*------- Text-Output: -----------------------*)

PROCEDURE WriteString*(win: dpl.WindowPtr; str: ARRAY OF CHAR);
(*
VAR
  len, i: INTEGER;
BEGIN
  i := 0;
  len := Strings.Length(str);
  WHILE i < len DO

  END;
*)
BEGIN
  dpl.Write(win,str);
END WriteString;

PROCEDURE Format*(win: dpl.WindowPtr; VAR str: String; data:LONGINT);
(* %% => %
     links  fhr.0   min.max Breite  longdata   dez|hex|string|char
   %  [-]    [0]      [123 [.123] ]     [l]        (d|x|s|c)

   Char ist immer in WORD, auch bei Angabe 'l'!!!
   String-Adresse ist immer LONG!!!
*)
(* niemals mehr als 79 Zeichen erzeugen! *)
BEGIN
  e.RawDoFmt(str,data,RFProc,s.ADR(ftemp));
  WriteString(win,ftemp);
END Format;


PROCEDURE WriteInt*(win: dpl.WindowPtr; x: LONGINT; n: INTEGER);
BEGIN
  e.RawDoFmt('%%%dld',s.ADR(n),RFProc,s.ADR(helpstr));
  Format(win,helpstr,s.ADR(x));
END WriteInt;


PROCEDURE WriteHex*(win: dpl.WindowPtr; x: LONGINT; n: INTEGER);
BEGIN
  IF n>=0 THEN (* RawDoFmt spinnt etwas bei neg. Zahlen und fhr. 0 *)
    e.RawDoFmt('%%0%dlx',s.ADR(n),RFProc,s.ADR(helpstr));
  ELSE
    n:=-n;
    e.RawDoFmt('%%-%dlx',s.ADR(n),RFProc,s.ADR(helpstr));
  END;
  Format(win,helpstr,s.ADR(x));
END WriteHex;


(*------- Text-Input: -----------------------*)

PROCEDURE Read*(win: dpl.WindowPtr; VAR ch: CHAR);
TYPE
 charArray = ARRAY 2 OF CHAR;
VAR
  msg: I.IntuiMessagePtr;
  x, y: INTEGER;
  old: BOOLEAN;
  ca: charArray;  (* tricky: so ca[0] is on an even address :-) *)
BEGIN
  IF ~(I.vanillaKey IN win.window.idcmpFlags) THEN HALT(20); END;
  old := dpl.SetCursor(win,TRUE);
  LOOP
    e.WaitPort(win.window.userPort);
    msg := e.GetMsg(win.window.userPort);
    WHILE msg#NIL DO
      IF I.vanillaKey IN msg.class THEN
        ch := CHR(msg.code);
        CASE ch OF
        "\n","\r": dpl.WriteLn(win); |
        "\b": dpl.CursorOff(win);
              x := win.curX-1;
              IF (x = 0) AND (win.curY > 0) THEN
                x := win.txtWidth-1;
                y := win.curY-1;
              END;
              dpl.Position(win,x,y);
              dpl.Text(win,win.curXAbs,win.curYAbs," ");
              dpl.CursorOn(win);
        ELSE
          ca[0] := ch; ca[1] := 0X;
          dpl.Write(win,ca);
        END;
        e.ReplyMsg(msg);
        IF dpl.SetCursor(win,old) THEN END;
        RETURN;
      END;
      e.ReplyMsg(msg);
      msg := e.GetMsg(win.window.userPort);
    END;
  END;
END Read;


PROCEDURE ReadString*(win: dpl.WindowPtr; VAR str: ARRAY OF CHAR);
VAR
  i: INTEGER;
BEGIN
  i := 0;
  REPEAT
    Read(win,str[i]);
    CASE str[i] OF "\n","\r",asc.eof,0X: str[i] := 0X; RETURN ELSE END;
    INC(i);
  UNTIL i=LEN(str);
END ReadString;


PROCEDURE ReadInt*(win: dpl.WindowPtr; VAR x: LONGINT): BOOLEAN;
VAR
  ch: CHAR;
  d: LONGINT;
  neg: BOOLEAN;
  str: String;
  i: INTEGER;
BEGIN
  x := 0; i := 0;
  ReadString(win,str);
  neg := FALSE;
  IF str[0]="-" THEN neg := TRUE; i := 1 END;
  LOOP
    ch := str[i];
    CASE ch OF
    0X: IF neg THEN x := -x END; RETURN TRUE |
    "0".."9":
      d := ORD(ch)-ORD("0");
      IF (MAX(LONGINT)-d) DIV 10 >= x THEN x := 10*x+d ELSE EXIT END |
    ELSE EXIT END;
    INC(i);
  END;
  RETURN FALSE;
END ReadInt;


PROCEDURE ReadInteger*(win: dpl.WindowPtr; VAR x: INTEGER): BOOLEAN;
VAR
  l: LONGINT;
BEGIN
  IF ReadInt(win,l) AND (l>=MIN(INTEGER)) AND (l<=MAX(INTEGER)) THEN
    x := SHORT(l);
    RETURN TRUE;
  END;
  RETURN FALSE;
END ReadInteger;


PROCEDURE ReadShortInt*(win: dpl.WindowPtr; VAR x: SHORTINT): BOOLEAN;
VAR
  l: LONGINT;
BEGIN
  IF ReadInt(win,l) AND (l>=MIN(SHORTINT)) AND (l<=MAX(SHORTINT)) THEN
    x := SHORT(SHORT(l));
    RETURN TRUE;
  END;
  RETURN FALSE;
END ReadShortInt;


PROCEDURE ReadHex*(win: dpl.WindowPtr; VAR x: LONGINT): BOOLEAN;
VAR
  ch: CHAR;
  d: LONGINT;
  str: String;
  i: INTEGER;
BEGIN
  x := 0; i := 0;
  ReadString(win,str);
  LOOP
    ch := str[i];
    CASE ch OF
    0X:       RETURN TRUE |
    "0".."9": DEC(ch,ORD("0")) |
    "A".."F": DEC(ch,ORD("A")-10) |
    "a".."f": DEC(ch,ORD("a")-10) |
    ELSE EXIT END;
    d := ORD(ch);
    IF (MAX(LONGINT)-d) DIV 16 >= x THEN x := 16*x+d ELSE EXIT END;
    INC(i);
  END;
  RETURN FALSE;
END ReadHex;

(*-------------------------------------------------------------------------*)

(* Synonyme fr Read-Funktionen ohne Ergebnis, falsche Eingabe fhrt also
   zu falschem Ergebnis: *)

PROCEDURE ReadIntOk     *{"DisplayIO.ReadInt"     }(win: dpl.WindowPtr; VAR x: LONGINT );
PROCEDURE ReadIntegerOk *{"DisplayIO.ReadInteger" }(win: dpl.WindowPtr; VAR x: INTEGER );
PROCEDURE ReadShortIntOk*{"DisplayIO.ReadShortInt"}(win: dpl.WindowPtr; VAR x: SHORTINT);
PROCEDURE ReadHexOk     *{"DisplayIO.ReadHex"     }(win: dpl.WindowPtr; VAR x: LONGINT );

(*------- Support -----------------------*)


PROCEDURE StripIntuiMessages(mp: e.MsgPortPtr; win: I.WindowPtr);
VAR
  msg, succ: I.IntuiMessagePtr;
BEGIN
  msg := s.VAL(I.IntuiMessagePtr,mp.msgList.head);
  WHILE msg # NIL DO
    succ := s.VAL(I.IntuiMessagePtr,msg.execMessage.node.succ);
    IF msg.idcmpWindow = win THEN
      e.Remove(msg);
      e.ReplyMsg(msg);
    END;
    msg := succ;
  END;
END StripIntuiMessages;


(*
 * Entfernt den mit AttachPort() angegngten IDCMP-Port von <win>
 *)
PROCEDURE DetachPort*(win: dpl.WindowPtr);
VAR
  iwin: I.WindowPtr;
BEGIN
  iwin := win.window;
  IF iwin.userPort = NIL THEN RETURN; END;
  e.Forbid();
  StripIntuiMessages(iwin.userPort,iwin);
  iwin.userPort := NIL;
  IF I.ModifyIDCMP(iwin,LONGSET{}) THEN END;
  e.Permit();
END DetachPort;

(*
 * Hngt <port> als IDCMP-Port an <win>, der die Flags <idcmp> erhlt
 *)
PROCEDURE AttachPort*(win: dpl.WindowPtr;
                      port: e.MsgPortPtr;
                      idcmp: LONGSET): BOOLEAN;
VAR
  iwin: I.WindowPtr;
BEGIN
  iwin := win.window;
  IF (iwin#NIL) & (iwin.idcmpFlags=LONGSET{}) & (idcmp#LONGSET{}) THEN
    iwin.userPort := port;
    RETURN I.ModifyIDCMP(iwin,idcmp) OR (I.int.libNode.version < 37);
  END;
  RETURN FALSE;
END AttachPort;

END DisplayIO.

