(*---------------------------------------------------------------------------
  :Program.     Metronom.mod
  :Contents.    Taktgeber
  :Author.      Christian Stiens
  :Address.     Heustiege 2, W-4710 Ldinghausen
  :Copyright.   PD
  :Language.    Oberon
  :Translator.  Amiga Oberon V2.12e [fbs]
  :History.     V1.0, 01-Nov-91
  :Imports.     AudioSupport (AMOK#58), R (Registernumbers)
---------------------------------------------------------------------------*)


MODULE Metronom;

  IMPORT
    arg: Arguments, as: AudioSupport, c  : Conversions, d : Dos,
    g  : Graphics,  hw: Hardware,     ic : Icon,        ol: OberonLib,
    e  : Exec,      wb: Workbench,    sys: SYSTEM,
    Cia,io,R;

  VAR
    chan    : SHORTINT;
    bpm,bpt : LONGINT;
    cnt,nr  : INTEGER;
    skip    : INTEGER;
    icon    : wb.DiskObjectPtr;
    int     : e.Interrupt;
    eClock  : LONGINT;
    intOn   : BOOLEAN;
    str     : e.STRING;
    tt      : e.STRPTR;


  (* $SaveRegs+ $StackChk- $RangeChk- $OvflChk- $NilChk- *)

  PROCEDURE * MyIntProc;
    CONST (* $DataChip+ *)
      wave = "\x00\x7F\x00\x80"; (* Dreieckwelle *)
    VAR
      per,cyc: INTEGER;
  BEGIN
    sys.SETREG(R.A5,sys.REG(R.A1)); (* VarBase setzen (fr small data) *)
    IF cnt = 0 THEN
      per := 1200; cyc := 80;
      IF nr = 0 THEN per := per DIV 2; cyc := cyc * 2 END;
      as.PlaySound(chan,sys.ADR(wave),4,per,64,cyc);
      INC(nr); IF nr = bpt THEN nr := 0 END;
    END;
    INC(cnt); IF cnt = skip THEN cnt := 0 END;
  END MyIntProc;

  (* $StackChk= $RangeChk= $OvflChk= $NilChk= *)


  PROCEDURE SetIntRate(intPerSec: REAL);
    VAR rateL: LONGINT;
        rate : INTEGER;
  BEGIN
    skip := 1;
    LOOP
      rateL := ENTIER(eClock/intPerSec+0.5);
      IF rateL < 010000H THEN EXIT END;
      skip := skip * 2;
      intPerSec := intPerSec * 2;
    END;
    (* $RangeChk- *)
    rate := SHORT(rateL);
    hw.ciab.talo := SHORT(sys.VAL(INTEGER,sys.VAL(SET,rate) * {0..7}));
    hw.ciab.tahi := SHORT(rate DIV 256);
    (* $RangeChk= *)
  END SetIntRate;


  PROCEDURE Assert(cc: BOOLEAN; txt: ARRAY OF CHAR); (* $CopyArrays- *)
  BEGIN
    IF NOT cc THEN
      io.WriteString(txt); io.WriteLn; HALT(0);
    END;
  END Assert;

BEGIN
  intOn := FALSE;

  IF e.exec.libNode.version >= 36 THEN  (* AmigaOS >= 2.0 ? *)
    eClock := e.exec.eClockFrequency;
  ELSE
    IF g.pal IN g.gfx.displayFlags THEN eClock := 709379
                                   ELSE eClock := 715909 END;
  END;

  IF ol.wbStarted THEN

    bpm := MIN(INTEGER);
    bpt := 4;
    arg.GetArg(0,str);

    icon := ic.GetDiskObject(str);
    IF icon # NIL THEN
      tt := ic.FindToolType(icon.toolTypes,"BPM");
      IF tt # NIL THEN
        IF NOT c.StringToInt(tt^,bpm) THEN bpm := MIN(INTEGER) END;
      END;
      tt := ic.FindToolType(icon.toolTypes,"BPT");
      IF tt # NIL THEN
        IF NOT c.StringToInt(tt^,bpt) THEN bpt := 4 END;
      END;
      ic.FreeDiskObject(icon);
    END;

    IF bpm = MIN(INTEGER) THEN
      io.closeDelay := 150;
      Assert(FALSE,"Workbench Usage: Set Tool Types BPM=#, [BPT=#]");
    END;

  ELSE

    arg.GetArg(1,str);
    Assert((arg.NumArgs() # 0) & (str[0] # "?") & c.StringToInt(str,bpm),
           "Usage: Metronom <bpm> [<bpt>]");

    arg.GetArg(2,str);
    IF (str[0]=0X) OR NOT c.StringToInt(str,bpt) THEN bpt := 4 END;

  END;

  Assert((bpm > 0) & (bpm <= 600),"<bpm> value must be 1 - 600");
  Assert((bpt > 0) & (bpt <=  16),"<bpt> value must be 1 - 16");

  Cia.base := e.OpenResource("ciab.resource");
  IF Cia.base=NIL THEN HALT(0) END;

  as.SetPriority(-50);
  chan := as.OpenChannel({});

  int := e.Interrupt(NIL,NIL,e.interrupt,0,NIL,NIL,NIL);
  int.code := MyIntProc;
  int.data := sys.REG(R.A5);

  cnt := 0;
  nr := 0;

  Assert(Cia.AddICRVector(hw.ta,sys.ADR(int)) = NIL,"Ciab timer A in use");
  hw.ciab.cra := SHORTSET{};
  intOn := TRUE;
  SetIntRate(bpm / 60);

  io.WriteString("Metronom by cs-soft (public domain). Press <Ctrl-C> to exit...\n");

  INCL(hw.ciab.cra,hw.craStart);

  sys.SETREG(0,e.Wait(LONGSET{d.ctrlC}));
  io.WriteString("*** BREAK\n");

CLOSE

  IF intOn THEN
    EXCL(hw.ciab.cra,hw.craStart);
    Cia.RemICRVector(hw.ta,sys.ADR(int))
  END;

END Metronom.


