(*
 * :Program.      AClock.mod
 * :Author.       Achim Siebert
 * :Address.      Nobileweg 67, D-7000 Stuttgart 40
 * :Shortcut.     [acs]
 * :Copyright.    FreeWare
 * :Language.     Oberon-2
 * :Translator.   Amiga Oberon Compiler V2.30 (inofficial version)
 * :History.      V1.1: added second hand
 * :Contents.     
 * :Usage.        AClock L=LEFT/N,T=TOP/N,W=WIDTH/N,H=HEIGHT/N,B=BACKCOL/N,S=SECCOL/N,
                  M=MINCOL/N,C=HOURCOL/N,P=POINTCOL/N,F1=FRAMECOL1/N,F2=FRAMECOL2/N,BACKDROP/S,NOSECS/S
 * :Remark.       Kick 37.xx needed
 *)
 
MODULE AClock;

IMPORT 
       D:  Dos,
       M:  MathTrans,
       G:  Graphics,
       GT: GadTools,
       I:  Intuition,
       E:  Exec,
       U:  Utility,
       S:  SYSTEM;

VAR
  rp        : G.RastPortPtr;
  winP      : I.WindowPtr;  
  argnumber : ARRAY 11 OF INTEGER;
  imsg      : I.IntuiMessagePtr;
  imsgClass : LONGSET;
  backdrop,
  secondhand: BOOLEAN;


PROCEDURE Box(rport:G.RastPortPtr;x,y,b,h: INTEGER;col1,col2,fillcol:INTEGER);

VAR x1,xb,yh: INTEGER;

BEGIN

  DEC(b); DEC(h);
  x1 := x+1;
  xb := x+b;
  yh := y+h;
  G.SetAPen(rport,col1);
  G.Move(rport,xb-1,y);
  G.Draw(rport,x,y);
  G.Draw(rport,x,yh);
  G.Move(rport,x1,y);
  G.Draw(rport,x1,yh-1);
  G.SetAPen(rport,col2);
  G.Move(rport,x1,yh);
  G.Draw(rport,xb,yh);
  G.Draw(rport,xb,y);
  G.Move(rport,xb-1,y+1);
  G.Draw(rport,xb-1,yh);
  IF fillcol # -1 THEN
     G.SetAPen(rport,fillcol);
     G.RectFill(rport,x+2,y+1,xb-2,yh-1);
  END;
END Box;


PROCEDURE OpenWindow();

VAR
  myNewWin: I.NewWindow;

BEGIN

    myNewWin := I.NewWindow(
       0,0,                (*  LeftEdge, TopEdge *)
       0,0,                (*  Width, Height *)
       0,0,                (*  DetailPen, BlockPen *)
       LONGSET{I.closeWindow,I.changeWindow,I.refreshWindow,I.activeWindow,I.inactiveWindow}, (*  IDCMPFlags *)
       LONGSET{I.windowSizing,I.windowDepth,I.windowDrag,I.windowClose,I.borderless}, (* Flags *)
       NIL,                (*  FirstGadget *)
       NIL,                (*  CheckMark *)
       NIL,                (*  Title *)
       NIL,                (*  Screen *)
       NIL,                (*  BitMap *)
       42, 20,             (*  MinWidth, MinHeight *)
       -1, -1,             (*  MaxWidth, MaxHeight *)
       {I.wbenchScreen});  (*  Type *)

   IF backdrop THEN
      INCL(myNewWin.flags,I.backDrop);
      EXCL(myNewWin.flags,I.windowSizing);
   END;

   winP := I.OpenWindowTags(myNewWin,
                            I.waWidth,argnumber[2],
                            I.waHeight,argnumber[3],
                            I.waLeft,argnumber[0],
                            I.waTop,argnumber[1],
                            U.done);

   IF winP = NIL THEN HALT(20) END;
   
   rp := winP.rPort;

END OpenWindow;


PROCEDURE Draw();


CONST  pi       = 3.141592653589793;
       pi180stel= pi/180;

VAR x,y: ARRAY 12 OF INTEGER;
    midx,midy,rx,ry,rxl,rxs,ryl,rys,offset: INTEGER;
    hx,hy,hxold,hyold,mx,my,mxold,myold,sx,sy,sxold,syold: INTEGER;
    oldsecs : LONGINT;

   PROCEDURE NewClock();
   
   VAR  date  : D.Date;
        minute, hour, second: INTEGER;

   BEGIN

   
      hxold := hx; hyold := hy;
      mxold := mx; myold := my;
      sxold := sx; syold := sy;

      D.DateStamp(date);
      
      IF secondhand THEN
         second := SHORT(date.tick DIV 50);
      ELSE   
         second := SHORT(date.minute MOD 60);
      END;

      IF oldsecs # second THEN
                  
         oldsecs := second;
         hour := SHORT(date.minute DIV 60);
         minute := SHORT(date.minute MOD 60);
         
         sx := midx + SHORT(SHORT(rxl*(M.Sin((second*6)*pi180stel))));
         sy := midy - SHORT(SHORT(ryl*(M.Cos((second*6)*pi180stel))));
         mx := midx + SHORT(SHORT(rxl*(M.Sin((minute*6+(second DIV 10)-1/6)*pi180stel))));
         my := midy - SHORT(SHORT(ryl*(M.Cos((minute*6+(second DIV 10)-1/6)*pi180stel))));
         hx := midx + SHORT(SHORT(rxs*(M.Sin((hour*30+minute DIV 2)*pi180stel))));
         hy := midy - SHORT(SHORT(rys*(M.Cos((hour*30+minute DIV 2)*pi180stel))));

         G.SetAPen(rp,argnumber[4]);
         G.Move(rp,midx,midy);
         G.Draw(rp,mxold,myold);
         G.Move(rp,midx,midy);
         G.Draw(rp,hxold,hyold);
                  
         IF secondhand THEN
         G.Move(rp,midx,midy);
         G.Draw(rp,sxold,syold);
         END;

         IF secondhand THEN
         G.SetAPen(rp,argnumber[5]);
         G.Move(rp,midx,midy);
         G.Draw(rp,sx,sy);
         END;
         
         G.SetAPen(rp,argnumber[7]);
         G.Move(rp,midx,midy);
         G.Draw(rp,hx,hy);

         G.SetAPen(rp,argnumber[6]);
         G.Move(rp,midx,midy);
         G.Draw(rp,mx,my);
         
         Box(rp,0,0,winP.width,winP.height,argnumber[9],argnumber[10],-1);

      END;

   END NewClock;


PROCEDURE Init();

VAR count : INTEGER;

BEGIN

   midx := (winP.width-1) DIV 2 +1;
   midy := (winP.height-1)DIV 2;
   rx := winP.width  DIV 2 - 6;
   ry := winP.height DIV 2 - 3;
   
   x[9] := midx-rx+1;
   x[3] := midx+rx-1;
   x[0] := midx; x[6] := midx;
   offset := SHORT(SHORT(rx*M.Cos(pi/6))); (* 30 *)
   x[1] := midx+offset;
   x[11]:= midx-offset;   
   x[5] := x[1];
   x[7] := x[11];
   offset := rx DIV 2; (* Cos(pi/3) = 0.5 *)
   x[2] := midx+offset;
   x[10]:= midx-offset;
   x[4] := x[2];
   x[8] := x[10];
   
   y[9] := midy;
   y[3] := midy;
   y[0] := midy-ry+1; y[6] := midy+ry-1;
   offset := ry DIV 2;
   y[1] := midy-offset;
   y[5] := midy+offset;
   y[11]:= y[1];
   y[7] := y[5];
   offset := SHORT(SHORT(ry*M.Sin(pi/3)));
   y[2] := midy-offset;
   y[4] := midy+offset;
   y[10]:= y[2];
   y[8] := y[4];
   
   rxl := rx - 2;
   rxs := rx DIV 3 * 2;
   
   ryl := ry - 2;
   rys := ry DIV 3 * 2;
   
   G.SetDrMd(rp,G.jam1);
   
   Box(rp,0,0,winP.width,winP.height,argnumber[9],argnumber[10],argnumber[4]);

   G.SetAPen(rp,argnumber[8]);
   FOR count := 0 TO 11 DO
      IF G.WritePixel(rp,x[count],y[count]) THEN END;
   END;

   oldsecs := -1;

END Init;

BEGIN

   Init();

   LOOP
      NewClock();
      D.Delay(9);
      LOOP
         imsg := GT.GetIMsg(winP.userPort);
         IF imsg#NIL THEN
            imsgClass := imsg.class;
            IF I.closeWindow IN imsgClass THEN
               GT.ReplyIMsg(imsg);
               HALT(0);
            ELSE
               Init(); oldsecs := -1;
            END;
            GT.ReplyIMsg(imsg);
         ELSE EXIT;
         END;
      END;
   END;

END Draw;


PROCEDURE GetArgs();

VAR  rdargs       : D.RDArgsPtr;
     argarray     : ARRAY 13 OF E.APTR;
     argPtr       : POINTER TO LONGINT;
     i : INTEGER;

BEGIN

 rdargs := D.ReadArgs("L=LEFT/N,T=TOP/N,W=WIDTH/N,H=HEIGHT/N,B=BACKCOL/N,S=SECCOL/N,M=MINCOL/N,C=HOURCOL/N,P=POINTCOL/N,F1=FRAMECOL1/N,F2=FRAMECOL2/N,BACKDROP/S,NOSECS/S",argarray,NIL);
 
 IF rdargs # NIL THEN
    FOR i := 0 TO 10 DO
       IF argarray[i]#0 THEN
          argPtr := argarray[i];
          argnumber[i] := SHORT(argPtr^);
       END;
    END;
    IF argarray[11]#0 THEN backdrop := TRUE END;
    IF argarray[12]#0 THEN secondhand := FALSE END;
 END;
 D.FreeArgs(rdargs);

END GetArgs;


BEGIN

   argnumber[0] := 20; argnumber[1] := 20; argnumber[2] := 80; argnumber[3] := 36;
   argnumber[4] := 0;  argnumber[5] := 3;  argnumber[6] := 2;  argnumber[7] := 1;
   argnumber[8] := 2;  argnumber[9] := 2;  argnumber[10]:= 1;  backdrop := FALSE;
   secondhand := TRUE;
   IF I.int.libNode.version<37 THEN HALT(20) END;
   GetArgs();
   IF argnumber[2]<42 THEN argnumber[2] := 42 END;
   IF argnumber[3]<20 THEN argnumber[3] := 20 END;
   OpenWindow();
   Draw();

CLOSE

   IF winP # NIL THEN I.CloseWindow(winP) END;

END AClock.

