(***************************************************************************
 :Program.    GBComp.mod
 :Author.     Jrgen Weinelt
 :Address.    Zur Kanzel 1, W-8783 Hammelburg, Germany
 :Copyright.  Public Domain, do whatever you want to
 :Language.   Modula-2
 :Translator. M2Amiga
 :History.    V1.0  Weinelt  Oct.90  (never released, first complete version)
 :History.    V1.1  Weinelt  02.Apr.91 (never released, added ARP filereq.)
 :History.    V2.0  Weinelt  11.May.91 (M2Amiga V4.0)
 :History.    V2.1  Weinelt  27.Sep.91 (images up to 160 bit width)
 :Imports.    GFXBits
 :Imports.    FileReq
 :Contents.   tool for converting data strings (defining bitmapped gfx)
 :Contents.   to m2amiga assemble statements
 **************************************************************************)

(*-------------------------------------------------------------------------
   please note: this program uses FileNames.GetExtension(), which contains
                two serious bugs in the 4.0 release. A+L was informed about
                these bugs. they will hopefully be fixed soon.
                if you want to rebuild GBComp from the source, you'll have
                to write a short procedure on your own as a replacement
                for FileNames.GetExtension().
                if you own the extended compiler package, you can fix
                the bugs yourself:
                  edit FileNames.mod
                  replace the following lines:

                    old line #116:   extension[i]:=fname[j]
                    new line #116:   extension[j-i-1]:=fname[j]

                    old line #118:   DEC(len,i);
                    new line #118:   len:=i;

                recompile FileNames.mod. that should be all.
-------------------------------------------------------------------------*)

(*$ StackChk:=FALSE
    ReturnChk:=FALSE
    Volatile:=FALSE
    LargeVars:=FALSE *)



MODULE GBComp;



IMPORT ASCII;
IMPORT Arts;
IMPORT Conversions;
IMPORT FileNames;
IMPORT FileReq;
IMPORT FileSystem;
IMPORT GFXBits;
IMPORT String;
IMPORT SYSTEM;



CONST
 maxPlanes=5;                   (* maximum image depth *)
 maxLines=64;                   (* maximum image height *)



TYPE
 ImgPlane=ARRAY[0..maxLines-1] OF GFXBits.PLine;
 Buffer=ARRAY[0..maxPlanes-1] OF ImgPlane;



VAR
 i: INTEGER;                    (* multi-purpose counter *)
 fnamef: FileReq.FileString;
 fname: GFXBits.DataStr;        (* input file name *)
 outname: GFXBits.DataStr;      (* mod output file name *)
 defname: GFXBits.DataStr;      (* def output file name *)
 procname: GFXBits.DataStr;     (* current procedure name *)
 inf,outf,deff: FileSystem.File;(* in/output files *)
 planes: INTEGER;               (* number of planes for current image *)
 x,y: INTEGER;                  (* width/height of current image *)
 len: INTEGER;                  (* buffer for string lengths *)
 buf: Buffer;                   (* image data buffer *)
 frd: FileReq.FileRequestData;  (* data for ARPFileRequester *)
 dum1: GFXBits.DataStr;         (* string dummy for unneeded proc results *)
 extens: GFXBits.DataStr;       (* holds filename extension *)
 modname: GFXBits.DataStr;      (* holds file name w/out path and extens. *)



(*------------------------------------------------------------------------
   :Semantic.   same as InOut.WriteString, but uses a File as output
   :Input.      f:      output file
   :Input.      s:      string to be put
   :Output.     -
   :Result.     -
  ------------------------------------------------------------------------*)

PROCEDURE WrStr(VAR f: FileSystem.File; s: ARRAY OF CHAR);
 VAR
  act: LONGINT;
 BEGIN
  FileSystem.WriteBytes(f,SYSTEM.ADR(s),String.Length(s),act);
 END WrStr;



(*------------------------------------------------------------------------
   :Semantic.   same as InOut.Write, but uses a File as output
   :Input.      f:      output file
   :Input.      c:      CHAR to be put
   :Output.     -
   :Result.     -
  ------------------------------------------------------------------------*)

PROCEDURE WrChr(VAR f: FileSystem.File; c: CHAR);
 BEGIN
  FileSystem.WriteChar(outf,c);
 END WrChr;



(*------------------------------------------------------------------------
   :Semantic.   convert numeric value to hex string
   :Input.      l:      numeric value
   :Input.      x:      width in bits (currently 16 or 32)
   :Output.     s:      hex string
   :Result.     -
   :Remark.     there seems to be a problem with the 4.0 compiler in
   :Remark.     68881 mode and the Conversions.ValToStr procedure
   :Remark.     (i'm still trying to find out what precisely went wrong)
  ------------------------------------------------------------------------*)

PROCEDURE Bin2Hex(l: LONGCARD; VAR s: ARRAY OF CHAR; x: INTEGER);
 VAR
  wid,i,k: INTEGER;
  hex: ARRAY[0..16] OF CHAR;
 BEGIN
  hex:="0123456789ABCDEF";
  wid:=(x+3) DIV 4;     (* compute number of hex digits *)
  s[wid]:=ASCII.nul;
  REPEAT
   DEC(wid);
   k:=l MOD 16;
   s[wid]:=hex[l MOD 16];
   l:=l DIV 16;
  UNTIL wid=0;
 END Bin2Hex;



(*------------------------------------------------------------------------
   :Semantic.   read a single line from the input file
   :Input.      f:      File structure
   :Output.     l:      line
   :Result.     -
  ------------------------------------------------------------------------*)

PROCEDURE GetLine(VAR f: FileSystem.File; VAR l: GFXBits.DataStr);
 VAR
  i: INTEGER;
 BEGIN
  i:=0;
  REPEAT
   FileSystem.ReadChar(f,l[i]);
   INC(i);
  UNTIL ((l[i-1]=ASCII.lf) OR (l[i-1]=ASCII.cr) OR (f.eof));
  l[i-1]:=ASCII.nul;
 END GetLine;



(*------------------------------------------------------------------------
   :Semantic.   read a single numeric value (ascii representation)
   :Semantic.   from the input file
   :Input.      f:      input file
   :Output.     -
   :Result.     numeric value or -1 if failed
  ------------------------------------------------------------------------*)

PROCEDURE GetNumeric(VAR f: FileSystem.File): INTEGER;
 VAR
  n: LONGINT;
  s: GFXBits.DataStr;
  err,sign: BOOLEAN;
 BEGIN
  GetLine(f,s);
  sign:=FALSE;
  Conversions.StrToVal(s,n,sign,10,err);
  IF (err OR (n>32767) OR sign) THEN
   RETURN -1;
  END;
  RETURN INTEGER(n);
 END GetNumeric;



(*------------------------------------------------------------------------
   :Semantic.   read the image data strings for a single image from the
   :Semantic.   input file and convert them to their binary
   :Semantic.   representation; store the result in the global data buffer
   :Input.      f:      input file
   :Input.      p:      number of planes expected
   :Input.      x:      image width expected (currently just 16 or 32)
   :Input.      y:      image height expected (currently no more than 64)
   :Output.     -
   :Result.     -
  ------------------------------------------------------------------------*)

PROCEDURE GetData(VAR f: FileSystem.File; p,x,y: INTEGER);
 VAR
  i,j: INTEGER;
  l: GFXBits.DataStr;
  line: GFXBits.MLine;
 BEGIN
  FOR i:=0 TO y-1 DO
   GetLine(f,l);
   Arts.Assert(String.Length(l)=x,SYSTEM.ADR("ERROR: illegal line length!"));
   GFXBits.DecodeLine(l,line);
   FOR j:=0 TO p-1 DO
    buf[j][i]:=line[j];
   END;
  END;
 END GetData;



(*------------------------------------------------------------------------
   :Semantic.   write a single cardinal (or longcard) to the output file
   :Semantic.   x determines the size of the numeric
   :Input.      k:      numeric to be written
   :Input.      x:      length in bits (currently just 16 or 32)
   :Output.     -
   :Result.     -
  ------------------------------------------------------------------------*)

PROCEDURE PutNumeric(k: LONGCARD; x: INTEGER);
 VAR
  s: GFXBits.DataStr;
 BEGIN
  CASE x OF
   |16: Bin2Hex(k,s,x);
   |32: Bin2Hex(k,s,x);
   |ELSE RETURN;
  END;
  WrChr(outf,"$"); WrStr(outf,s);
 END PutNumeric;



(*------------------------------------------------------------------------
   :Semantic.   write the entire image data (including procedure stuff)
   :Semantic.   to the output file; image data is in global data buffer
   :Input.      p:      number of planes
   :Input.      x:      number of bits (currently no more than 160)
   :Input.      y:      number of lines (currently no more than 64)
   :Input.      procname: procedure name for the image
   :Output.     -
   :Result.     -
  ------------------------------------------------------------------------*)

PROCEDURE PutData(p,x,y: INTEGER; procname: ARRAY OF CHAR);
 VAR
  h,i,j,wpl: INTEGER;
 BEGIN
  WrStr(outf,"\n(*$ EntryExitCode:=FALSE *)\n");
  WrStr(outf,"PROCEDURE "); WrStr(outf,procname); WrStr(outf,";\n");
  WrStr(deff,"PROCEDURE "); WrStr(deff,procname); WrStr(deff,";\n");
  WrStr(outf," BEGIN\n");
  FOR j:=0 TO p-1 DO
   WrStr(outf,"  (*===== plane #"); WrChr(outf,CHAR(j+48));
   WrStr(outf," =====*)\n");
   WrStr(outf,"  SYSTEM.ASSEMBLE(\n");
   WrStr(outf,"   DC.W\n");
   WrStr(outf,"    ");
   FOR i:=0 TO y-1 DO
    wpl:=(x+15) DIV 16;
    FOR h:=0 TO wpl-1 DO
     PutNumeric(buf[j][i][h],16);
     IF (i<y-1) OR (h<wpl-1) THEN
      WrChr(outf,",");
      IF ((i*wpl+h) MOD 8)=7 THEN
       WrStr(outf,"\n    ");
      END;
     ELSE
      WrStr(outf,"\n  END);\n");
     END;
    END;
   END;
(* if i<y-1 then wrln end; *)
  END;
  WrStr(outf," END "); WrStr(outf,procname); WrStr(outf,";\n");
 END PutData;



(*------------------------------------------------------------------------
   :Semantic.   replacement for the FileNames.GetExtension procedure
   :Semantic.   which seems to have a bug
   :Input.      n:      filename including extension
   :Output.     x:      extension (without "." char)
   :Output.     n:      filename without extension and "." char
   :Result.     -
  ------------------------------------------------------------------------*)

PROCEDURE GetExtension(VAR n,x: GFXBits.DataStr);
 VAR
  pos: LONGINT;
  i: LONGINT;
 BEGIN
  x:="";
  pos:=String.FirstPos(n,0,".");
  IF pos#String.noOccur THEN
   FOR i:=pos+1 TO String.Length(n) DO
    x[i-pos-1]:=n[i];
   END;
   n[pos]:=ASCII.nul;
  END;
 END GetExtension;



BEGIN
 FileReq.MakeFRD("Select input file","","",NIL,100,20,frd);
 FileReq.FileReq(frd,fnamef);

 String.Copy(fname,fnamef);
 IF String.Length(fname)>0 THEN
  outname:=fname;
  modname:=fname;
  len:=String.Length(modname);
  FileNames.GetPath(modname,dum1,len);
  len:=String.Length(modname);
  FileNames.GetExtension(modname,extens,len);
  len:=String.Length(outname);
  FileNames.GetExtension(outname,dum1,len);
  String.Copy(defname,outname);
  String.Concat(outname,".mod");
  String.Concat(defname,".def");

  FileSystem.Lookup(inf,fname,10000,FALSE);
  Arts.Assert(inf.res=FileSystem.done,SYSTEM.ADR("can't open input file!"));

  FileSystem.Lookup(outf,outname,10000,TRUE);
  Arts.Assert(outf.res=FileSystem.done,SYSTEM.ADR("can't open output file!"));

  FileSystem.Lookup(deff,defname,10000,TRUE);
  Arts.Assert(deff.res=FileSystem.done,SYSTEM.ADR("can't open def file!"));

  WrStr(outf,"IMPLEMENTATION MODULE "); WrStr(outf,modname); WrStr(outf,";\n");
  WrStr(outf,"(* created with GBComp V2.11 (PD)1991 by JOW *)\n");
  WrStr(outf,"(*$ ChipCODE:=TRUE *)\n");
  WrStr(outf,"\n\n\nIMPORT SYSTEM;\n\n\n");

  WrStr(deff,"DEFINITION MODULE "); WrStr(deff,modname); WrStr(deff,";\n");
  WrStr(deff,"(* created with GBComp V2.11 (PD)1991 by JOW *)\n");
  WrStr(deff,"\n\n\n(*$ ChipCODE:=TRUE *)\n\n\n\n");


  REPEAT
   planes:=GetNumeric(inf);
   x:=GetNumeric(inf);
   y:=GetNumeric(inf);
   GetLine(inf,procname);

   Arts.Assert((planes>0) OR inf.eof,SYSTEM.ADR("planes<1!"));
   Arts.Assert(planes<=maxPlanes,SYSTEM.ADR("too many planes!"));
   Arts.Assert(((x>7) & (x<161)) OR inf.eof,SYSTEM.ADR("x<8 or x>160!"));
   Arts.Assert((y>0) OR inf.eof,SYSTEM.ADR("y<1!"));
   Arts.Assert(y<=maxLines,SYSTEM.ADR("y too large!"));

   IF NOT inf.eof THEN
    GetData(inf,planes,x,y);
    PutData(planes,x,y,procname);
   END;

  UNTIL inf.eof;

  WrStr(outf,"\n\n\nEND "); WrStr(outf,modname); WrStr(outf,".mod\n");
  WrStr(deff,"\n\n\nEND "); WrStr(deff,modname); WrStr(deff,".def\n");
 END;
END GBComp.
