(*---------------------------------------------------------------------------
  :Program.     Print2.mod
  :Contents.    Zweispaltiger Ausdruck
  :Author.      Christian Stiens
  :Address.     Heustiege 2, W-4710 Ldinghausen
  :Copyright.   public domain
  :Language.    Oberon-2
  :Translator.  Amiga Oberon V2.25d
  :History.     V1.0, 04-Mar-92
---------------------------------------------------------------------------*)

MODULE Print2;

  IMPORT
    a  : Arguments,
    c  : Conversions,
    d  : Dos,
    e  : Exec,
    fr : FileReq,
    fs : FileSystem,
    I  : Intuition,
    sys: SYSTEM,
         io,Break;

  CONST
    maxLines = 100;

  VAR
    numargs   : INTEGER;
    arg       : e.STRING;
    filename  : e.STRING;
    condOn    : e.STRING;
    str       : e.STRING;
    condLen   : INTEGER;
    file      : fs.File;
    ff        : BOOLEAN;
    head      : BOOLEAN;
    prt       : d.FileHandlePtr;
    ch        : CHAR;
    num       : LONGINT;
    n,p,z,s,i : INTEGER;
    tab       : INTEGER;
    pageNr    : LONGINT;
    left,right: INTEGER;
    numLines  : INTEGER;
    page      : UNTRACED POINTER TO ARRAY maxLines,2,80 OF CHAR;
    pref      : I.Preferences;
    ver       : ARRAY 40 OF CHAR;

  PROCEDURE Usage;
  BEGIN
    io.WriteString(
      "Print2 by cs-soft in 1992\n"
      "Usage: Print2 [filename] [-f|h|c##]\n"
      "    f = No form feed after last page\n"
      "    h = No header line\n"
      "    c = Code for 'condensed on' in hex, default is 0F (epson)\n");
    HALT(0)
  END Usage;

  PROCEDURE FReq(VAR filename: ARRAY OF CHAR);
  BEGIN
    IF NOT fr.FileReq("Print2: Select file",filename) THEN HALT(0) END;
  END FReq;

  PROCEDURE ClearPage;
  BEGIN
    FOR z := 0 TO numLines-1 DO
      FOR s := 0 TO 79 DO
        page[z,0,s] := " ";
        page[z,1,s] := " ";
      END;
    END;
    z := 0; s := left; p := 0;
  END ClearPage;

  PROCEDURE PrintPage (last: BOOLEAN);
    TYPE
      MonatsLaengen = ARRAY 12 OF INTEGER;
    VAR
      header: ARRAY 80 OF CHAR;
      len: LONGINT;
      date: d.Date;
      i: LONGINT;
      n,j: INTEGER;
      s: ARRAY 16 OF CHAR;
      zeile: POINTER TO ARRAY 2*80 OF CHAR;
      tag,monat,jahr: INTEGER;
      mlaengen : MonatsLaengen;
  BEGIN
    INC(pageNr);
    IF head THEN
      IF (d.Write(prt,"\e[1w",4) # 4) OR  (* Elite off *)
         (d.Write(prt,"\e[3w",4) # 4) OR  (* Condensed off *)
         (d.Write(prt,"\e[1;80s",7) # 7)  (* left/right margin *)
      THEN HALT(20) END;
      header := "FILE:                                   DATE: xx.xx.xxxx xx:xx       PAGE: 000";
      len :=io.Length(filename); IF len > 35 THEN len := 35 END;
      e.CopyMem(filename,header[6],len);
      IF c.IntToStr(pageNr,s,10,3,"0") THEN e.CopyMem(s,header[75],3) END;
      d.DateStamp(date);
      tag := 1; monat := 1; jahr := 1978;
      mlaengen := MonatsLaengen(31,28,31,30,31,30,31,31,30,31,30,31);
      FOR i := 1 TO date.days DO
        INC(tag);
        IF tag > mlaengen[monat-1] THEN
          tag := 1; INC(monat);
          IF monat > 12 THEN
            monat := 1; INC(jahr); mlaengen[1] := 28;
            IF (jahr MOD 4=0) & ((jahr MOD 100#0) OR (jahr MOD 400=0)) THEN
              INC(mlaengen[1])
            END
          END
        END
      END;
      IF c.IntToStr(tag  ,s,10,2,"0") THEN e.CopyMem(s,header[46],2) END;
      IF c.IntToStr(monat,s,10,2,"0") THEN e.CopyMem(s,header[49],2) END;
      IF c.IntToStr(jahr ,s,10,4,"0") THEN e.CopyMem(s,header[52],4) END;
      IF c.IntToStr(date.minute DIV 60,s,10,2,"0") THEN e.CopyMem(s,header[57],2) END;
      IF c.IntToStr(date.minute MOD 60,s,10,2,"0") THEN e.CopyMem(s,header[60],2) END;
      IF d.Write(prt,header,78) <= 0 THEN HALT(20) END;
      IF d.Write(prt,"\n\n",2) # 2 THEN HALT(20) END;
    END;

    IF (d.Write(prt,"\e[2w",4) # 4) OR      (* Elite on *)
       (d.Write(prt,condOn,condLen)<=0) OR  (* Condensed on *)
       (d.Write(prt,"\e[1;160s",8) # 8)     (* left/right margin *)
    THEN HALT(20) END;
    IF p = 0 THEN n := z
             ELSE n := numLines-1 END;
    FOR z := 0 TO n DO
      Break.CheckBreak;
      zeile := sys.ADR(page[z]);
      j := 159;
      WHILE (j >= 0) & (zeile^[j] = " ") DO
        zeile^[j] := 0AX;
        DEC(j)
      END;
      IF j<159 THEN INC(j) END;
      IF d.Write(prt,zeile^,j+1) <= 0 THEN HALT(20) END;
    END;
    IF pref.paperType = I.single THEN
      IF NOT last THEN
        io.WriteString("Please put in a new sheet. Press <RETURN> when ready. ");
        io.ReadString(str);
      END;
    ELSE
      IF NOT (last & NOT ff) THEN
        IF d.Write(prt,"\f",1) # 1 THEN HALT(20) END;
      END;
    END;
    ClearPage;
  END PrintPage;

  PROCEDURE NewPage;
    VAR i: INTEGER;
  BEGIN
    z := 0;
    INC(p);
    IF p >= 2 THEN PrintPage(FALSE) END;
  END NewPage;

  PROCEDURE NewLine;
  BEGIN
    s := left;
    INC(z);
    IF z >= numLines THEN NewPage END;
  END NewLine;

  PROCEDURE GetCode;
    VAR ch : CHAR;
        code,n,j : INTEGER;
  BEGIN
    code := 0; n := 0; j := 6;
    LOOP
      IF i+1 >= SIZE(arg) THEN EXIT END;
      ch := CAP(arg[i+1]);
      IF (ch >= "0") & (ch <= "9") THEN
        code := code * 16 + (ORD(ch)-ORD("0"));
      ELSIF (ch >= "A") & (ch <= "F") THEN
        code := code * 16 + (ORD(ch)-ORD("A")+10);
      ELSE
        EXIT
      END;
      INC(n);
      IF n = 2 THEN
        condOn[j] := CHR(code); INC(j);
        n := 0; code := 0;
      END;
      INC(i);
    END;
    IF (j<=6) OR ODD(n) THEN Usage END;
    condLen := j;
    DEC(j,6);
    condOn[2] := CHR(j DIV 10 + ORD("0"));
    condOn[3] := CHR(j MOD 10 + ORD("0"));
  END GetCode;


BEGIN (* Print2 *)
  ver := "$VER: print2 1.0 (4.3.92)\n\r";

  prt := NIL;

  NEW(page);
  IF page = NIL THEN
    io.WriteString("Insufficient free store\n");
    HALT(20)
  END;

  numargs := a.NumArgs();

  a.GetArg(1,arg); IF arg[0]="?" THEN Usage END;

  filename := "";
  condOn := '\e[01"r\x0F';
  condLen := 7;

  ff := TRUE; head := TRUE;

  FOR n := 1 TO numargs DO
    a.GetArg(n,arg);
    IF arg[0] = "-" THEN
      i := 0;
      LOOP
        INC(i);
        IF i >= SIZE(arg) THEN EXIT END;
        CASE CAP(arg[i]) OF
          "F": ff := FALSE |
          "H": head := FALSE |
          "C": GetCode |
          0X : EXIT
            ELSE Usage;
        END;
      END;
    ELSE (* arg[0] # "-" *)
      IF filename[0] # 0X THEN Usage END;
      filename := arg
    END;
  END;

  n := io.Length(filename)-1; IF n<0 THEN n:=0 END;
  IF (filename[0]=0X) OR (filename[n]="/") OR (filename[n]=":") THEN
    FReq(filename)
  END;

  IF NOT fs.Open(file,filename,FALSE) THEN
    io.WriteString("Can't open the file\n");
    HALT(20)
  END;

  prt := d.Open("PRT:",d.newFile);

  IF prt = NIL THEN
    io.WriteString("Can't open the printer\n");
    HALT(20)
  END;

  I.GetPrefs(pref,SIZE(pref));
  left := pref.printLeftMargin-1; IF left < 0 THEN left := 0 END;
  right := pref.printRightMargin; IF right > 80 THEN right := 80 END;
  IF pref.paperLength < 10 THEN
    io.WriteString("Bad paper length\n"); HALT(20)
  END;
  IF pref.printSpacing = I.eightLPI THEN
    pref.paperLength := SHORT(ENTIER(pref.paperLength * 1.334));
  END;
  numLines := SHORT(ENTIER(pref.paperLength * 0.9));
  IF head THEN DEC(numLines,2) END;
  IF numLines > maxLines THEN numLines := maxLines END;

  IF (d.Write(prt,"\e#1",3)  # 3) OR   (* Initialize *)
     (d.Write(prt,"\e[0q",4) # 4) OR   (* no perf skip *)
     (d.Write(prt,"\e[1p",4) # 4)      (* proportional off *)
  THEN HALT(20) END;

  pageNr := 0;
  tab := 0;

  ClearPage;

  LOOP
    Break.CheckBreak;

    IF tab > 0 THEN
      ch := " ";
      DEC(tab);
    ELSE
      IF NOT fs.ReadChar(file,ch) THEN EXIT END;
    END;

    CASE ch OF
      007X: I.DisplayBeep(NIL) |
      009X: tab := 8 - s MOD 8 |
      00AX: NewLine |
      00CX: NewPage |
      01BX: |
      09BX: |
    ELSE
      IF ch >= " " THEN
        page[z,p,s] := ch;
        INC(s);
        IF s>=right THEN NewLine END;
      END;
    END;

  END;

  PrintPage(TRUE);

CLOSE

  IF prt # NIL THEN d.OldClose(prt) END;

END Print2.

