(**********************************************************************

    Modul      :  UserLineIO, V1.0
    Author     :  Raymond Horn
    Contents   :  Liest eine Eingabezeile vom Benutzer.
                  Die in der Eingabezeile enthaltenden Worte knnen
                  mit GetNextArg nacheinander abgerufen werden.

**********************************************************************)

IMPLEMENTATION MODULE UserLineIO;

FROM  Heap        IMPORT   Allocate, Deallocate;
FROM  Terminal    IMPORT   WriteLn, WriteString, ReadLn;
FROM  String      IMPORT   CopyPart, Copy;

IMPORT   ASCII;

CONST MaxZeilLen  =  160;

TYPE  ZeilenTyp            =  ARRAY [0..MaxZeilLen] OF CHAR;

      Eintragzeiger        =  POINTER TO Eintrag;
      Eintrag              =  RECORD
                                 Argument    :  ZeilenTyp;
                                 next        :  Eintragzeiger;
                              END;

VAR   Befehlszeile            :  ZeilenTyp;
      Pos, Laenge             :  INTEGER;
      valid                   :  BOOLEAN;
          (* zeigt an, ob gerade gltige Daten gespeichert sind *)

      Erster, Naechster       :  Eintragzeiger;

PROCEDURE ClearMem();
(* Lscht den  bisher Reservierten Speicher und setzt 'valid' auf falsch *)
(* 'Erster' Eintrag bleibt erhalten *)
   VAR Aktueller  :  Eintragzeiger;
   BEGIN
      Naechster := Erster^.next;
      WHILE Naechster # NIL DO
         Aktueller := Naechster;
         Naechster := Naechster^.next;
         Deallocate( Aktueller )
      END;  (* WHILE *)
      Naechster := Erster;
      valid := FALSE
   END ClearMem;

PROCEDURE SkipSpaces();
   BEGIN
      WHILE Befehlszeile[ Pos ] = ASCII.sp DO INC( Pos ) END
   END SkipSpaces;

PROCEDURE LenOfNextArg() : INTEGER;
   VAR Zaehler :  INTEGER;
   BEGIN
      Zaehler := 0;
      SkipSpaces;
      WHILE ( Befehlszeile[ Pos + Zaehler ] # ASCII.sp )
         AND ( Pos + Zaehler < Laenge ) DO
         INC( Zaehler )
      END;  (* WHILE *)
      RETURN Zaehler
   END LenOfNextArg;

PROCEDURE GetBefZeile( EingabeText : ARRAY OF CHAR ) : INTEGER;
(* Holt eine Befehlszeile vom Benutzer *)
(* Die Anzahl der Argumente in der Befehlszeile wird zurckgegeben *)
VAR      Len, Zaehler   :  INTEGER;
         Wort           :  ZeilenTyp;
   BEGIN
      IF Erster^.next # NIL THEN ClearMem; END;
      WriteString( EingabeText );
      ReadLn( Befehlszeile, Laenge );
      Pos := 0;
      Zaehler := 0;
      Len := LenOfNextArg();
      IF Len # 0 THEN
         valid := TRUE;
         CopyPart( Erster^.Argument, Befehlszeile, Pos, Len );
         INC( Zaehler );
         INC( Pos, Len );
         Len := LenOfNextArg();
         WHILE Len # 0 DO
            CopyPart( Wort, Befehlszeile, Pos, Len );
            Allocate( Naechster^.next, SIZE( Eintrag ) );
            IF Naechster^.next = NIL THEN
               WriteString( "*** Nicht mehr genug Speicher! " );
               WriteString( Wort ); WriteString( " ignoriert! ***\n" );
               Naechster := Erster;
               RETURN Zaehler
            END;
            Naechster := Naechster^.next;
            Naechster^.Argument := Wort;
            INC( Zaehler );
            INC( Pos, Len );
            Len := LenOfNextArg()
         END;   (* WHILE *)
         Naechster^.next := NIL;
         Naechster := Erster
      END;  (* IF *)
      RETURN Zaehler
   END GetBefZeile;

PROCEDURE GetNextArg( VAR Argument : ARRAY OF CHAR; VAR last : BOOLEAN );
(* Gibt das naechste in der Befehlszeile enthaltene Argument zurck
   und zeigt an, ob es das letzte gewesen ist. *)
   BEGIN
      IF valid THEN Copy( Argument, Naechster^.Argument )
      ELSE Argument[ 0 ] := ASCII.nul END;  (* IF *)
      IF Naechster^.next = NIL THEN (* letztes Argument wurde gelesen *)
         ClearMem;
         last := TRUE
      ELSE                 (* es gibt noch mehr Argumente *)
         Naechster := Naechster^.next;
         last := FALSE
      END  (* IF *)
   END GetNextArg;

BEGIN

   Allocate( Erster, SIZE( Eintrag ) );
   Naechster := Erster;
   valid := FALSE

END UserLineIO.
