(**********************************************************************

    Modul      :  ZPAusgabe, V1.0
    Author     :  Raymond Horn
    Contents   :  Modul fr 'ZapPrint'
                  Ausdruck, Anzeigen oder Statistik ber eine Textdatei
                  entsprechend der von ZPEingabe importierten
                  Druckparameter ( 'PPars' und 'Kopf' ).

**********************************************************************)

IMPLEMENTATION MODULE ZPAusgabe;

FROM  SYSTEM      IMPORT   LONGSET;
FROM  DosD        IMPORT   ctrlC, ctrlD, ctrlE;
FROM  Break       IMPORT   SetBreakMode, GetBreak, BreakText, InstallException;

FROM  InOut       IMPORT   Read, Write, WriteCard, done,
                           SetInput, CloseInput, SetOutput, CloseOutput;
FROM  Terminal    IMPORT   WriteLn, WriteInt, WriteString, ReadLn;
FROM  String      IMPORT   Length, Copy, Compare;
FROM  Conversions IMPORT   ValToStr;

FROM  ZPEingabe   IMPORT   PPars, Quiet;
FROM  ZPKopfdruck IMPORT   DruckeKopf;

IMPORT ASCII;

TYPE  Fehlerart               = ( Drucker, Datei, Argument, Escape );
(*    DruckDichten            = [ 1..6 ];
      DruckDichtenName        = ARRAY DruckDichten OF ARRAY [0..9] OF CHAR;
*)

CONST NummernPlatz            = 3;

VAR   Position,
      AktZeile, AktSeite,       (* relative Zhlung auf der aktuellen Seite *)
      Zeilenspeicher,           (* Speicher zu Druckbegin die Nummer der
                                   aktuellen Zeile zum Vergleich beim
                                   nchsten Aufruf. Ist manchmal notwendig
                                   fr pagenauer Drucken ungerade -> gerade *)
      Zeilennummer,
      Zeilenanzahl, Seitenanzahl,  (* gesamtzhlung fr Statistik *)
      Zeichenanzahl              : CARDINAL; (* absolute Zhlung *)

      EndeNachZeile,
      EndeNachSeite, Stop,
      Druckerflag, Dateiflag,
      NeueZeile, NeueSeite       : BOOLEAN;

      Name                    : ARRAY [0..79] OF CHAR;
            (* enthlt den aktuellen Dateinamen *)

      Zeichen                 : CHAR;
            (* enthlt das aktuell bearbeite Zeichen *)

(*      CPI                     : DruckDichten;
      DruckDichte             : DruckDichtenName;
*)
      BreakMode               : LONGSET;

PROCEDURE InitVars();
   BEGIN

      Position       := 0;       (* Pos in aktueller Zeile *)
      AktSeite       := 1;       (* Nummer der aktuellen Seite *)
      NeueZeile      := FALSE;   (* Flag fr Zeilenvorschub *)
      NeueSeite      := FALSE;   (* Flag fr Seitenvorschub *)
      Zeichenanzahl  := 0;       (* alle gedruckten Zeichen fr Statistik *)
      Zeilenanzahl   := 0;       (* alle gedruckten Zeilen fr Statistik *)
      Seitenanzahl   := 0;       (* alle gedruckten Seiten fr Statistik *)
      Zeilennummer   := 0;
      EndeNachZeile  := FALSE;
      EndeNachSeite  := FALSE;
      Dateiflag      := FALSE;
      Druckerflag    := FALSE;
      Stop           := FALSE;
   END InitVars;

PROCEDURE Fehler( Art : Fehlerart );
   BEGIN
      CASE Art OF
         Drucker  :  WriteString( "*** Konnte Drucker nicht ffnen!\n");
                     Stop := TRUE                                       |
         Datei    :  WriteString( "*** Konnte Datei '" );
                     WriteString( Name );
                     WriteString( "' nicht ffnen! ***\n" )             |
         Argument :  WriteString( " *** Kein Argument! ***\n" )         |
         Escape   :  WriteString( "*** Escapesequence nicht bekannt ! ***\n" )
      END (* CASE *)
   END Fehler;

PROCEDURE GibStatistik;
   BEGIN
      WriteInt( Zeichenanzahl, 1 );WriteString( " Zeichen in " );
      WriteInt( Zeilenanzahl, 1 );WriteString( " Zeilen auf " );
      WriteInt( Seitenanzahl, 1 );WriteString( " Seiten.\n\n" )
   END GibStatistik;
(*
PROCEDURE SetCPI();
   BEGIN
      CASE PPars.Zeichendichte OF
          5    :   CPI := 1   |
          6    :   CPI := 2   |
          10   :   CPI := 3   |
          12   :   CPI := 4   |
          17   :   CPI := 5   |
          20   :   CPI := 6
      ELSE
         WriteString( "*** Zeichendichteneingabe ignoriert! ***\n" )
      END (* CASE *)
   END SetCPI;
*)
PROCEDURE AbbruchTest;
   BEGIN
      BreakMode := GetBreak();
      IF ctrlD IN BreakMode THEN
         WriteString( BreakText );
         EndeNachZeile := TRUE;
      ELSIF ctrlE IN BreakMode THEN
         WriteString( "*** Drucke nur noch Seite " );
         WriteInt( AktSeite, 1 ); WriteString( "! ***\n" );
         EndeNachSeite := TRUE;
      END   (* IF 4 *)
   END AbbruchTest;

PROCEDURE OeffneDatei( Name : ARRAY OF CHAR ) : BOOLEAN;
(* ffnet die gewnschte Datei. Wenn es nicht klappt -> FALSE *)
   BEGIN
      IF Length( Name ) > 0 THEN
         SetInput( Name );
         IF done THEN Dateiflag := TRUE END
      ELSE Fehler( Argument )
      END;  (* IF *)
      RETURN Dateiflag
   END OeffneDatei;

PROCEDURE AllesSchliessen;
   BEGIN
      IF Dateiflag THEN CloseInput; Dateiflag := FALSE END;
      IF Druckerflag THEN CloseOutput; Druckerflag := FALSE END;
   END AllesSchliessen;

PROCEDURE IsDigit( VAR C : CHAR) : BOOLEAN;
(* berprft ob C eine Ziffer ist *)
   BEGIN
      RETURN ( C >= '0' ) AND ( C <= '9' )
   END IsDigit;

PROCEDURE IsLowerCase( VAR C : CHAR) : BOOLEAN;
(* berprft ob C eine Ziffer ist *)
   BEGIN
      RETURN ( C >= 'a' ) AND ( C <= 'z' )
   END IsLowerCase;

PROCEDURE Space( n : INTEGER; wirklich : BOOLEAN );
(* Fgt n Leerzeichen ein. Zeilen- und Seitenumbruch wird nicht beachtet !!! *)
   VAR   Zaehler   : INTEGER;
   BEGIN
      IF wirklich THEN
         FOR Zaehler := 1 TO n DO Write( ASCII.sp ) END;
      END;
      INC( Position, n )
   END Space;

PROCEDURE ZehnHoch( n : CARDINAL ) : CARDINAL;
   VAR Wert, Zaehler  : CARDINAL;
   BEGIN
      Wert := 1;
      FOR Zaehler := 1 TO n DO Wert := Wert * 10 END;
      RETURN Wert
   END ZehnHoch;

PROCEDURE DruckeNummer( wirklich : BOOLEAN ) : CARDINAL;
(* Druck bei Bedarf die Zeilennummer und gibt den bentigten Platz zurck *)
   VAR Zaehler  :  CARDINAL;
   BEGIN
      IF wirklich THEN
         INC( Zeilennummer );
         WriteCard( Zeilennummer, NummernPlatz );
         Write( ASCII.sp )
      END;
      Zaehler := ( NummernPlatz+1 );
      IF Zeilennummer >= ZehnHoch( NummernPlatz ) THEN INC( Zaehler ) END;
      IF Zeilennummer >= ZehnHoch( NummernPlatz+1 ) THEN INC( Zaehler ) END;
      RETURN Zaehler
   END DruckeNummer;

PROCEDURE PseudoFormFeed;
   BEGIN
      Write( ASCII.eol );
      WriteString( "-----ff-----\n" );
   END PseudoFormFeed;

PROCEDURE DruckerFormFeed;
   BEGIN
      Write( ASCII.eol );
      Write( ASCII.ff );
   END DruckerFormFeed;

PROCEDURE MacheFormFeed( Wo : Darstellungsart );
   VAR lokalesDruckerflag  :  BOOLEAN;
   BEGIN
      CASE Wo OF
         druck :  IF NOT Druckerflag THEN
                     SetOutput( "PRT:" );
                     IF done THEN lokalesDruckerflag := TRUE
                     ELSE Fehler( Drucker ); RETURN
                     END (* IF 2 *)
                  END; (* IF *)
                  DruckerFormFeed;
                  IF lokalesDruckerflag THEN CloseOutput END |
         zeige,
         stat  :  PseudoFormFeed
      END;   (* CASE *)
      INC( AktSeite );
      AktZeile := 1;          (* Zeilennummer in der aktuellen Seite *)
      Position := 0
   END MacheFormFeed;

PROCEDURE MacheLineFeed( Wo : Darstellungsart );
(* Schickt ein LF. Behandelt auch die notwendige Buchfhrung *)
   VAR lokalesDruckerflag  :  BOOLEAN;
   BEGIN
      IF ( Wo = druck ) AND NOT Druckerflag THEN
         SetOutput( "PRT:" );
         IF done THEN lokalesDruckerflag := TRUE
         ELSE Fehler( Drucker ); RETURN
         END  (* IF 2 *)
      END; (* IF *)
      Write( ASCII.lf );
      INC( AktZeile );
      Position := 0;
      IF lokalesDruckerflag THEN CloseOutput END
   END MacheLineFeed;

PROCEDURE SendESC;
(* sendet Druckersteuercode und gibt ein neues Zeichen aus
   dem Eingabestrom zurck *)
   BEGIN
      Write( Zeichen ); Read( Zeichen );
      CASE Zeichen OF
         'c','A'..'M'   :  |
         '#','('        :  Write( Zeichen ) ; Read( Zeichen )   |
         '['            :  WHILE NOT IsLowerCase( Zeichen ) DO
                              Write( Zeichen ); Read( Zeichen )
                           END  (* WHILE *)                    |
      ELSE Fehler( Escape ); RETURN
      END; (* CASE *)
      Write( Zeichen ); Read( Zeichen )
   END SendESC;

PROCEDURE CheckChar;
(* Untersucht ein Zeichen. Setzt 'NeueZeile' bzw. 'NeueSeite',
   wenn das Zeilen bzw. Seitenende erreicht wurde. *)
   BEGIN
      CASE Zeichen OF
         ASCII.esc            :  SendESC; CheckChar   |
         ASCII.eof            :  Stop := TRUE         |
         ASCII.ff             :  NeueSeite := TRUE;
                                 Zeichen := ASCII.nul |
                                    (* unterdrcke ff durch Zeichen *)
         ASCII.lf,ASCII.cr    :  NeueZeile := TRUE;
                                 IF AktZeile >= PPars.Seitenlaenge THEN
                                    NeueSeite := TRUE
                                 END;
                                 Zeichen := ASCII.nul |
                                    (* unterdrcke lf durch Zeichen *)
         ASCII.nul..ASCII.ht,
         ASCII.vt,
         ASCII.so..ASCII.sub,
         ASCII.gs..ASCII.us   :  Zeichen := ASCII.nul |
         ASCII.del            :  Write( Zeichen ); Read( Zeichen );
                                 CheckChar
      ELSE
         INC( Zeichenanzahl );
         INC( Position );
         IF Position >= PPars.Zeilenlaenge THEN
            NeueZeile := TRUE;
            IF AktZeile >= PPars.Seitenlaenge THEN NeueSeite := TRUE END
         END (* IF *)
      END (* CASE *)
   END CheckChar;

PROCEDURE InitNeueZeile( AllerersteZeile, wirklich : BOOLEAN );
(* Bereitet eine neue Zeile fr die Ausgabe vor *)
   VAR Stellen  :  CARDINAL;
   BEGIN
      NeueZeile := FALSE;
      INC( Zeilenanzahl );
      IF ( Zeichen = ASCII.nul ) OR AllerersteZeile THEN
         IF PPars.Zeilennummer THEN
            Position := DruckeNummer( wirklich );
            IF NOT wirklich THEN INC( Zeilennummer ) END
         END;  (* IF 2 *)
      ELSE     (* neue Zeile wg. Zeilenberlauf *)
         IF PPars.Zeilennummer THEN
            Stellen := DruckeNummer( FALSE )
         END;  (* IF 2 *)
         Space( PPars.Tabulator + Stellen, wirklich );
      END;
   END InitNeueZeile;

PROCEDURE InitNeueSeite( wirklich : BOOLEAN );
(* Bereitet eine neue Seite fr die Ausgabe vor *)
   BEGIN
      NeueSeite := FALSE;
      INC( Seitenanzahl );
      IF NOT Quiet AND wirklich THEN
         WriteString( "Drucke jetzt Seite " );
         WriteInt( AktSeite, 1 ); WriteString( ".\n" )
      END;  (* IF *)
      IF ( PPars.KopfAbSeite > 0 ) AND
         ( PPars.KopfAbSeite <= AktSeite ) THEN
         INC( AktZeile, DruckeKopf( Name, AktSeite, wirklich ) )
      END;   (* IF *)
      InitNeueZeile( AktSeite = 1, wirklich )
   END InitNeueSeite;

PROCEDURE SkipPage( Art : Darstellungsart );
(* berspringt eine Seite. Bei Zeilenberlauf der letzten Zeile,
   wird das erste Zeichen der neuen Seite zurckgegeben *)
   VAR Zeichenzahlspeicher,
       Zeilenzahlspeicher,
       Seitenzahlspeicher  : CARDINAL;
   BEGIN
      Zeichenzahlspeicher := Zeichenanzahl;
      Zeilenzahlspeicher := Zeilenanzahl;
      Seitenzahlspeicher := Seitenanzahl;
      IF NOT Quiet THEN
         WriteString( "berspringe jetzt Seite " );WriteInt( AktSeite, 1 );
         WriteString( ".\n" );
      END;  (* IF *)
      InitNeueSeite( FALSE );
      WHILE NOT NeueSeite AND NOT Stop DO
         IF NeueZeile THEN
            INC( AktZeile );
            Position := 0;
            InitNeueZeile( FALSE, FALSE )
         END;  (* IF *)
         Read( Zeichen );
         CheckChar
      END;    (* WHILE *)
      NeueSeite := TRUE;
      IF Art # stat THEN
         Zeichenanzahl := Zeichenzahlspeicher;
         Zeilenanzahl := Zeilenzahlspeicher;
         Seitenanzahl := Seitenzahlspeicher;
      END;
      INC( AktSeite );
      AktZeile := 1;
      Position := 0;
   END SkipPage;

PROCEDURE NochNicht() : BOOLEAN;
(* Liefert solange 'FALSE' bis ein Seitennummer erreicht ist, die gedruckt
   werden darf *)
   BEGIN
      IF AktSeite < PPars.ErsteSeite THEN RETURN TRUE
      ELSIF NOT PPars.SeitenSprung THEN RETURN FALSE
      ELSE RETURN ( PPars.NurGerade = ODD( AktSeite ) )
      END   (* IF *)
   END NochNicht;

PROCEDURE NochEineSeite() : BOOLEAN;
(* Liefert 'TRUE' solange noch eine Seite gedruckt werden darf *)
   BEGIN
      IF EndeNachSeite THEN RETURN FALSE
      ELSIF PPars.SeitenSprung THEN RETURN ( AktSeite < PPars.LetzteSeite - 1 )
      ELSE RETURN ( AktSeite < PPars.LetzteSeite )
      END   (* IF *)
   END NochEineSeite;

PROCEDURE FlagTest( Art : Darstellungsart );
(* Lst fr 'NeueSeite' und 'NeueZeile' entsprechende Handlungen aus *)
   BEGIN
      IF NeueSeite THEN
         IF NochEineSeite() THEN
            MacheFormFeed( Art );
            IF PPars.SeitenSprung THEN SkipPage( Art ) END;
            IF NOT Stop THEN InitNeueSeite( TRUE ) END
         ELSE Stop := TRUE; NeueZeile := FALSE
         END;
      END;  (* IF *)
      IF NeueZeile THEN
         MacheLineFeed( Art );
         IF NOT EndeNachZeile THEN InitNeueZeile( FALSE, TRUE )
         ELSE Stop := TRUE
         END   (* IF 2 *)
      END;  (* IF *)
   END FlagTest;

PROCEDURE LosGehts( Art : Darstellungsart );
(* Gibt den Text 'DateiName' auf dem vorher eingestellten Medium aus.
   Zur Formatierung werden die von ZPEingabe exportierte Variablen
   'PPars' herangezogen *)
   VAR   SkipFlag : BOOLEAN;
   BEGIN
      Zeilenspeicher := AktZeile; (* Speicher die Startposition *)
      IF NOT Quiet THEN
         WriteLn;
         WriteString("Drucke jetzt '");WriteString( Name );
         WriteString("'!\n")
      END;  (* IF 3 *)
      Read( Zeichen );      (* Vorbereitung *)
      CheckChar;
      WHILE NochNicht() AND NOT Stop DO SkipPage( Art ) END;
          (* Jetzt bin ich auf der richtigen Seite *)
      IF NOT Stop THEN InitNeueSeite( TRUE ) END;
      WHILE NOT Stop DO     (* Hier geht's wirklich los! *)
         Write( Zeichen );
         Read( Zeichen );
         CheckChar;
         AbbruchTest;
         FlagTest( Art )
      END;   (* WHILE *)         (* Das war's schon *)
      IF NOT NeueSeite THEN
         IF PPars.FFamEnde THEN MacheFormFeed( Art )
         ELSE MacheLineFeed( Art )
         END  (* IF 2 *)
      END; (* IF *)
      IF NOT PPars.SeitenSprung OR PPars.NurGerade THEN
         Name[ 0 ] := ASCII.nul (* lschen wegen pagenauem Drucken. *)
      END;  (* IF *)
      IF NOT Quiet THEN
         WriteString("\nIch bin fertig!\n ");
         GibStatistik
      END   (* IF *)
   END LosGehts;

PROCEDURE DruckeText( Dateiname : ARRAY OF CHAR;
                      Darstellung : Darstellungsart );
(* Gibt den Text 'Dateiname' auf dem Bildschirm/Drucker aus bzw.
   berechnet ( nur ) die Anzahl der zu druckenden Zeichen,...  *)
   VAR StatSpeicher  : CARDINAL;
   BEGIN
      InitVars;
      IF OeffneDatei( Dateiname ) THEN
         IF ( Compare( Name, Dateiname ) = 0 ) AND
            PPars.SeitenSprung AND PPars.NurGerade THEN
            AktZeile := Zeilenspeicher
               (* alte Startposition bleibt erhalten damit es immer pat,
                  auch wenn vorher die erste Seite nicht oben auf dem
                  ersten Blatt begonnen hat. *)
         END;  (* IF *)
         Copy( Name, Dateiname ); (* schneller mit globaler Variablen *)
         CASE Darstellung OF
            druck :  SetOutput( "PRT:" );
                     IF done THEN
                        Druckerflag := TRUE;
                        LosGehts( druck );
                        CloseOutput
                     ELSE  Fehler( Drucker )
                     END                                    |
            zeige :  LosGehts( zeige )                      |
            stat  :  StatSpeicher := AktZeile;
                     AktZeile := 1;
                     WHILE NOT Stop DO SkipPage( stat ) END;
                     AktZeile := StatSpeicher;
                     GibStatistik
         END (* CASE *)
      ELSE Copy( Name, Dateiname ); Fehler( Datei )
      END;
      AllesSchliessen;
   END DruckeText;

BEGIN  (* Initialisierung Ausgabe *)

   InitVars;
   AktZeile    := 1;          (* Zeilennummer in der aktuellen Seite *)
   INCL( BreakMode, ctrlC );  (* Programmabbruch *);
   INCL( BreakMode, ctrlD );  (* Druckende nach aktueller Zeile *)
   INCL( BreakMode, ctrlE );  (* Druckende nach aktueller Seite *)
   SetBreakMode( BreakMode );
   InstallException

END ZPAusgabe.
