(*
 * :Program.      Kwickbackup.mod
 * :Author.       Achim Siebert
 * :Address.      Nobileweg 67, D-7000 Stuttgart 40, Tel. (0)711/822509
 * :Shortcut.     [acs]
 * :Copyright.    Shareware
 * :Language.     Oberon-2
 * :Translator.   Amiga Oberon Compiler V2.25d (inofficial version)
 * :History.      V2.0: GUI; Wildcards;
 * :Contents.     Harddisk backup utility
 * :Usage.        Kwickbackup P=PARTITION/K,D=DRIVE/K,PATTERN/K,RESTORE/S,SETFLAG/S,REGARDFLAG/S
 * :Remark.       original version by Fridtjof Siebert
 *)

MODULE Kwikbackup;

IMPORT
   ASL,
   KWReq,
   Beep,
   Strings,
   Dos,
   E   : Exec,
   I   : Intuition,
   S   : SYSTEM,
   GT  : GadTools,
   KW  : KwikWindow,
   u   : Utility,
   TD  : TrackDisk,
   TDS : TrackDiskSupport;

TYPE

   pathType   = ARRAY 256 OF CHAR;
   nameType   = ARRAY 108 OF CHAR;

   ReqStrType = ARRAY 100 OF CHAR;

   MyFileType = STRUCT
      gorks: ARRAY 8 OF CHAR;    (*   0 =   0 *)
      byte:  LONGINT;            (*   8 =   8 *)
      prot:  LONGSET;            (*  12 =   C *)
      name:  nameType;           (*  16 =  10 *)
      comm:  ARRAY 116 OF CHAR;  (* 124 =  7C *)
      Size:  LONGINT;            (* 240 =  F0 *)
      path:  pathType;           (* 244 =  F4 *)
   END;                          (* 500 = 1F4 *)
   (* Danach MyFileType.size Bytes data und bis zu 3 pad-Bytes *)
   EndeType = STRUCT
      endeID: ARRAY 8 OF CHAR;
      byte: LONGINT;
   END;
   (* hinter letztem File *)
   (* BackUp - Format:
     Am Anfang jeder Diskkette :
     Byte 0..3 : "BkUp"
          4..5 : Identifier der BackUp-Reihe
          6    : Disketten Nummer
          7    : Version (0)
     Der Rest aller Disketten wird als ein groer Block angesehen.
     Er enthlt fr jedes File einen `MyFileType' gefolgt von dem
     Fileinhalt und  bis zu 3 Pad-Bytes. Hinter dem letzen File ist ein
     `EndeID'.  *)

CONST

   okstr     = " Okay ";
   cancelstr = " Cancel ";
   ignorestr = " Ignore ";
   retrystr  = " Retry ";

   cancel   = 0;
   ok       = 1;
   retry    = 1;
   continue = 2;

   TrackSize = 512*22;
   Gorks     = "Gorks!?!";
   EndeID    = "BkUpEnde";

   myAReqStruct = KWReq.AReqStruct(3,3,0,2,1,1,2,1,TRUE);   

VAR

   patbuffer         : ARRAY 256 OF CHAR;
   
   drive             : INTEGER;
   backup            : BOOLEAN;
   setarchs          : BOOLEAN;
   regardarchs       : BOOLEAN;
   match             : BOOLEAN;
   searching         : BOOLEAN;


   selectedMenu,
   selectedItem      : INTEGER;
   imsg              : I.IntuiMessagePtr;
   imsgClass         : LONGSET;
   imsgCode          : INTEGER;
   gadgetPtr         : I.GadgetPtr;   
   gstrInfo          : I.StringInfoPtr;

   aboutLock         : Dos.FileLockPtr;
   aboutInfo         : Dos.InfoData;

   TrackBuffer       : POINTER TO ARRAY TrackSize OF CHAR;
   ActTrack          : INTEGER;
   NumTracks         : INTEGER;
   TrackBufferCnt    : INTEGER;
   MySig             : INTEGER;
   snakeCnt          : INTEGER;
   DiskChange        : LONGINT;
   DiskNum           : INTEGER;
   Datum             : Dos.DatePtr;
   fehler            : BOOLEAN;
   lasterr,openerr   : SHORTINT;
   MyFileInfo        : MyFileType;
   MyLock            : Dos.FileLockPtr;
   File              : Dos.FileHandlePtr;
   Res               : INTEGER;
   ReqStr            : ReqStrType;
   path              : pathType;
   name              : nameType;


PROCEDURE Request(body:ReqStrType;ok,cancel,middle: ARRAY OF CHAR;loud:BOOLEAN;signal:INTEGER;twice:BOOLEAN):INTEGER;

VAR Str: ARRAY 1 OF ReqStrType;

BEGIN

  Str[0] := body;
  Beep.Beep(loud);
  IF loud THEN
     RETURN KWReq.ARequest(Str,KW.kwik,ok,cancel,middle,S.ADR(myAReqStruct),signal,twice);
  ELSE
     RETURN KWReq.ARequest(Str,KW.kwik,ok,cancel,middle,NIL,signal,twice);
  END;

END Request;


PROCEDURE Make2Digits(x,at: INTEGER);
(* macht aus x eine 2 - Ziffer Zahl an der Stelle at in ReqStr *)
BEGIN

  ReqStr[at  ] := CHR(ORD("0") + x DIV 10);
  ReqStr[at+1] := CHR(ORD("0") + x - (x DIV 10) * 10);

END Make2Digits;


(*------  Error Request:  ------*)


PROCEDURE Error(err: SHORTINT; Read: BOOLEAN): INTEGER;

VAR res:INTEGER;

BEGIN

  IF (err = TD.diskChanged) AND (lasterr = TD.writeProt) THEN 
     DiskChange := TDS.GetDiskChange(drive);
     lasterr := err; RETURN continue;
  END;
  lasterr := err;
  CASE err OF
     TD.notSpecified:  ReqStr := "(could not be determined)";   |
     TD.noSecHdr:      ReqStr := "no sector header";            |
     TD.badSecPreamble:ReqStr := "bad sector preamble";         |
     TD.badSecID:      ReqStr := "bad sector identifier";       |
     TD.badHdrSum:     ReqStr := "header-checksum error";       |
     TD.badSecSum:     ReqStr := "sector-checksum error";       |
     TD.tooFewSecs:    ReqStr := "too few sectors";             |
     TD.badSecHdr:     ReqStr := "bad sector header";           |
     TD.writeProt:     ReqStr := "disk is write-protected";     |
     TD.diskChanged:   ReqStr := "no disk present, disk change";|
     TD.seekError:     ReqStr := "seek error";                  |
     TD.noMem:         ReqStr := "not enough memory";           |
     TD.badUnitNum:    ReqStr := "drive not connected";         |
     TD.badDriveType:  ReqStr := "bad drive-type";              |
  ELSE
    ReqStr := "00"; Make2Digits(err,0);
  END;
  IF Read THEN
    Strings.Insert(ReqStr,0,"Read-Error: ");
  ELSE
    Strings.Insert(ReqStr,0,"Write-Error: ");
  END;
  IF TDS.Motor(drive,FALSE) THEN END;
  res := Request(ReqStr,retrystr,cancelstr,ignorestr,TRUE,MySig,TRUE);
  IF TDS.Motor(drive,TRUE) THEN END;  
  DiskChange := TDS.GetDiskChange(drive);
  RETURN res;

END Error;


PROCEDURE TestAbort():BOOLEAN;

BEGIN

   imsg := GT.GetIMsg(KW.winP.userPort);
   WHILE imsg#NIL DO
     imsgClass := imsg.class;
     imsgCode := imsg.code;
     gadgetPtr := imsg.iAddress;
     GT.ReplyIMsg(imsg);
     IF I.refreshWindow IN imsgClass THEN
       GT.BeginRefresh(KW.winP);
       GT.EndRefresh(KW.winP, I.LTRUE);
     END;
     IF I.menuPick IN imsgClass THEN
         WHILE imsgCode # I.menuNull DO
            selectedMenu := I.MenuNum(imsgCode);
            selectedItem := I.ItemNum(imsgCode);
            IF selectedMenu = 0 THEN
               CASE selectedItem OF
                  2: I.ZipWindow(KW.winP) |
                  6: IF Request("Do you wish to abort backup?"," Yes ",cancelstr,"",TRUE,-1,FALSE)=cancel THEN
                        RETURN FALSE;
                     ELSE
                        imsg := GT.GetIMsg(KW.winP.userPort);
                        WHILE imsg#NIL DO
                           GT.ReplyIMsg(imsg);
                           imsg := GT.GetIMsg(KW.winP.userPort);
                        END;
                        imsgClass := LONGSET{};
                        RETURN TRUE;
                     END|
               ELSE
               END;
            ELSE
            END;
            KW.nextMenu := I.ItemAddress(KW.menu^,imsgCode);
            imsgCode := KW.nextMenu.nextSelect;
         END;
     END;
     IF ((I.gadgetUp IN imsgClass) AND (gadgetPtr.gadgetID = KW.gStop)) OR
        (I.closeWindow IN imsgClass) OR
        ((I.vanillaKey IN imsgClass) AND ((imsgCode = ORD("S")) OR (imsgCode = ORD("s")))) THEN
         IF Request("Do you wish to abort backup?"," Yes ",cancelstr,"",TRUE,-1,FALSE)=cancel THEN
           RETURN FALSE;
         ELSE
           imsg := GT.GetIMsg(KW.winP.userPort);
           WHILE imsg#NIL DO
              GT.ReplyIMsg(imsg);
              imsg := GT.GetIMsg(KW.winP.userPort);
           END;
           imsgClass := LONGSET{};
           RETURN TRUE;
         END;
     END;
     imsg := GT.GetIMsg(KW.winP.userPort);
   END;
   RETURN FALSE;

END TestAbort;


PROCEDURE InitDrive(): BOOLEAN;

BEGIN

   KW.StopGads();
   IF I.RemoveGadget(KW.winP,KW.startGadget^)#0 THEN END;
   IF I.AddGadget(KW.winP,KW.stopGadget^,-1)#0 THEN END;
   I.RefreshGadgets(KW.stopglist,KW.winP,NIL);

   MySig := -1;

   LOOP
      IF TDS.OpenTrackDisk(drive,TRUE)#0 THEN
         ReqStr := "Can't open trackdisk device!";
         IF Request(ReqStr,retrystr,cancelstr,"",TRUE,-1,FALSE)=cancel THEN
            openerr := -2;
            RETURN TRUE;
         END;
      ELSE EXIT;
      END;
   END;
   NumTracks := SHORT(TDS.GetNumTracks(drive) DIV 2);
   DiskNum := 0;                      (* start disk #0 *)
   ActTrack := NumTracks;
   openerr := 0;

   LOOP
    MyLock := Dos.Lock(KW.partition,Dos.sharedLock);
    IF MyLock#NIL THEN EXIT END;
    ReqStr := "Can't lock ";
    Strings.Append(ReqStr,KW.partition);
    IF Request(ReqStr,retrystr,cancelstr,"",TRUE,-1,FALSE)=cancel THEN  RETURN TRUE END;
   END;

   IF ~Dos.Inhibit(KW.prDrive[drive]^,Dos.DOSTRUE) THEN openerr:= -3; RETURN TRUE END;

   MySig := TDS.AddChangeInt(drive);

   IF Dos.ParsePatternNoCase(KW.pattern,patbuffer,256)<0 THEN
      IF Request("Error while trying to parse file pattern !",okstr,"","",TRUE,-1,FALSE)#0 THEN END;
      RETURN TRUE;
   END;

   RETURN FALSE;

END InitDrive;


PROCEDURE ExitDrive();

BEGIN

  IF MyLock#NIL THEN Dos.UnLock(MyLock); MyLock:=NIL END;

  IF openerr#-2 THEN
     IF TDS.Motor(drive,FALSE) THEN END;
     IF (MySig#-1) AND (TDS.RemChangeInt(drive)#0) THEN END;
     TDS.CloseTrackDisk(drive);
     IF openerr#-3 THEN
       IF Dos.Inhibit(KW.prDrive[drive]^,Dos.DOSFALSE) THEN END;
     END;
  END;

  openerr := 0;
  GT.SetGadgetAttrs(KW.fileGadget^,KW.winP,NIL,GT.txText,S.ADR(KW.bigdone),u.done);
  path := "";
  GT.SetGadgetAttrs(KW.drawerGadget^,KW.winP,NIL,GT.txText,S.ADR(path),u.done);
  IF I.RemoveGadget(KW.winP,KW.stopGadget^)#0 THEN END;
  GT.SetGadgetAttrs(KW.stateGadget^,KW.winP,NIL,GT.txText,KW.rdvstr[0],u.done);
  IF I.AddGadget(KW.winP,KW.startGadget^,-1)#0 THEN END;
  I.RefreshGList(KW.startGadget,KW.winP,NIL,1);
  KW.RunGads();

END ExitDrive;


(*-----------------------  Create Backup:  --------------------------------*)


PROCEDURE BackUp();

VAR
  ID1,ID2   : CHAR;

  (*------  Move to next Track:  ------*)

  PROCEDURE NextTrack(): INTEGER;
  (* Result can be ok or cancel *)

  BEGIN
    IF ActTrack>=(NumTracks-1) THEN
      INC(DiskNum);
      IF TDS.Motor(drive,FALSE) THEN END;
      ReqStr := "Insert Disk Number 00 !";
      Make2Digits(DiskNum,19);
      CASE Request(ReqStr,okstr,cancelstr,"",FALSE,MySig,TDS.ChangeState(drive)) OF
         cancel: RETURN cancel;|
         ok: WHILE NOT(TDS.ChangeState(drive)) DO
               IF Request("No disk in drive !",retrystr,cancelstr,"",FALSE,MySig,FALSE)=cancel THEN
                  RETURN cancel;
               END;
             END;
      ELSE
      END;
      LOOP
        TDS.Clear(drive);
        IF TDS.ReadBlock(drive,0,TrackBuffer,1,TDS.GetDiskChange(drive))#0 THEN EXIT END;
        IF (TrackBuffer^[0]="B") AND (TrackBuffer^[1]="k") AND
           (TrackBuffer^[2]="U") AND (TrackBuffer^[3]="p") AND
           (TrackBuffer^[4]=ID1) AND (TrackBuffer^[5]=ID2) AND
           (TrackBuffer^[6]<CHR(DiskNum)) THEN
          IF TDS.Motor(drive,FALSE) THEN END;
          ReqStr := "This is disk # 00! Insert # 00!";
          Make2Digits(ORD(TrackBuffer^[6]),15);
          Make2Digits(DiskNum,28);
          CASE Request(ReqStr,retrystr,cancelstr,ignorestr,TRUE,MySig,TDS.ChangeState(drive)) OF
             continue: EXIT; |
             cancel: RETURN cancel;
          ELSE
          END;
        ELSIF (TrackBuffer^[0]="D") AND (TrackBuffer^[1]="O") AND (* let's see if it's a DOS-disk *)
              (TrackBuffer^[2]="S") THEN
            IF TDS.Motor(drive,FALSE) THEN END;
            CASE Request("This is a DOS disk!",retrystr,cancelstr,ignorestr,TRUE,MySig,TDS.ChangeState(drive)) OF
               continue: IF TDS.Motor(drive,TRUE) THEN END; EXIT; |
               cancel: RETURN cancel;
            ELSE
            END;
        ELSE
          EXIT;
        END;
      END;
      ActTrack := 0;
      TrackBuffer^ := "BkUp";                         (* Disk ID        *)
      TrackBuffer^[4] := ID1; TrackBuffer^[5] := ID2; (* Backup ID      *)
      TrackBuffer^[6] := CHR(DiskNum);                (* Disk #         *)
      TrackBuffer^[7] := 0X;                          (* BackUp Version *)
      TrackBufferCnt  := 8;                           (* 8 Bytes in Buffer *)
      GT.SetGadgetAttrs(KW.diskGadget^,KW.winP,NIL,GT.nmNumber,DiskNum,u.done);
      DiskChange := TDS.GetDiskChange(drive);
    ELSE
      INC(ActTrack);
      IF TestAbort() THEN RETURN cancel END;
    END;
    GT.SetGadgetAttrs(KW.trackGadget^,KW.winP,NIL,GT.nmNumber,ActTrack,u.done);
    RETURN ok;
  END NextTrack;

  (*------  Write to TrackDisk:  ------*)

  PROCEDURE WriteTrack(Buffer: POINTER TO BYTE; Size: LONGINT): INTEGER;

  VAR
    err: SHORTINT;

  BEGIN
    WHILE Size>=(TrackSize-TrackBufferCnt) DO
      E.CopyMem(Buffer^,TrackBuffer[TrackBufferCnt],
              TrackSize-TrackBufferCnt);
      Buffer := S.VAL(E.ADDRESS,S.VAL(LONGINT,Buffer)+TrackSize-TrackBufferCnt);
      DEC(Size,TrackSize-TrackBufferCnt);
      TrackBufferCnt := 0;
      LOOP
        GT.SetGadgetAttrs(KW.stateGadget^,KW.winP,NIL,GT.txText,KW.rdvstr[2],u.done);
        err := TDS.FormatTrack(drive,ActTrack,TrackBuffer,1,DiskChange);
        IF err=0 THEN
          GT.SetGadgetAttrs(KW.stateGadget^,KW.winP,NIL,GT.txText,KW.rdvstr[3],u.done);
          err := TDS.ReadCycSec(drive,ActTrack,0,0,TrackBuffer,22,DiskChange);
          IF err=0 THEN EXIT END;
        END;
        CASE Error(err,FALSE) OF
          continue:
            IF NextTrack()=cancel THEN RETURN cancel END;
            RETURN continue; |
          cancel:
            RETURN cancel; |
        ELSE
        END;
      END;
      IF TestAbort() THEN RETURN cancel END;
      IF NextTrack()=cancel THEN RETURN cancel END;
    END;
    IF Size#0 THEN
      E.CopyMem(Buffer^,TrackBuffer[TrackBufferCnt],Size);
      INC(TrackBufferCnt,SHORT(Size));
    END;
    RETURN ok;
  END WriteTrack;

  (*------  Write last Track:  ------*)

  PROCEDURE WriteLast(): INTEGER;

  VAR err: SHORTINT;
      Ende: EndeType;

  BEGIN
    LOOP
      Ende.endeID := EndeID;
      Ende.byte := TrackBufferCnt + 11264 * LONG(ActTrack);
      IF WriteTrack(S.ADR(Ende),SIZE(Ende))=cancel THEN RETURN cancel END;
      IF (TrackBufferCnt#0) OR (ActTrack#0) THEN
        err := TDS.FormatTrack(drive,ActTrack,TrackBuffer,1,DiskChange);
        IF err=0 THEN EXIT END;
        CASE Error(err,FALSE) OF
          continue: RETURN continue; |
          cancel:   RETURN cancel;   |
        ELSE
        END;
      ELSE
        EXIT;
      END;
    END;
    IF TDS.Motor(drive,FALSE) THEN END;
    RETURN ok;
  END WriteLast;

  (*------  Write File to Tracks:  ------*)

  PROCEDURE WriteFile(File: Dos.FileHandlePtr; Size: LONGINT): INTEGER;

  VAR
    err: SHORTINT;
    len: LONGINT;

  BEGIN
    Size := ((Size+3) DIV 4) * 4;                        (* add Pad bytes *)
    WHILE Size>=(TrackSize-TrackBufferCnt) DO
      len := Dos.Read(File,TrackBuffer[TrackBufferCnt],TrackSize-TrackBufferCnt);
      DEC(Size,TrackSize-TrackBufferCnt);
      TrackBufferCnt := 0;
      LOOP
        GT.SetGadgetAttrs(KW.stateGadget^,KW.winP,NIL,GT.txText,KW.rdvstr[2],u.done);
        err := TDS.FormatTrack(drive,ActTrack,TrackBuffer,1,DiskChange);
        IF err=0 THEN
          GT.SetGadgetAttrs(KW.stateGadget^,KW.winP,NIL,GT.txText,KW.rdvstr[3],u.done);
          err := TDS.ReadCycSec(drive,ActTrack,0,0,TrackBuffer,22,DiskChange);
          IF err=0 THEN EXIT END;
        END;
        CASE Error(err,FALSE) OF
          continue:
            IF NextTrack()=cancel THEN RETURN cancel END;
            RETURN continue; |
          cancel:
            RETURN cancel; |
        ELSE
        END;
      END;
      IF NextTrack()=cancel THEN RETURN cancel END;
    END;
    IF Size#0 THEN
      len := Dos.Read(File,TrackBuffer[TrackBufferCnt],Size);
      INC(TrackBufferCnt,SHORT(Size));
    END;
    IF TestAbort() THEN RETURN cancel END;
    RETURN ok;
  END WriteFile;

  (*------  Rekursive backup procedure:  ------*)

  PROCEDURE DoBackUp(Lock: Dos.FileLockPtr): INTEGER;

  VAR
    FileInfo: Dos.FileInfoBlockPtr;
    DosErr: LONGINT;
    res: INTEGER;
    Lock2,old: Dos.FileLockPtr;
    l: INTEGER;
    c: CHAR;

  BEGIN
    res := ok;
    path := MyFileInfo.path;
    GT.SetGadgetAttrs(KW.drawerGadget^,KW.winP,NIL,GT.txText,S.ADR(path),u.done);
    LOOP
      NEW(FileInfo);
      IF FileInfo#NIL THEN EXIT END;
      IF Request("Out of memory",retrystr,cancelstr,"",TRUE,-1,FALSE)#retry THEN EXIT END;
    END;
    IF Dos.Examine(Lock,FileInfo^) THEN
      IF FileInfo.dirEntryType>0 THEN
        old := Dos.CurrentDir(Lock);
            WHILE Dos.ExNext(Lock,FileInfo^) AND (res#cancel) DO
              IF FileInfo.dirEntryType>0 THEN
                        (*------  Directory:  ------*)
                LOOP
                  Lock2 := Dos.Lock(FileInfo.fileName,Dos.sharedLock);
                  IF Lock2#NIL THEN EXIT END;
                  ReqStr := "Can't lock ";
                  Strings.Append(ReqStr,FileInfo.fileName);
                  IF Request(ReqStr,retrystr,cancelstr,"",TRUE,-1,FALSE)=cancel THEN
                    res := cancel;
                    EXIT;
                  END;
                END;
                IF Lock2#NIL THEN
                  l := Strings.Length(MyFileInfo.path);
                  IF l>253 THEN
                    IF Request("Path tooo looong!!!",okstr,"","",TRUE,-1,FALSE)#0 THEN END;
                    res := cancel;
                  ELSE
                    IF l#0 THEN
                       MyFileInfo.path[l] := "/";
                       MyFileInfo.path[l+1] := 0X;
                    END;
                    Strings.Append(MyFileInfo.path,FileInfo.fileName);
                    res := DoBackUp(Lock2);
                    MyFileInfo.path[l] := 0X;
                    path := MyFileInfo.path;
                    GT.SetGadgetAttrs(KW.drawerGadget^,KW.winP,NIL,GT.txText,S.ADR(path),u.done);
                  END;
                  Dos.UnLock(Lock2);
                END;
              ELSE
                        (*------  File:  ------*)
                IF (NOT(Dos.archive IN FileInfo.protection) OR
                   (~regardarchs)) AND Dos.MatchPatternNoCase(patbuffer,FileInfo.fileName) THEN
                  COPY(FileInfo.fileName,name);
                  GT.SetGadgetAttrs(KW.fileGadget^,KW.winP,NIL,GT.txText,S.ADR(name),u.done);
                  MyFileInfo.byte := TrackBufferCnt + 11264*LONG(ActTrack);
                  MyFileInfo.prot := FileInfo.protection;
                  COPY(FileInfo.fileName,MyFileInfo.name);
                  COPY(FileInfo.comment,MyFileInfo.comm);
                  MyFileInfo.Size := FileInfo.size;
                  LOOP
                    File := Dos.Open(FileInfo.fileName,Dos.oldFile);
                    IF File#NIL THEN EXIT END;
                    ReqStr := "Can't open ";
                    Strings.Append(ReqStr,FileInfo.fileName);
                    Strings.Append(ReqStr," !");
                    CASE Request(ReqStr,retrystr,cancelstr,ignorestr,TRUE,-1,FALSE) OF
                      cancel: res := cancel; EXIT; |
                      continue: EXIT; |
                    ELSE
                    END;
                  END;
                  IF File#NIL THEN
                    CASE WriteTrack(S.ADR(MyFileInfo),SIZE(MyFileType)) OF
                    ok: res := WriteFile(File,FileInfo.size);
                        Dos.OldClose(File);
                        IF (res=ok) AND setarchs THEN
                          IF Dos.SetProtection(FileInfo.fileName,FileInfo.protection +
                               LONGSET{Dos.archive}) THEN END;
                        END; |
                    cancel: Dos.OldClose(File); res := cancel; |
                    ELSE
                      Dos.OldClose(File);
                    END;
                  END;
                END;   (* IF archived IN protection AND RegardArchived THEN *)
              END;   (* IF direntryType>0 THEN ELSE *)
              IF TestAbort() THEN res := cancel END;
            END;   (* WHILE Dos.ExNext()#0 DO *)
        old := Dos.CurrentDir(old);
      END;   (* IF FileInfo^.dirEntryType>0 THEN *)
    END;   (* IF Dos.Examine()=0 THEN *)
    IF FileInfo#NIL THEN DISPOSE(FileInfo) END;
    RETURN res;
  END DoBackUp;


BEGIN

   fehler := InitDrive();

(*------  Start:  ------*)

   IF ~fehler THEN
     MyFileInfo.gorks := Gorks;
     Dos.DateStamp(Datum^);                   (* take ticks as ID for this BackUp *)
     ID1 := CHR( Datum.tick - (Datum.tick DIV 256) * 256);
     ID2 := CHR((Datum.tick - (Datum.tick DIV 65536) * 65536) DIV 256);
     IF NextTrack()#cancel THEN             (* insert first Disk          *)
       MyFileInfo.path := "";
       IF DoBackUp(MyLock)#cancel THEN      (* Back it up                 *)
         IF WriteLast()=cancel THEN END;    (* Write last track           *)
       END;
     END;
   END;

   ExitDrive();

END BackUp;


(*----- an absolutely redundant procedure: -----*)


PROCEDURE Snake();

VAR j: INTEGER;

BEGIN

IF snakeCnt<12 THEN
   CASE snakeCnt OF
      |0: path := "";
      |1: path := "_";
      |2: path := "\\_";
      |3: path := "___";
      |4: path := "/\\_";
      |5: path := "____";
      |6: path := "_/\\_";
      |7: path := "_____";
      |8: path := "__/\\_";
      |9: path := "______";
      |10:path := " __/\\_";
      |11:path := " ______";
    END;
ELSE
   j := snakeCnt DIV 2;
   IF snakeCnt MOD 2 = 0 THEN
      path[j -1] := "\\";
      path[j -2] := "/";
      path[j -5]:= " ";
   ELSE
      path[j -1] := "_";
      path[j -2] := "_";
      path[j -5] := "_";
      Strings.Insert(path,0," ");
   END;   
END;
GT.SetGadgetAttrs(KW.drawerGadget^,KW.winP,NIL,GT.txText,S.ADR(path),u.done);
INC(snakeCnt); IF (j>KW.snakeMax) OR (j=253) THEN snakeCnt:=0 END;

END Snake;


(*----------------------------  Restore:  ---------------------------------*)


PROCEDURE Restore();

VAR
  DiskID: ARRAY 8 OF CHAR;
  ID1,ID2: CHAR;
  result: SHORTINT;
  fehler: BOOLEAN;
  oldlock: Dos.FileLockPtr;

(*------  Move to next Track:  ------*)

  PROCEDURE NextReadTrack(): SHORTINT;
  (* Result can be ok or cancel *)

  VAR
    err: SHORTINT;

  BEGIN
    IF TestAbort() THEN RETURN cancel END;
    IF ActTrack>=(NumTracks-1) THEN
      ActTrack := 0;
      INC(DiskNum);
      IF TDS.Motor(drive,FALSE) THEN END;
      ReqStr := "Insert disk number 00 !";
      Make2Digits(DiskNum,19);
      CASE Request(ReqStr,okstr,cancelstr,"",FALSE,MySig,TDS.ChangeState(drive)) OF
         cancel: RETURN cancel;|
         ok: WHILE NOT(TDS.ChangeState(drive)) DO
               IF Request("No disk in drive !",retrystr,cancelstr,"",FALSE,MySig,FALSE)=cancel THEN
                  RETURN cancel;
               END;
             END;
      ELSE
      END;
      LOOP
        TDS.Clear(drive);
        err := TDS.ReadBlock(drive,0,TrackBuffer,22,TDS.GetDiskChange(drive));
        IF err#0 THEN
          CASE Error(err,TRUE) OF
            cancel: RETURN cancel; |
            continue: RETURN continue; |
          ELSE
          END;
        ELSIF (TrackBuffer^[0]#"B") OR (TrackBuffer^[1]#"k") OR
              (TrackBuffer^[2]#"U") OR (TrackBuffer^[3]#"p") THEN
          CASE Request("That's no Backup-Disk",retrystr,cancelstr,ignorestr,TRUE,MySig,TDS.ChangeState(drive)) OF
            cancel: RETURN cancel; |
            continue: RETURN continue; |
          ELSE
          END;
        ELSIF TrackBuffer[6]#CHR(DiskNum) THEN
          IF TDS.Motor(drive,FALSE) THEN END;
          ReqStr := "This is disk # 00! Insert # 00!";
          Make2Digits(ORD(TrackBuffer[6]),15);
          Make2Digits(DiskNum,28);
          CASE Request(ReqStr,retrystr,cancelstr,ignorestr,TRUE,MySig,TDS.ChangeState(drive)) OF
             continue: RETURN continue; |
             cancel: RETURN cancel;
          ELSE
          END;
        ELSIF TrackBuffer[7]#0X THEN
          IF TDS.Motor(drive,FALSE) THEN END;
          CASE Request("Wrong backup version!",retrystr,cancelstr,ignorestr,TRUE,MySig,TDS.ChangeState(drive)) OF
            continue: RETURN continue; |
            cancel: RETURN cancel;
          ELSE
          END;
        ELSIF DiskNum=1 THEN
          ID1 := TrackBuffer[4]; ID2 := TrackBuffer[5]; EXIT;
        ELSIF (TrackBuffer[4]#ID1) OR (TrackBuffer[5]#ID2) THEN
          IF TDS.Motor(drive,FALSE) THEN END;
          CASE Request("Wrong Backup-identifier!",retrystr,cancelstr,ignorestr,TRUE,MySig,TDS.ChangeState(drive)) OF
            continue: RETURN continue; |
            cancel: RETURN cancel;
          ELSE
          END;
        ELSE
          EXIT;
        END;
      END;   (* LOOP *)
      GT.SetGadgetAttrs(KW.diskGadget^,KW.winP,NIL,GT.nmNumber,DiskNum,u.done);
      DiskChange := TDS.GetDiskChange(drive);
      TrackBufferCnt := 8;
    ELSE
      INC(ActTrack);
      err := TDS.ReadBlock(drive,22*ActTrack,TrackBuffer,22,DiskChange);
      IF err#0 THEN
        CASE Error(err,FALSE) OF
          cancel: RETURN cancel; |
          continue: RETURN continue; |
        ELSE
        END;
      END;
      TrackBufferCnt := 0;
    END;
    GT.SetGadgetAttrs(KW.trackGadget^,KW.winP,NIL,GT.nmNumber,ActTrack,u.done);
    IF searching THEN Snake(); END;
    RETURN ok;
  END NextReadTrack;

  (*------  Read Bytes from TrackDisk:  ------*)

  PROCEDURE ReadTrack(Buffer: POINTER TO BYTE; Size: LONGINT): SHORTINT;

  BEGIN
    WHILE Size>0 DO
      IF Size>=TrackSize-TrackBufferCnt THEN
      E.CopyMem(TrackBuffer[TrackBufferCnt],Buffer^,
              TrackSize-TrackBufferCnt);
        DEC(Size,TrackSize-TrackBufferCnt);
        Buffer := S.VAL(E.ADDRESS,S.VAL(LONGINT,Buffer)+TrackSize-TrackBufferCnt);
        CASE NextReadTrack() OF
           cancel: RETURN cancel; |
           continue: RETURN continue; |
        ELSE
        END;
      ELSE
        E.CopyMem(TrackBuffer[TrackBufferCnt],Buffer^,
                Size);
        INC(TrackBufferCnt,SHORT(Size));
        Size := 0;
      END;
    END;
    RETURN ok;
  END ReadTrack;

  (*------  Make Directory:  ------*)

  PROCEDURE GetDir(VAR dirname: ARRAY OF CHAR; VAR Lock: Dos.FileLockPtr): BOOLEAN;
  (* Returns TRUE if error occured *)

  VAR
    lck: Dos.FileLockPtr;
    len: INTEGER;
    c: CHAR;

  BEGIN
    Lock := Dos.Lock(dirname,Dos.sharedLock);
    IF Lock#NIL THEN
      RETURN FALSE;
    ELSE
      Lock := Dos.CreateDir(dirname);
      IF Lock#NIL THEN
        RETURN FALSE;
      ELSE
        len := Strings.Length(dirname) - 1;
        WHILE (len>0) AND (dirname[len]#"/") DO DEC(len) END;
        IF len=0 THEN RETURN TRUE END;
        c := dirname[len];
        dirname[len] := 0X;
        IF GetDir(dirname,lck) THEN
          dirname[len] := c;
          RETURN TRUE; (* Error *)
        ELSE
          Dos.UnLock(lck);
          dirname[len] := c;
          Lock := Dos.Lock(dirname,Dos.sharedLock);
          IF Lock#NIL THEN
            RETURN FALSE;
          ELSE
            Lock := Dos.CreateDir(dirname);
            RETURN Lock=NIL;
          END;
        END;
      END;
    END;
  END GetDir;

  (*------  Read File from TrackDisk:  ------*)

  PROCEDURE ReadFile(seek: BOOLEAN): SHORTINT;

  VAR
    err: SHORTINT;
    adr: LONGINT;
    len: LONGINT;
    lock,old: Dos.FileLockPtr;
    file: Dos.FileHandlePtr;
    res: SHORTINT;

  BEGIN
    res := ok;
    adr := TrackBufferCnt + LONG(ActTrack) * 11264;
    IF seek THEN
      CASE ReadTrack(S.ADR(MyFileInfo.prot),SIZE(MyFileInfo)-12) OF
         cancel: RETURN cancel; |
         continue: RETURN continue; |
      ELSE
      END;
      MyFileInfo.gorks := Gorks;
      MyFileInfo.byte := adr;
    ELSE
      CASE ReadTrack(S.ADR(MyFileInfo),SIZE(MyFileInfo)) OF
         cancel: RETURN cancel; |
         continue: RETURN continue; |
      ELSE
      END;
    END;
      IF (MyFileInfo.gorks=Gorks) AND (MyFileInfo.byte=adr) THEN
          IF Dos.MatchPatternNoCase(patbuffer,MyFileInfo.name) THEN
             match := TRUE; snakeCnt := 0; searching := FALSE;
          ELSE
             IF match THEN
                GT.SetGadgetAttrs(KW.fileGadget^,KW.winP,NIL,GT.txText,S.ADR(KW.bigsearch),u.done);
             END;
             match := FALSE; searching := TRUE;
          END;
          IF match THEN
             name := MyFileInfo.name;
             GT.SetGadgetAttrs(KW.fileGadget^,KW.winP,NIL,GT.txText,S.ADR(name),u.done);
             IF Strings.Length(MyFileInfo.path)=0 THEN
               lock := NIL;
               GT.SetGadgetAttrs(KW.drawerGadget^,KW.winP,NIL,GT.txText,S.ADR(""),u.done);
             ELSE
               GT.SetGadgetAttrs(KW.drawerGadget^,KW.winP,NIL,GT.txText,S.ADR(MyFileInfo.path),u.done);
               LOOP
                 IF GetDir(MyFileInfo.path,lock) THEN
                   CASE Request("Can't create directory!",retrystr,cancelstr,ignorestr,TRUE,-1,FALSE) OF
                     cancel: res := cancel; EXIT; |
                     continue: res := continue; EXIT; |
                   ELSE
                   END;
                 ELSE
                   EXIT;
                 END;
               END;
             END;
          END;
          IF res=ok THEN
            IF match THEN
               IF lock#NIL THEN old := Dos.CurrentDir(lock) END;
               LOOP
                 file := Dos.Open(MyFileInfo.name,Dos.newFile);
                 IF file#NIL THEN EXIT END;
                 ReqStr := "Can't open ";
                 Strings.Append(ReqStr,MyFileInfo.name);
                 CASE Request(ReqStr,retrystr,cancelstr,ignorestr,TRUE,-1,FALSE) OF
                   cancel: res := cancel; EXIT; |
                   continue: res := continue; EXIT; |
                 ELSE
                 END;
               END;
            END;
            IF (file#NIL) OR ~match THEN
              WHILE (MyFileInfo.Size#0) AND (res=ok) DO
                IF MyFileInfo.Size>=TrackSize-TrackBufferCnt THEN
                  IF match THEN
                    len := Dos.Write(file,TrackBuffer[TrackBufferCnt],TrackSize-TrackBufferCnt);
                  END;
                  DEC(MyFileInfo.Size,TrackSize-TrackBufferCnt);
                  CASE NextReadTrack() OF
                    cancel: res := cancel; |
                    continue: res := continue; |
                  ELSE
                  END;
                ELSE
                  IF match THEN
                  len := Dos.Write(file,TrackBuffer[TrackBufferCnt],
                               MyFileInfo.Size);
                  END;
                  INC(TrackBufferCnt,SHORT(MyFileInfo.Size));
                  MyFileInfo.Size := 0;
                END;
              END;
              TrackBufferCnt := ((TrackBufferCnt + 3) DIV 4) * 4;
              IF TrackBufferCnt>TrackSize THEN
                CASE NextReadTrack() OF
                  cancel: res := cancel; |
                  continue: res := continue; |
                ELSE
                END;
              END;
              IF match THEN
                 Dos.OldClose(file);
                 IF res#ok THEN
                   IF Dos.DeleteFile(MyFileInfo.name) THEN END;
                 ELSE
                   IF Dos.SetProtection(MyFileInfo.name,MyFileInfo.prot) THEN END;
                   IF Dos.SetComment(MyFileInfo.name,MyFileInfo.comm) THEN END;
                 END;
              END;
            END;
            IF match AND (lock#NIL) THEN
              old := Dos.CurrentDir(old);
              Dos.UnLock(lock);
            END;
            RETURN res;
          END;
      ELSIF MyFileInfo.gorks=EndeID THEN
        RETURN cancel;
      ELSE
        ReqStr := "Wrong data found! Continue anyway?";
        IF Request(ReqStr," Continue ",cancelstr,"",FALSE,-1,FALSE)=cancel THEN
          RETURN cancel;
        ELSE
          RETURN continue;
        END;
      END;
    RETURN res;
  END ReadFile;

  (*------  Seek:  ------*)

  PROCEDURE Seek(): SHORTINT;

  VAR ID: STRUCT
           go: ARRAY 8 OF CHAR;
           by: LONGINT;
          END;
      adr: LONGINT;
      err: SHORTINT;

  BEGIN
    searching := TRUE; snakeCnt := 0;
    GT.SetGadgetAttrs(KW.fileGadget^,KW.winP,NIL,GT.txText,S.ADR(KW.bigsearch),u.done);
    LOOP
      DiskChange := TDS.GetDiskChange(drive);
      err := TDS.ReadBlock(drive,0,TrackBuffer,22,DiskChange);
      IF err#0 THEN
        CASE Error(err,TRUE) OF
          cancel: RETURN cancel; |
          continue: EXIT; |
        ELSE END;
      ELSE
        ID1 := TrackBuffer^[4];
        ID2 := TrackBuffer^[5];
        DiskNum := ORD(TrackBuffer^[6]);
        GT.SetGadgetAttrs(KW.diskGadget^,KW.winP,NIL,GT.nmNumber,DiskNum,u.done);
        EXIT;
      END;
    END;
    IF TrackBufferCnt>TrackSize THEN TrackBufferCnt := TrackSize END;
    IF ActTrack>NumTracks THEN ActTrack := NumTracks END;
    adr := TrackBufferCnt + LONG(ActTrack) * 11264;
    LOOP
      IF TrackBufferCnt>=TrackSize THEN
        LOOP
          CASE NextReadTrack() OF
            cancel: RETURN cancel; |
            ok: EXIT; |
          ELSE END;
        END;
        IF ActTrack=0 THEN
          ID1 := TrackBuffer^[4];
          ID2 := TrackBuffer^[5];
          DiskNum := ORD(TrackBuffer^[6]);
          GT.SetGadgetAttrs(KW.diskGadget^,KW.winP,NIL,GT.nmNumber,DiskNum,u.done);
        END;
        adr := TrackBufferCnt + LONG(ActTrack) * 11264;
      END;
      IF (TrackBuffer^[TrackBufferCnt]="G") THEN
        IF (TrackBuffer^[TrackBufferCnt+1]="o") AND
           (TrackBuffer^[TrackBufferCnt+2]="r") AND
           (TrackBuffer^[TrackBufferCnt+3]="k") THEN
          CASE ReadTrack(S.ADR(ID),12) OF
           cancel: RETURN cancel; |
          ELSE
            IF (ID.go[4]="s") AND (ID.go[5]="!") AND
               (ID.go[6]="?") AND (ID.go[7]="!") AND
               (ID.by = adr) THEN
              searching := FALSE; path := "";
              RETURN ReadFile(TRUE);
            END;
          END;
        END;
      END;
      INC(TrackBufferCnt,4);
      INC(adr,4);
    END;
  END Seek;

BEGIN

   fehler := InitDrive();

(*------  Start:  ------*)

   IF ~fehler THEN
     match := TRUE; snakeCnt := 0; searching := FALSE;
     oldlock := Dos.CurrentDir(MyLock);
     result := NextReadTrack();              (* insert first Disk          *)
     IF result=continue THEN result := Seek() END;
     IF result=ok THEN
       GT.SetGadgetAttrs(KW.stateGadget^,KW.winP,NIL,GT.txText,KW.rdvstr[1],u.done);
       LOOP
         CASE ReadFile(FALSE) OF
           continue: IF Seek()=cancel THEN EXIT END; |
           cancel: EXIT; |
         ELSE
         END;
       END;
     END;
     oldlock := Dos.CurrentDir(oldlock);
   END;

   ExitDrive();

END Restore;

(*----- before backup or restore check for existing drawer -----*)

PROCEDURE TestPartition():BOOLEAN;

VAR plock: Dos.FileLockPtr;

BEGIN

   IF KW.partition#"" THEN
      plock := Dos.Lock(KW.partition,Dos.sharedLock);
      IF plock # NIL THEN Dos.UnLock(plock);
         RETURN TRUE;
      END;
   END;
   RETURN Request("Please select an existing partition or drawer first!",okstr,"","",FALSE,-1,FALSE)=5;

END TestPartition;



PROCEDURE About();

VAR   used: LONGINT;
      pos:  INTEGER;
      ch: ARRAY 1 OF CHAR;
      volume: ARRAY 32 OF CHAR;
      reqstring: ARRAY 4 OF ARRAY 80 OF CHAR;
      numstring: ARRAY 20 OF CHAR;


   PROCEDURE Int2Str(num: LONGINT);

   BEGIN

     numstring := "";
     WHILE num>0 DO
       ch[0] := CHR(num MOD 10 + ORD("0"));
       Strings.Insert(numstring,0,ch);
       num := num DIV 10;
     END;

   END Int2Str;

BEGIN

   reqstring[0] := "\o$VER: Kwikbackup 2.0 (13.03.92)";
   COPY(KW.title,reqstring[0]);
   reqstring[1] := "Used blocks on volume ";
   pos := Strings.Occurs(KW.partition,":");
   IF pos > 0 THEN Strings.Cut(KW.partition,0,pos+1,volume);
      aboutLock := Dos.Lock(volume,Dos.accessRead);
      IF (aboutLock # NIL) AND Dos.Info(aboutLock,aboutInfo) THEN
         COPY(aboutInfo.volumeNode.name^,volume);
         Strings.Delete(volume,0,1);
         Strings.Append(reqstring[1],volume);
         Strings.Append(reqstring[1],": ");
         used := aboutInfo.numBlockUsed;
         Int2Str(used);
         Strings.Append(reqstring[1],numstring);
      ELSE
         Strings.Append(reqstring[1],"???");
      END
   ELSE
      Strings.Append(reqstring[1],"(none specified)");
   END;
   Dos.UnLock(aboutLock); aboutLock := NIL;
   reqstring[2] := "Number of disks probably needed for a";
   reqstring[3] := "backup of the whole volume: ";
   IF used > 0 THEN
       Int2Str(used DIV 1760+1);
       Strings.Append(reqstring[3],numstring);
   ELSE
      Strings.Append(reqstring[3],"--");
   END;
   KW.StopGads();
   IF KWReq.ARequest(reqstring,KW.kwik,okstr,"","",NIL,-1,FALSE)#0 THEN END;
   KW.RunGads();

END About;


PROCEDURE GetDrawer();

   VAR ok: BOOLEAN;
       fr: ASL.FileRequesterPtr;

BEGIN

   ok := FALSE;
   IF ASL.asl#NIL THEN
      IF I.RemoveGList(KW.winP,KW.glist,-1)#0 THEN END;
      fr := ASL.AllocAslRequest(ASL.fileRequest,NIL);
      fr.dir := S.ADR(KW.partition);
      fr.width := 200; fr.height := 180;
      IF fr=NIL THEN HALT(30) END;
      ok := ASL.AslRequestTags(fr,
                       ASL.hail,     S.ADR("Select HDPartition/Drawer"),
                       ASL.window,   KW.winP,
                       ASL.extFlags1,ASL.noFiles,
                       u.done)#NIL;
      IF I.AddGList(KW.winP,KW.glist,-1,-1,NIL)#0 THEN END;
      IF ok THEN
         COPY(fr.dir^,KW.partition);
         GT.SetGadgetAttrs(KW.hdGadget^,KW.winP,NIL,GT.stString,S.ADR(KW.partition),u.done);
      END;
      ASL.FreeAslRequest(fr);
   ELSE
      IF Request("Couldn't open ASL.library!",okstr,"","",TRUE,-1,FALSE)#0 THEN END;
   END;

END GetDrawer;


PROCEDURE HandleMenuEvent();

BEGIN

   WHILE imsgCode # I.menuNull DO
      selectedMenu := I.MenuNum(imsgCode);
      selectedItem := I.ItemNum(imsgCode);
      IF selectedMenu = 0 THEN
         CASE selectedItem OF
            0: GetDrawer()|
            2: I.ZipWindow(KW.winP) |
            4: About()    |
            6: HALT(0)    |
         ELSE
         END;
      ELSE
      END;
      KW.nextMenu := I.ItemAddress(KW.menu^,imsgCode);
      imsgCode    := KW.nextMenu.nextSelect;
   END;

END HandleMenuEvent;


PROCEDURE HandleEvent(gad: I.GadgetPtr; code: INTEGER);

VAR lbool: LONGINT;

BEGIN

   IF I.vanillaKey IN imsgClass THEN

      CASE code OF
         ORD("S"),ORD("s"): IF TestPartition() THEN IF backup THEN BackUp() ELSE Restore() END;END;|
         ORD("A"),ORD("a"): IF backup THEN setarchs := ~(I.selected IN KW.archGadget.flags);
                               lbool := I.BoolToLong(setarchs);
                               GT.SetGadgetAttrs(KW.archGadget^,KW.winP,NIL,GT.cbChecked,lbool,u.done);
                            END;
                            |
         ORD("R"),ORD("r"): IF backup THEN regardarchs := ~(I.selected IN KW.regardGadget.flags);
                               lbool := I.BoolToLong(regardarchs);
                               GT.SetGadgetAttrs(KW.regardGadget^,KW.winP,NIL,GT.cbChecked,lbool,u.done);
                            END; 
                            |
         ORD("H"),ORD("h"): IF I.ActivateGadget(KW.hdGadget^,KW.winP,NIL) THEN END |
         ORD("P"),ORD("p"): IF I.ActivateGadget(KW.patGadget^,KW.winP,NIL) THEN END |
         ORD("D"),ORD("d"): IF code = ORD("d") THEN
                               INC(drive); IF drive = 4 THEN drive := 0 END;
                            ELSE DEC(drive); IF drive < 0 THEN drive := 3 END;
                            END;
                            GT.SetGadgetAttrs(KW.driveGadget^,KW.winP,NIL,GT.cyActive,drive,u.done);
                            |
         ORD(">"),ORD("<"): IF backup THEN
                              GT.SetGadgetAttrs(KW.backupGadget^,KW.winP,NIL,GT.cyActive,1,u.done);
                              GT.SetGadgetAttrs(KW.archGadget^,KW.winP,NIL,I.gaDisabled,I.LTRUE,u.done);
                              GT.SetGadgetAttrs(KW.regardGadget^,KW.winP,NIL,I.gaDisabled,I.LTRUE,u.done);
                            ELSE
                              GT.SetGadgetAttrs(KW.backupGadget^,KW.winP,NIL,GT.cyActive,0,u.done);
                              GT.SetGadgetAttrs(KW.archGadget^,KW.winP,NIL,I.gaDisabled,I.LFALSE,u.done);
                              GT.SetGadgetAttrs(KW.regardGadget^,KW.winP,NIL,I.gaDisabled,I.LFALSE,u.done);
                            END;
                            backup := ~backup;
                            |
      ELSE
      END;

   ELSE
      CASE gad.gadgetID OF

         KW.gStart : IF TestPartition() THEN IF backup THEN BackUp() ELSE Restore() END;END;|
         KW.gBack  : backup := (code = 0);
                     IF ~backup THEN
                        GT.SetGadgetAttrs(KW.archGadget^,KW.winP,NIL,I.gaDisabled,I.LTRUE,u.done);
                        GT.SetGadgetAttrs(KW.regardGadget^,KW.winP,NIL,I.gaDisabled,I.LTRUE,u.done);
                     ELSE
                        GT.SetGadgetAttrs(KW.archGadget^,KW.winP,NIL,I.gaDisabled,I.LFALSE,u.done);
                        GT.SetGadgetAttrs(KW.regardGadget^,KW.winP,NIL,I.gaDisabled,I.LFALSE,u.done);
                     END |
         KW.gArch  : setarchs    := I.selected IN gadgetPtr.flags|
         KW.gRegard: regardarchs := I.selected IN gadgetPtr.flags|
         KW.gHardd : gstrInfo := gad.specialInfo;
                     COPY(gstrInfo.buffer^,KW.partition)|
         KW.gPat   : gstrInfo := gad.specialInfo;
                     COPY(gstrInfo.buffer^,KW.pattern);
                     IF KW.pattern="" THEN KW.pattern := "#?";
                        GT.SetGadgetAttrs(KW.patGadget^,KW.winP,NIL,GT.stString,S.ADR(KW.pattern),u.done);
                     END|
         KW.gDrive : drive := code|

      ELSE
      END;
   END;

END HandleEvent;


BEGIN

   IF I.int.libNode.version<37 THEN HALT(20) END;
   TrackBuffer := E.AllocMem(SIZE(TrackBuffer^),LONGSET{E.chip});
   NEW(Datum);
   IF (TrackBuffer=NIL) OR (Datum=NIL) THEN HALT(10) END;
   
   backup      := KW.backup;
   drive       := KW.drive;
   setarchs    := KW.setarchs;
   regardarchs := KW.regardarchs;
   
   LOOP
      REPEAT UNTIL E.Wait(LONGSET{KW.winP.userPort.sigBit}) # LONGSET{};
      LOOP
         imsg := GT.GetIMsg(KW.winP.userPort);
         IF imsg=NIL THEN EXIT END;
         imsgClass := imsg.class;
         imsgCode := imsg.code;

         gadgetPtr   := imsg.iAddress;

         GT.ReplyIMsg(imsg);

         IF I.refreshWindow IN imsgClass THEN
           GT.BeginRefresh(KW.winP);
           GT.EndRefresh(KW.winP, I.LTRUE);
         END;
         IF I.closeWindow IN imsgClass THEN
           HALT(0);
         END;
         IF (I.gadgetUp IN imsgClass) OR (I.vanillaKey IN imsgClass) THEN
            HandleEvent(gadgetPtr,imsgCode);
         END;
         IF I.menuPick IN imsgClass THEN
            HandleMenuEvent();
         END;
      END;
   END;

CLOSE

   IF Datum # NIL THEN DISPOSE(Datum) END;
   IF TrackBuffer#NIL THEN E.FreeMem(TrackBuffer,SIZE(TrackBuffer^)) END;

END Kwikbackup.
