MODULE KWReq;

IMPORT
   Strings,
   E: Exec,
   S: SYSTEM,
   G: Graphics,
   U: Utility,
   I: Intuition;

TYPE 
     AReqStructPtr* = POINTER TO AReqStruct;
     AReqStruct*    = RECORD
                      WindowCol*     : SHORTINT;
                      InWindowCol*   : SHORTINT;
                      BoxCol*        : SHORTINT;
                      OutBorderCol1* : SHORTINT;
                      OutBorderCol2* : SHORTINT;
                      InBorderCol1*  : SHORTINT;
                      InBorderCol2*  : SHORTINT;
                      TextCol*       : SHORTINT;
                      Pattern*       :  BOOLEAN;
                      END;
                  
TYPE pattype = ARRAY 2 OF SET;

CONST 
      pattern = pattype(-{0,2,4,6,8,10,12,14},{0,2,4,6,8,10,12,14});
      mydefs = AReqStruct(3,0,0,2,1,1,2,1,TRUE);

VAR ReqNw: I.NewWindow;
    reqgads: ARRAY 3 OF I.Gadget;
    myreqStruct: AReqStruct;
    ReqIMess     : I.IntuiMessage;
    ReqIMessPtr  : I.IntuiMessagePtr;
    ReqWinP      : I.WindowPtr;
    gPtr         : I.GadgetPtr;
    rp : G.RastPortPtr;

PROCEDURE InitRequest;

VAR i: INTEGER;

BEGIN

   i := 0;
   REPEAT
      reqgads[i].flags    := I.gadgHComp;
      reqgads[i].activation := {I.relVerify};
      reqgads[i].gadgetType := I.boolGadget;
      reqgads[i].mutualExclude:= LONGSET{};
      reqgads[i].gadgetID := i;
      INC(i);
   UNTIL i = 3;
   reqgads[2].nextGadget := NIL;
   ReqNw.firstGadget   := S.ADR(reqgads[0]);
      
END InitRequest;

PROCEDURE Simplebox*(rport:G.RastPortPtr;x,y,b,h: INTEGER;col1,col2,fillcol:INTEGER);

VAR xb,yh: INTEGER;

BEGIN

  DEC(b); DEC(h);
  xb := x+b;
  yh := y+h;
  G.SetAPen(rport,col1);
  G.Move(rport,xb,y);
  G.Draw(rport,x,y);
  G.Draw(rport,x,yh);
  G.SetAPen(rport,col2);
  G.Move(rport,x+1,yh);
  G.Draw(rport,xb,yh);
  G.Draw(rport,xb,y+1);
  IF fillcol # -1 THEN
     G.SetAPen(rport,fillcol);
     G.RectFill(rport,x+1,y+1,xb-1,yh-1);
  END;
END Simplebox;


PROCEDURE Box*(rport:G.RastPortPtr;x,y,b,h: INTEGER;col1,col2,fillcol:INTEGER);

VAR x1,xb,yh: INTEGER;

BEGIN

  DEC(b); DEC(h);
  x1 := x+1;
  xb := x+b;
  yh := y+h;
  G.SetAPen(rport,col1);
  G.Move(rport,xb-1,y);
  G.Draw(rport,x,y);
  G.Draw(rport,x,yh);
  G.Move(rport,x1,y);
  G.Draw(rport,x1,yh-1);
  G.SetAPen(rport,col2);
  G.Move(rport,x1,yh);
  G.Draw(rport,xb,yh);
  G.Draw(rport,xb,y);
  G.Move(rport,xb-1,y+1);
  G.Draw(rport,xb-1,yh);
  IF fillcol # -1 THEN
     G.SetAPen(rport,fillcol);
     G.RectFill(rport,x+2,y+1,xb-2,yh-1);
  END;
END Box;

PROCEDURE ARequest*(bodytexts:ARRAY OF ARRAY OF CHAR;titletext,oktext,canceltext,middletext:ARRAY OF CHAR;defines:AReqStructPtr;signal:INTEGER;twice:BOOLEAN):SHORTINT;

VAR yfsize,baseline,leftoffset,rightoffset,lines,i: INTEGER;
    bodyleft,bodytop,minwidth : INTEGER;
    oklength,cancellength,middlelength : INTEGER;
    bodywidth,oldbodywidth,okwidth,cancelwidth,middlewidth : INTEGER;
    messet,waitset: LONGSET;
    cancel,middle : BOOLEAN;
    ScP   : I.ScreenPtr;

BEGIN

   IF defines # NIL THEN
      myreqStruct := defines^;
   ELSE
      myreqStruct := mydefs;
   END;
   lines := SHORT(LEN(bodytexts));
   i := 0; bodywidth := 0; oldbodywidth := 0;
   cancel := canceltext # "";
   middle := middletext # "";
   IF oktext = "" THEN oktext := "Okay" END;
   oklength := Strings.Length(oktext);
   IF cancel THEN
      cancellength := Strings.Length(canceltext);
   END;
   IF middle THEN
      middlelength := Strings.Length(middletext);
   END;
   
   ReqNw.detailPen   := myreqStruct.TextCol;
   ReqNw.blockPen    := myreqStruct.WindowCol;
   ReqNw.idcmpFlags    := LONGSET{I.gadgetUp,I.vanillaKey,I.inactiveWindow,I.activeWindow};
   ReqNw.flags         := LONGSET{I.rmbTrap,I.windowDrag};
   IF signal=-1 THEN INCL(ReqNw.flags,I.activate) END;
   ReqNw.type := I.customScreen;
   INCL(ReqNw.flags,I.windowDepth);
   ScP := I.LockPubScreen(NIL);
   IF ScP = NIL THEN RETURN -1 END;
      
   yfsize      := ScP.rastPort.font.ySize+1;
   baseline    := ScP.rastPort.txBaseline;
   leftoffset  := ScP.wBorLeft;
   rightoffset := ScP.wBorRight;
      
   REPEAT
      oldbodywidth := G.TextLength(S.ADR(ScP.rastPort),bodytexts[i],Strings.Length(bodytexts[i]));
      IF bodywidth < oldbodywidth THEN
         bodywidth := oldbodywidth;
      END;
      INC(i);
   UNTIL i = lines;

   okwidth:= G.TextLength(S.ADR(ScP.rastPort),oktext,oklength);
   IF cancel THEN
      cancelwidth:= G.TextLength(S.ADR(ScP.rastPort),canceltext,cancellength);
   END;
   IF middle THEN
      middlewidth := G.TextLength(S.ADR(ScP.rastPort),middletext,middlelength);
   END;
   
   minwidth := okwidth+leftoffset+rightoffset+8+24;
   IF cancel THEN minwidth := minwidth+cancelwidth+16+24 END;
   IF middle THEN minwidth := minwidth+middlewidth+16+24 END;
   ReqNw.width       := bodywidth+32+leftoffset+rightoffset;
   IF ReqNw.width < minwidth THEN ReqNw.width := minwidth; END;
   ReqNw.height      := (2+lines)*yfsize+28;
   ReqNw.screen      := ScP;
   IF titletext # "" THEN ReqNw.title := S.ADR(titletext)
   ELSE ReqNw.title := NIL;
   END;

   reqgads[0].topEdge  := (1+lines)*yfsize+19;
   reqgads[0].width    := okwidth+24;
   reqgads[0].height   := yfsize+5;

   IF ~cancel THEN
      reqgads[0].leftEdge := ReqNw.width-leftoffset-okwidth-24-4;
      reqgads[0].nextGadget := NIL;
      reqgads[1].width := reqgads[0].width;
   ELSE
      reqgads[0].leftEdge := leftoffset+4;
      reqgads[0].nextGadget := S.ADR(reqgads[1]);
      reqgads[1].leftEdge := ReqNw.width-leftoffset-cancelwidth-24-4;
      reqgads[1].topEdge  := reqgads[0].topEdge;
      reqgads[1].width    := cancelwidth+24;
      IF middle THEN
         reqgads[1].nextGadget := S.ADR(reqgads[2]);
         reqgads[2].width    := middlewidth+24;
         reqgads[2].leftEdge := ReqNw.width DIV 2 - reqgads[2].width DIV 2;
         IF (reqgads[2].leftEdge < reqgads[0].leftEdge+reqgads[0].width+16) OR
            (reqgads[2].leftEdge+reqgads[2].width >= reqgads[2].leftEdge-16)
         THEN reqgads[2].leftEdge := (ReqNw.width - reqgads[0].width - reqgads[1].width) DIV 2
              + reqgads[0].width - reqgads[2].width DIV 2;
         END;
         reqgads[2].topEdge  := reqgads[0].topEdge;
      ELSE reqgads[1].nextGadget := NIL;
      END;
   END;
   reqgads[1].height := reqgads[0].height;
   reqgads[2].height := reqgads[0].height;
   
   IF ~cancel THEN ReqNw.leftEdge := ScP.mouseX-ReqNw.width+6+(reqgads[0].width DIV 2);
   ELSE ReqNw.leftEdge := ScP.mouseX-ReqNw.width DIV 2;
   END;
   IF ReqNw.leftEdge < 0 THEN ReqNw.leftEdge := 0;END;
   IF ReqNw.leftEdge > ScP.width-ReqNw.width THEN ReqNw.leftEdge := ScP.width-ReqNw.width;END;
   ReqNw.topEdge := ScP.mouseY-ReqNw.height+4+(reqgads[0].height DIV 2);
   IF ReqNw.topEdge < 0 THEN ReqNw.topEdge := 0;END;
   IF ReqNw.topEdge > ScP.height-ReqNw.height THEN ReqNw.topEdge := ScP.height-ReqNw.height;END;

   (********************** Open Window : **********************)

   ReqWinP := I.OpenWindow(ReqNw);
   IF ReqWinP # NIL THEN
      rp := ReqWinP.rPort;
      G.SetFont(rp,ScP.rastPort.font);
   END;
   I.UnlockPubScreen(NIL, ScP);

   IF ReqWinP=NIL THEN RETURN -1;END;

   IF myreqStruct.Pattern THEN
      G.SetBPen(rp,myreqStruct.InBorderCol2);
      G.SetDrMd(rp,G.jam2);
      G.SetAfPt(rp,S.ADR(pattern),1);
   END;
   G.SetAPen(rp,myreqStruct.InWindowCol);
   G.RectFill(rp,ReqWinP.borderLeft,ReqWinP.borderTop,ReqWinP.width-ReqWinP.borderRight-1,ReqWinP.height-ReqWinP.borderBottom-1);
   G.SetAfPt(rp,NIL,0);
   G.SetDrMd(rp,G.jam1);
   Box(rp,ReqWinP.borderLeft+4,ReqWinP.borderTop+2,ReqWinP.width-8-ReqWinP.borderLeft-ReqWinP.borderRight,13+yfsize*lines,myreqStruct.InBorderCol1,myreqStruct.InBorderCol2,myreqStruct.BoxCol);
   Box(rp,reqgads[0].leftEdge,reqgads[0].topEdge,reqgads[0].width,reqgads[0].height,myreqStruct.InBorderCol2,myreqStruct.InBorderCol1,myreqStruct.BoxCol);
   IF cancel THEN
      Box(rp,reqgads[1].leftEdge,reqgads[1].topEdge,reqgads[1].width,reqgads[1].height,myreqStruct.InBorderCol2,myreqStruct.InBorderCol1,myreqStruct.BoxCol);
      IF middle THEN
         Box(rp,reqgads[2].leftEdge,reqgads[2].topEdge,reqgads[2].width,reqgads[2].height,myreqStruct.InBorderCol2,myreqStruct.InBorderCol1,myreqStruct.BoxCol);
      END;
   END;   
   Simplebox(rp,ReqWinP.borderLeft-1,ReqWinP.borderTop-1,ReqWinP.width-6,ReqWinP.height-yfsize-2,myreqStruct.InBorderCol1,myreqStruct.InBorderCol2,-1);
   Simplebox(rp,0,0,ReqWinP.width,ReqWinP.height,myreqStruct.OutBorderCol1,myreqStruct.OutBorderCol2,-1);
   G.SetAPen(rp,1);
   G.Move(rp,ReqWinP.borderLeft-1,ReqWinP.borderTop-1);
   G.Draw(rp,ReqWinP.width,ReqWinP.borderTop-1);
   G.SetDrMd(rp,G.jam1);
   G.SetAPen(rp,myreqStruct.TextCol);
   bodyleft := (ReqWinP.width-bodywidth) DIV 2;
   bodytop := ReqWinP.borderTop+9+baseline;
   i := 0;
   REPEAT
      G.Move(rp,bodyleft,bodytop);
      G.Text(rp,bodytexts[i],Strings.Length(bodytexts[i]));
      INC(bodytop,yfsize);
      INC(i);
   UNTIL i = lines;

   G.Move(rp,reqgads[0].leftEdge+12,reqgads[0].topEdge+baseline+3);
   G.Text(rp,oktext,oklength);
   IF cancel THEN
      G.Move(rp,reqgads[1].leftEdge+12,reqgads[1].topEdge+baseline+3);
      G.Text(rp,canceltext,cancellength);
      IF middle THEN
         G.Move(rp,reqgads[2].leftEdge+12,reqgads[2].topEdge+baseline+3);
         G.Text(rp,middletext,middlelength);
      END;
   END;
   
   waitset := LONGSET{ReqWinP.userPort.sigBit};
   IF signal#-1 THEN INCL(waitset,signal) END;
   LOOP
      messet := E.Wait(waitset);
      IF ReqWinP.userPort.sigBit IN messet THEN
         LOOP
            ReqIMessPtr := E.GetMsg(ReqWinP.userPort);
            IF ReqIMessPtr = NIL THEN EXIT END;
            ReqIMess := ReqIMessPtr^;
            E.ReplyMsg(ReqIMessPtr);
         
            IF (I.gadgetUp IN ReqIMess.class) THEN
            
               gPtr := ReqIMess.iAddress;
           
               CASE gPtr.gadgetID OF
                  0,1: I.CloseWindow(ReqWinP);RETURN 1-SHORT(gPtr.gadgetID) |
                  2  : I.CloseWindow(ReqWinP);RETURN 2 |
               ELSE
               END;
               
            ELSIF (I.vanillaKey IN ReqIMess.class) THEN
            
              CASE ReqIMess.code OF
                ORD("y"),ORD("Y"),ORD("v"),ORD("V"),ORD("r"),ORD("R"),13: I.CloseWindow(ReqWinP); RETURN 1|
                27,ORD("n"),ORD("N"),ORD("b"),ORD("B"): IF cancel THEN I.CloseWindow(ReqWinP); RETURN 0; END|
                ORD("i"),ORD("I"),ORD("m"),ORD("M"): IF middle THEN I.CloseWindow(ReqWinP); RETURN 2; END|
              ELSE
              END;
               
            ELSE
               Simplebox(rp,ReqWinP.borderLeft-1,ReqWinP.borderTop-1,ReqWinP.width-6,ReqWinP.height-yfsize-2,myreqStruct.InBorderCol1,myreqStruct.InBorderCol2,-1);
               Simplebox(rp,0,0,ReqWinP.width,ReqWinP.height,myreqStruct.OutBorderCol1,myreqStruct.OutBorderCol2,-1);
               G.SetAPen(rp,1);
               G.Move(rp,ReqWinP.borderLeft-1,ReqWinP.borderTop-1);
               G.Draw(rp,ReqWinP.width,ReqWinP.borderTop-1);
            END;
         END;
      END;
      IF (signal#-1) AND (signal IN messet) THEN
         IF ~twice THEN I.CloseWindow(ReqWinP); RETURN 3
         ELSE twice := FALSE;
         END;
      END;
   END;
      
END ARequest;

BEGIN

   InitRequest;

END KWReq.
