(*---------------------------------------------------------------------------
    :Program.    LQPrint.mod
    :Author.     Achim Siebert
    :Address.    Nobileweg 67, D-7000 Stuttgart-40
    :Phone.      0711/822509
    :Shortcut.   [acs]
    :Version.    1.3
    :Date.       27-Mar-92
    :Copyright.  Shareware
    :Language.   Oberon
    :Translator. Oberon
    :Imports.    none.
    :UpDate.     1.1: still in Modula2; 1.2: changed to Oberon;
                 1.3.:added special characters; 1.5: nearly all special characters;
                 now multiple files/fonts/modes; faster;
                 2.0: GUI;
    :Contents.   Program for ASCII-text printouts on Epson LQ.
    :Usage.      LQPrint File(s)/M,NW=NOWINDOW/S,SETTINGS=FROM/K
    :Remark.     none.
---------------------------------------------------------------------------*)

MODULE LQPrint;

IMPORT Dos,
       Exec,
       AReq20,
       Menu,
       OL:OberonLib,
       FR: FileReq,
       FS:FileSystem,
       I: Intuition,
       S: SYSTEM,
       GT: GadTools,
       u:Utility,
       WB:Workbench,
       g:Graphics,
       Icon,
       Arguments,
       Strings;
       
TYPE prfonttype      = ARRAY 11 OF Exec.STRPTR;
     prCPItype       = ARRAY 4 OF Exec.STRPTR;
     checktextstype  = ARRAY 10 OF Exec.STRPTR;
     reqtype         = ARRAY 2 OF ARRAY 40 OF CHAR;

CONST errstring         = "Error while printing!";
      configID          = "LQPrint Settings  1992 Achim Siebert";
      defaultConfigName = "LQPrint.prefs";
      offline           = reqtype("Your printer is offline,","please switch to online!");
      configerr         = reqtype("Couldn't read settings file:","\"ENVARC:LQPrint.prefs\" !");
      configerr2        = reqtype("Probably not a LQPrint settings file!","Loading failed!");
      version           = "\o$VER: LQPrint 2.0 (27.03.92)";
      
      Topaz80Const      = g.TextAttr(
          S.ADR("topaz.font"),   (*  Name *)
          8,                     (*  YSize *)
          SHORTSET{},            (*  Style *)
          SHORTSET{});           (*  Flags *)
    
      prfonts = prfonttype(
                  S.ADR("Draft"),
                  S.ADR("Roman"),
                  S.ADR("Sans Serif"),
                  S.ADR("Courier"),
                  S.ADR("Prestige"),
                  S.ADR("Script"),
                  S.ADR("Orator"),
                  S.ADR("Orator-S"),
                  S.ADR("OCR-B"),
                  S.ADR("OCR-A"),
                NIL);

      prCPI   = prCPItype( 
                  S.ADR("Pica"),
                  S.ADR("Elite"),
                  S.ADR("Micro"),
                NIL);

      checktexts = checktextstype(
                  S.ADR("Formfeed"),
                  S.ADR("Proportional"),                  
                  S.ADR("Condensed"),
                  S.ADR("1.5 space"),
                  S.ADR("Skip Perf."),
                  S.ADR("Doublestrike"),
                  S.ADR("Unidirectional"),
                  S.ADR("E x p a n d"),
                  S.ADR("|  Justify  |"), 
                  S.ADR("Bold"));

      gFormf0     = 1;     (* Gadget-IDs *)
      gProp1      = 2;
      gCond2      = 3;
      gSpace3     = 4;
      gSkip4      = 5;
      gDouble5    = 6;
      gUnidir6    = 7;
      gExpand7    = 8;
      gJust8      = 9;
      gBold9      = 10;      
      gSELECT     = 11;
      gPRINT      = 12;
      gEXIT       = 13;
      gSAVE       = 14;
      gCycleFonts = 15;
      gCycleCPI   = 16;
      gINTLeft    = 17;
      gINTRight   = 18;
      gINTtab     = 19;
      gTabSlide   = 20;
      gFName      = 21;
      

   (* $DataChip+ *)
   
     ZZData = "\x00\x00\x00\x00\x04\x00\x07\xC0\x00\x00\x07\xC0\x01\x00\x03\x80"
              "\x00\x00\x07\xE0\x07\xC0\x1F\xF8\x1F\xF0\x3F\xEC\x3F\xF8\x7F\xDE"
              "\x3F\xF8\x7F\xBE\x7F\xFC\xFF\x7F\x7E\xFC\xFF\xFF\x7F\xFC\xFF\xFF"
              "\x3F\xF8\x7F\xFE\x3F\xF8\x7F\xFE\x1F\xF0\x3F\xFC\x07\xC0\x1F\xF8"
              "\x00\x00\x07\xE0\x00\x00\x00\x00";
   
VAR

  Printer,TextFile   : Dos.FileHandlePtr;
  fileinfo           : Dos.FileInfoBlock;
  FileName,DirName   : ARRAY 256 OF CHAR;
  PathName           : Exec.STRING;
  filesize,length,rc : LONGINT;
  ConfFile           : FS.File;
  ConfID, TestID     : ARRAY 50 OF CHAR;
  Buffer             : ARRAY 512 OF CHAR;
  Buffer2            : ARRAY 512*3 OF CHAR;
  prreg[12570624]    : BYTE;  
  
  gstate,gstateOld   : STRUCT
                          prfontnum,cpinum,tabnum,leftnum,rightnum: INTEGER;
                          gchecked                                : SET;
                       END;

  strinfo, findinfo  : I.StringInfoPtr;
  gad,findGad        : I.GadgetPtr;
  lbool              : I.LONGBOOL;

  font               : g.TextFontPtr;
  scP                : I.ScreenPtr;
  glist,stopglist    : I.GadgetPtr;
  mlist              : I.MenuPtr;
  winP               : I.WindowPtr;
  rp                 : g.RastPortPtr;
  vi                 : Exec.APTR;

  topaz80,top80bold  : g.TextAttr;

  imsg               : I.IntuiMessagePtr;
  imsgClass          : LONGSET;
  nextMenu           : I.MenuItemPtr;
  topborder, numargs,
  argcount, selectedMenu,
  selectedItem,imsgCode,i     : INTEGER;
  cricon,wbStarted,confloaded : BOOLEAN;

  rdargs             : Dos.RDArgsPtr;
  argarray           : ARRAY 3 OF LONGINT;
  setnamePtr         : Exec.STRPTR;
  fnames             : POINTER TO ARRAY 100 OF Exec.STRPTR;
  anchor             : Dos.AnchorPath;

  reqarray: ARRAY 1 OF ARRAY 256 OF CHAR;


PROCEDURE Inform(info:ARRAY OF CHAR);

BEGIN

   IF ~ wbStarted THEN Dos.PrintF(info);
   ELSE
      COPY(info,reqarray[0]);
      IF AReq20.ARequest(reqarray,"LQPrint"," Okay ","","",NIL)#0 THEN END;
   END;

END Inform;

PROCEDURE CreateAllGadgets(): BOOLEAN;

VAR
  ng: GT.NewGadget;
  gad,stopgad: I.GadgetPtr;
  index: INTEGER;

BEGIN
  gad := GT.CreateContext(glist);
  stopgad := GT.CreateContext(stopglist);

  ng.leftEdge := 8;
  ng.topEdge := 2+topborder;
  ng.width := 132;
  ng.height := 14;
  ng.gadgetText := S.ADR("Select File");
  ng.textAttr := S.ADR(topaz80);
  ng.gadgetID := gSELECT;
  ng.flags := LONGSET{};
  ng.visualInfo := vi;
  gad := GT.CreateGadget(GT.buttonKind, gad, ng, u.done);

  ng.leftEdge := 178;
  ng.gadgetText := S.ADR("PRINT !");
  ng.gadgetID := gPRINT;
  gad := GT.CreateGadget(GT.buttonKind, gad, ng, u.done);
  
  ng.gadgetText := S.ADR("STOP !");
  ng.gadgetID := 100;
  ng.textAttr := S.ADR(top80bold);
  stopgad := GT.CreateGadget(GT.buttonKind, stopgad, ng, u.done);
  ng.textAttr := S.ADR(topaz80);
 
  ng.topEdge := 160+topborder;
  ng.gadgetText := S.ADR("Exit");
  ng.gadgetID := gEXIT;
  gad := GT.CreateGadget(GT.buttonKind, gad, ng, u.done);
   
  ng.leftEdge := 8;
  ng.gadgetText := S.ADR("Save Settings");
  ng.gadgetID := gSAVE;
  gad := GT.CreateGadget(GT.buttonKind, gad, ng, u.done);
  
  ng.topEdge := 20+topborder;
  ng.width   := 302;
  ng.gadgetText := NIL;
  ng.gadgetID := gFName;
  gad := GT.CreateGadget(GT.stringKind, gad, ng,
                         GT.stMaxChars, 512,
                         u.done);
 
  ng.leftEdge := 198;
  ng.topEdge  := 106+topborder;
  ng.width    := 112;
  ng.height   := 14;
  ng.gadgetText := S.ADR("Font:");
  ng.gadgetID := gCycleFonts;
  gad := GT.CreateGadget(GT.cycleKind, gad, ng,
                         GT.cyLabels, S.ADR(prfonts),
                         GT.cyActive, gstate.prfontnum,
                         u.done);

  ng.topEdge  := 124+topborder;
  ng.gadgetID := gCycleCPI;
  ng.gadgetText := S.ADR("CPI:");
  gad := GT.CreateGadget(GT.cycleKind, gad, ng,
                         GT.cyLabels, S.ADR(prCPI),
                         GT.cyActive, gstate.cpinum,
                         u.done);
  
  ng.leftEdge := 8;
  ng.topEdge  := 26+topborder;
  ng.height   := 12;
  INCL(ng.flags,GT.placeTextRight);
  index := 1;
  REPEAT
  	 IF index = 6 THEN ng.leftEdge := 152; ng.topEdge := 26+topborder END;
    INC(ng.topEdge,12);
    ng.gadgetID := index;
    ng.gadgetText := checktexts[index-1];
    lbool := I.BoolToLong(index IN gstate.gchecked);
    gad := GT.CreateGadget(GT.checkBoxKind, gad, ng,
                           GT.cbChecked, lbool,
                           u.done);
    INC(index);
  UNTIL index = 11;

  ng.leftEdge := 8;
  ng.topEdge  := 142 + topborder;
  ng.height   := 14;
  ng.width    := 45;
  ng.gadgetID := gINTLeft;
  ng.gadgetText := S.ADR("left     margins");
  gad := GT.CreateGadget(GT.integerKind, gad, ng,
                         GT.inNumber, gstate.leftnum,
                         GT.inMaxChars, 2,
                         u.done);

  ng.leftEdge := 265;
  ng.gadgetID := gINTRight;
  EXCL(ng.flags,GT.placeTextRight);
  INCL(ng.flags,GT.placeTextLeft);
  ng.gadgetText := S.ADR("right");
  gad := GT.CreateGadget(GT.integerKind, gad, ng,
                  GT.inNumber, gstate.rightnum,
                  GT.inMaxChars, 3,
                  u.done);

  ng.leftEdge := 58;
  ng.topEdge  := 112 + topborder;
  ng.gadgetID := gINTtab;
  EXCL(ng.flags,GT.placeTextLeft);
  INCL(ng.flags,GT.placeTextAbove);
  ng.gadgetText := S.ADR("Tab size:");
  gad := GT.CreateGadget(GT.integerKind, gad, ng,
                  GT.inNumber, gstate.tabnum,
                  GT.inMaxChars, 2,
                  u.done);

  ng.leftEdge := 18;
  ng.topEdge := 130+topborder;
  ng.height := 8;
  ng.width := 120;
  ng.flags := LONGSET{};
  ng.gadgetText := NIL;
  ng.gadgetID := gTabSlide;
  gad := GT.CreateGadget(GT.sliderKind, gad, ng,
                  GT.slMin, 1,
                  GT.slMax, 10,
                  GT.slLevel, gstate.tabnum,
                  I.gaImmediate, I.LTRUE,
                  I.gaRelVerify, I.LTRUE,
                  u.done);

  RETURN (gad#NIL) AND (stopgad#NIL);

END CreateAllGadgets;


PROCEDURE CreateMenu(VAR mlist: I.MenuPtr);

BEGIN

   Menu.StartMenu(winP);

   Menu.NewMenu("Project");

   Menu.NewItem("Open...","O");
   Menu.Separator;
   Menu.NewItem("Print","P");
   Menu.Separator;
   Menu.NewItem("About...","B");
   Menu.Separator;
   Menu.NewItem("Quit","Q");

   Menu.NewMenu("Settings");

   Menu.NewItem("Last Saved","L");
   Menu.NewItem("Load Settings...","C");
   Menu.NewItem("Save Settings","S");
   Menu.NewItem("Save Settings As...","A");
   Menu.NewItem("Restore","R");
   Menu.Separator;
   Menu.NewItemChecked("Create Icons?","I",cricon);

   mlist := Menu.EndMenu();

END CreateMenu;


PROCEDURE OpenWindow();

VAR

  myNewWin: I.NewWindow;
  leftEdge,topEdge: INTEGER;

CONST
   myNewWinConst = I.NewWindow(
       0, 0,               (*  LeftEdge, TopEdge *)
       310, 176,           (*  Width, Height *)
       -1, -1,             (*  DetailPen, BlockPen *)
       LONGSET{I.mouseButtons,I.gadgetUp,I.menuPick,I.closeWindow,I.refreshWindow}, (*  IDCMPFlags *)
       LONGSET{I.activate,I.windowDrag,I.windowDepth,I.windowClose}, (* Flags *)
       NIL,                (*  FirstGadget *)
       NIL,                (*  CheckMark *)
       S.ADR("LQPrint V2.0  1992 Achim Siebert"), (*  Title *)
       NIL,                (*  Screen *)
       NIL,                (*  BitMap *)
       90, 14,             (*  MinWidth, MinHeight *)
       310, 176,           (*  MaxWidth, MaxHeight *)
       {I.wbenchScreen});  (*  Type *)

BEGIN

   topaz80 := Topaz80Const;
   
   top80bold := topaz80;
   top80bold.style := SHORTSET{g.bold};
   
   font := g.OpenFont(topaz80);
   IF font=NIL THEN HALT(21) END;

   scP := I.LockPubScreen(NIL);
   IF scP=NIL THEN HALT(22) END;

   vi := GT.GetVisualInfo(scP,u.done);
   IF vi=NIL THEN HALT(23) END;

   topborder := scP.wBorTop + scP.font.ySize + 1;

   IF ~ CreateAllGadgets() THEN
      HALT(24);
   END;

   myNewWin := myNewWinConst;
   myNewWin.firstGadget := glist;

   leftEdge := scP.mouseX-68;
   topEdge  := scP.mouseY-topborder-10;

   winP := I.OpenWindowTags(myNewWin,
                            I.waInnerHeight,myNewWin.height,
                            I.waInnerWidth,myNewWin.width,
                            I.waAutoAdjust,I.LTRUE,
                            I.waLeft,leftEdge,
                            I.waTop,topEdge,
                            I.waPubScreen,  scP,
                            u.done);
   IF winP = NIL THEN HALT(20) END;
  
   IF scP#NIL THEN
      I.UnlockPubScreen(NIL, scP);
   END;
   
   CreateMenu(mlist);
   IF ~ I.SetMenuStrip(winP,mlist^) THEN HALT(20) END;

   GT.RefreshWindow(winP, NIL);
   
   rp := winP.rPort;

END OpenWindow;


PROCEDURE GetGadget(firstgad:I.GadgetPtr;id: INTEGER): I.GadgetPtr;

BEGIN

   IF firstgad.gadgetID = id THEN RETURN firstgad
   ELSE
      RETURN GetGadget(firstgad.nextGadget,id)
   END;

END GetGadget;


PROCEDURE StopGads();

BEGIN

   I.SetPointer (winP,ZZData,16,16,-6,0);
   IF (glist # NIL) AND (I.RemoveGList(winP,glist,-1)#0) THEN END;

END StopGads;


PROCEDURE RunGads();

BEGIN

   IF (glist # NIL) AND (I.AddGList(winP,glist,-1,-1,NIL)#0) THEN END;
   I.ClearPointer(winP);

END RunGads;


PROCEDURE InitStates();

BEGIN

  gstate.gchecked  := {};
  gstate.prfontnum := 1;
  gstate.cpinum    := 0;
  gstate.tabnum    := 3;
  gstate.leftnum   := 8;
  gstate.rightnum  := 68;

END InitStates;


PROCEDURE GetStates();

VAR gadnum : INTEGER;

BEGIN

   FOR gadnum := gFormf0 TO gBold9 DO
      findGad := GetGadget(winP.firstGadget,gadnum);
      IF I.selected IN findGad.flags THEN
         INCL(gstate.gchecked,gadnum)
      ELSE EXCL(gstate.gchecked,gadnum)
      END;
   END;
   findGad := GetGadget(winP.firstGadget,gINTLeft);
   strinfo := findGad.specialInfo;
   gstate.leftnum := SHORT(strinfo.longInt);
   findGad := GetGadget(winP.firstGadget,gINTRight);
   strinfo := findGad.specialInfo;
   gstate.rightnum := SHORT(strinfo.longInt);
   findGad := GetGadget(winP.firstGadget,gINTtab);
   strinfo := findGad.specialInfo;
   gstate.tabnum := SHORT(strinfo.longInt);

END GetStates;


PROCEDURE SetFileGad();

BEGIN

   findGad := GetGadget(winP.firstGadget,gFName);
   GT.SetGadgetAttrs(findGad^,winP,NIL,GT.stString,S.ADR(FileName),u.done);

END SetFileGad;


PROCEDURE GetFileSize();

BEGIN

   TextFile := Dos.Open(FileName,Dos.oldFile);
   IF TextFile # NIL THEN
      IF ~ Dos.ExamineFH(TextFile,fileinfo) THEN HALT(20) END;
      filesize := fileinfo.size;
   END;
   IF Dos.Close(TextFile) THEN TextFile := NIL END;

END GetFileSize;

PROCEDURE GetFile();

VAR ok : BOOLEAN;

BEGIN

   StopGads();
   REPEAT
      ok := FR.FileReqWin("Select file to print",FileName,winP);
   UNTIL ~ok OR (ok AND FS.Exists(FileName));
   RunGads();
   IF ok THEN
      SetFileGad();   
      GetFileSize();
   END;

END GetFile;

PROCEDURE PrintOut();

TYPE umrechn = ARRAY 256 OF BYTE;

CONST umr = umrechn( "\x00","\x01","\x02","\x03","\x04","\x05","\x06","\x07",
                     "\x08","\x09","\x0A","\x0B","\x0C","\x0D","\x0E","\x0F",
                     "\x10","\x11","\x12","\x13","\x14","\x15","\x16","\x17",
                     "\x18","\x19","\x1A","\x1B","\x1C","\x1D","\x1E","\x1F",
                       " " ,  "!" ,  '"' ,  "#" ,  "$" ,  "%" ,  "&" ,  "'" ,
                       "(" ,  ")" ,  "*" ,  "+" ,  "," ,  "-" ,  "." ,  "/" ,
                       "0" ,  "1" ,  "2" ,  "3" ,  "4" ,  "5" ,  "6" ,  "7" ,
                       "8" ,  "9" ,  ":" ,  ";" ,  "<" ,  "=" ,  ">" ,  "?" ,
                       "@" ,  "A" ,  "B" ,  "C" ,  "D" ,  "E" ,  "F" ,  "G" ,
                       "H" ,  "I" ,  "J" ,  "K" ,  "L" ,  "M" ,  "N" ,  "O" ,
                       "P" ,  "Q" ,  "R" ,  "S" ,  "T" ,  "U" ,  "V" ,  "W" ,
                       "X" ,  "Y" ,  "Z" ,  "[" , "\\" ,  "]" ,  "^" ,  "_" ,
                       "`" ,  "a" ,  "b" ,  "c" ,  "d" ,  "e" ,  "f" ,  "g" ,
                       "h" ,  "i" ,  "j" ,  "k" ,  "l" ,  "m" ,  "n" ,  "o" ,
                       "p" ,  "q" ,  "r" ,  "s" ,  "t" ,  "u" ,  "v" ,  "w" ,
                       "x" ,  "y" ,  "z" ,  "{" ,  "|" ,  "}" ,  "~" ,"\xB2",
                     "\x80","\x81","\x82","\x83","\x84","\x85","\x86","\x87",
                     "\x88","\x89","\x8A","\x8B","\x8C","\x8D","\x8E","\x8F",
                     "\x90","\x91","\x92","\x93","\x94","\x95","\x96","\x97",
                     "\x98","\x99","\x9A","\x9B","\x9C","\x9D","\x9E","\x9F",
                       " " ,"\xAD","\x9B","\x9C",  " " ,"\x9D",  "|" ,"\x15",
                       " " ,  "c" ,"\xA6","\xAE","\xAA",  "-" ,  "r" ,"\xC4",
                     "\xF8","\xF1",  " " ,  " " ,  "'" ,"\xE6","\xE3","\xF9",
                       "," ,  " " ,"\xA7","\xAF","\xAC","\xAB",  " " ,"\xA8",
                       "A" ,   "A",  "A" ,  "A" ,"\x8E","\x8F","\x92","\x80",
                       "E" ,"\x90",  "E" ,  "E" ,  "I" ,  "I" ,  "I" ,  "I" ,
                       " " ,"\xA5",  "O" ,  "O" ,  "O" ,  "O" ,"\x99",  "x" ,
                       " " ,  "U" ,  "U" ,  "U" ,"\x9A",  "Y" ,  " " ,"\xE1",
                     "\x85","\xA0","\x83",  "a" ,"\x84","\x86","\x91","\x87",
                     "\x8A","\x82","\x88","\x89","\x8D","\xA1","\x8C","\x8B",
                     "\xEB","\xA4","\x95","\xA2","\x93",  "o" ,"\x94","\xF6",
                     "\xED","\x97","\xA3","\x96","\x81",  "y" ,  " " ,"\x98");

VAR k,j,len: LONGINT;
  
BEGIN
   len := 0;
   k := 0;
   j := 0;
   REPEAT
         CASE Buffer[k] OF
         |"" : Buffer2[j] := "\x1B"; Buffer2[j+1] := "R"; Buffer2[j+2] := "\x05";
                Buffer2[j+3] := "$"; Buffer2[j+4] := "\x1B"; Buffer2[j+5] := "R";
                Buffer2[j+6] := "\x00"; INC(j,6);
         |"","","","","" :
                Buffer2[j] := "\x1B"; Buffer2[j+1] := "S"; Buffer2[j+2] := "\x00";
                IF Buffer[k] = "" THEN
                   Buffer2[j+3] := "1";
                ELSIF Buffer[k] = "" THEN
                   Buffer2[j+3] := "2";
                ELSIF Buffer[k] = "" THEN
                   Buffer2[j+3] := "-"
                ELSE Buffer2[j+3] := "3";
                END;
                Buffer2[j+4] := "\x1B"; Buffer2[j+5] := "T"; INC(j,5);
                IF Buffer[k] = "" THEN
                   Buffer2[j+1] := "\x08"; Buffer2[j+2] := "\x1B"; Buffer2[j+3] := "S";
                   Buffer2[j+4] := "\x01"; Buffer2[j+5] := "4";
                   Buffer2[j+6] := "\x1B"; Buffer2[j+7] := "T";
                   Buffer2[j+8] := "\x08"; Buffer2[j+9] := "-";
                   INC(j,9);
                END;
         |"" : Buffer2[j] := "-"; Buffer2[j+1] := "\x08"; Buffer2[j+2] := "D"; INC(j,2);
         |"","" : Buffer2[j] := "b"; Buffer2[j+1] := "\x08"; Buffer2[j+2] := "p"; INC(j,2);
         |"" : Buffer2[j] := "O"; Buffer2[j+1] := "\x08"; Buffer2[j+2] := "\x2F";INC(j,2);
         |"","","","" : Buffer2[j] := CHR(umr[ORD(Buffer[k])]); Buffer2[j+1] := "\x08"; Buffer2[j+2] := "\x7E";INC(j,2);
         |"","","" : Buffer2[j] := CHR(umr[ORD(Buffer[k])]); Buffer2[j+1] := "\x08";
                Buffer2[j+2] := "\x1B"; Buffer2[j+3] := "R"; Buffer2[j+4] := "\x01";
                Buffer2[j+5] := "~"; Buffer2[j+6] := "\x1B"; Buffer2[j+7] := "R";
                Buffer2[j+8] := "\x00";
                INC(j,8);
         |"","","","","" : Buffer2[j] := CHR(umr[ORD(Buffer[k])]); Buffer2[j+1] := "\x08";
                Buffer2[j+2] := "^"; INC(j,2);
         | ELSE Buffer2[j] := CHR(umr[ORD(Buffer[k])]) END;
         INC(k);
         INC(j);
         IF j > 512*3-20 THEN
            HALT(20);
         END;
   UNTIL k = length;
   len := Dos.Write(Printer,Buffer2,j);
   IF len # j THEN
      Inform(errstring);
      HALT(20);
   END;
      
END PrintOut;


PROCEDURE Print();

VAR i,j,numblocks,x,lastx,sent : INTEGER;

   PROCEDURE WaitForPrinter() : BOOLEAN;
   
   VAR l: INTEGER;
   
   BEGIN
   
      IF prreg # 0FCX THEN
         REPEAT
            IF AReq20.ARequest(offline,"LQPrint","Retry","Cancel","",NIL)=0 THEN RETURN FALSE END;
         UNTIL (prreg = 0FCX);
      END;
      RETURN TRUE;
   
   END WaitForPrinter;

   PROCEDURE Send(a,b,c : INTEGER);
   
   VAR n,printed : LONGINT;
   
   BEGIN
     
     n := 1;
     Buffer[0] := CHR(a);
     IF b # -1 THEN
       INC(n);
       Buffer[1] := CHR(b);
       IF c # -1 THEN
          INC(n);
          Buffer[2] := CHR(c);
       END;
     END;
     
     printed := Dos.Write(Printer,Buffer,n);
     IF printed # n THEN
        Inform(errstring);
        HALT(20);
     END;

END Send;

BEGIN

   IF wbStarted THEN
      GetStates();
      StopGads();
   END;
   IF (FileName = "") OR (~ FS.Exists(FileName)) THEN
     Inform("No existing file selected!");
   ELSE
      Printer := Dos.Open("PAR:",Dos.newFile);
      IF (Printer = NIL) THEN Inform("Couldn't open PAR: for output")
      ELSE
         IF WaitForPrinter() THEN
         
   
            Send(27,64,-1);  (* Init *)
            Send(27,116,1);  (* Graphic character set for German Umlaute etc. *)
   
            Send(27,120,1);  (* Letter Quality *)
   
            CASE gstate.prfontnum OF
               0 :    Send(27,120,0)|                   (* Draft *)
               1..5 : Send(27,107,gstate.prfontnum-1);| (* Roman, SansSerif, Courier, Prestige, Script *)
               6,7  : Send(27,107,gstate.prfontnum+1);  (* Orator(-S), always unidirectional print! *)
                      Send(27,85,1)|
               8,9  : Send(27,107,gstate.prfontnum-3);| (* OCR-B, OCR-A *)
            ELSE
            END;
   
            CASE gstate.cpinum OF
               0: Send(27,80, -1); Send(27,50,-1)|   (* Pica,  pitch 10, 1/6" line space *)
               1: Send(27,77, -1); Send(27,50,-1)|   (* Elite, pitch 12, 1/6" line space *)
               2: Send(27,103,-1); Send(27,48,-1)|   (* Micro print,     1/8" line space *)
            ELSE
            END;

            IF gBold9   IN gstate.gchecked THEN Send(27,69,-1); END;
            IF gCond2   IN gstate.gchecked THEN Send(15,-1,-1); END;
            IF gDouble5 IN gstate.gchecked THEN Send(27,71,-1); END;
            IF gJust8   IN gstate.gchecked THEN Send(27,97, 3); END;
            IF gProp1   IN gstate.gchecked THEN Send(27,112,1); END;
            IF gSkip4   IN gstate.gchecked THEN Send(27,78, 6); END;
            IF gUnidir6 IN gstate.gchecked THEN Send(27,85, 1); END;
            IF gExpand7 IN gstate.gchecked THEN Send(27,87, 1); END;
            IF gSpace3  IN gstate.gchecked THEN Send(27,65,15); END;
   
            Send(27,108,gstate.leftnum);   (* Left margin  *)
            Send(27,81,gstate.rightnum);   (* Right margin *)
   
            i :=  gstate.tabnum;
            Send(27,68,0);         (* tab reset             *)
            Send(27,68,i);         (* first tab             *)
            j := 1;
            LOOP
               INC(i,gstate.tabnum);INC(j);
               IF (j <= 32) AND (i <= 255) THEN
                     Send(i,-1,-1) (* following tabs        *)
               ELSE Send(0,-1,-1); EXIT;
               END;
            END;

            GetFileSize();
            TextFile := Dos.Open(FileName,Dos.oldFile);
            IF TextFile # NIL THEN
               IF (filesize > 512) AND wbStarted THEN
                  IF I.AddGList(winP,stopglist,-1,-1,NIL)#0 THEN END;
                  I.RefreshGadgets(stopglist,winP,NIL);
               END;
               IF filesize > 0 THEN
                  IF wbStarted THEN
                     numblocks := SHORT(filesize DIV 512);
                     IF filesize MOD 512 # 0 THEN INC(numblocks);END;
                     g.SetAPen(rp,0);
                     g.SetDrMd(rp,SHORTSET{g.complement});
                     sent := 0; lastx:= 12; x := 0;
                  END;
                  REPEAT
                     length := Dos.Read(TextFile,Buffer,512);
                     IF length > 0 THEN
                        PrintOut();
                        IF wbStarted THEN
                           imsg := GT.GetIMsg(winP.userPort);
                           IF imsg#NIL THEN
                              imsgClass := imsg.class;
                              imsgCode := imsg.code;
                              gad := imsg.iAddress;
                              GT.ReplyIMsg(imsg);
                              IF I.gadgetUp IN imsgClass THEN
                                 IF gad.gadgetID = 100 THEN
                                    reqarray[0]:="Stop current print job?";
                                    IF AReq20.ARequest(reqarray,"LQPrint"," YES! Stop it! "," Cancel ","",NIL) = 1 THEN
                                       length := -1;
                                    END;
                                 END;
                              END;
                           END;
                           IF length # -1 THEN
                           INC(sent);
                           x := SHORT((LONG(sent*294)) DIV numblocks)+12;
                           IF x > lastx THEN
                              g.RectFill(rp,lastx,22+topborder,x-1,31+topborder);
                           END;
                           lastx := x;
                           END;
                        END;
                     END;
                  UNTIL length <= 0;
                  IF wbStarted THEN
                     Dos.Delay(25);
                     g.RectFill(rp,12,22+topborder,x-1,31+topborder);
                     g.SetDrMd(rp,g.jam1);
                  END;
               END;

               IF gFormf0 IN gstate.gchecked THEN Send(12,-1,-1); ELSE Send(10,13,-1) END;
               IF Dos.Close(TextFile) THEN TextFile := NIL END;
               IF wbStarted AND (I.RemoveGList(winP,stopglist,-1)#0) THEN END;

            ELSE Inform("Could'nt open file");
            END;
         END;
      END;
      IF Dos.Close(Printer) THEN Printer  := NIL END;
   END;
   
   IF wbStarted THEN RunGads();
      findGad := GetGadget(winP.firstGadget,gPRINT);
      I.RefreshGList(findGad,winP,NIL,1);
   END;

END Print;


PROCEDURE SaveConfig(default,quit:BOOLEAN);

VAR ok         : BOOLEAN;
    ConfigName : ARRAY 256 OF CHAR;
    diskobjPtr : WB.DiskObjectPtr;

BEGIN
   
   GetStates();
   ok := TRUE;
   ConfigName := "ENVARC:";
   Strings.Append(ConfigName,defaultConfigName);
   IF default AND (~ FS.Exists("ENVARC:")) THEN ok := FALSE;
   ELSIF quit THEN
      ConfigName := "ENV:";
      IF ~ FS.Exists(ConfigName) THEN ok := FALSE
      ELSE Strings.Append(ConfigName,defaultConfigName);
      END;
   END;
   IF ~(default OR quit) THEN
      StopGads();
      ok := FR.FileReqWinSave("Save config file",ConfigName,winP);
      RunGads();
   END;
   IF ok AND FS.Open(ConfFile,ConfigName,TRUE) AND
      FS.WriteString(ConfFile,ConfID)
      AND FS.Write(ConfFile,gstate)
      AND FS.Close(ConfFile) THEN
      COPY(ConfigName,PathName); Strings.Append(PathName,".info");
      IF cricon AND (~FS.Exists(PathName)) THEN
         diskobjPtr := Icon.GetDefDiskObject(WB.project);
         IF diskobjPtr#NIL THEN
            IF DirName # "" THEN
               diskobjPtr.defaultTool := S.ADR(DirName);
            ELSE
               diskobjPtr.defaultTool := S.ADR("SYS:Tools/LQPrint");
            END;
            IF Icon.PutDiskObject(ConfigName,diskobjPtr) THEN END;
            Icon.FreeDiskObject(diskobjPtr);
         END;
      END;
   ELSIF ~quit AND ok THEN Inform("Saving config file failed!");
   END;

END SaveConfig;


PROCEDURE SetStates();

VAR gadnum : INTEGER;

BEGIN

   FOR gadnum := gFormf0 TO gBold9 DO
      findGad := GetGadget(winP.firstGadget,gadnum);
      lbool := I.BoolToLong(gadnum IN gstate.gchecked);
      GT.SetGadgetAttrs(findGad^,winP,NIL,GT.cbChecked,lbool,u.done);
   END;
   findGad := GetGadget(winP.firstGadget,gINTLeft);
   GT.SetGadgetAttrs(findGad^,winP,NIL,GT.inNumber,gstate.leftnum,u.done);
   findGad := GetGadget(winP.firstGadget,gINTRight);
   GT.SetGadgetAttrs(findGad^,winP,NIL,GT.inNumber,gstate.rightnum,u.done);
   findGad := GetGadget(winP.firstGadget,gINTtab);
   GT.SetGadgetAttrs(findGad^,winP,NIL,GT.inNumber,gstate.tabnum,u.done);
   findGad := GetGadget(winP.firstGadget,gTabSlide);
   GT.SetGadgetAttrs(findGad^,winP,NIL,
                     GT.slLevel,gstate.tabnum,u.done);
   findGad := GetGadget(winP.firstGadget,gCycleFonts);
   GT.SetGadgetAttrs(findGad^,winP,NIL,GT.cyActive,gstate.prfontnum,u.done);
   findGad := GetGadget(winP.firstGadget,gCycleCPI);
   GT.SetGadgetAttrs(findGad^,winP,NIL,GT.cyActive,gstate.cpinum,u.done);

END SetStates;


PROCEDURE OpenConfig(default,first:BOOLEAN);

VAR ok: BOOLEAN;
    ConfigName : ARRAY 256 OF CHAR;

BEGIN
   
   ok := TRUE;
   ConfigName := "ENVARC:";
   Strings.Append(ConfigName,defaultConfigName);
   IF first THEN InitStates();
      ConfigName := "ENV:";
      Strings.Append(ConfigName,defaultConfigName);
   ELSIF default THEN
      IF ~ FS.Exists(ConfigName) THEN
         IF AReq20.ARequest(configerr,"LQPrint"," Okay ","","",NIL)=0 THEN END;
         ok := FALSE;
      END;
   ELSE
      StopGads();
      ok := FR.FileReqWin("Open config file",ConfigName,winP);
      RunGads();
   END;
   IF ok THEN
      gstateOld := gstate;
      IF FS.Exists(ConfigName) AND FS.Open(ConfFile,ConfigName,FALSE) THEN
         IF FS.ReadString(ConfFile,TestID) AND
            (TestID = ConfID) AND FS.Read(ConfFile,gstate) AND
            FS.Close(ConfFile) AND ~ first THEN
            SetStates();
         ELSIF ~first THEN IF AReq20.ARequest(configerr2,"LQPrint"," Okay ","","",NIL)#0 THEN END;
         END;
      END;
   END;

END OpenConfig;


PROCEDURE CliConfig(from:Exec.STRING):BOOLEAN;

BEGIN

   IF (from # "") AND FS.Exists(from) AND FS.Open(ConfFile,from,FALSE) AND
         FS.ReadString(ConfFile,TestID) AND
         (TestID = ConfID) AND FS.Read(ConfFile,gstate) AND
         FS.Close(ConfFile) THEN RETURN TRUE;
   END;
   RETURN FALSE;

END CliConfig;


PROCEDURE HandleGadgetEvent(gad: I.GadgetPtr; code: INTEGER);

BEGIN

   CASE gad.gadgetID OF
   
      gEXIT       : HALT(0)
                    |
      gSELECT     : GetFile();
                    |
      gPRINT      : Print();
                    |
      gSAVE       : SaveConfig(TRUE,FALSE)
                    |
      gCycleFonts : gstate.prfontnum := code
                    |
      gCycleCPI   : gstate.cpinum := code
                    |
      gINTLeft    : strinfo := gad.specialInfo;
                    gstate.leftnum := SHORT(strinfo.longInt);
                    IF gstate.leftnum <= 0 THEN
                      GT.SetGadgetAttrs(gad^,winP,NIL,GT.inNumber,0,u.done);
                      gstate.leftnum := 0;
                    END;
                    |
      gINTRight   : strinfo := gad.specialInfo;
                    gstate.rightnum := SHORT(strinfo.longInt);
                    findGad := GetGadget(winP.firstGadget,gINTLeft);
                    findinfo := findGad.specialInfo;
                    IF strinfo.longInt <= findinfo.longInt+2 THEN
                      gstate.rightnum := SHORT(findinfo.longInt)+3;
                      GT.SetGadgetAttrs(gad^,winP,NIL,GT.inNumber,gstate.rightnum,u.done);
                    ELSIF strinfo.longInt > 255 THEN
                      gstate.rightnum := 255;
                      GT.SetGadgetAttrs(gad^,winP,NIL,GT.inNumber,gstate.rightnum,u.done);
                    END;
                    |
      gINTtab     : strinfo := gad.specialInfo;
                    IF strinfo.longInt < 1 THEN
                      GT.SetGadgetAttrs(gad^,winP,NIL,GT.inNumber,1,u.done);
                    ELSIF strinfo.longInt > 10 THEN
                      GT.SetGadgetAttrs(gad^,winP,NIL,GT.inNumber,10,u.done);
                    END;
                    findGad := GetGadget(winP.firstGadget,gTabSlide);
                    GT.SetGadgetAttrs(findGad^,winP,NIL,
                                      GT.slLevel,strinfo.longInt,u.done);
                    gstate.tabnum := SHORT(strinfo.longInt);
                    |
      gTabSlide   : findGad := GetGadget(winP.firstGadget,gINTtab);
                    GT.SetGadgetAttrs(findGad^,winP,NIL,
                                      GT.inNumber,code,u.done);
                    gstate.tabnum := code;
                    |
      gFName      : strinfo := gad.specialInfo;
                    COPY(strinfo.buffer^,FileName);
                    IF ~ FS.Exists(FileName) THEN
                       Inform("File does not exist!");
                       IF I.ActivateGadget(gad^,winP,NIL) THEN END;
                    ELSE GetFileSize();
                    END;
                    |
   ELSE
   END;

END HandleGadgetEvent;


PROCEDURE About();

TYPE mestype = ARRAY 3 OF ARRAY 50 OF CHAR;

VAR message : mestype;

   PROCEDURE Int2Str(num: LONGINT);

   VAR    ch: ARRAY 1 OF CHAR;

   BEGIN

     WHILE num>0 DO
       ch[0] := CHR(num MOD 10 + ORD("0"));
       Strings.Insert(message[2],0,ch);
       num := num DIV 10;
     END;

   END Int2Str;

BEGIN

   message[0] := "LQPrint version 2.0    1992 Achim Siebert";
   message[1] := "Size of selected file: ";
   IF FileName # ""
   THEN Int2Str(filesize);
        Strings.Append(message[1],message[2]);
        Strings.Append(message[1]," chars");
   ELSE Strings.Append(message[1], "(none selected)");
   END;
   message[2] := "This program is shareware!";
   IF AReq20.ARequest(message,"LQPrint"," Okay ","","",NIL)#0 THEN END;

END About;


PROCEDURE HandleMenuEvent(selMenu,selItem:INTEGER);

BEGIN

   IF selMenu = 0 THEN
      CASE selItem OF
         0: GetFile() |
         2: Print(); |
         4: About(); |
         6: HALT(0) |
      ELSE
      END;
   ELSIF selMenu = 1 THEN
      CASE selItem OF
         0: OpenConfig(TRUE,FALSE) |
         1: OpenConfig(FALSE,FALSE)|
         2: SaveConfig(TRUE,FALSE) |
         3: SaveConfig(FALSE,FALSE)|
         4: gstate := gstateOld; SetStates()|
         6: cricon := ~cricon|
      ELSE
      END;
   ELSE
   END;

END HandleMenuEvent;


BEGIN  (*----------------------------MAIN-------------------------------------*)


  cricon := OL.wbStarted;
  wbStarted := cricon;
  
  DirName := version; DirName := "";   (* dummy to preserve version string *)

  IF I.int.libNode.version<37 THEN
     Inform("Requires KickStart version 37.xx");
     HALT(10);
  END;

  IF Dos.GetCurrentDirName(DirName,256) THEN
     IF (DirName#"") AND (DirName[Strings.Length(DirName)-1] # ":") THEN
        Strings.AppendChar(DirName,"/");
     END;
     Arguments.GetArg(0,reqarray[0]);
     Strings.Append(DirName,reqarray[0]); numargs := Arguments.NumArgs();
  END;

  confloaded := FALSE;

  ConfID := configID;
  
  IF ~ wbStarted THEN
     
     anchor.breakBits := LONGSET{Dos.ctrlC,Dos.ctrlD,Dos.ctrlE,Dos.ctrlF};
     anchor.strLen := 256;
     anchor.flags := SHORTSET{Dos.doWild};
     rdargs := Dos.ReadArgs("File(s)/M,NW=NOWINDOW/S,SETTINGS=FROM/K",argarray,NIL);
     
     IF rdargs = NIL THEN Inform("Usage: LQPrint File(s)/M,NW=NOWINDOW/S,SETTINGS=FROM/K\n"); HALT(0) END;
     
     IF argarray[2]#0 THEN setnamePtr := S.VAL(Exec.STRPTR,argarray[2]);
        COPY(setnamePtr^,PathName);
        IF ~CliConfig(PathName) THEN
            Inform("Opening settings file failed, using defaults!\n");
            OpenConfig(FALSE,TRUE);
        END;
     ELSE OpenConfig(FALSE,TRUE);
     END;
     confloaded := TRUE;
     IF argarray[1]#0 THEN
        IF argarray[0]#0 THEN
           fnames := S.VAL(Exec.APTR,argarray[0]);
           i := 0;
           WHILE (i < 100) AND (fnames[i] # NIL) DO
              rc := Dos.MatchFirst(fnames[i]^,anchor);
              WHILE rc = 0 DO
                 COPY(anchor.buf,FileName);
                 Inform("Printing: ");Inform(FileName);Inform("\n");
                 Print();
                 rc := Dos.MatchNext(anchor);
              END;
              IF rc = 205 THEN Inform(fnames[i]^);Inform(" not found!\n"); END;
              Dos.MatchEnd(anchor);
              INC(i);
           END;
        END;
     ELSE
        wbStarted := TRUE;
     END;
  END;

  IF wbStarted THEN

     IF ~confloaded THEN OpenConfig(FALSE,TRUE) END;

     OpenWindow();
       
     IF confloaded THEN
        IF argarray[0]#0 THEN
           fnames := S.VAL(Exec.APTR,argarray[0]);
           i := 0;
           WHILE (i < 100) AND (fnames[i] # NIL) DO
              rc := Dos.MatchFirst(fnames[i]^,anchor);
              WHILE rc = 0 DO
                 COPY(anchor.buf,FileName);
                 SetFileGad();
                 GetFileSize();
                 Print();
                 rc := Dos.MatchNext(anchor);
              END;
              Dos.MatchEnd(anchor);
              INC(i);
           END;
        END;
     ELSIF numargs > 0 THEN
        Arguments.GetArg(1,PathName);
        IF CliConfig(PathName) THEN SetStates();
        ELSE COPY(PathName,FileName); SetFileGad(); Print();
        END;
        FOR argcount := 2 TO numargs DO
        Arguments.GetArg(argcount,FileName);
        SetFileGad();
        Print();
        END;
     END;

     LOOP
        REPEAT UNTIL Exec.Wait(LONGSET{winP.userPort.sigBit}) # LONGSET{};
        LOOP
          imsg := GT.GetIMsg(winP.userPort);
          IF imsg=NIL THEN EXIT END;
          imsgClass := imsg.class;
          imsgCode := imsg.code;
          
          gad := imsg.iAddress;
          
          GT.ReplyIMsg(imsg);
          
          IF I.refreshWindow IN imsgClass THEN
            GT.BeginRefresh(winP);
            GT.EndRefresh(winP, I.LTRUE);
          END;
          IF I.closeWindow IN imsgClass THEN
            HALT(0);
          END;
          IF I.gadgetUp IN imsgClass THEN
             HandleGadgetEvent(gad,imsgCode);
          END;
          IF I.menuPick IN imsgClass THEN
            WHILE imsgCode # I.menuNull DO
               selectedMenu := I.MenuNum(imsgCode);
               selectedItem := I.ItemNum(imsgCode);
               HandleMenuEvent(selectedMenu,selectedItem);
               nextMenu := I.ItemAddress(mlist^,imsgCode);
               imsgCode := nextMenu.nextSelect;
            END;
          END;
        END;
     END;
  END;

CLOSE

   IF wbStarted THEN SaveConfig(FALSE,TRUE) END;

   IF (Printer  # NIL) AND (Dos.Close(Printer)) THEN END;
   IF (TextFile # NIL) AND (Dos.Close(TextFile)) THEN END; 

   IF winP # NIL THEN I.CloseWindow(winP) END;

   IF glist # NIL THEN GT.FreeGadgets(glist) END;
   IF stopglist # NIL THEN GT.FreeGadgets(stopglist) END;
   IF vi # NIL THEN GT.FreeVisualInfo(vi) END;

   IF mlist # NIL THEN Menu.DisposeMenu(mlist) END;

   IF font # NIL THEN g.CloseFont(font) END;
   
   IF rdargs # NIL THEN Dos.FreeArgs(rdargs) END;

END LQPrint.
