(*************************************************************************

:Program.       PL0Parser.mod
:Contents.      Parser for PL0-Complier
:Author.        N. With, ported to Oberon by hartmut Goebel
:Language.      Oberon
:Translator.    Amiga Oberon

:Imports.       TextWindows (hartmut Goebel)
:Imports.       MultiMem (hartmut Goebel)

*************************************************************************)

MODULE PL0Parser;

IMPORT
  scan: PL0Scanner,
  gen: PL0Generator,
  i: PL0Interpreter,
  mm: MultiMem,
  sys: SYSTEM,
  ol: OberonLib,
  tw: TextWindows;

VAR
  noerr*: BOOLEAN;
  heap: mm.HeapPtr;

TYPE
  ObjPtr = POINTER TO Object;
  HeaderPtr = POINTER TO Header;
  Object = RECORD
    name: INTEGER;
    next: ObjPtr;
  END;
  Const = RECORD (Object)
    val: INTEGER;
  END;
  Var = RECORD (Object)
    lev, adr: INTEGER;
  END;
  Proc = RECORD (Object)
    lev, adr, size: INTEGER;
  END;
  Header = RECORD (Object)
    last, down: ObjPtr;
  END;

CONST
  (* object type *)
  tConst = 0;
  tVar = 1;
  tProc = 2;
  tHeader = 3;

VAR
  topScope: HeaderPtr;
  bottom: ObjPtr;
  win: tw.TxtWinPtr;
  curlev: INTEGER;

PROCEDURE err(n: INTEGER);
BEGIN
  noerr := FALSE;
  scan.Mark(n);
  tw.Inversid(win);
  tw.WriteInt(win,n,2);
  tw.Inversid(win);
END err;


PROCEDURE test(s: INTEGER; n: INTEGER);
BEGIN
  IF scan.sym < s THEN err(n);
    REPEAT scan.GetSym UNTIL s >= s;
  END;
END test;


PROCEDURE NewObj(k: INTEGER): ObjPtr;
VAR
  obj: ObjPtr;
BEGIN (* check for multible definition *)
  obj := topScope.next;
  WHILE obj # NIL DO
    IF scan.Diff(scan.id,obj.name) = 0 THEN err(25); END;
    obj := obj.next;
  END;
  (* now enter object into list *)
  CASE k OF (* $TypeChk- *)
    tVar:    mm.New(heap,obj,sys.SIZE(Var));    sys.INIT(obj(Var)); |
    tConst:  mm.New(heap,obj,sys.SIZE(Const));  sys.INIT(obj(Const)); |
    tProc:   mm.New(heap,obj,sys.SIZE(Proc));   sys.INIT(obj(Proc)); |
    tHeader: mm.New(heap,obj,sys.SIZE(Header)); sys.INIT(obj(Header));
  END; (* $TypeChk= *)
  IF obj=NIL THEN HALT(20); END;
  obj.name := scan.id;
  obj.next := NIL;
  scan.KeepId;
  topScope.last.next := obj;
  topScope.last := obj;
  RETURN obj;
END NewObj;


PROCEDURE find(id: INTEGER): ObjPtr;
VAR
  hd: HeaderPtr;
  obj: ObjPtr;
BEGIN
  hd := topScope;
  WHILE hd # NIL DO
    obj := hd.next;
    WHILE obj#NIL DO
      IF scan.Diff(id,obj.name) = 0 THEN RETURN obj;
      ELSE obj := obj.next;
      END;
    END;
    hd := hd.down(Header);
  END;
  err(11); RETURN NIL;
END find;


PROCEDURE expression;
VAR
  addop: INTEGER;

  PROCEDURE factor;
  VAR
    obj: ObjPtr;
  BEGIN
    tw.WriteString(win,"factor\n");
    test(scan.lparen,6);
    IF scan.sym = scan.ident THEN
      obj := find(scan.id);
      IF obj IS Const THEN gen.Gen(i.LIT,0, obj(Const).val)
      ELSIF obj IS Var THEN gen.Gen(i.LOD,curlev-obj(Var).lev,obj(Var).adr)
      ELSIF obj IS Proc THEN err(21)
      ELSE END;
      scan.GetSym;
    ELSIF scan.sym = scan.number THEN
      gen.Gen(i.LIT,0,scan.num);
      scan.GetSym;
    ELSIF scan.sym = scan.lparen THEN
      scan.GetSym;
      expression;
      IF scan.sym = scan.rparen THEN scan.GetSym; ELSE err(7); END;
    ELSE err(8);
    END;
  END factor;

  PROCEDURE term;
  VAR
    mulopt: INTEGER;
  BEGIN
    tw.WriteString(win,"term\n");
    factor;
    WHILE (scan.times <= scan.sym) & (scan.sym <= scan.div) DO
      mulopt := scan.sym;
      scan.GetSym;
      factor;
      IF mulopt = scan.times THEN gen.Gen(i.OPR,0,i.times);
      ELSE gen.Gen(i.OPR,0,i.div);
      END;
    END;
  END term;

BEGIN
  tw.WriteString(win,"expression\n");
  IF (scan.plus <= scan.sym) & (scan.sym <= scan.minus) THEN
    addop := scan.sym;
    scan.GetSym;
    term;
    IF addop = i.minus THEN gen.Gen(i.OPR,0,i.neg); END;
  ELSE term;
  END;
  WHILE (scan.plus <= scan.sym) & (scan.sym <= scan.minus) DO
    addop := scan.sym;
    scan.GetSym;
    term;
    IF addop = i.plus THEN gen.Gen(i.OPR,0,i.plus);
    ELSE gen.Gen(i.OPR,0,i.minus);
    END;
  END;
END expression;


PROCEDURE condition;
VAR
  relop: INTEGER;
BEGIN
  tw.WriteString(win,"condition\n");
  IF scan.sym = scan.odd THEN
    scan.GetSym;
    expression;
    gen.Gen(i.OPR,0,i.odd);
  ELSE
    expression;
    IF (scan.eql <= scan.sym) & (scan.sym <= scan.geq) THEN
      relop := scan.sym;
      scan.GetSym;
      expression;
      CASE relop OF
        scan.eql : gen.Gen(i.OPR,0,i.eql);|
        scan.neq : gen.Gen(i.OPR,0,i.neq);|
        scan.lss : gen.Gen(i.OPR,0,i.lss);|
        scan.geq : gen.Gen(i.OPR,0,i.geq);|
        scan.gtr : gen.Gen(i.OPR,0,i.gtr);|
        scan.leq : gen.Gen(i.OPR,0,i.leq);|
      END;
    ELSE err(20);
    END;
  END;
END condition;


PROCEDURE statement;
VAR
  obj: ObjPtr;
  L0, L1: INTEGER;
BEGIN
  tw.WriteString(win,"statement\n");
  test(scan.ident,10);
  IF scan.sym = scan.ident THEN
    obj := find(scan.id);
    IF NOT (obj IS Var) THEN
      err(12); obj := NIL; END;
    scan.GetSym;
    IF scan.sym = scan.becomes THEN scan.GetSym;
    ELSE err(13);
    END;
    expression;
    IF obj # NIL THEN
      gen.Gen(i.STO,curlev-obj(Var).lev,obj(Var).adr); (* store *)
    END;
  ELSIF scan.sym = scan.call THEN
    scan.GetSym;
    IF scan.sym = scan.ident THEN
      obj := find(scan.id);
      IF obj IS Proc THEN
        gen.Gen(i.CAL,curlev-obj(Proc).lev,obj(Proc).adr); (* call *)
      ELSE err(15);
      END;
      scan.GetSym;
    ELSE err(14);
    END;
  ELSIF scan.sym = scan.begin THEN
    scan.GetSym;
    LOOP
      statement;
      IF scan.sym = scan.semicolon THEN scan.GetSym;
      ELSIF scan.sym = scan.end THEN scan.GetSym; EXIT;
      ELSIF scan.sym < scan.const THEN err (17);
      ELSE err(17); EXIT;
      END;
    END;
  ELSIF scan.sym = scan.if THEN
    scan.GetSym;
    condition;
    IF scan.sym = scan.then THEN scan.GetSym; ELSE err(16); END;
    L0 := gen.Label();
    gen.Gen(i.JPC,0,0);
    statement;
    gen.fixup(L0);
  ELSIF scan.sym = scan.while THEN
    L0 := gen.Label();
    scan.GetSym;
    condition;
    L1 := gen.Label();
    gen.Gen(i.JPC,0,0);
    IF scan.sym = scan.do THEN scan.GetSym; ELSE err(18); END;
    statement;
    gen.Gen(i.JMP,0,L0);
    gen.fixup(L1);
  ELSIF scan.sym = scan.read THEN
    scan.GetSym;
    IF scan.sym = scan.ident THEN
      obj := find(scan.id);
      IF obj IS Var THEN
        gen.Gen(i.OPR,0,i.readVar);
        gen.Gen(i.STO,curlev-obj(Var).lev,obj(Var).adr);
      ELSE
        err(12)
      END;
    ELSE err(14);
    END;
    scan.GetSym;
  ELSIF scan.sym = scan.write THEN
    scan.GetSym;
    expression;
    gen.Gen(i.OPR,0,i.writeVar);
  END;
  test(scan.ident,19);
END statement;


PROCEDURE block;
VAR
  adr: INTEGER; (* data address *)
  L0: INTEGER; (* internal code index *)
  hd: HeaderPtr;
  obj: ObjPtr;

  PROCEDURE ConstDeclaration;
  VAR
    obj: ObjPtr;
  BEGIN
    tw.WriteString(win,"ConstDeclaration\n");
    IF scan.sym = scan.ident THEN
      scan.GetSym;
      IF (scan.sym = scan.eql) OR (scan.sym = scan.const) THEN
        IF scan.sym = scan.becomes THEN err(1); END;
        scan.GetSym;
        IF scan.sym = scan.number THEN
          obj := NewObj(tConst);
          obj(Const).val := scan.num;
          scan.GetSym;
        ELSE err(2);
        END;
      ELSE err(3);
      END;
    ELSE err(4);
    END;
  END ConstDeclaration;

  PROCEDURE VarDeclaration;
  VAR
    obj: ObjPtr;
  BEGIN
    tw.WriteString(win,"VarDeclaration\n");
    IF scan.sym = scan.ident THEN
      obj := NewObj(tVar);
      scan.GetSym;
      obj(Var).lev := curlev;
      obj(Var).adr := adr;
      INC(adr);
    ELSE err(4);
    END;
  END VarDeclaration;

BEGIN
  tw.WriteString(win,"block\n");
  INC(curlev); adr := 3;
  mm.New(heap,hd,sys.SIZE(hd));
  IF hd=NIL THEN HALT(20); END;
  hd.name := 0;
  hd.next := NIL;
  hd.last := hd;
  hd.down := topScope;
  topScope := hd;
  L0 := gen.Label();
  gen.Gen(i.JMP,0,0); (* jump *)
  IF scan.sym = scan.const THEN
    scan.GetSym;
    LOOP
      ConstDeclaration;
      IF scan.sym = scan.comma THEN scan.GetSym;
      ELSIF scan.sym = scan.semicolon THEN scan.GetSym; EXIT;
      ELSIF scan.sym = scan.ident THEN err(5);
      ELSE err(5); EXIT;
      END;
    END;
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
  END;
  IF scan.sym = scan.var THEN
    scan.GetSym;
    LOOP
      VarDeclaration;
      IF scan.sym = scan.comma THEN scan.GetSym;
      ELSIF scan.sym = scan.semicolon THEN scan.GetSym; EXIT;
      ELSIF scan.sym = scan.ident THEN err(5);
      ELSE err(5); EXIT;
      END;
    END;
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
  END;
  WHILE scan.sym = scan.procedure DO
    scan.GetSym;
    IF scan.sym = scan.ident THEN scan.GetSym; ELSE err(4); END;
    obj := NewObj(tProc);
    obj(Proc).lev := curlev;
    obj(Proc).adr := gen.Label();
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
    block;
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
  END;
  gen.fixup(L0);
  gen.Gen(i.INT,0,adr); (* enter *)
  statement;
  gen.Gen(i.OPR,0,i.ret); (* return *)
  topScope := topScope.down(Header);
  DEC(curlev);
END block;


PROCEDURE Parse*;
BEGIN
  noerr := TRUE;
  topScope := NIL;
  tw.ClrHome(win); (* FreeMem *)
  scan.GetSym;
  block;
  IF scan.sym # scan.period THEN err(9); END;
END Parse;

BEGIN
 win := tw.OpenTextWin("PARSE",408,110,204,140);
 IF (win=NIL) THEN HALT(20) END;
 mm.NewHeap(heap);
 IF heap=NIL THEN HALT(20) END;

CLOSE
  IF heap#NIL THEN mm.DisposeHeap(heap); END;
  IF win # NIL THEN tw.CloseTextWin(win); END;
END PL0Parser.


