
MODULE PL0ParserKap8;

IMPORT
  scan: PL0Scanner,
  gen: PL0Generator,
  i: PL0Interpreter,
  mm: MultiMem,
  sys: SYSTEM,
  tw: TextWindows;

VAR
  noerr*: BOOLEAN;
  heap: mm.HeapPtr;

TYPE
  ObjPtr = POINTER TO Object;
  Object = RECORD
    name: INTEGER;
    next: ObjPtr;
    kind: INTEGER;
    last, down: ObjPtr;
  END;

CONST
  (* Object.kind *)
  Const = 0;
  Var = 1;
  Proc = 2;
  Header = 3;

VAR
  topScope, bottom, undef: ObjPtr;
  win: tw.TxtWinPtr;
  curlev: INTEGER;

PROCEDURE err(n: INTEGER);
BEGIN
  noerr := FALSE;
  scan.Mark(n);
  tw.Inversid(win);
  tw.WriteInt(win,n,2);
  tw.Inversid(win);
END err;


PROCEDURE test(s: INTEGER; n: INTEGER);
BEGIN
  IF scan.sym < s THEN err(n);
    REPEAT scan.GetSym UNTIL s >= s;
  END;
END test;

PROCEDURE NewObj(k: INTEGER): ObjPtr;
VAR
  obj: ObjPtr;
BEGIN (* check for multible definition *)
  obj := topScope.next;
  WHILE obj # NIL DO
    IF scan.Diff(scan.id,obj.name) = 0 THEN err(25); END;
    obj := obj.next;
  END;
  (* now enter object into list *)
  mm.New(heap,obj,sys.SIZE(obj));
  IF obj=NIL THEN HALT(20); END;
  obj.name := scan.id;
  obj.kind := k;
  obj.next := NIL;
  scan.KeepId;
  topScope.last.next := obj;
  topScope.last := obj;
  RETURN obj;
END NewObj;


PROCEDURE find(id: INTEGER): ObjPtr;
VAR
  hd, obj: ObjPtr;
BEGIN
  hd := topScope;
  WHILE hd # NIL DO
    obj := hd.next;
    WHILE obj#NIL DO
      IF scan.Diff(id,obj.name) = 0 THEN RETURN obj;
      ELSE obj := obj.next;
      END;
    END;
    hd := hd.down;
  END;
  err(11); RETURN NIL;
END find;


PROCEDURE expression;
VAR
  addop: INTEGER;

  PROCEDURE factor;
  VAR
    obj: ObjPtr;
  BEGIN
    tw.WriteString(win,"factor\n");
    test(scan.lparen,6);
    IF scan.sym = scan.ident THEN
      obj := find(scan.id);
      CASE obj.kind OF
        Const: gen.Gen(i.LIT,0, obj.val) |
        Var: gen.Gen(iLOD,curlev-vlev,vadr) |
        Proc: err(21) |
      ELSE END;
      scan.GetSym;
    ELSIF scan.sym = scan.number THEN
      gen.Gen(i.LIT,0,num);
      scan.GetSym;
    ELSIF scan.sym = scan.lparen THEN
      scan.GetSym;
      expression;
      IF scan.sym = scan.rparen THEN scan.GetSym; ELSE err(7); END;
    ELSE err(8);
    END;
  END factor;

  PROCEDURE term;
  VAR
    mulopt: INTEGER;
  BEGIN
    tw.WriteString(win,"term\n");
    factor;
    WHILE (scan.times <= scan.sym) & (scan.sym <= scan.div) DO
      mulopt := scan.sym;
      scan.GetSym;
      factor;
      IF mulopt = scan.times THEN gen.Gen(i.OPR,0,i.times);
      ELSE gen.Gen(i.OPR,0,i.div);
      END;
    END;
  END term;

BEGIN
  tw.WriteString(win,"expression\n");
  IF (scan.plus <= scan.sym) & (scan.sym <= scan.minus) THEN
    addop := scan.sym;
    scan.GetSym;
    term;
    IF addop = i.minus THEN gen.Gen(i.OPR,0,i.neg);
  ELSE term;
  END;
  WHILE (scan.plus <= scan.sym) & (scan.sym <= scan.minus) DO
    addop := scan.sym;
    scan.GetSym;
    term;
    IF addop = i.plus THEN gen.Gen(i.OPR,0,i.plus);
    ELSE gen.Gen(i.OPR,0,i.minus);
    END;
  END;
END expression;


PROCEDURE condition;
VAR
  relop: INTEGER;
BEGIN
  tw.WriteString(win,"condition\n");
  IF scan.sym = scan.odd THEN
    scan.GetSym;
    expression;

  ELSE
    expression;
    IF (scan.eql <= scan.sym) & (scan.sym <= scan.geq) THEN
      scan.GetSym;
      expression;
    ELSE err(20);
    END;
  END;
END condition;


PROCEDURE statement;
VAR
  obj: ObjPtr;
BEGIN
  tw.WriteString(win,"statement\n");
  IF scan.sym = scan.ident THEN
    obj := find(scan.id);
    scan.GetSym;
    IF scan.sym = scan.becomes THEN scan.GetSym;
    ELSE err(13);
    END;
    expression;
  ELSIF scan.sym = scan.call THEN
    scan.GetSym;
    IF scan.sym = scan.ident THEN
      obj := find(scan.id);
      scan.GetSym;
    ELSE err(14);
    END;
  ELSIF scan.sym = scan.begin THEN
    scan.GetSym;
    statement;
    WHILE scan.sym = scan.semicolon DO
      scan.GetSym;
      statement;
    END;
    IF scan.sym = scan.end THEN scan.GetSym; ELSE err(17); END;
  ELSIF scan.sym = scan.if THEN
    scan.GetSym;
    condition;
    IF scan.sym = scan.then THEN scan.GetSym; ELSE err(16); END;
    statement;
  ELSIF scan.sym = scan.while THEN
    scan.GetSym;
    condition;
    IF scan.sym = scan.do THEN scan.GetSym; ELSE err(18); END;
    statement;
  ELSIF scan.sym = scan.read THEN
    scan.GetSym;
    IF scan.sym = scan.ident THEN obj := find(scan.id);
    ELSE err(14);
    END;
    scan.GetSym;
  ELSIF scan.sym = scan.write THEN
    scan.GetSym;
    expression;
  END;
END statement;


PROCEDURE block;
VAR
  hd, obj: ObjPtr;

  PROCEDURE ConstDeclaration;
  VAR
    obj: ObjPtr;
  BEGIN
    tw.WriteString(win,"ConstDeclaration\n");
    IF scan.sym = scan.ident THEN
      scan.GetSym;
      IF scan.sym = scan.eql THEN
        scan.GetSym;
        IF scan.sym = scan.number THEN
          obj := NewObj(Const);
          scan.GetSym;
        ELSE err(2);
        END;
      ELSE err(3);
      END;
    ELSE err(4);
    END;
  END ConstDeclaration;

  PROCEDURE VarDeclaration;
  VAR
    obj: ObjPtr;
  BEGIN
    tw.WriteString(win,"VarDeclaration\n");
    IF scan.sym = scan.ident THEN
      obj := NewObj(Var);
      scan.GetSym;
    ELSE err(4);
    END;
  END VarDeclaration;

BEGIN
  tw.WriteString(win,"block\n");
  mm.New(heap,hd,sys.SIZE(hd));
  IF hd=NIL THEN HALT(20); END;
  hd.kind := Header;
  hd.name := 0;
  hd.next := NIL;
  hd.last := hd;
  hd.down := topScope;
  topScope := hd;
  IF scan.sym = scan.const THEN
    scan.GetSym;
    ConstDeclaration;
    WHILE scan.sym = scan.comma DO
      scan.GetSym;
      ConstDeclaration;
    END;
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
  END;
  IF scan.sym = scan.var THEN
    scan.GetSym;
    VarDeclaration;
    WHILE scan.sym = scan.comma DO
      scan.GetSym;
      VarDeclaration;
    END;
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
  END;
  WHILE scan.sym = scan.procedure DO
    scan.GetSym;
    IF scan.sym = scan.ident THEN scan.GetSym; ELSE err(4); END;
    obj := NewObj(Proc);
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
    block;
    IF scan.sym = scan.semicolon THEN scan.GetSym; ELSE err(5); END;
  END;
  statement;
  topScope := topScope.down;
END block;


PROCEDURE Parse;
BEGIN
  noerr := TRUE;
  topScope := NIL;
  tw.ClrHome(win); (* FreeMem *)
  scan.GetSym;
  block;
  IF scan.sym # scan.period THEN err(9); END;
  mm.EmptyHeap(heap);
END Parse;

BEGIN
  win := tw.OpenTextWin("PARSE",408,110,204,140);
  IF (win=NIL) THEN HALT(20) END;
  mm.NewHeap(heap);
  IF heap=NIL THEN HALT(20) END;

CLOSE
  IF heap#NIL THEN mm.DisposeHeap(heap); END;
  IF win # NIL THEN tw.CloseTextWin(win); END;
END PL0ParserKap8.


